/*	disp-lbp.c
	interface between fortran and graphics/printer subroutines
	entry names in this library are based on DISP convention

	functions with "_" at the end can be called from fortran
	fortran subroutine names should not exceed 16 characters

	int = long (4-byte) integer = fortran integer
*/

#include <stdio.h>
#include <string.h>
#include <math.h>

int lbp_echo, disp_echo;

hcopy_()
{
	if (lbp_echo)	LBP_hcopy_();
	else		DISP_hcopy_();
}

initialize_(initmode)
	int *initmode;
{
	int vpunits = 1;
	/*	default viewport units = 1/10mm	*/

	if (*initmode < 0 ) 
		disp_echo = 0;
	else
		disp_echo = 1;

	if ( (0 < abs(*initmode) && abs(*initmode) <= 6) ||
	     (10 < abs(*initmode) && abs(*initmode) <= 16) )
		lbp_echo = 1;
	else
		lbp_echo = 0;

	if (disp_echo) DISP_initialize_(initmode);
	if (lbp_echo) LBP_initialize_(initmode);
	viewportunits_(&vpunits);
}

reset_()
{
	if (disp_echo) DISP_reset_();
	if (lbp_echo) LBP_reset_();
}

terminate_()
{
	if (disp_echo) DISP_terminate_();
	if (lbp_echo) LBP_terminate_();
}

redraw_()
{
	if (disp_echo) return(DISP_redraw_());
	return(0);
}

clear_()
{
	if (disp_echo) DISP_clear_();
	if (lbp_echo) LBP_clear_();
}

bgcolor_(r,g,b)
	int *r,*g,*b;
{
	if (disp_echo) DISP_bgcolor_(r,g,b);
	if (lbp_echo) LBP_bgcolor_(r,g,b);
}

vdcextent_(x1,y1,x2,y2)
	int *x1, *y1, *x2, *y2;
{
	if (disp_echo) DISP_vdcextent_( x1, y1, x2, y2 );
	if (lbp_echo) LBP_vdcextent_( x1, y1, x2, y2 );
}

viewportunits_(units)
	int *units;
/*	units=0:				relative
	     =1(default here):		in 1/10 mm
	     =2:				absolute		*/
{
	if (disp_echo) DISP_viewportunits_(units);
	if (lbp_echo) LBP_viewportunits_(units);
}

viewport_(x1,y1,x2,y2)
	int *x1,*y1,*x2,*y2;
/*	viewport specification
	this routine can be used for viewportunits=1,2	*/
{
	if (disp_echo) DISP_viewport_( x1, y1, x2, y2 );
	if (lbp_echo) LBP_viewport_( x1, y1, x2, y2 );
}

viewportf_( fx1,fy1,fx2,fy2)
      float *fx1,*fy1,*fx2,*fy2;
/*    viewport specification
      this routine can be used for viewportunits=0  */
{
      if (disp_echo) DISP_viewportf_( fx1, fy1, fx2, fy2 );
      if (lbp_echo) LBP_viewportf_( fx1, fy1, fx2, fy2 );
}

viewportmapping_(isotropy,horizontal,vertical)
	int *isotropy,*horizontal,*vertical;
/*	isotropy=0(default here):	not forced
	        =1(news-cgi default):	forced
	horizontal=0:		center
	          =1(default):	left
	          =2:		right
	vertical=0:		center
	        =3:		top
	        =4(default):	bottom		*/
{
	if (disp_echo) DISP_viewportmapping_(isotropy,horizontal,vertical);
	if (lbp_echo) LBP_viewportmapping_(isotropy,horizontal,vertical);
}

clipindicator_(indicator)
	int *indicator;
/*	indicator=0(default):	clip rectangle
	         =1:		view surface
	         =2:		off		*/
{
	if (disp_echo) DISP_clipindicator_(indicator);
	if (lbp_echo) LBP_clipindicator_(indicator);
}

cliprectangle_(x1,y1,x2,y2)
	int *x1,*y1,*x2,*y2;
{
	if (disp_echo) DISP_cliprectangle_( x1, y1, x2, y2 );
	if (lbp_echo) LBP_cliprectangle_( x1, y1, x2, y2 );
}

polyline_(np,x,y)
	int *np,x[],y[];
{

	if (disp_echo) DISP_polyline_( np, x, y );
	if (lbp_echo) LBP_polyline_( np, x, y );
}

disjointline_(np,x,y)
	int *np,x[],y[];
{
	if (disp_echo) DISP_disjointline_(np, x, y );
	if (lbp_echo) LBP_disjointline_(np, x, y );
}

arc_(cx,cy,r,angles,anglee)
	int *cx,*cy,*r;
	float *angles,*anglee;
{
	if (disp_echo) DISP_arc_(cx,cy,r,angles,anglee);
	if (lbp_echo) LBP_arc_(cx,cy,r,angles,anglee);
}

arc3_(x1,y1,x2,y2,x3,y3)
	int *x1,*y1,*x2,*y2,*x3,*y3;
{
	if (disp_echo) DISP_arc3_(x1,y1,x2,y2,x3,y3);
	if (lbp_echo) LBP_arc3_(x1,y1,x2,y2,x3,y3);
}

elliparc_(cx,cy,a,b,anglea,angles,anglee)
	int *cx,*cy,*a,*b;
	float *anglea,*angles,*anglee;
{
	if(disp_echo) DISP_elliparc_(cx,cy,a,b,anglea,angles,anglee);
	if(lbp_echo) LBP_elliparc_(cx,cy,a,b,anglea,angles,anglee);
}

linetype_(type)
	int *type;
/*	type=1(default):	solid
	    =2:			dashed
	    =3:			dotted
	    =4:			dash-dot
	    =5:			dash-dot-dot		*/
{
	if (disp_echo) DISP_linetype_(type);
	if (lbp_echo) LBP_linetype_(type);
}

linewidth_(iwidth)
	int *iwidth;
/*	linewidth specification in absolute mode */
{
	if (disp_echo) DISP_linewidth_(iwidth);
	if (lbp_echo) LBP_linewidth_(iwidth);
}

linefwidth_(fwidth)
	float *fwidth;
/*	linewidth specification in scaled mode */
{
	if (disp_echo) DISP_linefwidth_(fwidth);
	if (lbp_echo) LBP_linefwidth_(fwidth);
}

linecolor_(r,g,b)
	int *r,*g,*b;
/*	r is also used as color index	*/
{
	if (disp_echo) DISP_linecolor_(r,g,b);
	if (lbp_echo) LBP_linecolor_(r,g,b);
}

polymarker_(np,x,y)
	int *np,x[],y[];
{
	if (disp_echo) DISP_polymarker_(np, x, y );
	if (lbp_echo) LBP_polymarker_(np, x, y );
}

markertype_(type)
	int *type;
/*	type=1:		dot
	    =2:		plus
	    =3(default)	asterisk
	    =4:		circle
	    =5:		cross			*/
{
	if (disp_echo) DISP_markertype_(type);
	if (lbp_echo) LBP_markertype_(type);
}

markersize_(isize)
	int *isize;
/*	marker size specification in absolute mode	*/
{
	if (disp_echo) DISP_markersize_(isize);
	if (lbp_echo) LBP_markersize_(isize);
}

markerfsize_(fsize)
	float *fsize;
/*	marker size specification in scaled mode	*/
{
	if (disp_echo) DISP_markerfsize_(fsize);
	if (lbp_echo) LBP_markerfsize_(fsize);
}

markercolor_(r,g,b)
	int *r,*g,*b;
/*	r is also used as color index	*/
{
	if (disp_echo) DISP_markercolor_(r,g,b);
	if (lbp_echo) LBP_markercolor_(r,g,b);
}

text_(x,y,string,len)
	int *x,*y,len;
	char *string;
/*	FORTRAN calling sequence: call text(x,y,string)	*/
{
	if (disp_echo) DISP_text_( x, y, string, len);
	if (lbp_echo) LBP_text_( x, y, string, len);
}

textnf_(x,y,string,len)
	int *x,*y,len;
	char *string;
{
	if (disp_echo) DISP_textnf_( x, y, string, len);
	if (lbp_echo) LBP_textnf_( x, y, string, len);
}

restrictedtext_(wx,wy,x,y,flag,string,len)
	int *wx,*wy,*x,*y,*flag,len;
	char *string;
/*	FORTRAN calling sequence: call restrictedtext(wx,wy,x,y,flag,string)
	flag=0: string not final
	    =1: final			*/
{
	if (disp_echo) DISP_restrictedtext_(wx,wy,x,y,flag,string,len);
	if (lbp_echo) LBP_restrictedtext_(wx,wy,x,y,flag,string,len);
}

appendtext_(flag,string,len)
	int *flag,len;
	char *string;
/*	FORTRAN calling sequence: call appendtext(flag,string)
	flag=0: string not final
	    =1: final			*/
{
	if (disp_echo) DISP_appendtext_(flag,string,len);
	if (lbp_echo) LBP_appendtext_(flag,string,len);
}

textcolor_(r,g,b)
	int *r,*g,*b;
/*	r is also used as color index	*/
{
	if (disp_echo) DISP_textcolor_(r,g,b);
	if (lbp_echo) LBP_textcolor_(r,g,b);
}

chrheight_(height)
	int *height;
{
/*	height is specified in VDC unit 	*/

	if (disp_echo) DISP_chrheight_(height);
	if (lbp_echo) LBP_chrheight_(height);
}

chrwidth_(ratio)
	float *ratio;
{
	if (disp_echo) DISP_chrwidth_(ratio);
	if (lbp_echo) LBP_chrwidth_(ratio);
}

chrspacing_(spacing)
	float *spacing;
{
	if (disp_echo) DISP_chrspacing_(spacing);
	if (lbp_echo) LBP_chrspacing_(spacing);
}

chrorientation_(angleb,angleu)
	float *angleb,*angleu;
{
	if (disp_echo) DISP_chrorientation_(angleb,angleu);
	if (lbp_echo) LBP_chrorientation_(angleb,angleu);
}

textpath_(path)
	int *path;
/*	path=0:			left
	    =1:			up
	    =2(default):	right
	    =3:			down		*/
{
	if (disp_echo) DISP_textpath_(path);
	if (lbp_echo) LBP_textpath_(path);
}

textalignment_(halign,valign,chalign,cvalign)
	int *halign,*valign;
	float *chalign,*cvalign;
/*	halign=0:	left
	      =1:	center
	      =2:	right
	      =3:	normal hirizontal
	      =4:	continuous horizontal
	valign=10:	top
	      =11:	cap
	      =12:	half
	      =13:	base
	      =14:	bottom
	      =15:	normal vertical
	      =16:	continuous vertical		*/

{
	if (disp_echo) DISP_textalignment_(halign,valign,chalign,cvalign);
	if (lbp_echo) LBP_textalignment_(halign,valign,chalign,cvalign);
}

fontindex_(index)
	int *index;
{
	if (disp_echo) DISP_fontindex_(index);
	if (lbp_echo) LBP_fontindex_(index);
}

polygon_(np,x,y)
	int *np,x[],y[];
{
	if (disp_echo) DISP_polygon_( np, x, y );
	if (lbp_echo) LBP_polygon_( np, x, y );
}

polygonset_(np,x,y,flag)
	int *np,x[],y[];
	char *flag[];
/*	flag=0:		invisible
	    =1:		visible
	    =2:		close_invisible
	    =3:		close_visible		*/
{
	if (disp_echo) DISP_polygonset_(np, x, y, flag);
	if (lbp_echo) LBP_polygonset_(np, x, y, flag);
}


rectangle_(x1,y1,x2,y2)
	int *x1,*y1,*x2,*y2;
{
	if (disp_echo) DISP_rectangle_( x1, y1, x2, y2 );
	if (lbp_echo) LBP_rectangle_( x1, y1, x2, y2 );
}

circle_(cx,cy,r)
	int *cx,*cy,*r;
{
	if (disp_echo) DISP_circle_(cx,cy,r);
	if (lbp_echo) LBP_circle_(cx,cy,r);
}

arcclose_(cx,cy,r,angles,anglee,type)
	int *cx,*cy,*r,*type;
	float *angles, *anglee;
/*	type=0:			chord
	    =1(default):	pie		*/
{
	if (disp_echo) DISP_arcclose_(cx,cy,r,angles,anglee,type);
	if (lbp_echo) LBP_arcclose_(cx,cy,r,angles,anglee,type);
}

arc3close_(x1,y1,x2,y2,x3,y3,type)
	int *x1,*y1,*x2,*y2,*x3,*y3,*type;
/*	type=0:			chord
	    =1(default):	pie		*/
{	
	if (disp_echo) DISP_arc3close_(x1,y1,x2,y2,x3,y3,type);
	if (lbp_echo) LBP_arc3close_(x1,y1,x2,y2,x3,y3,type);
}

ellipse_(cx,cy,a,b,anglea)
	int *cx,*cy,*a,*b;
	float *anglea;
{
	if (disp_echo) DISP_ellipse_(cx,cy,a,b,anglea);
	if (lbp_echo) LBP_ellipse_(cx,cy,a,b,anglea);
}

elliparcclose_(cx,cy,a,b,anglea,angles,anglee,type)
	int *cx,*cy,*a,*b,*type;
	float *anglea,*angles,*anglee;

/*	type=0:			chord
	    =1(default):	pie		*/
{
	if (disp_echo) DISP_elliparcclose_(cx,cy,a,b,anglea,angles,anglee,type);
	if (lbp_echo) LBP_elliparcclose_(cx,cy,a,b,anglea,angles,anglee,type);
}

fillcolor_(r,g,b)
	int *r,*g,*b;
/*	r is also used as color index	*/
{
	if (disp_echo) DISP_fillcolor_(r,g,b);
	if (lbp_echo) LBP_fillcolor_(r,g,b);
}

interiorstyle_(style)
	int *style;
/*	style=1(default):	hollow
	     =2:		solid
	     =3:		pattern
	     =4:		hatch
	     =5:		empty		*/
{
	if (disp_echo) DISP_interiorstyle_(style);
	if (lbp_echo) LBP_interiorstyle_(style);
}

hatchindex_(index)
	int *index;
/*	index=1:		horizontal equally spaced parallel lines
	     =2(default):	vertical equally spaced parallel lines
	     =3:		positive slope equally spaced parallel lines
	     =4:		negative slope equally spaced parallel lines
	     =5:		horizontal/vertical crosshatch
	     =6:		positive/nagative slope crosshatch	*/
{
	if (disp_echo) DISP_hatchindex_(index);
	if (lbp_echo) LBP_hatchindex_(index);
}

patternindex_(index)
	int *index;
{
	if (disp_echo) DISP_patternindex_(index);
}

patternsize_(hx,hy,wx,wy)
	int *hx,*hy,*wx,*wy;
{
	if (disp_echo) DISP_patternsize_(hx,hy,wx,wy);
}

patterntable_(index,nx,ny,plane,pattern)
	int *index,*nx,*ny,*plane;
	int *pattern;
{
/*	incomplete */
}

fillpoint_(x,y)
	int *x,*y;
{
	if (disp_echo) DISP_fillpoint_(x,y);
}

edgetype_(type)
	int *type;
{
	if (disp_echo) DISP_edgetype_(type);
}

edgewidth_(iwidth)
	int *iwidth;
/*	edge width specification in absolute mode	*/
{
	if (disp_echo) DISP_edgewidth_(iwidth);
}

edgefwidth_(fwidth)
	float *fwidth;
/*	edge width specification in scaled mode	*/
{
	if (disp_echo) DISP_edgefwidth_(fwidth);
}

edgecolor_(r,g,b)
	int *r,*g,*b;
/*	r is also used as color index	*/
{
	if (disp_echo) DISP_edgecolor_(r,g,b);
}

edgevisibility_(flag)
	int *flag;
{
	if (disp_echo) DISP_edgevisibility_(flag);
	if (lbp_echo) LBP_edgevisibility_(flag);

}

cellarray_(x1,y1,x2,y2,x3,y3,nx,ny,precision,r,g,b,i)
	int *x1,*y1,*x2,*y2,*x3,*y3,*nx,*ny,*precision,*r,*g,*b,*i;
{
/* incomplete */
}

colormode_(mode)
	int *mode;
/*	mode=0:			direct
	    =1(default):	indexed		*/
{
	if (disp_echo) DISP_colormode_(mode);
	if (lbp_echo) LBP_colormode_(mode);
}

colortable_(startindex,ncolor,r,g,b)
	int *startindex,*ncolor,r[],g[],b[];
{
	if (disp_echo) DISP_colortable_(startindex,ncolor,r,g,b);
	if (lbp_echo) LBP_colortable_(startindex,ncolor,r,g,b);
}

auxcolor_(r,g,b)
	int *r,*g,*b;
/*	r is also used as color index		*/
{
	if (disp_echo) DISP_auxcolor_(r,g,b);
}

transparency_(trans)
	int *trans;
/*	trans=0:		transparent
	     =1:		opaque		*/
{
	if (disp_echo) DISP_transparency_(trans);
}

byteimage_(xo,yo,nx,ny,x1,x2,y1,y2,image)
	int *xo,*yo,*nx,*ny,*x1,*x2,*y1,*y2;
	unsigned char *image;
{
	if (disp_echo)	DISP_byteimage_(xo,yo,nx,ny,x1,x2,y1,y2,image);
	if (lbp_echo)	LBP_byteimage_(xo,yo,nx,ny,x1,x2,y1,y2,image);
}

bitwiseimage_(xo,yo,nx,ny,x1,x2,y1,y2,nplane,bitmap)
	int *xo,*yo,*nx,*ny,*x1,*x2,*y1,*y2,*nplane;
	unsigned short *bitmap;
/*	nplane = 1	dithering
	nplane = 8	halftone map (no magnification)
	nplane = 2, 4: halftone with magnification 2 or 4 (LBP)
*/
{
	if (disp_echo)	DISP_bitwiseimage_(xo,yo,nx,ny,x1,x2,y1,y2,
							nplane,bitmap);
	if (lbp_echo)	LBP_bitwiseimage_(xo,yo,nx,ny,x1,x2,y1,y2,
							nplane,bitmap);
}

pointermode_(mode)
	int *mode;
{
	if (disp_echo)	DISP_pointermode_(mode);
}

pointerprompt_(onoff)
	int *onoff;
{
	if (disp_echo)	DISP_pointerprompt_(onoff);
}

pointerack_(onoff)
	int *onoff;
{
	if (disp_echo)	DISP_pointerack_(onoff);
}



