/*	return the character code of the first character in the input buffer
	if no input exists, return -1
*/

/* ------------------------- sony news version ------------------------------*/

#ifdef sony
#include <sgtty.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/file.h>

kbhit_()
{
	static int fd, init=1;
	char c;
	int code, no;
	static unsigned short orig, cbreak;
	unsigned long ll;
	static struct sgttyb arg;

	if (init )	{
		fd = open("/dev/tty", O_RDWR, 0666);
		ioctl(fd, TIOCGETP, &arg);
			/*	store original arg	*/
/*          orig = arg.sg_flags;    */
            orig = 030;
		cbreak = orig|2;
		init = 0;
	}

	arg.sg_flags = cbreak;
	ioctl(fd, TIOCSETN, &arg);
		/*	start CBREAK mode (unbuffered input),
			without flushing i/o buffer	*/

	code = -1;
	arg.sg_flags = orig;

	ioctl( fd, FIONREAD, &no);
		/*	get number of characters in the input buffer	*/

	if(no > 0) 	read(fd, &c, 1);
	ioctl(fd, TIOCSETN, &arg);
		/*	restore original mode (buffered input)	*/
/*
	It seems that if the keyboard is hit between read and ioctl,
	the character is echoed on the screen but the input queue is
	flushed by ioctl. Therefore such input is not detected by
	this function. In order to make this possibility lower, 
	a delay loop is inserted below.

	BS is only occasionally detected, probably because in buffered
	input mode BS is ignored unless some preceding characters exist.
	If the following delay loop is eliminated, BS is detected.
	On the other hand DEL is always detected.

	If the cooked mode is restored when the input buffer is not
	empty, the content of the input buffer is echoed on the screen.
	This can be avoided by using TIOCSETP, but then continuous
	key input (such as from arrow keys) is not fully received.
	Therefore TIOCSETN is used here.
*/

	if(no > 0) 	code = (unsigned int) c;
	for (ll=0; ll<100000; ll++) ;

	return (code);
}

/*	Reference: from tty(4).

	Terminal mode:

	COOKED    The normal mode.  In this mode lines of input are
	collected and input editing is done.  The edited
	line is made available when it is completed by a
	newline, or when the t_brkc character (normally
	undefined) or t_eofc character (normally an EOT,
	control-D, hereafter ^D) is entered.  A carriage
	return is usually made synonymous with newline in
	this mode, and replaced with a newline whenever it
	is typed.  All driver functions (input editing,
	interrupt generation, output processing such as
	delay generation and tab expansion, etc.) are
	available in this mode.

	CBREAK    This mode eliminates the character, word, and line
	editing input facilities, making the input charac-
	ter available to the user program as it is typed.
	Flow control, literal-next and interrupt process-
	ing are still done in this mode.  Output process-
	ing is done.

	RAW       This mode eliminates all input processing and
	makes all input characters available as they are
	typed; no output processing is done either.
	
	The kbhit routine should use CBREAK mode.

	Declare
		struct sgttyb arg;
	where
		struct sgttyb {
			char   sg_ispeed;
			char   sg_ospeed;
			char   sg_erase;
			char   sg_kill;
			short  sg_flags;
		};

	The default setup of sg_flags is known by:
		ioctl(fd, TIOCGETP, &arg);
	where
		TIOCGETP     Fetch the basic parameters associated with the
		terminal, and store in the pointed-to sgttyb
		structure.

	The result is arg.sg_flags = 030, i.e.
		CRMOD     0000020  Map CR into LF; output LF as CR-LF
		ECHO      0000010  Echo (full duplex)

	In order to change to CBREAK mode, we should include
		CBREAK    0000002  Return each character as soon as typed

	That is,
		arg.sg_flags = 032;
		ioctl(fd, code, &arg);
	where code is
		TIOCSETP     Set the parameters according to the pointed-to
		sgttyb structure.  The interface delays until
		output is quiescent, then throws away any
		unread characters, before changing the modes.
	or
		TIOCSETN     Set the parameters like TIOCSETP but do not
		delay or flush input.  Input is not preserved,
		however, when changing to or from RAW.

	The number of characters in the input buffer is known by:
		ioctl(fd, code, &ncharacter);
	where code is
		FIONREAD     returns in the int pointed to by arg the number
		of characters immediately readable from the
		argument descriptor.  This works for files,
		pipes, and terminals.
*/

#define KBHIT_DEFINED
#endif

/* ------------------------- sun workstation version ------------------------*/

#ifdef sun

#include <sys/termios.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/uio.h>

kbhit_()
{
	static int fd, init=1;
	char c;
	int code, no;
	static unsigned long orig, modify_echo, modify_noecho;
	static unsigned char orig_min, min=0;
	static struct termios arg;
	unsigned long ll;

	if (init )	{
		fd = open("/dev/tty", O_RDWR, 0666);
		ioctl(fd, TCGETS, &arg);
			/*	store original arg	*/
		orig = arg.c_lflag;
		modify_noecho = orig & 0xfffffff5;
		modify_echo   = orig & 0xfffffffd;
		orig_min = arg.c_cc[VEOF];
		init = 0;
	}

	for(ll=0; ll< 100000; ll++)	;

	arg.c_cc[VEOF] = min;
	arg.c_lflag = modify_noecho;
	ioctl(fd, TCSETSW, &arg);
	arg.c_lflag = modify_echo;
	ioctl(fd, TCSETSW, &arg);

/*	Start non-canonical mode (unbuffered input),
	first with echo off and then make echo on.
	If the keyboard has been hit before this routine
	is called (and the input character was echoed),
	switching to non-canonical mode with echo on makes
	the character in the input buffer printed again.
	If the keyboard is hit between the two ioctls above,
	the input character is not echoed. In order to make
	this possibility lower, a delay loop is inserted at
	the beginning of this routine.
*/
	code = -1;
	arg.c_lflag = orig;
	arg.c_cc[VEOF] = orig_min;

	no = 	read(fd, &c, 1);
	ioctl(fd, TCSETSF, &arg);
		/*	resume canonical mode	*/

	if(no > 0)	code = (unsigned int) c;

	return (code);
}

/*	Reference: from termio(4).

	Non-Canonical Mode Input Processing
	In non-canonical mode input processing, input characters are
	not assembled into lines, and erase and kill processing does
	not occur.  The MIN and TIME values are  used  to  determine
	how to process the characters received.

	MIN represents the minimum number of characters that  should
	be  received when the read is satisfied (when the characters
	are returned to the user).  TIME is a timer of  0.10  second
	granularity  that  is  used to timeout bursty and short term
	data transmissions.  The four possible values  for  MIN  and
	TIME and their interactions are described below.

		...............

	Case D: MIN = 0, TIME = 0
	In this case return is immediate.  The minimum of either the
	number  of  characters requested or the number of characters
	currently available will be  returned  without  waiting  for
	more characters to be input.

		...............

	The kbhit routine should use non-canonical mode with MIN=0, TIME=0.

	Declare
		struct termios arg;
	where
		#define      NCCS 17
		struct       termios {
			unsigned    long    c_iflag;* input modes *
			unsigned    long    c_oflag;* output modes *
			unsigned    long    c_cflag;* control modes *
			unsigned    long    c_lflag;* local modes *
			unsigned    char    c_line;* line discipline *
			unsigned    char    c_cc[NCCS];* control chars *
		};

	The special control characters  are  defined  by  the  array
	c_cc.   The  relative  positions and initial values for each
	function are as follows:

		0   VINTR    ETX
		1   VQUIT    FS
		2   VERASE   DEL
		3   VKILL    NAK
		4   VEOF     EOT
		5   VEOL     NUL
		6   VEOL2    NUL
		7   VSWTCH   NUL
		8   VSTART   DC1
		9   VSTOP    DC3
		10  VSUSP    EM
		12  VREPRINT DC2
		13  VDISCARD SI
		14  VWERASE  ETB
		15  VLNEXT   SYN
	The MIN value is stored in the  VMIN  element  of  the  c_cc
	array,  and the TIME value is stored in the VTIME element of
	the c_cc array.  The VMIN element is the same element as the
	VEOF  element,  and the VTIME element is the same element as
	the VEOL element.


	The default of MIN, TIME, and c_lflag (local mode flag) is known by:
		ioctl(fd, TIOCGETP, &arg);
	where
		TCGETS	The argument is a  pointer  to  a  termios
		structure.   The  current terminal parame-
		ters are  fetched  and  stored  into  that
		structure.   This  call  is allowed from a
		background process; however, the  informa-
		tion  may  subsequently  be  changed  by a
		foreground process.

	The result is MIN=4, TIME=0, and c_lflag=0105073, i.e.
		ISIG    0000001   Enable signals.
		ICANON  0000002   Canonical input (erase and kill processing).
		ECHO    0000010   Enable echo.
		ECHOE   0000020   Echo erase character as BS-SP-BS.
		ECHOK   0000040   Echo NL after kill character.
		ECHOCTL 0001000   Echo control characters as ^chr, delete as ^?.
		ECHOKE  0004000   BS-SP-BS erase entire line on line kill.
		IEXTEN  0100000   Recognize all specials (if clear, POSIX only).

	In order to change into non-canonical mode, suppress ICANON.
	That is,
		arg.c_lflag = 0105071;	echo on	
	or	arg.c_lflag = 0105061;	echo off
		ioctl(fd, code, &arg);
	where code is
		TCSETS	The argument is a  pointer  to  a  termios
		structure.   The  current terminal parame-
		ters are set from  the  values  stored  in
		that structure.  The change is immediate.
	or
		TCSETSW	The argument is a  pointer  to  a  termios
		structure.   The  current terminal parame-
		ters are set from  the  values  stored  in
		that  structure.   The change occurs after
		all characters queued for output have been
		transmitted.   This  form  should  be used
		when changing parameters that will  affect
		output.
	or
		TCSETSF	The argument is a  pointer  to  a  termios
		structure.   The  current terminal parame-
		ters are set from  the  values  stored  in
		that  structure.   The change occurs after
		all characters queued for output have been
		transmitted;  all  characters  queued  for
		input are discarded and  then  the  change
		occurs.
*/

#define KBHIT_DEFINED
#endif

#ifndef KBHIT_DEFINED

/* ----------------------for other machines --------------------------------*/

kbhit_()
{
	return (getchar());
}

#endif
