/*	newslib.c
	News CGI library
*/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <cgi.h>

#include "makebitwiseimage.c"

#ifdef DISPONLY
#include "disponly.h"
#endif

/*	parameters for display and frame buffer	*/
#include "crt.h"

/*    int = long (4-byte) integer = fortran integer   */

unsigned int	int_r[256]={0,255},
			int_g[256]={0,255},
			int_b[256]={0,255},
			max_colorindex =2;
#define DIRECTCOLOR 0
#define INDEXEDCOLOR 1
int colormode = INDEXEDCOLOR;

#define AMPLITUDE 100
#define MAXFONTNAMES 2
#define CGIMAXBUF 256 
/*	polyline becomes disjointed if CGIMAXBUF>256	*/

static char *fontnamelist[MAXFONTNAMES]={"rom-ascii","vsfont32"};
static Point pointlist[CGIMAXBUF];
static FlaggedPoint *fplist[CGIMAXBUF];

DC dc1,dc2;		/*	viewport specified in devide coordinates	*/

static int vpunits = 1;
/*	default viewport units = 1/10mm	*/

static int xs=0, ys=0;				/*    start point of VDC      */
static int xe=32767, ye=32767;            /*    end point of VDC  */
static float xfactor, yfactor;


/*	internal functions	*/

static Point point(x,y)
      int x,y;
{
      Point p;
      p.x=x; p.y=y;
      return(p);
}

static FlaggedPoint flaggedpoint(x,y,flag)
      int x,y;
      char flag;
{
      Point point();
      FlaggedPoint f;
      f.point=point(x,y); f.flag=flag;
      return(f);
}


static Vector vector(x,y)
      int x,y;
{
      Vector v;
      v.x=x; v.y=y;
      return(v);
}

static Colour_Direct direct(r,g,b)
      int r,g,b;
{
      Colour_Direct d;
      d.red=r; d.green=g; d.blue=b;
      return(d);
}

static Vector angle_to_vector(r,angle)
	float angle;
	int r;
{
	Vector v;
	double pi=3.141592;
	v.x = r * cos ( pi*angle/180.0 );
	v.y = r * sin ( pi*angle/180.0 );
	return(v);
}
 
static Point addpoint(point,vector)
	Point point;
	Vector vector;
{
	Point p;
	p.x = point.x + vector.x;
	p.y = point.y + vector.y;
	return(p);
}


DISP_initialize_(initmode)
	int *initmode;
{
	CGI_Initialize();

	CGI_Device_Viewport_Specification_Mapping(0,1,4);
		/*	isotropy not forced	*/

	CGI_Initialize_Input_Device(LOCATOR,0);
	CGI_Set_Echo_State(LOCATOR,0,ON);
		/*	enable mouse	*/

	CGI_Font_List(MAXFONTNAMES,fontnamelist);	
/*      int_r[0]=0;       int_g[0]=0;       int_b[0]=0;
      int_r[1]=255;     int_g[1]=255;     int_b[1]=255;
      max_colorindex=2;	*/
/*    register two colors     */
}

DISP_reset_()
{
	 CGI_Reset_To_Defaults();
}

DISP_redraw_()
{
}

DISP_terminate_()
{
	CGI_Set_Echo_State(LOCATOR,0,OFF);
	CGI_Release_Input_Device(LOCATOR,0);
	CGI_Terminate();
}

DISP_clear_()
{
	int soft=CGI_CTL_SOFT_CLEAR, hard=CGI_CTL_HARD_NOP;
	CGI_Clear_View_Surface(soft,hard);
}

DISP_bgcolor_(r,g,b)
	int *r,*g,*b;
{
	Colour_Direct direct();
	CGI_Background_Colour(direct(*r,*g,*b));
}

DISP_vdcextent_(x1,y1,x2,y2)
	int *x1, *y1, *x2, *y2;
{
	Point point();
	CGI_VDC_Extent(point(*x1,*y1),point(*x2,*y2));
      xs = *x1;   ys = *y1;
      xe = *x2;   ye = *y2;
}

DISP_viewportunits_(units)
	int *units;
/*	units=0:				percent relative
	     =1(default here):		in 1/10 mm
	     =2:				absolute		*/
{
	CGI_Device_Viewport_Specification_Units(*units);
	vpunits = *units;
}

DISP_viewport_(x1,y1,x2,y2)
	int *x1,*y1,*x2,*y2;
/*	viewport specification
	this routine can be used for viewportunits=1,2

	(0,0) is at the upper left corner of the screen
	VDC (x1,y1),(x2,y2) are mapped to viewport (x1,y1),(x2,y2)
*/
{
	Point point();
      float xx1,yy1,xx2,yy2;

	if (vpunits == 1)	{	/*	unit=1/10mm	*/
		dc1.Dec.x= *x1;
		dc1.Dec.y= *y1;
		dc2.Dec.x= *x2;
		dc2.Dec.y= *y2;
		CGI_Device_Viewport(dc1,dc2);

		xx1 = *x1;	yy1 = *y1;
		xx2 = *x2;	yy2 = *y2;
	}
	if (vpunits == 2)	{	/*	unit=dot	*/
		dc1.Nat=point(*x1,*y1);
		dc2.Nat=point(*x2,*y2);
		CGI_Device_Viewport(dc1,dc2);

		xx1 = (*x1)*DISPdotsize;
		yy1 = (*y1)*DISPdotsize;
		xx2 = (*x2)*DISPdotsize;
		yy2 = (*y2)*DISPdotsize;
	}
      xfactor = ( xx2 - xx1 )/( xe - xs );
      yfactor = ( yy2 - yy1 )/( ye - ys );
		/*	x,yfactor * VDC length  -> length in 1/10mm	*/
}

DISP_viewportf_( fx1,fy1,fx2,fy2)
      float *fx1,*fy1,*fx2,*fy2;
/*    viewport specification
      this routine can be used for viewportunits=0  */
{
      Point point();
      float xx1,yy1,xx2,yy2;

	if (vpunits == 0) {	/*	unit = percent	*/
		dc1.Per.x= *fx1;
		dc1.Per.y= *fy1;
		dc2.Per.x= *fx2;
		dc2.Per.y= *fy2;
		CGI_Device_Viewport(dc1,dc2);

		xx1 = DISPxmax * 0.01*(*fx1);
		yy1 = DISPymax * 0.01*(*fy1);
		xx2 = DISPxmax * 0.01*(*fx2);
		yy2 = DISPymax * 0.01*(*fy2);		/*	these are in 1/10mm	*/
      	xfactor = ( xx2 - xx1 )/( xe - xs );
      	yfactor = ( yy2 - yy1 )/( ye - ys );
			/*	x,yfactor * VDC length  -> length in 1/10mm	*/
		
	}
}

DISP_viewportmapping_(isotropy,horizontal,vertical)
	int *isotropy,*horizontal,*vertical;
/*	isotropy=0(default here):	not forced
	        =1(news cgi default):	forced
	horizontal=0:		center
	          =1(default):	left
	          =2:		right
	vertical=0:		center
	        =3:		top
	        =4(default):	bottom		*/
{
	CGI_Device_Viewport_Specification_Mapping(
			*isotropy,*horizontal,*vertical);
}

DISP_clipindicator_(indicator)
	int *indicator;
/*	indicator=0(default):	clip rectangle
	         =1:		view surface
	         =2:		off		*/
{
	CGI_Clip_Indicator(*indicator);
}

DISP_cliprectangle_(x1,y1,x2,y2)
	int *x1,*y1,*x2,*y2;
{
	Point point();
	CGI_Clip_Rectangle(point(*x1,*y1),point(*x2,*y2));
}

DISP_hcopy_()
{
}

DISP_polyline_(np,x,y)
	int *np,x[],y[];
{

	int i, is=0, n;
	Point point();

	do {
		n = ( *np -is > CGIMAXBUF) ? CGIMAXBUF : (*np -is);
		for (i=0; i<n; i++) pointlist[i]=point(x[i+is],y[i+is]);
		CGI_Polyline(n,pointlist);
		is += n-1;
	} while ( is < *np -1 );
}

DISP_disjointline_(np,x,y)
	int *np,x[],y[];
{
	int i, is=0, ie, n;
	Point point();

	do {
		n = ( *np -is > CGIMAXBUF) ? CGIMAXBUF : (*np -is);
		ie += n-1;
		for (i=0; i<n; i++) pointlist[i]=point(x[i+is],y[i+is]);
		CGI_Disjoint_Polyline(n,pointlist);
		is += n;
	} while ( ie < *np - 1 );
}

DISP_arc_(cx,cy,r,angles,anglee)
	int *cx,*cy,*r;
	float *angles,*anglee;
{
	Point point(),addpoint();
	Vector angle_to_vector();
	int amplitude = AMPLITUDE;
	float angs, ange, aux;

      angs = *angles;
      ange = *anglee;
      if (xfactor < 0.0)      {
            angs = 180.0 - angs;
            ange = 180.0 - ange;
      }
      if (yfactor < 0.0)      {
            angs = -angs;
            ange = -ange;
      }
      if (xfactor*yfactor < 0.0)    {
            aux = angs;
            angs = ange;
            ange = aux;
      }

	CGI_Circular_Arc_Center(point(*cx,*cy),
		angle_to_vector(amplitude,angs),
		angle_to_vector(amplitude,ange),
		*r);
}

DISP_arc3_(x1,y1,x2,y2,x3,y3)
	int *x1,*y1,*x2,*y2,*x3,*y3;
{
	Point point();
	CGI_Circular_Arc_3_Point(point(*x1,*y1),point(*x2,*y2),point(*x3,*y3));
}

DISP_elliparc_(cx,cy,a,b,anglea,angles,anglee)
	int *cx,*cy,*a,*b;
	float *anglea,*angles,*anglee;
{
	float angs, ange, anga, angb, aux;
	Point point(),addpoint();
	Vector angle_to_vector();
	int amplitude=AMPLITUDE;

      angs = *angles;
      ange = *anglee;
	angb = *anglea + 90.0;
      if (xfactor < 0.0)      {
            angs = 180.0 - angs;
            ange = 180.0 - ange;
            anga = 180.0 - anga;
            angb = 180.0 - angb;
      }
      if (yfactor < 0.0)      {
            angs = -angs;
            ange = -ange;
            anga = -anga;
            angb = -angb;
      }
      if (xfactor*yfactor < 0.0)    {
            aux = angs;
            angs = ange;
            ange = aux;
      }

	CGI_Elliptical_Arc(point(*cx,*cy),
		addpoint(point(*cx,*cy),angle_to_vector(*a, anga)),
		addpoint(point(*cx,*cy),angle_to_vector(*b, angb)),
		angle_to_vector(amplitude, angs),
		angle_to_vector(amplitude, ange));
}

DISP_linetype_(type)
	int *type;
/*	type=1(default):		solid
	    =2:			dashed
	    =3:			dotted
	    =4:			dash-dot
	    =5:			dash-dot-dot		*/
{
	if (*type >=1 && *type <= 5)	CGI_Line_Type(*type);
}

DISP_linewidth_(iwidth)
	int *iwidth;
/*	linewidth specification in absolute mode (in VDC unit, integer)	*/
{
	Size size;

	CGI_Line_Width_Specification_Mode(0);
	size.vdc=(*iwidth);
	CGI_Line_Width(size);
}

DISP_linefwidth_(fwidth)
	float *fwidth;
/*	linewidth specification in scaled mode (in pixel unit, floating)	*/
{
	Size size;

	CGI_Line_Width_Specification_Mode(1);
	size.scaled=(*fwidth);
	CGI_Line_Width(size);
}

DISP_linecolor_(r,g,b)
	int *r,*g,*b;
/*	r is also used as color index	*/
{
	Colour c;
	Colour_Direct direct();
	c.Direct=direct(*r,*g,*b);
	CGI_Line_Colour(c);
}

DISP_polymarker_(np,x,y)
	int *np,x[],y[];
{
	Point point();
	int i, is=0, n;

	do {
		n = ( *np -is > CGIMAXBUF) ? CGIMAXBUF : (*np -is);
		for (i=0; i<n; i++) pointlist[i]=point(x[i+is],y[i+is]);
		CGI_Polymarker(n,pointlist);
		is += n;
	} while ( is < *np );
}

DISP_markertype_(type)
	int *type;
/*	type=1:		dot
	    =2:		plus
	    =3(default)	asterisk
	    =4:		circle
	    =5:		cross			*/
{
	if (*type >= 1 && *type <= 5) CGI_Marker_Type(*type);
}

DISP_markersize_(isize)
	int *isize;
/*	marker size specification in absolute mode (in VDC unit, integer)	*/
{
	Size s;

	CGI_Marker_Size_Specification_Mode(0);
	s.vdc=(*isize);
	CGI_Marker_Size(s);
}

DISP_markerfsize_(fsize)
	float *fsize;
/*	marker size specification in scaled mode (relative, in floating)	*/
{
	Size s;

	CGI_Marker_Size_Specification_Mode(1);
	s.scaled=(*fsize);
	CGI_Marker_Size(s);
}

DISP_markercolor_(r,g,b)
	int *r,*g,*b;
/*	r is also used as color index	*/
{
	Colour c;
	Colour_Direct direct();
	c.Direct=direct(*r,*g,*b);
	CGI_Marker_Colour(c);
}

DISP_text_(x,y,string,len)
	int *x,*y,len;
	char *string;
/*	FORTRAN calling sequence: call text(x,y,string)	*/
{
	int flag = 1;	/*	string is final	*/
	Point point();
	char buf[257],*strncpy();
	strncpy(buf,string,len);
	buf[len]='\0';
	CGI_Text(point(*x,*y),flag,buf);
}

DISP_textnf_(x,y,string,len)
	int *x,*y,len;
	char *string;
{
	int flag = 0;	/*	string is not final	*/
	Point point();
	char buf[257],*strncpy();
	strncpy(buf,string,len);
	buf[len]='\0';
	CGI_Text(point(*x,*y),flag,buf);
}

DISP_restrictedtext_(wx,wy,x,y,flag,string,len)
	int *wx,*wy,*x,*y,*flag,len;
	char *string;
/*	FORTRAN calling sequence: call restrictedtext(wx,wy,x,y,flag,string)
	flag=0: string not final
	    =1: final			*/
{
	Point point();
	char buf[257],*strncpy();
	strncpy(buf,string,len);
	buf[len]='\0';
	CGI_Restricted_Text(*wx,*wy,point(*x,*y),*flag,buf);
}

DISP_appendtext_(flag,string,len)
	int *flag,len;
	char *string;
/*	FORTRAN calling sequence: call appendtext(flag,string)
	flag=0: string not final
	    =1: final			*/
{
	char buf[257],*strncpy();
	strncpy(buf,string,len);
	buf[len]='\0';
	CGI_Append_Text(*flag,buf);
}

DISP_textcolor_(r,g,b)
	int *r,*g,*b;
/*	r is also used as color index	*/
{
	Colour c;
	Colour_Direct direct();
	c.Direct=direct(*r,*g,*b);
	CGI_Text_Colour(c);
}

DISP_chrheight_(height)
	int *height;
{
/*	height is in VDC unit 
	argument of news cgi's chrheight is in DC unit, not in VDC as in manual
*/
	CGI_Character_Height( (int) (fabs((double)yfactor)*(*height)/DISPdotsize) );
}

DISP_chrwidth_(ratio)
	float *ratio;
{
	CGI_Character_Expansion_Factor( (double)(*ratio) );
}

DISP_chrspacing_(spacing)
	float *spacing;
{
	CGI_Character_Spacing( (double)(*spacing) );
}

DISP_chrorientation_(angleb,angleu)
	float *angleb,*angleu;
{
	Vector angle_to_vector();
	int amplitude=AMPLITUDE;
	CGI_Character_Orientation(
			angle_to_vector(amplitude,*angleu),
			angle_to_vector(amplitude,*angleb));
}

DISP_textpath_(path)
	int *path;
/*	path=0:			left
	    =1:			up
	    =2(default):	right
	    =3:			down		*/
{
	CGI_Text_Path(*path);
}

DISP_textalignment_(halign,valign,chalign,cvalign)
	int *halign,*valign;
	float *chalign,*cvalign;
/*	halign=0:	left
	      =1:	center
	      =2:	right
	      =3:	normal hirizontal
	      =4:	continuous horizontal
	valign=10:	top
	      =11:	cap
	      =12:	half
	      =13:	base
	      =14:	bottom
	      =15:	normal vertical
	      =16:	continuous vertical		*/

{
	CGI_Text_Alignment(*halign,*valign,(double)*chalign,(double)*cvalign );
}

DISP_fontindex_(index_f)
	int *index_f;
{
/*	index_f=1:	rom-ascii
	        2:	vsfont32	*/

	int index;
	index = *index_f -1;
	if (index == 0 || index == 1) CGI_Text_Font_Index(index+1);
/*								why not index?	*/
}

DISP_polygon_(np,x,y)
	int *np,x[],y[];
{
	Point point();
	int i, is=0, n;

	do {
		n = ( *np -is > CGIMAXBUF) ? CGIMAXBUF : (*np -is);
		for (i=0; i<n; i++) pointlist[i]=point(x[i+is],y[i+is]);
		CGI_Polygon(n,pointlist);
		is += n-1;
	} while ( is < *np -1 );
}

DISP_polygonset_(np,x,y,flag)
	int *np,x[],y[];
	char *flag[];
/*	flag=0:		invisible
	    =1:		visible
	    =2:		close_invisible
	    =3:		close_visible		*/
{
	FlaggedPoint flaggedpoint();
	int i, is=0, ie, n;
	
	do {
		n = ( *np -is > CGIMAXBUF) ? CGIMAXBUF : (*np -is);
		ie += n-1;
		for (i=0; i<n; i++)
		*fplist[i]=flaggedpoint(x[i+is],y[i+is],*flag[i+is]);
		CGI_Polygon_Set(n,fplist);
		is += n;
	} while ( ie < *np - 1 );
}


DISP_rectangle_(x1,y1,x2,y2)
	int *x1,*y1,*x2,*y2;
{
	Point point();
	CGI_Rectangle(point(*x1,*y1),point(*x2,*y2));
}

DISP_circle_(cx,cy,r)
	int *cx,*cy,*r;
{
	Point point();
	CGI_Circle(point(*cx,*cy),*r);
}

DISP_arcclose_(cx,cy,r,angles,anglee,type)
	int *cx,*cy,*r,*type;
	float *angles, *anglee;
/*	type=0:			chord
	    =1(default):	pie		*/
{
	Point point();
	Vector angle_to_vector();
	int amplitude=AMPLITUDE;
	float angs, ange, aux;

      angs = *angles;
      ange = *anglee;
      if (xfactor < 0.0)      {
            angs = 180.0 - angs;
            ange = 180.0 - ange;
      }
      if (yfactor < 0.0)      {
            angs = -angs;
            ange = -ange;
      }
      if (xfactor*yfactor < 0.0)    {
            aux = angs;
            angs = ange;
            ange = aux;
      }

	CGI_Circular_Arc_Center_Close(point(*cx,*cy),
		angle_to_vector(amplitude,angs),
		angle_to_vector(amplitude,ange),
		*r,*type);
}

DISP_arc3close_(x1,y1,x2,y2,x3,y3,type)
	int *x1,*y1,*x2,*y2,*x3,*y3,*type;
/*	type=0:			chord
	    =1(default):	pie		*/
{	Point point();
	CGI_Circular_Arc_3_Point_Close(
		point(*x1,*y1),point(*x2,*y2),point(*x3,*y3),*type);
}

DISP_ellipse_(cx,cy,a,b,anglea)
	int *cx,*cy,*a,*b;
	float *anglea;
{
	Point point(),addpoint();
	Vector angle_to_vector();
	float angleb;
	angleb = (*anglea) + 90.0;
	CGI_Ellipse(point(*cx,*cy),
		addpoint(point(*cx,*cy),angle_to_vector(*a,*anglea)),
		addpoint(point(*cx,*cy),angle_to_vector(*b, angleb)));
}

DISP_elliparcclose_(cx,cy,a,b,anglea,angles,anglee,type)
	int *cx,*cy,*a,*b,*type;
	float *anglea,*angles,*anglee;

/*	type=0:			chord
	    =1(default):	pie		*/
{
	Point point(),addpoint();
	Vector angle_to_vector();
	float angs, ange, anga, angb, aux;
	int amplitude=AMPLITUDE;

      angs = *angles;
      ange = *anglee;
      anga = *anglea;
      angb = *anglea +90.0;
      if (xfactor < 0.0)      {
            angs = 180.0 - angs;
            ange = 180.0 - ange;
            anga = 180.0 - anga;
            angb = 180.0 - angb;
      }
      if (yfactor < 0.0)      {
            angs = -angs;
            ange = -ange;
            anga = -anga;
            angb = -angb;
      }
      if (xfactor*yfactor < 0.0)    {
            aux = angs;
            angs = ange;
            ange = aux;
      }

	CGI_Elliptical_Arc_Close(point(*cx,*cy),
		addpoint(point(*cx,*cy),angle_to_vector(*a, anga)),
		addpoint(point(*cx,*cy),angle_to_vector(*b, angb)),
		angle_to_vector(amplitude, angs),
		angle_to_vector(amplitude, ange),
		*type);
}

DISP_fillcolor_(r,g,b)
	int *r,*g,*b;
/*	r is also used as color index	*/
{
	Colour c;
	Colour_Direct direct();
	c.Direct=direct(*r,*g,*b);
	CGI_Fill_Colour(c);
}

DISP_interiorstyle_(style)
	int *style;
/*	style=1(default):	hollow
	     =2:		solid
	     =3:		pattern
	     =4:		hatch
	     =5:		empty		*/
{
	if (*style >= 1 && *style <= 5) CGI_Interior_Style(*style);
}

DISP_hatchindex_(index)
	int *index;
/*	index=1:		horizontal parallel lines
	     =2(default):	vertical parallel lines
	     =3:		positive slope parallel lines
	     =4:		negative slope parallel lines
	     =5:		horizontal/vertical crosshatch
	     =6:		positive/negative slope crosshatch	*/
{
	if (*index >= 1 && *index <= 6) CGI_Hatch_Index(*index);
}

DISP_patternindex_(index)
	int *index;
/*	index=1: vetrical stripes
	     =2: horizontal stripes
	     =3: checker pattern	*/
{
	if (*index >= 1 && *index <= 3) CGI_Pattern_Index(*index);
}

DISP_patternsize_(hx,hy,wx,wy)
	int *hx,*hy,*wx,*wy;
{
	Vector vector();
	 CGI_Pattern_Size(vector(*hx,*hy),vector(*wx,*wy));
}

DISP_patterntable_(index,nx,ny,plane,pattern)
	int *index,*nx,*ny,*plane;
	int *pattern;
{
/*	incomplete
	CGI_Pattern_Table(*index,*nx,*ny,*plane,*pattern);
								*/
}

DISP_fillpoint_(x,y)
	int *x,*y;
{
	Point point();
	CGI_Fill_Reference_Point(point(*x,*y));
}

DISP_edgetype_(type)
	int *type;
{
	CGI_Edge_Type(*type);
}

DISP_edgewidth_(iwidth)
	int *iwidth;
/*	edge width specification in absolute mode	(in VDC unit, integer)	*/
{
	Size s;

	CGI_Edge_Width_Specification_Mode(0);
	s.vdc=(*iwidth);
	CGI_Edge_Width(s);
}

DISP_edgefwidth_(fwidth)
	float *fwidth;
/*	edge width specification in scaled mode (in pixel unit, floating)	*/
{
	Size s;

	CGI_Edge_Width_Specification_Mode(1);
	s.scaled=(*fwidth);
	CGI_Edge_Width(s);
}

DISP_edgecolor_(r,g,b)
	int *r,*g,*b;
/*	r is also used as color index	*/
{
	Colour c;
	Colour_Direct direct();
	c.Direct=direct(*r,*g,*b);
	CGI_Edge_Colour(c);
}

DISP_edgevisibility_(edge)
	int *edge;
/*	edge = 0: visible
	       1: invisivble (default)	*/
{
	CGI_Edge_Visibility(*edge);

}

DISP_cellarray_(x1,y1,x2,y2,x3,y3,nx,ny,precision,r,g,b,i)
	int *x1,*y1,*x2,*y2,*x3,*y3,*nx,*ny,*precision,*r,*g,*b,*i;
{
	Point point();
/* incomplete 
	Colour *cellcolour,colour();	
	
		CGI_Cell_Array(point(*x1,*y1),point(*x2,*y2),
			point(*x3,*y3),*nx,*ny,*precision,cellcolour);
				*/
}

DISP_colormode_(mode)
	int *mode;
/*	mode=0:			direct
	    =1(default):	indexed		*/
{
	CGI_Colour_Selection_Mode(*mode);
	colormode = *mode;
}

DISP_colortable_(startindex_f,ncolor,r,g,b)
	int *startindex_f,*ncolor,r[],g[],b[];
/*
	startindex_f >=1
	ncolor <=256
*/
{
	Colour_Direct colourlist[256],direct();
	int i, startindex;

	startindex = *startindex_f -1;
      if (startindex+(*ncolor) > max_colorindex)
		max_colorindex = startindex +(*ncolor);

	for(i=0; i<*ncolor; i++)	{
		colourlist[i]=direct(r[i],g[i],b[i]);
            int_r[startindex +i] = r[i];
            int_g[startindex +i] = g[i];
            int_b[startindex +i] = b[i];
	}
	CGI_Colour_Table(startindex+1,(*ncolor)-1,colourlist);
/*	why not CGI_Colour_Table(startindex,*ncolor,colourlist); ??	*/
}

DISP_auxcolor_(r,g,b)
	int *r,*g,*b;
/*	r is also used as color index		*/
{
	Colour c;
	Colour_Direct direct();

	c.Direct=direct(*r,*g,*b);
	CGI_Auxiliary_Colour(c);
}

DISP_transparency_(trans)
	int *trans;
/*	trans=0:		transparent
	     =1:		opaque		*/
{
	CGI_Transparency(*trans);
}

DISP_byteimage_(xo,yo,nx,ny,x1,x2,y1,y2,image)
	int *xo,*yo,*nx,*ny,*x1,*x2,*y1,*y2;
      unsigned char *image;
/*	(xo,yo) = VDC coordinate of upper left corner of the image
	          to be shown on the screen
	(nx,ny) = size of the full image array
	(x1,y1)-(x2,y2) = range of image subarray to be displayed
      x1>=1, x2<=nx, y1>=1, y2<=ny (as fortran subscripts)
*/
{
	int nplane=8;
	unsigned short *bitmap;
	unsigned int size;

	size = (((*nx)+15)/16)*(*ny)*16;
	bitmap = (unsigned short *)malloc(size);
	makebitwiseimage(image,*nx,*ny,nplane,bitmap);
	DISP_bitwiseimage_(xo,yo,nx,ny,x1,x2,y1,y2,nplane,bitmap);
	free(bitmap);
}

DISP_bitwiseimage_(xo,yo,nx,ny,x1,x2,y1,y2,nplane,bitmap)
	int *xo,*yo,*nx,*ny,*x1,*x2,*y1,*y2,*nplane;
	unsigned short *bitmap;
/*	(xo,yo) = VDC coordinate of upper left corner of the image
	          to be shown on the screen
	(nx,ny) = size of the full image array
	(x1,y1)-(x2,y2) = range of image subarray to be displayed
	x1>=1, x2<=nx, y1>=1, y2<=ny (as fortran subscripts)

	nplane = 1	dithering
	nplane = 8	halftone map (no magnification)
	nplane = 2, 4: halftone with magnification 2 or 4 (LBP)
*/
{
	Point point();
	int cgi_nplane;
	if (*nplane == 1)	cgi_nplane = 1;
	else			cgi_nplane = 8;

	CGI_Pixel_Array(point(*xo,*yo),*nx,*ny,
			(*x1)-1,(*x2)-1,(*y1)-1,(*y2)-1,cgi_nplane,bitmap);
}

static makebitwiseimage(image,nx,ny,nplane,bitmap)
	unsigned char *image;
	unsigned short *bitmap;
	int nx,ny,nplane;
{
	int x,y,ni,xx,yy,xr,yr,z,linesize,mapsize;
	unsigned short wmask, *wp, w[8], avr;
	unsigned char b, *bp, *bp0, *ip, *ip0;
	static int level[64]	= {
		0,0,0,0,0,0,0,0,0,0,
		1,1,1,1,
		2,2,2,2,
		3,3,3,3,
		4,4,4,4,
		5,5,5,5,
		6,6,6,6,
		7,7,7,7,
		8,8,8,8,
		9,9,9,9,
		10,10,10,10,
		11,11,11,11,
		12,12,12,12,12,12,12,12,12,12	};
	static unsigned char dither8x8[13][8] =	{
		0,0,0,0,0,0,0,0,					/*	0 dot	*/
		0,0,0, 0x08, 0,0,0,0,				/*	1 dot	*/
		0,0,0, 0x22, 0,0,0,0,				/*	2 dots	*/
		0,0x08, 0, 0x08, 0, 0x08, 0,0,		/*	3 dots	*/
		0, 0x44, 0,0, 0, 0x44, 0,0,			/*	4 dots	*/
		0, 0x42, 0, 0x08, 0x20, 0, 0x42, 0,		/*	6 dots	*/
		0, 0x44, 0, 0x88, 0, 0x44, 0, 0x88,		/*	8 dots	*/
		0x11, 0, 0x55, 0, 0x11, 0, 0x55, 0,		/*	12 dots	*/
		0x55, 0, 0x55, 0, 0x55, 0, 0x55, 0,		/*	16 dots	*/
		0x55, 0x44, 0x55, 0x88, 0x55, 0x44, 0x55, 0x88,	/*	24 dots	*/
		0xcc, 0x55, 0xcc, 0x55, 0xcc, 0x55, 0xcc, 0x55,	/*	32 dots	*/
		0xff, 0xcc, 0xff, 0xcc, 0xff, 0xcc, 0xff, 0xcc,	/*	48 dots	*/
		0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff	/*	64 dots	*/
		};
	static unsigned char dither4x4[17][4] = 	{
		0, 0, 0, 0,		/*	0	*/
		0, 4, 0, 0,		/*	1	*/
		0, 5, 0, 0,		/*	2	*/
		0, 5, 0, 4,		/*	3	*/
		0, 5, 0, 5,		/*	4	*/
		0, 5, 2, 5,		/*	5	*/
		2, 5, 2, 5,		/*	6	*/
		2, 5, 10, 5,	/*	7	*/
		6, 5, 10, 5,	/*	8	*/
		6, 13, 10, 5,	/*	9	*/
		7, 13, 10, 5,	/*	10	*/
		7, 13, 10, 13,	/*	11	*/
		7, 13, 11, 13,	/*	12	*/
		7, 13, 11, 15,	/*	13	*/
		7, 15, 11, 15,	/*	14	*/
		7, 15, 15, 15,	/*	15	*/
		15, 15, 15, 15	/*	16	*/
		};

/*
	image(x,y)=image[y][x] -- 8-bit-deep image data
	bitmap(i,y,z)=bitmap[z][y][i] -- bitmap
		y = 0 to ny
		x = 0 to nx
		i = 0 to ni
	size of one bitmap plane = mapsize *16 bits
	if nplane =1, halftone is represented by dithering
*/

	ni = ( (15+ nx) / 16 )  -1;
	if (nplane == 1) goto dither4;
	if (nplane < 1 ) goto dither8;

	mapsize = (ni+1)*ny;	/*	in words	*/
	for (y=0; y< ny; y++)	{
		for (x=0; x< nx-15; x+=16, bitmap++ )	{
			for (xx=0; xx<16 ; xx++, image++)	{
				b= *image;
/*
				for (z=0; z<8; z++)	{
					w[z] <<= 1;
					w[z] |= (b & 1);
					b >>= 1;
				}
				expand do loop	*/
				w[0] <<= 1;
				w[0] |= (b & 1);
				b >>= 1;
				w[1] <<= 1;
				w[1] |= (b & 1);
				b >>= 1;
				w[2] <<= 1;
				w[2] |= (b & 1);
				b >>= 1;
				w[3] <<= 1;
				w[3] |= (b & 1);
				b >>= 1;
				w[4] <<= 1;
				w[4] |= (b & 1);
				b >>= 1;
				w[5] <<= 1;
				w[5] |= (b & 1);
				b >>= 1;
				w[6] <<= 1;
				w[6] |= (b & 1);
				b >>= 1;
				w[7] <<= 1;
				w[7] |= (b & 1);
				b >>= 1;
			}
/*
			for (z=0,wp=bitmap; z<8; z++, wp+=mapsize)
				*wp = w[z];	
			expand do loop	*/
			wp=bitmap;
			*wp=w[0]; wp+=mapsize;
			*wp=w[1]; wp+=mapsize;
			*wp=w[2]; wp+=mapsize;
			*wp=w[3]; wp+=mapsize;
			*wp=w[4]; wp+=mapsize;
			*wp=w[5]; wp+=mapsize;
			*wp=w[6]; wp+=mapsize;
			*wp=w[7]; wp+=mapsize;
		}
/*		for remaining columns if any	*/
		if ( x == nx )	continue;
		wmask = 1;
		for (xx=0 ; xx<16; xx++)	{
			if (xx >= nx - x ) wmask=0;
			b= *image;
			for (z=0; z<8; z++)	{
				w[z] <<= 1;
				w[z] |= ( wmask & ( b & 1) );
				b >>= 1;
			}
			if (wmask) ++image;
		}
		for (z=0,wp=bitmap; z<8; z++, wp+=mapsize)
			*wp = w[z];	
		bitmap++;
	}
	return;

dither4:
	linesize= 2*(ni+1);		/*	in bytes	*/
	ip0 = image;
	bp0 = (char *) bitmap;
      for (y=0; y< ny ; y+=4 )  {
		if ( ny - y >= 4 )	yr = 4;
		else				yr = ny - y;

		for (x=0; x< nx-7; x+=8, ip0 +=8, bp0++ )	{
			avr=0;
			for (yy=0, ip=ip0; yy<yr; yy++, ip+=nx )	{
				for (xx=0; xx<4; xx++)	avr+= *(ip++);
				ip -= 4;
			}
			avr /= (4*yr);
			avr = (avr+1)/16;	/*	avr <= 16	*/

			for (yy=0, bp=bp0; yy<yr; yy++, bp+=linesize)
				*bp = dither4x4[avr][yy] << 4;

			avr=0;
			for (yy=0, ip=ip0+4; yy<yr; yy++, ip+=nx )	{
				for (xx=4; xx<8; xx++)	avr+= *(ip++);
				ip -= 4;
			}
			avr /= (4*yr);
			avr = (avr+1)/16;	/*	avr <= 16	*/

			for (yy=0, bp=bp0; yy<yr; yy++, bp+=linesize)	
				*bp |= dither4x4[avr][yy];
		}

/*		remaining columns if any	*/
		if ( ( xr = nx-x ) > 4 ) {
           		avr=0;
			for (yy=0, ip=ip0; yy<yr; yy++, ip+=nx )	{
				for (xx=0; xx< 4; xx++)	avr+= *(ip++);
				ip -= 4;
			}
			avr /= (4*yr);
			avr = (avr+1)/16;	/*	avr <= 16	*/
			for (yy=0, bp=bp0; yy<yr; yy++, bp+=linesize)
				*bp = dither4x4[avr][yy] << 4;

			xr -=4;
           		avr=0;
			for (yy=0, ip=ip0+4; yy<yr; yy++, ip+=nx )	{
				for (xx=0; xx< xr; xx++)	avr+= *(ip++);
				ip -= xr;
			}
			avr /= xr*yr;
			avr = (avr+1)/16;	/*	avr <= 16	*/
			for (yy=0, bp=bp0; yy<yr; yy++, bp+=linesize)	{
				*bp |= dither4x4[avr][yy];
				*bp >>= (4-xr);
				*bp <<= (4-xr);
			}
			x+=8;
			ip0+= 4+xr;
			bp0++;
		}
		else if ( xr > 0 )	{
           		avr=0;
			for (yy=0, ip=ip0; yy<yr; yy++, ip+=nx )	{
				for (xx=0; xx< xr; xx++)	avr+= *(ip++);
				ip -= xr;
			}
			avr /= (xr*yr);
			avr = (avr+1)/16;	/*	avr <= 16	*/

			for (yy=0, bp=bp0; yy<yr; yy++, bp+=linesize)	{
				*bp = dither4x4[avr][yy];
				*bp >>= (4-xr);
				*bp <<= (4-xr);
				*bp <<= 4;
			}
			ip0 += xr;
			bp0++;
			x +=8;
		}
		if ( x % 16 == 8 )	{
			for (yy=0, bp=bp0; yy<yr; yy++, bp+=linesize)	*bp = 0;
			bp0++;
		}
	ip0 += (yr-1)*nx;
	bp0 += (yr-1)*linesize;
	}

	return;

dither8:
	linesize= 2*(ni+1);		/*	in bytes	*/
	ip0 = image;
	bp0 = (char *) bitmap;
      for (y=0; y< ny ; y+=8 )  {
		if ( ny - y >= 8 )	yr = 8;
		else				yr = ny - y;

		for (x=0; x< nx-7; x+=8, ip0 +=8, bp0++ )	{
			avr=0;
			for (yy=0, ip=ip0; yy<yr; yy++, ip+=nx )	{
				for (xx=0; xx<8; xx++)	avr+= *(ip++);
				ip -= 8;
			}
			avr /= (4*8*yr);	/* 	avr < 64, level[avr] < 13	*/

			for (yy=0, bp=bp0; yy<yr; yy++, bp+=linesize)	
				*bp = dither8x8[level[avr]][yy];
		}

/*		remaining columns if any	*/
		if ( ( xr = nx-x ) > 0 ) {
           		avr=0;
			for (yy=0, ip=ip0; yy<yr; yy++, ip+=nx )	{
				for (xx=0; xx< xr; xx++)	avr+= *(ip++);
				ip -= xr;
			}
			avr /= (4*xr*yr);
			for (yy=0, bp=bp0; yy<yr; yy++, bp+=linesize)	{
				*bp = dither8x8[level[avr]][yy];
				*bp >>= (8-xr);
				*bp <<= (8-xr);
			}
			ip0 += xr;
			bp0++;
			x +=8;
		}
		if ( x % 16 == 8 )	{
			for (yy=0, bp=bp0; yy<yr; yy++, bp+=linesize)	*bp = 0;
			bp0++;
		}
	ip0 += (yr-1)*nx;
	bp0 += (yr-1)*linesize;
	}
}

DISP_pointermode_(mode)
	int *mode;
/*	mode=1: cross hair
	     2: tracking cross
	     3: rubber line
	     4: rubber rectangle
	     5: hardware cursor
*/
{
	EchoDataRecord echodata;
	int shape;
	shape = *mode;
	if (shape <1 || shape >5)	shape=1;

	CGI_Set_Echo_Data(LOCATOR,0,shape+1,PROMPT_TONE,ACK_TONE,
		dc1,dc2,echodata);
}

DISP_pointerprompt_(onoff)
	int *onoff;
{
	CGI_Set_Prompt_State(LOCATOR,0,1-(*onoff));
}

DISP_pointerack_(onoff)
	int *onoff;
{
	CGI_Set_Acknowledgement_State(LOCATOR,0,1-(*onoff));
}

DISP_pointerrequest_(x,y,buttonnumber)
	int *buttonnumber, *x,*y;
/*	trigger =1: keyboard return        mapped to 0
	        =2: keyboard change                  0
	        =3: pfkey change                     0
	        =4: mouse move                       0
	        =5: left button                      1
	        =6: right button                     2
	        =7: right button                     3
	        =8: keyboard backspace               0
	        =9: keyboard delete                  0

	returned values (x,y) are in VDC(?)
*/
{
	int trigger,status,valid;
	static int triggermapping[]={0,0,0,0,1,2,3,0,0};
	InputValue value;

	CGI_Request(LOCATOR,0,-1, &status,&valid,&trigger, &value);

	if ( trigger >= 1 && trigger <= 9)	{
		*buttonnumber = triggermapping[trigger-1];
		if (trigger != TRIGGER_FIRED || valid != VALUE_OK)
			*buttonnumber =0;
	} else	*buttonnumber =0;
	*x = value.locator_Value.x;
	*y = value.locator_Value.y;
}

DISP_pointersample_(x,y,status)
	int *x,*y,*status;
{
	InputValue value;
	CGI_Sample(LOCATOR,0, &value,status);
	*x = value.locator_Value.x;
	*y = value.locator_Value.y;
}

