/*	x11lib.c
	X-window library based on cgi conventions
*/

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>

#ifdef DISPONLY
#include "disponly.h"
#endif

/*	parameters for display and frame buffer	*/
#include "crt.h"

/*	int = long (4-byte) integer = fortran integer	*/

extern struct common_sys
	{int mycode,myint,keyin,mag_stdin,lst,msg,msglvl,errflg;
	 float ufl; short int iufl2,maxi2; long int iufl4,maxi4;}
	sys_;	/*	fortran common/sys/	*/

#define DISPstrlen 80
extern struct common_disp
        {char dumpfile[DISPstrlen];}
        dispstr_;        /*      fortran common/dispstr/  */

Display *display;
Window window, root, parent;
GC gc, gc_text, gc_marker, gc_fill, gc_image;
XEvent event;

#define FILL_HOLLOW 1
#define FILL_SOLID 2
static int fillstyle = 1;

#define DIRECTCOLOR 0
#define INDEXEDCOLOR 1
#define WRITABLEMAP 1
#define READONLYMAP 0
#ifndef SYSTEMCOLOR
#define SYSTEMCOLOR 16
#endif
unsigned int int_r[256],int_g[256],int_b[256], max_colorindex;
static unsigned int colorbase=SYSTEMCOLOR;

static Colormap colormap,defaultmap;
int colormode = INDEXEDCOLOR;
/*	static unsigned long colorindex = SYSTEMCOLOR;	*/
static int colormapmode;

#define MAXFONTNAMES 5
/*	font names taken from /usr/lib/X11/fonts/misc	*/
static char *fontnamelist[MAXFONTNAMES]={
	"7x13",
	"5x8",
	"6x10",
	"8x13",
	"9x15"};
static Font fontlist[MAXFONTNAMES];

#define DISPMAXBUF 1024
static XPoint pointlist[DISPMAXBUF];
static XSegment segmentlist[DISPMAXBUF];

static int vpunits = 1;
/*    default viewport units = 1/10mm     */

static int xs=0, ys=0;              /*    start point of VDC      */
static int xe=32767, ye=32767;            /*    end point of VDC  */
static float xfactor, yfactor, scalefactor;
static int xoffset, yoffset;
static int disp_xmaxdot, disp_ymaxdot;
static int linetype=1, linewidth=1;
static int markertype=1;	/*	marker=dot	*/
static int markersize=10;

/*	16X16 stipple patterns	*/
static unsigned char hatch1[]=	{	0xff,0xff, 0x00,0x00, 0x00,0x00, 0x00,0x00,
					0xff,0xff, 0x00,0x00, 0x00,0x00, 0x00,0x00,
					0xff,0xff, 0x00,0x00, 0x00,0x00, 0x00,0x00,
					0xff,0xff, 0x00,0x00, 0x00,0x00, 0x00,0x00},
		hatch2[]=	{	0x88,0x88, 0x88,0x88, 0x88,0x88, 0x88,0x88,
					0x88,0x88, 0x88,0x88, 0x88,0x88, 0x88,0x88,
					0x88,0x88, 0x88,0x88, 0x88,0x88, 0x88,0x88,
					0x88,0x88, 0x88,0x88, 0x88,0x88, 0x88,0x88},
		hatch3[]=	{	0x80,0x80, 0x01,0x01, 0x02,0x02, 0x04,0x04,
					0x08,0x08, 0x10,0x10, 0x20,0x20, 0x40,0x40,
					0x80,0x80, 0x01,0x01, 0x02,0x02, 0x04,0x04,
					0x08,0x08, 0x10,0x10, 0x20,0x20, 0x40,0x40},
		hatch4[]=	{	0x80,0x80, 0x40,0x40, 0x20,0x20, 0x10,0x10,
					0x08,0x08, 0x04,0x04, 0x02,0x02, 0x01,0x01,
					0x80,0x80, 0x40,0x40, 0x20,0x20, 0x10,0x10,
					0x08,0x08, 0x04,0x04, 0x02,0x02, 0x01,0x01},
		hatch5[]=	{	0xff,0xff, 0x88,0x88, 0x88,0x88, 0x88,0x88,
					0xff,0xff, 0x88,0x88, 0x88,0x88, 0x88,0x88,
					0xff,0xff, 0x88,0x88, 0x88,0x88, 0x88,0x88,
					0xff,0xff, 0x88,0x88, 0x88,0x88, 0x88,0x88},
		hatch6[]=	{	0x80,0x80, 0x41,0x41, 0x22,0x22, 0x14,0x14,
					0x08,0x08, 0x14,0x14, 0x22,0x22, 0x41,0x41,
					0x80,0x80, 0x41,0x41, 0x22,0x22, 0x14,0x14,
					0x08,0x08, 0x14,0x14, 0x22,0x22, 0x41,0x41},
		hatch0[]=	{	0x00,0x00, 0x00,0x00, 0x00,0x00, 0x00,0x00,
					0x00,0x00, 0x00,0x00, 0x00,0x00, 0x00,0x00,
					0x00,0x00, 0x00,0x00, 0x00,0x00, 0x00,0x00,
					0x00,0x00, 0x00,0x00, 0x00,0x00, 0x00,0x00};
static Pixmap hatch[7];

/*	internal functions	*/

static XPoint point(x,y)
	int x,y;
{
	XPoint p;
	p.x= xconv(x);
	p.y= yconv(y);
	return(p);
}

static XSegment segment(x1,y1,x2,y2)
	int x1,y1,x2,y2;
{
	XSegment seg;
	seg.x1 = xconv(x1);
	seg.y1 = yconv(y1);
	seg.x2 = xconv(x2);
	seg.y2 = yconv(y2);
	return (seg);
}

static XRectangle rectangle(x1,y1,x2,y2)
	int x1,y1,x2,y2;
{
	XRectangle rec;
	int xul,yul;

	xul = (x1<x2)? x1:x2;
	yul = (y1<y2)? y1:y2;
	rec.x= xconv(xul);
	rec.y= yconv(yul);
	rec.width  = abs(xconv(x2) -xconv(x1));
	rec.height = abs(yconv(y2) -yconv(y1));
	if (yfactor < 0) rec.y -= rec.height;
	return(rec);
}

/*    conversion from VDC to window coordinates    */

static xconv(x)
      int x;
{
      return ( (int)( xfactor * x + xoffset ) );
}

static yconv(y)
      int y;
{
      return ( (int)( yfactor * y + yoffset ) );
}

static lconv(a)
      int a;
{
      return ( (int)( scalefactor * a ) );
}

static unsigned int get_colorindex(r,g,b)
	int r,g,b;
{
	int index;
	
	for (index=0; index<max_colorindex; index++)	{
		if (r == int_r[index] && g == int_g[index] && b == int_b[index] )
			return (index);
	}
	index=max_colorindex;
	max_colorindex++;
	int_r[index]=r;
	int_g[index]=g;
	int_b[index]=b;
	return (index);
}
	

DISP_initialize_(initmode)
	int *initmode;
{
	int revert, f;

	display = XOpenDisplay (NULL);
	XGetInputFocus (display, &parent, &revert);
		/*	parent is the id of the window initiated this program	*/
	root = RootWindow (display,0);
	defaultmap = DefaultColormap (display, 0);

	for (f=0; f<MAXFONTNAMES; f++)
		fontlist[f] = XLoadFont (display, fontnamelist[f]);

	hatch[0] = XCreateBitmapFromData(display,root,hatch0,16,16);
	hatch[1] = XCreateBitmapFromData(display,root,hatch1,16,16);
	hatch[2] = XCreateBitmapFromData(display,root,hatch2,16,16);
	hatch[3] = XCreateBitmapFromData(display,root,hatch3,16,16);
	hatch[4] = XCreateBitmapFromData(display,root,hatch4,16,16);
	hatch[5] = XCreateBitmapFromData(display,root,hatch5,16,16);
	hatch[6] = XCreateBitmapFromData(display,root,hatch6,16,16);
}

DISP_reset_()
{
}

DISP_redraw_()
/*	if exposure event is detected, return 1	*/
{
/*	if (XCheckMaskEvent ( display, ExposureMask, &event)) return (1);	*/
	if (XCheckWindowEvent ( display, window, ExposureMask, &event)) return (1);
	return(0);
}

DISP_terminate_()
{
	int i;
	if (colormapmode == WRITABLEMAP)	{
		XUninstallColormap(display,colormap);
		XFreeColormap(display,colormap);
	}
	for (i=0; i<7; i++)	XFreePixmap(display,hatch[i]);

	XFlush(display);
/*	getchar();	*/
	XCloseDisplay (display);
}

DISP_clear_()
{
	XClearWindow (display, window);
}

DISP_bgcolor_(r,g,b)
	unsigned int *r,*g,*b;
/*	r,g,b = 0 to 255	
	if colormode=INDEXEDCOLOR, r contains color index	*/
{
	XColor color;
	unsigned int short mult=256, red,green,blue;

	if (colormode == DIRECTCOLOR)	{
		red   = mult * (*r);
		green = mult * (*g);
		blue  = mult * (*b);
		color.red   = red;
		color.green = green;
		color.blue  = blue;
		color.flags = DoRed|DoGreen|DoBlue;
		if (colormapmode == WRITABLEMAP)	{
			color.pixel = colorbase + get_colorindex(*r,*g,*b);
			XStoreColor (display, colormap, &color);
			XSetWindowBackground (display, window, color.pixel);
		}
		else	{
			XAllocColor (display, colormap, &color);
			XSetWindowBackground (display, window, color.pixel);
		}	
	}
	else	{
		XSetWindowBackground (display, window, (*r)-1+colorbase);
		if ( sys_.msglvl > 3 )
			printf("\n (x11lib/bgcolor) xcgi colorindex=%d, system index=%ld", *r, (*r)-1+colorbase);
	}
}

DISP_vdcextent_(x1,y1,x2,y2)
	int *x1, *y1, *x2, *y2;
{
      xs = *x1;   ys = *y1;
      xe = *x2;   ye = *y2;
}

DISP_viewportunits_(units)
	int *units;
/*	units=0:				percent relative
	     =1(default here):		in 1/10 mm
	     =2:				absolute		*/
{
	vpunits = *units;
}

DISP_viewport_(x1,y1,x2,y2)
	int *x1,*y1,*x2,*y2;
/*	viewport specification
	this routine can be used for viewportunits=1,2

      (0,0) is at the upper left corner of the screen
      VDC (xs,ys),(xe,ye) are mapped to viewport (x1,y1),(x2,y2)
	the initial window of width x2-x1 and height y2-y1 appears at (x1,y1)
*/
{
	float xx1,yy1,xx2,yy2;
	unsigned int xul,yul,width,height;
	unsigned int xoff=10, yoff=10;

	XColor color;
	Visual myvisual;
	unsigned int bordercolor, backgroundcolor;
	unsigned int borderwidth =1;
	XSetWindowAttributes attr;
	unsigned int index, startindex_f=1,ncolor=2;
	unsigned int nplanes=0, ncolors=256;
	unsigned long planes[8],pixels[256];
	static char *visualclass[]={	"StaticGray",
						"GrayScale",
						"StaticColor",
						"PseudoColor",
						"TrueColor",
						"DirectColor"};

	if (vpunits == 1)	{	/*	(x1,y1),(x2,y2) are in units of 1/10mm	*/
		xx1 = (*x1)/DISPdotsize;
		yy1 = (*y1)/DISPdotsize;
		xx2 = (*x2)/DISPdotsize;
		yy2 = (*y2)/DISPdotsize;
	}
	if (vpunits == 2)	{	/*	(x1,y1),(x2,y2) are in dots	*/
		xx1 = *x1;	yy1 = *y1;
		xx2 = *x2;	yy2 = *y2;
	}

      xfactor = ( xx2 - xx1 )/( xe - xs );
      yfactor = ( yy2 - yy1 )/( ye - ys );
		/*	x,yfactor * VDC length  -> dots	*/
      if ( fabs((double)xfactor) > fabs((double)yfactor) )
            scalefactor= fabs(yfactor);
      else
            scalefactor= fabs(xfactor);

	xul = (xx1<xx2)? xx1:xx2;
	yul = (yy1<yy2)? yy1:yy2;	/*	(xul,yul)=upper left corner	*/
	width =  fabs((double)(xx2-xx1) ) + 2*xoff;
	height = fabs((double)(yy2-yy1) ) + 2*yoff;

	xoffset = xx1 - xfactor * xs - xul + xoff;
	yoffset = yy1 - yfactor * ys - yul + yoff;
	/*	xul,yul are subtracted because plotting is made on the current
		window, not on the root window	*/

	bordercolor = BlackPixel(display,0);
	backgroundcolor = BlackPixel(display,0);

	window = XCreateSimpleWindow (display,root, 
			xul+xoff, yul+yoff, width, height,
			borderwidth, bordercolor, backgroundcolor);

	myvisual = *(DefaultVisual(display, 0));
	if (sys_.msglvl > 2)	{
		printf("\n (DISP_viewport) visual structure values:");
		printf("\n visualid=%d",myvisual.visualid);
		printf("\n class =%d",myvisual.class);
		printf("\n masks =%ul %ul %ul",
		myvisual.red_mask, myvisual.green_mask, myvisual.blue_mask);
		printf("\n bits per rgb=%d",myvisual.bits_per_rgb);
	}
/*	visual class:
	StaticGray = 0
	GrayScale  = 1
	StaticColor= 2
	PseudoColor= 3
	TrueColor  = 4
	DirectColor= 5
*/
	
	if (myvisual.class == GrayScale ||
	    myvisual.class == PseudoColor )	{
		colormap = XCreateColormap(display, window,
                	&myvisual, AllocAll);
		colormapmode = WRITABLEMAP;
	}
	else if (myvisual.class == DirectColor)	{
		colormap = XCreateColormap(display, window,
                	&myvisual, AllocNone);
		XAllocColorCells(display, colormap, False, 
			planes,nplanes,pixels,ncolors);
		colormapmode = WRITABLEMAP;
	}
	else	{
		colormap = DefaultColormap (display, 0);
		colormapmode = READONLYMAP;
/*		this case, colormap = defaultmap	*/
	}


	for (index=0; index<colorbase; index++)       {
		color.pixel = index;
		color.flags = DoRed|DoGreen|DoBlue;
		XQueryColor(display, defaultmap, &color);
		if (colormapmode == WRITABLEMAP)
			XStoreColor (display, colormap, &color);
			/*      copy colors from standard color map     */
		if (sys_.msglvl > 2)	{
			printf("\n (DISP_viewport) index,r,g,b=%d, %ld,%ld,%ld",
			color.pixel,color.red,color.green,color.blue);
			printf(" pixel=%ld",pixels[index]);
		}
        }
/*      generally index=0,1 are black and white */

	if (sys_.msglvl > 1)	{
		printf("\n (DISP_viewport) colorbase=%d",colorbase);
		printf("\n       visual type=%s",visualclass[myvisual.class]);
		printf("\n       map entries=%ld\n",myvisual.map_entries);
	}

	DISP_colortable_( &startindex_f,&ncolor,int_r,int_g,int_b);
	max_colorindex=2;
/*	register two colors	*/

	XStoreName (display, window, "magpack2");
	if (colormapmode == WRITABLEMAP)	{
		XInstallColormap ( display, colormap );
		XSetWindowColormap(display, window, colormap);
	}


	attr.backing_store = WhenMapped;
/*	attr.backing_store = Always;            does not work   */
	XChangeWindowAttributes (display, window, CWBackingStore, &attr);

	XSelectInput (display,window,
		ExposureMask|ResizeRedirectMask|ButtonPressMask|ButtonMotionMask );
	XMapWindow(display, window);

	gc = XCreateGC (display,window,0,0);
	gc_text = XCreateGC (display,window,0,0);
	gc_marker = XCreateGC (display,window,0,0);
	gc_fill = XCreateGC (display,window,0,0);
	gc_image = XCreateGC (display,window,0,0);
	XSetFont(display,gc_text,fontlist[0]);

/*	default fill style is 'hollow'	*/
	XSetStipple(display,gc_fill,hatch[0]);
	XSetFillStyle(display,gc_fill,FillStippled);

/*	wait until the window is exposed for the first time	*/
	while(1)	{
		XNextEvent ( display,&event);
		if (event.type == Expose) break;
	}
/*	usleep(100000);	*/
	sleep(1);
	XSetInputFocus (display, parent, RevertToPointerRoot, CurrentTime);
		/*	change focus window to the parent window
			this should be done after XMoveResize	*/
	XRaiseWindow(display, parent);
		/*	For SunWorkstations, the window where the cursor is located
			is the focused window which can accept keyboard input. 
			The focused window is not necessarily the top-level window.
			Make the parent window at the top level.	*/

	if (sys_.msglvl > 2)	{
		printf("\n (DISP_viewport)");
		printf("\n      vpunits,factor=%d,%g,%g", vpunits,xfactor,yfactor);
		printf("\n      xs,ys,xe,ye=%d,%d,%d,%d",xs,ys,xe,ye);
		printf("\n      x,yoffset=%d,%d",xoffset,yoffset);
		printf("\n      x1,y1,x2,y2=%d,%d,%d,%d",*x1,*y1,*x2,*y2);
		printf("\n      xx1,yy1,xx2,yy2=%g,%g,%g,%g",xx1,yy1,xx2,yy2);
		printf("\n      window geometry=%d,%d,%d,%d",xul,yul,width,height);
	}
}

DISP_viewportf_( fx1,fy1,fx2,fy2)
      float *fx1,*fy1,*fx2,*fy2;
/*    viewport specification
      this routine can be used for viewportunits=0 (percent relative) */
{
	unsigned int x1,y1,x2,y2;

	if (vpunits == 0) {
		
		x1 = 0.01*(*fx1)*disp_xmaxdot;
		y1 = 0.01*(*fy1)*disp_ymaxdot;
		x2 = 0.01*(*fx2)*disp_xmaxdot;
		y2 = 0.01*(*fy2)*disp_ymaxdot;

		vpunits = 2;	/*	converted to dots	*/
		DISP_viewport_( &x1,&y1,&x2,&y2);
		vpunits = 0;
	}
}

DISP_viewportmapping_(isotropy,horizontal,vertical)
	int *isotropy,*horizontal,*vertical;
{
}

DISP_clipindicator_(indicator)
	int *indicator;
/*	indicator=0(default):	clip rectangle
	         =1:		view surface
	         =2:		off		*/
{
}

DISP_cliprectangle_(x1,y1,x2,y2)
	int *x1,*y1,*x2,*y2;
{
	XRectangle rectangle(), rec[1];

	rec[0] = rectangle ( *x1, *y1, *x2, *y2 );
	XSetClipRectangles (display, gc, 0,0, rec, 1, YXSorted);
	XSetClipRectangles (display, gc_fill, 0,0, rec, 1, YXSorted);
	XSetClipRectangles (display, gc_text, 0,0, rec, 1, YXSorted);
	XSetClipRectangles (display, gc_marker, 0,0, rec, 1, YXSorted);
	XSetClipRectangles (display, gc_image, 0,0, rec, 1, YXSorted);
}

DISP_hcopy_()
{
	char cbuffer[100];
	int status, i;

	for (i=DISPstrlen-2; i>0; i--)	{
		if (dispstr_.dumpfile[i] != ' ')	{
			dispstr_.dumpfile[i+1] = '\0';
			sprintf(cbuffer,
			"xwd -id %ld %s",window, dispstr_.dumpfile);

/*			<dumpfile> can be "-out <outputfile>"	*/

        		if( (status = system(cbuffer)) != 0 )	{
				printf(
	"\n (x11lib/hcopy) error: command execution failed, command=%s",
                		cbuffer);
			} else	sleep(1);

        		return (status);
		}
	}
	if (sys_.msglvl > 0)	{
		printf("\n (x11lib/hcopy) dumpfile name is empty");
	}
	return;
}

DISP_polyline_(np,x,y)
	int *np,x[],y[];
{
	int i, is=0, n;
	XPoint point();

	if ( *np == 1 || *np == 2 && x[0] == x[1] && y[0] == y[1] )	{
		pointlist[0]=point(x[0],y[0]);
		XDrawPoints (display, window, gc, pointlist, 1, CoordModeOrigin);
	}
	else	{
		do {
			n = ( *np -is > DISPMAXBUF) ? DISPMAXBUF : (*np -is);
			for (i=0; i<n; i++) pointlist[i]=point(x[i+is],y[i+is]);
			XDrawLines (display, window, gc, pointlist, n,
					CoordModeOrigin);
			is += n-1;
		} while ( is < *np -1 );
	}
	XFlush(display);
}

DISP_disjointline_(np,x,y)
	int *np,x[],y[];
{
	int i, is=0, ie, n, n_even;
	XSegment segment();

	n_even = ((*np)/2)*2;
	do {
		n = ( n_even -is > DISPMAXBUF) ? DISPMAXBUF : (n_even -is);
		ie += n-1;
		for (i=0; i<n; i+=2)
			segmentlist[i/2]=segment(x[i+is],y[i+is], x[i+is+1],y[i+is+1]);
		XDrawSegments (display, window, gc, segmentlist, n_even/2);
		is += n;
	} while ( ie < n_even - 1 );
	XFlush(display);
}

DISP_arc_(cx,cy,r,angles,anglee)
	int *cx,*cy,*r;
	float *angles,*anglee;
{
	XRectangle rectangle(),rec;
	unsigned int x1,y1,x2,y2;
	float angs,ange;
	int angle1, angle2;

	x1 = *cx - *r;
	y1 = *cy - *r;
	x2 = x1 + *r;
	y2 = y1 + *r;
	rec = rectangle( x1,y1,x2,y2);

	angs = *angles;
	ange = *anglee;
      if (xfactor < 0.0)	{
		angs = 180.0 - angs;
		ange = 180.0 - ange;
	}
      if (-yfactor < 0.0)	{
		angs = -angs;
		ange = -ange;
	}
	angle1 = 64 *angs;
	angle2 = 64 *(ange-angs);

	XDrawArc (display, window, gc, rec.x, rec.y,
			rec.width, rec.height, angle1,angle2);
	XFlush(display);
}

DISP_arc3_(x1,y1,x2,y2,x3,y3)
	int *x1,*y1,*x2,*y2,*x3,*y3;
{
}

DISP_elliparc_(cx,cy,a,b,anglea,angles,anglee)
	int *cx,*cy,*a,*b;
	float *anglea,*angles,*anglee;
{
	XRectangle rectangle(),rec;
	unsigned int x1,y1,x2,y2;
	float angs, ange;
	int angle1, angle2;

	x1 = *cx - *a;
	y1 = *cy - *b;
	x2 = x1 + *a;
	y2 = y1 + *b;
	rec = rectangle( x1,y1,x2,y2);

      angs = *angles;
      ange = *anglee;
      if (xfactor < 0.0)      {
            angs = 180.0 - angs;
            ange = 180.0 - ange;
      }
      if (-yfactor < 0.0)      {
            angs = -angs;
            ange = -ange;
      }
	angle1 = 64 *angs;
	angle2 = 64 *(ange-angs);
	
	XDrawArc (display, window, gc, rec.x, rec.y,
			rec.width, rec.height, angle1,angle2);
	XFlush(display);
}

static lineattribute(type,width)
	int type, width;
/*	type=1(default):		solid
	    =2:			dashed
	    =3:			dotted
	    =4:			dash-dot
	    =5:			dash-dot-dot

	width in pixels		*/
{
	static char dashed[] ={ 4,4};
	static char dotted[] ={ 2,2};
	static char dashdot[] ={ 8,2,2,2};
	static char dashdotdot[] ={ 8, 2,2,2,2,2};
 
	switch (type)	{
	case 1:	XSetLineAttributes (display, gc, width, LineSolid,
					CapButt, JoinMiter);
			break;
	case 2:	XSetLineAttributes (display, gc, width, LineOnOffDash,
					CapButt, JoinMiter);
			XSetDashes (display, gc, 0, dashed, 2);
			break;
	case 3:	XSetLineAttributes (display, gc, width, LineOnOffDash,
					CapButt, JoinMiter);
			XSetDashes (display, gc, 0, dotted, 2);
			break;
	case 4:	XSetLineAttributes (display, gc, width, LineOnOffDash,
					CapButt, JoinMiter);
			XSetDashes (display, gc, 0, dashdot, 4);
			break;
	case 5:	XSetLineAttributes (display, gc, width, LineOnOffDash,
					CapButt, JoinMiter);
			XSetDashes (display, gc, 0, dashdotdot, 6);
			break;
	}
}

DISP_linetype_(type)
	int *type;
{
	linetype = *type;
	lineattribute( linetype, linewidth);
}
DISP_linewidth_(iwidth)
	int *iwidth;
/*	linewidth specification in absolute mode (in VDC unit, integer)	*/
{
	linewidth = lconv(*iwidth);
	lineattribute( linetype, linewidth);
}

DISP_linefwidth_(fwidth)
	float *fwidth;
/*	linewidth specification in scaled mode (in pixel unit, floating)	 */
{
	linewidth = (int)(*fwidth);
	lineattribute( linetype, linewidth);
}

static fgcolor(r,g,b, gc1)
	int r,g,b;
	GC gc1;
/*    r,g,b = 0 to 255
      if colormode=INDEXEDCOLOR, r contains color index     */
{
	XColor color;

	unsigned short mult=256, red,green,blue;

	if (colormode == DIRECTCOLOR) {
		red   = mult * r;
		green = mult * g;
		blue  = mult * b;
		color.red   = red;
		color.green = green;
		color.blue  = blue;
		color.flags = DoRed|DoGreen|DoBlue;
		if (colormapmode == WRITABLEMAP)	{
			color.pixel = colorbase + get_colorindex(r,g,b);
			XStoreColor (display, colormap, &color);
			XSetForeground(display, gc1, color.pixel);
		}
		else	{
			XAllocColor (display, colormap, &color);
			XSetForeground(display, gc1, color.pixel);
		}	
	}
	else	{
		XSetForeground(display, gc1, (r-1)+colorbase);
		if ( sys_.msglvl > 3 )
			printf("\n (x11lib/fgcolor) xcgi colorindex=%d, system index=%ld", r, (r-1)+colorbase);
	}
}

DISP_linecolor_(r,g,b)
	int *r,*g,*b;
/*    r,g,b = 0 to 255
      if colormode=INDEXEDCOLOR, r contains color index     */
{
	fgcolor(*r,*g,*b,gc);
}

DISP_polymarker_(np,x,y)
	int *np,x[],y[];
{
/*	markertype	=1:	dot (default of cgi)
	    		=2:	plus
			=3:	asterisk
			=4:	circle
			=5:	cross
			=6:	square		(added for x11)
			=7:	filled squares	(added for x11)
			=8:	triangle		(added for x11)
			=9:	filled triangle	(added for x11)
*/
	XPoint point();
	XSegment segment();
	XRectangle rectangle(),rec;
	unsigned int x1,y1,x2,y2;
	int i, j;
	static float	plusx[]=	{ -1, 1, 0, 0},
		 		plusy[]=	{  0, 0,-1, 1},

				astx[]=	{  0, 0, -0.866, 0.866, 0.866, -0.866},
				asty[]=	{ -1, 1, -0.5,   0.5,   -0.5,   0.5},

				crossx[]=	{ -1, 1, -1, 1},
				crossy[]=	{ -1, 1, 1, -1},

				trix[]=	{  0, -0.866, 0.866, 0},
				triy[]=	{  1, -0.5,   -0.5,  1};

	for( i=0; i< *np; i++)	{
		switch (markertype)	{

		case 1:	/*	dots	*/
			pointlist[0]=point(x[i],y[i]);
			XDrawPoints (display, window, gc_marker,
				pointlist, 1, CoordModeOrigin);
			break;

		case 2:	/*	plus signs	*/
			for( j=0; j<4; j+=2 )
				segmentlist[j/2]=segment(
					x[i] +(int)(markersize*plusx[j]),
					y[i] +(int)(markersize*plusy[j]),
					x[i] +(int)(markersize*plusx[j+1]),
					y[i] +(int)(markersize*plusy[j+1]) );
			XDrawSegments (display, window, gc_marker,
				segmentlist, 2);
			break;

		case 3:	/*	asterisks	*/
			for( j=0; j<6; j+=2 )
				segmentlist[j/2]=segment(
					x[i] +(int)(markersize*astx[j]),
					y[i] +(int)(markersize*asty[j]),
					x[i] +(int)(markersize*astx[j+1]),
					y[i] +(int)(markersize*asty[j+1]) );
			XDrawSegments (display, window, gc_marker,
				segmentlist, 3);
			break;

		case 4:	/*	circles	*/
			x1 = x[i] -markersize;
			y1 = y[i] -markersize;
			x2 = x[i] +markersize;
			y2 = y[i] +markersize;
			rec = rectangle( x1,y1,x2,y2);
			XDrawArc (display, window, gc_marker, rec.x, rec.y,
				rec.width, rec.height, 0, 64*360);
			break;

		case 5:	/*	crosses	*/
			for( j=0; j<4; j+=2 )
				segmentlist[j/2]=segment(
					x[i] +(int)(markersize*crossx[j]),
					y[i] +(int)(markersize*crossy[j]),
					x[i] +(int)(markersize*crossx[j+1]),
					y[i] +(int)(markersize*crossy[j+1]) );
			XDrawSegments (display, window, gc_marker,
				segmentlist, 2);
			break;

		case 6:	/*	squares	*/
			x1 = x[i] -markersize;
			y1 = y[i] -markersize;
			x2 = x[i] +markersize;
			y2 = y[i] +markersize;
			rec = rectangle( x1,y1,x2,y2);
			XDrawRectangle (display, window, gc_marker, 
				rec.x, rec.y, rec.width, rec.height);
			break;

		case 7:	/*	filled squares	*/
			x1 = x[i] -markersize;
			y1 = y[i] -markersize;
			x2 = x[i] +markersize;
			y2 = y[i] +markersize;
			rec = rectangle( x1,y1,x2,y2);
			XDrawRectangle (display, window, gc_marker, 
				rec.x, rec.y, rec.width, rec.height);
			XFillRectangle (display, window, gc_marker, 
				rec.x, rec.y, rec.width, rec.height);
			break;
			
		case 8:	/*	triangles	*/
			for (j=0; j<4; j++)
				pointlist[j]=point(
					x[i] +(int)(markersize*trix[j]),
					y[i] +(int)(markersize*triy[j]) );
			XDrawLines (display, window, gc_marker, pointlist, 4,
				CoordModeOrigin);
			break;

		case 9:	/*	filled triangles	*/
			for (j=0; j<4; j++)
				pointlist[j]=point(
					x[i] +(int)(markersize*trix[j]),
					y[i] +(int)(markersize*triy[j]) );
			XDrawLines (display, window, gc_marker, pointlist, 4,
				CoordModeOrigin);
			XFillPolygon (display, window, gc_marker, pointlist,
				 3, Nonconvex, CoordModeOrigin);
			break;
		}
	}
	XFlush(display);
}

DISP_markertype_(type)
	int *type;
{
	markertype = *type;
}

DISP_markersize_(isize)
	int *isize;
/*	marker size specification in absolute mode (in VDC unit, integer)	*/
{
	markersize = lconv(*isize);
}

DISP_markerfsize_(fsize)
	float *fsize;
/*	marker size specification in scaled mode (in pixel unit, floating)*/
{
	markersize = (*fsize) * DISPdotsize;
}

DISP_markercolor_(r,g,b)
	int *r,*g,*b;
/*    r,g,b = 0 to 255
      if colormode=INDEXEDCOLOR, r contains color index     */
{
      fgcolor(*r,*g,*b,gc_marker);
}

DISP_text_(x,y,string,len)
	int *x,*y,len;
	char *string;
/*	FORTRAN calling sequence: call text(x,y,string)	*/
{
	XDrawString (display, window, gc_text, xconv(*x),yconv(*y), string, len);
	XFlush(display);
}

DISP_textnf_(x,y,string,len)
	int *x,*y,len;
	char *string;
{
	XDrawString (display, window, gc_text, xconv(*x),yconv(*y), string, len);
	XFlush(display);
}

DISP_restrictedtext_(wx,wy,x,y,flag,string,len)
	int *wx,*wy,*x,*y,*flag,len;
	char *string;
/*	FORTRAN calling sequence: call restrictedtext(wx,wy,x,y,flag,string)
	flag=0: string not final
	    =1: final			*/
{
	XDrawString (display, window, gc_text, xconv(*x),yconv(*y), string, len);
	XFlush(display);
}

DISP_appendtext_(flag,string,len)
	int *flag,len;
	char *string;
/*	FORTRAN calling sequence: call appendtext(flag,string)
	flag=0: string not final
	    =1: final			*/
{
/*	not supported	*/
}

DISP_textcolor_(r,g,b)
	int *r,*g,*b;
/*    r,g,b = 0 to 255
      if colormode=INDEXEDCOLOR, r contains color index     */
{
      fgcolor(*r,*g,*b,gc_text);
}

DISP_chrheight_(height)
	int *height;
{
/*	height is in VDC unit 
*/
/*	needs font selection: not implemented	*/
}

DISP_chrwidth_(ratio)
	float *ratio;
{
/*	needs font selection: not implemented	*/
}

DISP_chrspacing_(spacing)
	float *spacing;
{
/*	needs font selection: not implemented	*/
}

DISP_chrorientation_(angleb,angleu)
	float *angleb,*angleu;
{
/*	not implemented	*/
}

DISP_textpath_(path)
	int *path;
/*	path=0:			left
	    =1:			up
	    =2(default):	right
	    =3:			down		*/
{
/*	not implemented	*/
}

DISP_textalignment_(halign,valign,chalign,cvalign)
	int *halign,*valign;
	float *chalign,*cvalign;
/*	halign=0:	left
	      =1:	center
	      =2:	right
	      =3:	normal hirizontal
	      =4:	continuous horizontal
	valign=10:	top
	      =11:	cap
	      =12:	half
	      =13:	base
	      =14:	bottom
	      =15:	normal vertical
	      =16:	continuous vertical		*/

{
/*	not implemented	*/
}

DISP_fontindex_(index_f)
	int *index_f;
/*	index_f = 1,2,3,,,	*/
{
	int index;
	index = *index_f -1;
	XSetFont (display, gc_text, fontlist[index]);
}

DISP_polygon_(np,x,y)
	int *np,x[],y[];
{
	XPoint point();
	int i;

	for (i=0; i< (*np) && i< DISPMAXBUF -1; i++)	{
		pointlist[i]=point(x[i],y[i]);
	}


	XFillPolygon (display, window, gc_fill, pointlist, *np, 
			Nonconvex, CoordModeOrigin);

	if (fillstyle == FILL_HOLLOW)	{
		pointlist[i] = point(x[0],y[0]);
		XDrawLines (display, window, gc, pointlist, (*np)+1,
					CoordModeOrigin);
	}
/*	Xfillpolygon does not draw borders	*/
	XFlush(display);
}

DISP_polygonset_(np,x,y,flag)
	int *np,x[],y[];
	char *flag[];
/*	flag=0:		invisible
	    =1:		visible
	    =2:		close_invisible
	    =3:		close_visible		*/
{
/*	not supported	*/
}

DISP_rectangle_(x1,y1,x2,y2)
	int *x1,*y1,*x2,*y2;
{
	XRectangle rectangle(),rec;

	rec = rectangle(*x1,*y1,*x2,*y2);
	XFillRectangle (display, window, gc_fill, 
		rec.x, rec.y, rec.width, rec.height);
	if (fillstyle == FILL_HOLLOW)
		XDrawRectangle (display, window, gc, 
		rec.x, rec.y, rec.width, rec.height);
/*	Xfillrectangle does not draw borders	*/
	XFlush(display);
}

DISP_circle_(cx,cy,r)
	int *cx,*cy,*r;
{
	XRectangle rectangle(),rec;
	unsigned int x1,y1,x2,y2;
	int angle1, angle2;

	x1 = *cx - *r;
	y1 = *cy - *r;
	x2 = x1 + *r;
	y2 = y1 + *r;
	rec = rectangle( x1,y1,x2,y2);
	angle1=0;
	angle2=64*360;
	XFillArc (display, window, gc_fill, rec.x,rec.y, 
			rec.width, rec.height, angle1,angle2);
	if (fillstyle == FILL_HOLLOW)
		XDrawArc (display, window, gc, rec.x,rec.y, 
			rec.width, rec.height, angle1,angle2);
/*	Xfillarc does not draw borders	*/
	XFlush(display);
}

DISP_arcclose_(cx,cy,r,angles,anglee,type)
	int *cx,*cy,*r,*type;
	float *angles, *anglee;
/*	type=0:			chord
	    =1(default):	pie		*/
{
	XRectangle rectangle(),rec;
	unsigned int x1,y1,x2,y2;
	int angle1, angle2;
	float angs, ange;

	x1 = *cx - *r;
	y1 = *cy - *r;
	x2 = x1 + *r;
	y2 = y1 + *r;
	rec = rectangle( x1,y1,x2,y2);

      angs = *angles;
      ange = *anglee;
      if (xfactor < 0.0)      {
            angs = 180.0 - angs;
            ange = 180.0 - ange;
      }
      if (-yfactor < 0.0)      {
            angs = -angs;
            ange = -ange;
      }
	angle1 = 64 *angs;
	angle2 = 64 *(ange-angs);
	
	if (*type == 1)
		XSetArcMode (display, gc_fill, ArcPieSlice);
	else
		XSetArcMode (display, gc_fill, ArcChord);
	
	XFillArc (display, window, gc_fill, rec.x, rec.y,
			rec.width, rec.height, angle1,angle2);

	if (fillstyle == FILL_HOLLOW)
		XDrawArc (display, window, gc, rec.x, rec.y,
			rec.width, rec.height, angle1,angle2);
/*	Xfillarc does not draw borders	*/
	XFlush(display);
}

DISP_arc3close_(x1,y1,x2,y2,x3,y3,type)
	int *x1,*y1,*x2,*y2,*x3,*y3,*type;
/*	type=0:			chord
	    =1(default):	pie		*/
{
/*	not supported	*/
}

DISP_ellipse_(cx,cy,a,b,anglea)
	int *cx,*cy,*a,*b;
	float *anglea;
{
	XRectangle rectangle(),rec;
	unsigned int x1,y1,x2,y2;
	int angle1, angle2;

	x1 = *cx - *a;
	y1 = *cy - *b;
	x2 = x1 + *a;
	y2 = y1 + *b;
	rec = rectangle( x1,y1,x2,y2);
	angle1=0;
	angle2=64*360;
	XFillArc (display, window, gc_fill, rec.x, rec.y,
		rec.width, rec.height, angle1,angle2);
	if (fillstyle == FILL_HOLLOW)
		XDrawArc (display, window, gc, rec.x, rec.y,
		rec.width, rec.height, angle1,angle2);
/*	Xfillarc does not draw borders	*/

	XFlush(display);
}

DISP_elliparcclose_(cx,cy,a,b,anglea,angles,anglee,type)
	int *cx,*cy,*a,*b,*type;
	float *anglea,*angles,*anglee;

/*	type=0:			chord
	    =1(default):	pie		*/
{
	XRectangle rectangle(),rec;
	unsigned int x1,y1,x2,y2;
	int angle1, angle2;
	float angs, ange;

	x1 = *cx - *a;
	y1 = *cy - *b;
	x2 = x1 + *a;
	y2 = y1 + *b;
	rec = rectangle( x1,y1,x2,y2);

      angs = *angles;
      ange = *anglee;
      if (xfactor < 0.0)      {
            angs = 180.0 - angs;
            ange = 180.0 - ange;
      }
      if (-yfactor < 0.0)      {
            angs = -angs;
            ange = -ange;
      }
	angle1 = 64 *angs;
	angle2 = 64 *(ange-angs);

	if (*type == 1)
		XSetArcMode (display, gc_fill, ArcPieSlice);
	else
		XSetArcMode (display, gc_fill, ArcChord);
		
	XFillArc (display, window, gc_fill, rec.x, rec.y,
			rec.width, rec.height, angle1,angle2);
	if (fillstyle == FILL_HOLLOW)
		XDrawArc (display, window, gc, rec.x, rec.y,
			rec.width, rec.height, angle1,angle2);
/*	Xfillarc does not draw borders	*/
	XFlush(display);
}

DISP_fillcolor_(r,g,b)
	int *r,*g,*b;
/*    r,g,b = 0 to 255
      if colormode=INDEXEDCOLOR, r contains color index     */
{
      fgcolor(*r,*g,*b,gc_fill);
}

DISP_interiorstyle_(style)
	int *style;
/*	style=1(default):	hollow
	     =2:		solid
	     =3:		pattern
	     =4:		hatch
	     =5:		empty		*/
{
	fillstyle = *style;
	switch (*style)	{
		case 1:	XSetStipple(display,gc_fill,hatch[0]);
				XSetFillStyle(display,gc_fill,FillStippled);
				break;
		case 2:	XSetFillStyle(display,gc_fill,FillSolid);
				break;
		case 4:	XSetFillStyle(display,gc_fill,FillStippled);
				break;
	}
}

DISP_hatchindex_(index)
	int *index;
/*	index=1:		horizontal parallel lines
	     =2(default):	vertical parallel lines
	     =3:		positive slope parallel lines
	     =4:		negative slope parallel lines
	     =5:		horizontal/vertical crosshatch
	     =6:		positive/negative slope crosshatch	*/
{
	if (*index >= 1 && *index <= 6)
		XSetStipple(display,gc_fill,hatch[*index]);
}

DISP_patternindex_(index)
	int *index;
{
/*	not implemented yet	*/
}

DISP_patternsize_(hx,hy,wx,wy)
	int *hx,*hy,*wx,*wy;
{
/*	not implemented yet	*/
}

DISP_patterntable_(index,nx,ny,plane,pattern)
	int *index,*nx,*ny,*plane;
	int *pattern;
{
/*	not implemented yet	*/
}

DISP_fillpoint_(x,y)
	int *x,*y;
{
/*	not supported	*/
}

DISP_edgetype_(type)
	int *type;
{
/*	not supported	*/
}

DISP_edgewidth_(iwidth)
	int *iwidth;
/*	edge width specification in absolute mode	*/
{
/*	not supported	*/
}

DISP_edgefwidth_(fwidth)
	float *fwidth;
/*	edge width specification in scaled mode	*/
{
/*	not supported	*/
}

DISP_edgecolor_(r,g,b)
	int *r,*g,*b;
/*	r is also used as color index	*/
{
	DISP_linecolor_(r,g,b);
}

DISP_edgevisibility_(edge)
	int *edge;
/*	edge = 0: visible
	       1: invisivble (default)	*/
{
/*	not supported	*/
}

DISP_cellarray_(x1,y1,x2,y2,x3,y3,nx,ny,precision,r,g,b,i)
	int *x1,*y1,*x2,*y2,*x3,*y3,*nx,*ny,*precision,*r,*g,*b,*i;
{
/*	not supported	*/
}

DISP_colormode_(mode)
	int *mode;
/*	mode=0:			direct
	    =1(default):	indexed		*/
{
	colormode = *mode;
/*
	if the colormode (pseudocolor or directcolor) is different
	from the default colormode of the X-server, it will be emulated
	}
*/
}

DISP_colortable_(startindex_f,ncolor,r,g,b)
	unsigned int *startindex_f,*ncolor,r[],g[],b[];
/*	startindex_f = 1,2,,,256
	ncolor = 1,2,,,256	*/
{
	XColor color;
	unsigned int mult=256;
	int i, startindex;

	startindex = *startindex_f -1;
	if (startindex+(*ncolor) > max_colorindex)
		max_colorindex = startindex +(*ncolor);

	for(i=0; i< *ncolor; i++)	{
		color.red   = mult*r[i];
		color.green = mult*g[i];
		color.blue  = mult*b[i];
		int_r[startindex +i] = r[i];
		int_g[startindex +i] = g[i];
		int_b[startindex +i] = b[i];
		color.pixel = startindex + i + colorbase;
		color.flags = DoRed|DoGreen|DoBlue;
		if (colormapmode == WRITABLEMAP)
			XStoreColor (display, colormap, &color );
		else	{
			XAllocColor (display, colormap, &color);
		}

		if (sys_.msglvl > 3)	{
			XQueryColor (display, colormap, &color);
			printf("\n (DISP_colortable) color No.%3d: intended r,g,b=%ld,%ld,%ld",
					startindex+i,
					mult*r[i],mult*g[i],mult*b[i]);
			printf("\n                                 results=%ld,%ld,%ld internal No=%ld",
                  		color.red, color.green, color.blue, color.pixel);
		}
	}
	XFlush (display);
}

DISP_auxcolor_(r,g,b)
	int *r,*g,*b;
/*	r is also used as color index		*/
{
/*	not supported	*/
}

DISP_transparency_(trans)
	int *trans;
/*	trans=0:		transparent
	     =1:		opaque		*/
{
/*	not supported	*/
}

DISP_byteimage_(xo,yo,nx,ny,x1,x2,y1,y2,image)
	int *xo,*yo,*nx,*ny,*x1,*x2,*y1,*y2;
	unsigned char *image;
/*	(xo,yo) = VDC coordinates of upper left corner of the image
	          to be displayed on the screen
	(nx,ny) = size of the full image array
	(x1,y1)-(x2,y2)= range of image subarray to be displayed
      x1>=1, x2<=nx, y1>=1, y2<=ny (as fortran subscripts)
*/
{
	XImage *ximage,*XCreateImage();
	unsigned int depth,offset,width,height,
		srcx,srcy,destx,desty,subwidth,subheight;
	int bytes_per_line,bitmap_pad;

	depth=8;
	offset=0;
	width= *nx;
	height= *ny;
	bytes_per_line = ((*nx +15)/16)*2;
	bitmap_pad = 16;
	ximage = XCreateImage(display,DefaultVisual(display,0),depth,ZPixmap,
			offset,image,width,height,bitmap_pad,bytes_per_line);
	srcx = xconv((*x1)-1);	/* in units of device dots	*/
	srcy = yconv((*y1)-1);
	destx = xconv((*xo)-1);	/* in units of device dots	*/
	desty = yconv((*yo)-1);
	subwidth = abs((*x2)-(*x1)+1);
	subheight= abs((*y2)-(*y1)+1);
	XPutImage(display,window,gc_image,ximage,
			srcx,srcy,destx,desty,subwidth,subheight);
}

DISP_bitwiseimage_(xo,yo,nx,ny,x1,x2,y1,y2,nplane,bitmap)
      int *xo,*yo,*nx,*ny,*x1,*x2,*y1,*y2,*nplane;
      unsigned short *bitmap;
/*	(xo,yo) = VDC coordinates of upper left corner of the image
	          to be displayed on the screen
	(nx,ny) = size of the full image array
	(x1,y1)-(x2,y2) = range of image subarray to be displayed
      x1>=1, x2<=nx, y1>=1, y2<=ny (as fortran subscripts)
*/
{
	unsigned int depth,offset,width,height,
		srcx,srcy,destx,desty,subwidth,subheight;
	int bytes_per_line,bitmap_pad;
	XImage *ximage,*XCreateImage();

	depth= *nplane;
	offset=0;
	width= *nx;
	height= *ny;
	bytes_per_line = ((*nx +15)/16)*2;
	bitmap_pad = 16;
	ximage = XCreateImage(display,DefaultVisual(display,0),depth,XYPixmap,
			offset,(char *)bitmap,width,height,bitmap_pad,bytes_per_line);
	srcx = xconv((*x1)-1);	/* in units of device dots	*/
	srcy = yconv((*y1)-1);
	destx = xconv((*xo)-1);	/* in units of device dots	*/
	desty = yconv((*yo)-1);
	subwidth = abs((*x2)-(*x1)+1);
	subheight= abs((*y2)-(*y1)+1);
	XPutImage(display,window,gc_image,ximage,
			srcx,srcy,destx,desty,subwidth,subheight);
}

DISP_initpointer_()
{
}

DISP_releasepointer_()
{
}

DISP_pointermode_(mode)
	int *mode;
{
	int shape;
	static int cursormapping[]={	XC_crosshair,
						XC_cross,
						XC_arrow, XC_arrow,
						XC_X_cursor};
	Cursor cursor;

	shape = *mode;
	if (shape <1 || shape >5)	shape=1;
	shape = cursormapping[shape-1];
	cursor = XCreateFontCursor(display,shape);
	XDefineCursor(display,window,cursor);
}

DISP_pointerecho_(onoff)
	int *onoff;
{
}

DISP_pointerprompt_(onoff)
      int *onoff;
{
}

DISP_pointerack_(onoff)
      int *onoff;
{
}

DISP_pointerrequest_(x,y,buttonnumber)
	int *x,*y,*buttonnumber;
/*	returned values (x,y) are in device dots	*/
{
	XEvent e;

	XWindowEvent (display,window,ButtonPress,&e);
	*x = e.xbutton.x;
	*y = e.xbutton.y;
	*buttonnumber = e.xbutton.button;
	if (sys_.msglvl > 2)	{
		printf("\nbutton number=%d", *buttonnumber);
		printf("\nbutton state=%x", e.xbutton.state);
		printf("\nlocal (x,y)=%d,%d",*x,*y);
		printf("\nglobal (x,y)=%d,%d\n",
			e.xbutton.x_root,e.xbutton.y_root);
	}
}

DISP_pointersample_(x,y,status)
      int *x,*y,*status;
{
      XEvent e;
      
      XWindowEvent (display,window,ButtonPress|MotionNotify,&e);
      if ( e.type == ButtonPress)  {
            *x = e.xbutton.x;
            *y = e.xbutton.y;
            *status= e.xbutton.state;
            if (sys_.msglvl > 2)    {
                  printf("\nbutton number=%d", e.xbutton.button);
                  printf("\nbutton state=%x", *status);
                  printf("\nlocal (x,y)=%d,%d",*x,*y);
                  printf("\nglobal (x,y)=%d,%d\n",
                        e.xbutton.x_root,e.xbutton.y_root);
            }
	}
      if (e.type == MotionNotify )  {
            *x = e.xmotion.x;
            *y = e.xmotion.y;
            *status= e.xmotion.state;
            if (sys_.msglvl > 2)    {
                  printf("\nmotion state=%x", *status);
                  printf("\nlocal (x,y)=%d,%d",*x,*y);
                  printf("\nglobal (x,y)=%d,%d\n",
                        e.xbutton.x_root,e.xbutton.y_root);
            }
	}
}

