c
c  dipole field
c  xx: point where the field bb is evaluated
c  xxs: source point
c  dmom: dipole moment
c  dir: direction of dipole moment (unit vector)
c
	subroutine dipole(xx,bb,psi,xxs,dir,dmom)
	dimension xx(3),bb(3),dir(3),xxs(3),dp1(3),dp2(3),xx0(3)
c
	do 10 m=1,3
 10	dp1(m)=0.0
c
	if(abs(dir(1)).le.abs(dir(2)) .and. 
     &	abs(dir(1)).le.abs(dir(3))) then
	  dp1(1)=1.0
	else if(abs(dir(2)).le.abs(dir(1)) .and. 
     &	abs(dir(2)).le.abs(dir(3))) then
	  dp1(2)=1.0
	else
	  dp1(3)=1.0
	endif
c
	dp2(1)=dir(2)*dp1(3)-dir(3)*dp1(2)
	dp2(2)=dir(3)*dp1(1)-dir(1)*dp1(3)
	dp2(3)=dir(1)*dp1(2)-dir(2)*dp1(1)
c
	dmag=sqrt(dp2(1)**2+dp2(2)**2+dp2(3)**2)
	do 20 m=1,3
 20	dp2(m)=dp2(m)/dmag
c
	dp1(1)=dp2(2)*dir(3)-dp2(3)*dir(2)
	dp1(2)=dp2(3)*dir(1)-dp2(1)*dir(3)
	dp1(3)=dp2(1)*dir(2)-dp2(2)*dir(1)
c
	do 30 m=1,3
 30	xx0(m)=xx(m)-xxs(m)
	r=sqrt(xx0(1)**2+xx0(2)**2+xx0(3)**2)
	x=xx0(1)*dp1 (1)+xx0(2)*dp1 (2)+xx0(3)*dp1 (3)
	y=xx0(1)*dp2 (1)+xx0(2)*dp2 (2)+xx0(3)*dp2 (3)
	z=xx0(1)*dir(1)+xx0(2)*dir(2)+xx0(3)*dir(3)
	psi=dmom*z/r**3
	bx=dmom*3.0*x*z/r**5
	by=dmom*3.0*y*z/r**5
	bz=dmom*(2.0*z**2-x**2-y**2)/r**5
c
	do 40 m=1,3
 40	bb(m)=bx*dp1(m)+by*dp2(m)+bz*dir(m)
	return
	end
c
c  non-linear force-free field solution of Low and Lou
c  (Astrophys. J. 352, 343-352, 1990)
c
	subroutine nlfff(xx,bb,alpha,xxs,dir,b0,al,
     &	           order,isym,ak,dmu,nmu,pval,pdval)
c
	dimension xx(3),bb(3),xxs(3),dir(3)
	dimension xx0(3),er(3),et(3),ef(3),pval(nmu),pdval(nmu)
c
	do 10 m=1,3
 10	xx0(m)=xx(m)-xxs(m)
	r=sqrt(xx0(1)**2+xx0(2)**2+xx0(3)**2)
	do 20 m=1,3
 20	er(m)=xx0(m)/r
c
	ef(1)=dir(2)*er(3)-dir(3)*er(2)
	ef(2)=dir(3)*er(1)-dir(1)*er(3)
	ef(3)=dir(1)*er(2)-dir(2)*er(1)
	sinth=min(sqrt(ef(1)**2+ef(2)**2+ef(3)**2),1.0)
	if(sinth.gt.0.0) then
	  do 30 m=1,3
 30	  ef(m)=ef(m)/sinth
	endif
c
	et(1)=ef(2)*er(3)-ef(3)*er(2)
	et(2)=ef(3)*er(1)-ef(1)*er(3)
	et(3)=ef(1)*er(2)-ef(2)*er(1)
c
c  pval(1) = p(costh=-1)
c  pcal(nmu) = p(costh=0)
c  pdval = dp/d(mu)
c
	costh=sqrt(abs(1.0-sinth**2))
	if(dir(1)*er(1)+dir(2)*er(2)+dir(3)*er(3) .lt.0.0)
     &	  costh=-costh
	imu = (-abs(costh)+1.0)/dmu +1.0
	rmu = (-abs(costh)+1.0)/dmu -(imu-1)
	pval1=pval(imu)*(1.0-rmu) + pval(imu+1)*rmu
	pdval1=pdval(imu)*(1.0-rmu) + pdval(imu+1)*rmu
	if(costh.ge.0.0) then
	  if(isym.eq.-1) then
	    pval1=-pval1
	  else
	    pdval1=-pdval1
	  endif
	endif
c
	denom=(r/al)**(2.0+order)
	br=-pdval1/denom
	if(sinth.gt.0.0) then
	  bt= order*pval1/(denom*sinth)
	  bf= ak*abs(pval1)*abs(pval1)**(1.0/order)/(denom*sinth)
	else
	  bt=0.0
	  bf=0.0
	endif
c
	do 40 m=1,3
 40	bb(m)=b0*(br*er(m)+bt*et(m)+bf*ef(m))
	alpha=ak*(1.0+1.0/order)/r*pval1*abs(pval1)**(1.0/order)
	return
	end
c
c  an example of a non-force-free field
c  vector potential A=(0,Ay,0), Ay=a0*exp(-ax*x**2-ay*y**2-akz*z)
c  aj = curl B
c
	subroutine nonfff(xx,bb,aj,xxs,a0,ax,ay,akz)
c
	dimension xx(3),bb(3),aj(3),xxs(3)
c
	x=xx(1)-xxs(1)
	y=xx(2)-xxs(2)
	z=xx(3)
	e=exp(-ax*x*x-ay*y*y-akz*z)
	bb(1)= a0*akz*e
	bb(2)= 0.0
	bb(3)=-2.0*a0*ax*x*e
	aj(1)= 4.0*a0*ax*ay*x*y*e
	aj(2)=(2.0*ax-4.0*(ax*x)**2-akz*akz)*a0*e
	aj(3)= 2.0*a0*ay*akz*y*e
	return
	end
c
c  read dipole data
c
	subroutine readdp(iu,iproj,kindm1,
     &	            nx1,ny1,xs1,ys1,dx1,dy1,rsp1)
c
	include 'sys.fc'
	include 'coord.fc'
	include 'auxparam.fc'
	dimension xx(3),bb(3),bb1(3),bb2(3),dir(3),xxs(3)
c
	kindm1=2
c
	read(iu,*) bmax,dir,xxs
	read(iu,*) iproj,kindm1
c
c  kindm1=2 - pseudo-vector
c  kindm1=3 - real vector
c  bx,by,bz are evaluated on the following meshes
c  iproj=0: plane, face-on
c        1: plane, oblique
c        2: sphere, with mirror source (bl=0 at the plane of the sky)
c        3: sphere, no mirror source
c          (assume p=b0=0, which is different from mesh=3)
c
	read(iu,*) nx1,ny1,xs1,ys1,dx1,dy1,rsp1
c  assume ypos=0; but this setting is done in the calling program
c  because solar.fc is not linked to this program 
c
c  xs1,ys1,dx1,dy1 are in C(=D)-coordinates
c
	rsp=rsp1
	xxsp(1)=0.0
	xxsp(2)=0.0
	xxsp(3)=-rsp1
	call setdu
	xtod=0.0
	ytod=0.0
	if(iproj.ne.0) then
	  xtod=xs1+0.5*dx1*float(nx1-1)
	  ytod=ys1+0.5*dy1*float(ny1-1)
	  xxtod(1)=xtod
	  xxtod(2)=ytod
	  call projsr(xxtod,rsp1)
	  call dtoc(xxtod,xxto)
	  call settu
	endif
c
	d=sqrt(dir(1)**2+dir(2)**2+dir(3)**2)
	do 10 m=1,3
 10	dir(m)=dir(m)/d
c
	if(iproj.eq.2) then
c  location of mirror source
	  xxsm(1)=xxs(1)
	  xxsm(2)=xxs(2)
	  xxsm(3)=-2.0*rsp1-xxs(3)
	  dirm(1)= dir(1)
	  dirm(2)= dir(2)
	  dirm(3)=-dir(3)
	endif
c
	dmom=1.0
	bmax1=0.0
	bmax2=0.0
	bmax12=0.0
	bb2(1)=0.0
	bb2(2)=0.0
	bb2(3)=0.0
c
	do 20 j=1,ny1
	do 20 i=1,nx1
	xx(1)=xs1+dx1*float(i-1)
	xx(2)=ys1+dy1*float(j-1)
	xx(3)=0.0
	if(iproj.eq.1) then
	  call projt(xx,xt,yt)
	else if(iproj.eq.2) then
	  call projsr(xx,rsp1)
	  call dipole(xx,bb2,psi,xxsm,dirm,dmom)
	  bmax2=max(bmax2,sqrt(bb2(1)**2+bb2(2)**2+bb2(3)**2))
	else if(iproj.eq.3) then
	  call projsr(xx,rsp1)
	endif
c
	call dipole(xx,bb1,psi,xxs,dir,dmom)
c
	bmax1=max(bmax1,sqrt(bb1(1)**2+bb1(2)**2+bb1(3)**2))
	bb(1)=bb1(1)+bb2(1)
	bb(2)=bb1(2)+bb2(2)
	bb(3)=bb1(3)+bb2(3)
	bmax12=max(bmax12,sqrt(bb(1)**2+bb(2)**2+bb(3)**2) )
 20	continue
	if(msglvl.gt.1) then
	  if(iproj.eq.2) then
     	    write(lst,*) 'bmax1,2,12=',bmax1,bmax2,bmax12
	  else
	    write(lst,*) 'bmax=',bmax1
	  endif
	endif
c
	dmom=bmax/bmax12
	do 30 i=1,3
	xxsaux(i)=xxs(i)
	diraux(i)=dir(i)
 30	continue
	return
	end
c
c  read data for nonlinear force-free field of Low and Lou
c
	subroutine readff(iu,iproj,kindm1,
     &	            nx1,ny1,xs1,ys1,dx1,dy1,rsp1)
c
	parameter (NMU0=100)
	external rnlfff
	include 'sys.fc'
	include 'coord.fc'
	include 'auxparam.fc'
	dimension xx(3),bb(3),dir(3),xxs(3)
	dimension xvec(1),gvec(1),h(5,1)
	common/cnlfff/order,isym,ak,dmu,nmu,pval1(NMU0),pdval1(NMU0)
c
	kindm1=2
c
	read(iu,*) bmax,dir,xxs,al
	read(iu,*) iproj,kindm1
c  kindm1=2 - pseudo-vector
c  kindm1=3 - real vector
c  kindm1=2003 - vector and alpha
c  bx,by,bz are evaluated on the following meshes
c  iproj=0: plane, face-on
c        1: plane, oblique
c      2,3: sphere
	read(iu,*) nx1,ny1,xs1,ys1,dx1,dy1,rsp1
c
	rsp=rsp1
	xxsp(1)=0.0
	xxsp(2)=0.0
	xxsp(3)=-rsp1
	call setdu
	xtod=0.0
	ytod=0.0
	if(iproj.ne.0) then
	  xtod=xs1+0.5*dx1*float(nx1-1)
	  ytod=ys1+0.5*dy1*float(ny1-1)
	  xxtod(1)=xtod
	  xxtod(2)=ytod
	  call projsr(xxtod,rsp1)
	  call dtoc(xxtod,xxto)
	  call settu
	endif
c
	d=sqrt(dir(1)**2+dir(2)**2+dir(3)**2)
	do 10 m=1,3
 10	dir(m)=dir(m)/d
c
	order=1
	isym=-1
	ak=sqrt(0.425)
	read(iu,*) order,isym,ak
	eps1=1.0e-4
	delx1=0.001
	ss1=0.025
	limit=5
	ipr1=1
	nmu=NMU0
	read(iu,*) nmu,limit,eps1,delx1,ss1,ipr1
	if(nmu.gt.NMU0) then
	  write(msg,*) '(subr.rdfff ) error: nmu=',nmu
	  errflg=2
	  return
	endif
c
	xvec(1)=ak
	neq=1
	call mnnl(rnlfff,rnlfff,neq,xvec,f,gvec,eps1,delx1,ss1,
     &	          limit,ipr1,h)
	ak=xvec(1)
c
	b0=1.0
	bmax1=0.0
	do 20 j=1,ny1
	do 20 i=1,nx1
	xx(1)=xs1+dx1*float(i-1)
	xx(2)=ys1+dy1*float(j-1)
	xx(3)=0.0
	if(iproj.eq.1) then
	  call projt(xx,xt,yt)
	else if(iproj.ge.2) then
	  call projsr(xx,rsp1)
	endif
c
	call nlfff(xx,bb,alpha,xxs,dir,b0,al,
     &	     order,isym,ak,dmu,nmu,pval1,pdval1)
c
	bmax1=max(bmax1,sqrt(bb(1)**2+bb(2)**2+bb(3)**2))
 20	continue
c
	if(msglvl.gt.1) write(lst,*) 'bmax,bmax1=',bmax,bmax1
c
	b0fff=bmax*b0/bmax1
	alfff=al
	odrfff=order
	symfff=isym
	akfff=ak
	dmufff=dmu
	nmufff=nmu
	do 30 i=1,nmu
	pval(i)=pval1(i)
	pdval(i)=pdval1(i)
 30	continue
	do 40 i=1,3
	xxsaux(i)=xxs(i)
	diraux(i)=dir(i)
 40	continue
c
	return
	end
c
	subroutine rnlfff(n1,x,f,g)
	parameter (NMU0=100)
	external odell
	dimension x(1),g(1),wk(3,2),y0(3),ya(3,NMU0),fa(3,NMU0)
	common/cnlfff/order,isym,ak,dmu,nmu,pval(NMU0),pdval(NMU0)
c
	ak=x(1)
	nd=3
	dmu=1.0/float(nmu-1)
c  y(1)=p
c  y(2)=dp/dmu
c  y(3)=mu
c  isym=-1: antisymmeric with respect to mu=0, therefore p(0)=0
c  isym= 1: symmetric with respect to mu=0, therefore dp/dmu(0)=0
	y0(1)=0.0
	y0(2)=10.0
	y0(3)=-1.0
	call rk4(odell,nd,y0,dmu,nmu,nd,ya,fa,wk)
	if(isym.eq.-1) then
	  g(1)=ya(1,nmu)
	else
	  g(1)=ya(2,nmu)
	endif
	f=g(1)**2
	do 10 i=1,nmu
	pval(i)=ya(1,i)
	pdval(i)=ya(2,i)
 10	continue
c  pval stores the values of p from mu=-1 to 0
c  pdval stores the values of dp/dmu from mu=-1 to 0
	return
	end
c
c  ordinary differential equation of Low and Lou
c
	subroutine odell(n1,y,f)
c
	parameter (NMU0=100)
	common/cnlfff/order,isym,ak,dmu,nmu,pval(NMU0),pdval(NMU0)
	dimension y(3),f(3)
c
c  y(1)=p
c  y(2)=dp/dmu
c  y(3)=mu(=costh)
c  f=dy/dmu
c
	amu=y(3)
	p=y(1)
	dp=y(2)
	f(1)=dp
	f(3)=1.0
	if(abs(y(3)).eq.1.0) then
	  f(2)=order*(order+1.0)*p/(2.0*amu)
	else
	  f(2)=-order*(order+1.0)*p*
     &	 (1.0+(ak/order)**2*abs(p)**(2.0/order))
     &	/(1.0-amu*amu)
	endif
	return
	end
c
c  read data for a non-force-free field
c
	subroutine readnf(iu,iproj,kindm1,
     &	            nx1,ny1,xs1,ys1,dx1,dy1,rsp1)
c
	include 'sys.fc'
	include 'coord.fc'
	include 'auxparam.fc'
	dimension xx(3),bb(3),aj(3),xxs(3),vn(3)
c
	kindm1=2
c
	read(iu,*) bmax0,wx,wy,hz,xxs
	read(iu,*) iproj,kindm1
c  kindm1=2 - pseudo-vector
c  kindm1=3 - real vector
c  bx,by,bz are evaluated on the following meshes
c  iproj=0: plane, face-on
c        1: plane, oblique
c      2,3: sphere
	read(iu,*) nx1,ny1,xs1,ys1,dx1,dy1,rsp1
c
	rsp=rsp1
	xxsp(1)=0.0
	xxsp(2)=0.0
	xxsp(3)=-rsp1
	call setdu
	xtod=0.0
	ytod=0.0
c
	if(iproj.ne.0) then
	  xtod=xs1+0.5*dx1*float(nx1-1)
	  ytod=ys1+0.5*dy1*float(ny1-1)
	  xxtod(1)=xtod
	  xxtod(2)=ytod
	  call projsr(xxtod,rsp1)
	  call dtoc(xxtod,xxto)
	  call settu
	endif
c
	if(iproj.eq.0) then
	  do 10 m=1,3
 10	  vn(m)=zdu(m)
	else if(iproj.eq.1) then
	  do 11 m=1,3
 11	  vn(m)=ztu(m)
	endif
c
	a0=1.0
	ax=1.0/wx**2
	ay=1.0/wy**2
	akz=1.0/hz
	bmax=0.0
c
	do 20 j=1,ny1
	do 20 i=1,nx1
	xx(1)=xs1+dx1*float(i-1)
	xx(2)=ys1+dy1*float(j-1)
	xx(3)=0.0
	if(iproj.eq.1) then
	  call projt(xx,xt,yt)
	else if(iproj.ge.2) then
	  call projs(xx,ap,al)
	  do 21 m=1,3
 21	  vn(m)=(xx(m)-xxsp(m))/rsp1
	endif
	call nonfff(xx,bb,aj,xxs,a0,ax,ay,akz)
	ajn=aj(1)*vn(1)+aj(2)*vn(2)+aj(3)*vn(3)
c
	bmax=max(bmax,sqrt(bb(1)**2+bb(2)**2+bb(3)**2) )
 20	continue
c
	a0=bmax0*a0/bmax
c
	a0nff=a0
	axnff=ax
	aynff=ay
	akznff=akz
	do 30 i=1,3
 	xxsaux(i)=xxs(i)
	diraux(i)=vn(i)
 30	continue
	return
	end
