# magpack2 makefile for sony news (m680xx cpu)
#
# executable programs are installed in BINDIR and UTILDIR
# these directories should be on the command search path
BINDIR	=	${HOME}/bin
UTILDIR	=	${HOME}/bin

# name of compilers
FC	=	f77
CC	=	cc

FFLAGS=
CFLAGS=

INSTALL	=	install -c -s

IO	=	unixio
MISC	=	unixmisc
OBSTOP	=	oaotop
OBSTOB	=	mtktob
POLTOB	=	poltob

PLOTDIR	=	../plotlib
LBPDIR	=	../lbplib
DISPDIR	=	../displib
CNTLDIR	=	../cntl
STYDIR	=	../sty

PRINTER	=	lips2
# PRINTER	=	lnop
PLOTDEV	=	4013
# PLOTDEV	=	4014
# PLOTDEV	=	pnop
PLOTLIB1	=	${PLOTDIR}/plot-lbp.o \
			${PLOTDIR}/${PLOTDEV}lib.o \
			${LBPDIR}/${PRINTER}lib.o
PLOTLIB2	=	-l4013
# PLOTLIB2	=	-l4014

DISPDEV	=	x11
# DISPDEV	=	news
# DISPDEV	=	dnop
DISPLIB1	=	${DISPDIR}/disp-lbp.o \
			${DISPDIR}/kbhit.o \
			${DISPDIR}/${DISPDEV}lib.o \
			${LBPDIR}/${PRINTER}lib.o
DISPLIB2	=	-lX11
# DISPLIB2	=	-lcginews

# output of plot to x-window for debug
# PLOTDEV	=	x11
# PLOTLIB1	=	${PLOTDIR}/displib.o \
# 			${DISPLIB1}
# PLOTLIB2	=	${DISPLIB2}

AOBJS	=initc.o assign.o load.o conv.o diagnose.o spec.o \
       readm.o writem.o readl.o writel.o poltob.o \
       ${IO}.o ${MISC}.o transc.o transm.o rotm.o fits.o \
       ${OBSTOP}.o \
       ${OBSTOB}.o \
       energy.o \
       vector.o virial.o cff.o lff.o curlb.o \
       sets12.o reads1.o fpt.o field.o auxfield.o pot.o integ.o \
       test.o eval.o suncal.o \
       mathsub.o mnnl.o graphsub.o version.o

FOBJS	= fffini.o select.o jxb.o fffio.o fffitr.o spllof.o

BIN	=	prog \
		disp

FFF	=	fff

UTIL	=	dumpm \
		dumpl \
		sunparam \
		dipole \
		delta \
		nonfff \
		nlfff \
		gendata \
		sample1 \
		sample2 \
		sample3


STARS	=	binary dispbin

INSTALLBIN	=	${BINDIR}/prog \
			${BINDIR}/disp

INSTALLFFF	=	${BINDIR}/fff

INSTALLUTIL	=	${UTILDIR}/dumpm \
		${UTILDIR}/dumpl \
		${UTILDIR}/sunparam \
		${UTILDIR}/dipole \
		${UTILDIR}/delta \
		${UTILDIR}/nonfff \
		${UTILDIR}/nlfff

INSTALLSTARS	=	${BINDIR}/binary \
			${BINDIR}/dispbin

# for debugging
L	= 
# L	= -y_smooth_
# CISCNEWS ... -y at the end of f77 options 
#
# L	= -ytext_
# RISCNEWS ... -y at the beginning of f77 options

bin:	${BIN}
util:	${UTIL}
stars:	${STARS}
all:	${BIN} ${UTIL} ${FFF}
install:	${INSTALLBIN}
install_fff:	${INSTALLFFF}
install_util:	${INSTALLUTIL}
install_all:	${INSTALLBIN} ${INSTALLUTIL}
install_stars:	${INSTALLSTARS}


alib.a:	${AOBJS}
	ar cur alib.a ${AOBJS}
	ranlib alib.a

${PLOTDIR}/displib.o:
	(cd ${PLOTDIR};\
	make displib.o)

${PLOTDIR}/plot-lbp.o:
	(cd ${PLOTDIR};\
	make plot-lbp.o)

${PLOTDIR}/${PLOTDEV}lib.o:
	(cd ${PLOTDIR};\
	make ${PLOTDEV}lib.o)

oaobook:	oaobook.o cntr.o ${PLOTDIR}/displib.o ${DISPLIB1} alib.a
	${FC} -o oaobook oaobook.o cntr.o ${PLOTDIR}/displib.o \
		${DISPLIB1} alib.a ${DISPLIB2}

mtkbook:	mtkbook.o cntr.o ${PLOTDIR}/displib.o ${DISPLIB1} alib.a
	${FC} -o mtkbook mtkbook.o cntr.o ${PLOTDIR}/displib.o \
		${DISPLIB1} alib.a ${DISPLIB2}

${DISPDIR}/disp-lbp.o:
	(cd ${DISPDIR};\
	make disp-lbp.o)

${DISPDIR}/kbhit.o:
	(cd ${DISPDIR};\
	make kbhit.o)

${DISPDIR}/${DISPDEV}lib.o:	${DISPDIR}/crt.h
	(cd ${DISPDIR};\
	make ${DISPDEV}lib.o)

${LBPDIR}/${PRINTER}lib.o:	${LBPDIR}/lbpnop.c
	(cd ${LBPDIR}; \
	${CC} -c ${PRINTER}lib.c)

${BINDIR}/prog:	prog
	${INSTALL} prog ${BINDIR}
	
prog:	progmain.o alib.a
	${FC} -o prog progmain.o alib.a

${BINDIR}/fff:	fff
	${INSTALL} fff ${BINDIR}

fff:	fffmain.o ${FOBJS} alib.a
	${FC} -o fff fffmain.o ${FOBJS} alib.a

binary:		binmain.o alib.a
      ${FC} -o binary binmain.o alib.a

dispbin:	dispbin.o dispmb.o sphere.o ${DISPLIB1} alib.a
	${FC} -o dispbin dispbin.o dispmb.o sphere.o \
	${DISPLIB1} alib.a ${DISPLIB2}

${BINDIR}/plot:	plot
	${INSTALL} plot ${BINDIR}

plot:	plotmain.o plotm.o plotr.o cntr2.o sphere.o \
		${PLOTLIB1} alib.a
	${FC} -o plot plotmain.o plotm.o plotr.o cntr2.o sphere.o \
		${PLOTLIB1} alib.a ${PLOTLIB2}

${BINDIR}/disp:	disp
	${INSTALL} disp ${BINDIR}

disp:	dispmain.o dispm.o dispr.o cntr2.o sphere.o \
		${DISPLIB1} alib.a
	${FC} -o disp dispmain.o dispm.o dispr.o cntr2.o sphere.o \
		${DISPLIB1} alib.a ${DISPLIB2}

${UTILDIR}/dipole:	dipole
	${INSTALL} dipole ${UTILDIR}

dipole:	gendpl.o alib.a
	${FC} -o dipole gendpl.o alib.a

${UTILDIR}/delta:	delta
	${INSTALL} delta ${UTILDIR}

delta:	gendlt.o alib.a
	${FC} -o delta gendlt.o alib.a

${UTILDIR}/nonfff:	nonfff
	${INSTALL} nonfff ${UTILDIR}

nonfff:	gennff.o alib.a
	${FC} -o nonfff gennff.o alib.a

${UTILDIR}/nlfff:	nlfff
	${INSTALL} nlfff ${UTILDIR}

nlfff:	genfff.o alib.a
	${FC} -o nlfff genfff.o alib.a

${UTILDIR}/dumpm:	dumpm
	${INSTALL} dumpm ${UTILDIR}

dumpm:	dumpm.o alib.a
	${FC} -o dumpm dumpm.o alib.a

${UTILDIR}/dumpl:	dumpl
	${INSTALL} dumpl ${UTILDIR}

dumpl:	dumpl.o alib.a
	${FC} -o dumpl dumpl.o alib.a

${UTILDIR}/sunparam:	sunparam
	${INSTALL} sunparam ${UTILDIR}

sunparam:	sunparam.o alib.a
	${FC} -o sunparam sunparam.o alib.a

${UTILDIR}/gendata:     gendata
      ${INSTALL} gendata ${UTILDIR}

gendata:    gendata.o alib.a
      ${FC} -o gendata gendata.o alib.a

${UTILDIR}/sample1:     sample1
      ${INSTALL} sample1 ${UTILDIR}

sample1:    sample1.o alib.a
      ${FC} -o sample1 sample1.o alib.a

${UTILDIR}/sample2:     sample2
      ${INSTALL} sample2 ${UTILDIR}

sample2:    sample2.o alib.a
      ${FC} -o sample2 sample2.o alib.a

${UTILDIR}/sample3:     sample3
      ${INSTALL} sample3 ${UTILDIR}

sample3:    sample3.o alib.a
      ${FC} -o sample3 sample3.o alib.a

sysconf.cfg:	sysconf
	sysconf

init:	sysconf
	rm sysconf.cfg
	sysconf

sysconf:	sysconf.o diagnose.o conv.o ${MISC}.o
	${FC} -o sysconf sysconf.o diagnose.o conv.o ${MISC}.o

link:;	makelink

update:;	makeupdt

depend:
	mkmf 
	mkmf -f Sun.mak
	mkmf -f Mips.mak
	mkmf -f Facomvpp.mak

index:;	ctags -xw *.f* | sort -f

tags:	*.f*
	ctags *.f*


###
assign.o: fcntl.fc procs.fc sys.fc fileio.fc auxparam.fc mdata.fc
auxfield.o: sys.fc coord.fc auxparam.fc
cff.o: sys.fc coord.fc lparam.fc mparam.fc mdata.fc fileio.fc
cntr.o: sys.fc fline.fc
cntr2.o: sys.fc fline.fc
conv.o: sys.fc
curlb.o: sys.fc coord.fc solar.fc mdata.fc mparam.fc fileio.fc auxparam.fc
curlf.o: ff0.fc fff.fc s3.fc
diagnose.o: sys.fc
dispm.o: sys.fc mdata.fc coord.fc solar.fc mparam.fc string.fc fileio.fc \
	fline.fc graph.fc
dispmain.o: sys.fc fileio.fc string.fc coord.fc lparam.fc mparam.fc \
	mdata.fc fcntl.fc procs.fc
dispr.o: graph.fc coord.fc fline.fc mdata.fc
dumpl.o: sys.fc coord.fc fileio.fc fcntl.fc string.fc fline.fc lparam.fc
dumpm.o: sys.fc mdata.fc mparam.fc coord.fc solar.fc fileio.fc fcntl.fc \
	string.fc
energy.o: sys.fc solar.fc coord.fc ss.fc s1.fc s2.fc s3.fc ff0.fc
eval.o: sys.fc coord.fc lparam.fc mparam.fc mdata.fc fline.fc fileio.fc \
	string.fc
fffini.o: sys.fc mparam.fc lparam.fc auxparam.fc coord.fc fline.fc ss.fc \
	s3.fc ff0.fc fff.fc
fffio.o: sys.fc ff0.fc fff.fc coord.fc ss.fc s1.fc s2.fc s3.fc lparam.fc \
	mparam.fc auxparam.fc solar.fc string.fc
fffitr.o: sys.fc auxparam.fc ff0.fc fff.fc mdata.fc
fffmain.o: sys.fc fileio.fc string.fc coord.fc lparam.fc mparam.fc mdata.fc \
	fcntl.fc procs.fc auxparam.fc ff0.fc fff.fc
field.o: sys.fc lparam.fc mparam.fc ss.fc s1.fc s2.fc s3.fc ff0.fc coord.fc \
	auxparam.fc
fits.o: sys.fc coord.fc solar.fc fileio.fc mparam.fc string.fc mdata.fc
fpt.o: sys.fc coord.fc fline.fc mdata.fc ss.fc s1.fc s2.fc mparam.fc \
	lparam.fc fileio.fc
gendlt.o: sys.fc mdata.fc mparam.fc coord.fc solar.fc fileio.fc fcntl.fc \
	string.fc
gendpl.o: sys.fc mdata.fc mparam.fc coord.fc solar.fc fileio.fc fcntl.fc \
	string.fc auxparam.fc
genfff.o: sys.fc mdata.fc mparam.fc coord.fc solar.fc fileio.fc fcntl.fc \
	string.fc auxparam.fc
genmag.o: sys.fc mparam.fc fileio.fc mdata.fc string.fc solar.fc coord.fc
gennff.o: sys.fc mdata.fc mparam.fc coord.fc solar.fc fileio.fc fcntl.fc \
	string.fc auxparam.fc
graphsub.o: sys.fc
ibmio.o: sys.fc fileio.fc fcntl.fc
ibmmisc.o: convi.fi sys.fc
initc.o: fcntl.fc fileio.fc procs.fc mdata.fc sys.fc coord.fc ff0.fc \
	lparam.fc mparam.fc ss.fc solar.fc string.fc auxparam.fc graph.fc
integ.o: sys.fc fline.fc coord.fc lparam.fc mparam.fc ss.fc s1.fc s2.fc
jxb.o: sys.fc coord.fc mparam.fc ff0.fc fff.fc ss.fc s3.fc lparam.fc
load.o: sys.fc coord.fc fileio.fc mparam.fc lparam.fc mdata.fc fcntl.fc
mipsio.o: sys.fc fileio.fc fcntl.fc
mnnl.o: sys.fc ff0.fc fff.fc
mtkbook.o: sys.fc fileio.fc string.fc coord.fc lparam.fc mparam.fc mdata.fc \
	fcntl.fc procs.fc auxparam.fc solar.fc graph.fc
mtktob.o: sys.fc coord.fc fileio.fc fcntl.fc string.fc mdata.fc mparam.fc \
	lparam.fc auxparam.fc solar.fc
oaobook.o: sys.fc fileio.fc string.fc coord.fc lparam.fc mparam.fc mdata.fc \
	fcntl.fc procs.fc auxparam.fc solar.fc graph.fc
oaotop.o: sys.fc coord.fc fileio.fc fcntl.fc string.fc mdata.fc mparam.fc \
	lparam.fc auxparam.fc solar.fc oaolib.fi
plotm.o: sys.fc mdata.fc coord.fc solar.fc mparam.fc string.fc fileio.fc \
	fline.fc graph.fc
plotmain.o: sys.fc fileio.fc string.fc coord.fc lparam.fc mparam.fc \
	mdata.fc fcntl.fc procs.fc
plotr.o: graph.fc coord.fc fline.fc mdata.fc
poltob.o: sys.fc string.fc mdata.fc fileio.fc mparam.fc auxparam.fc
pot.o: sys.fc coord.fc ss.fc s1.fc s2.fc s3.fc lparam.fc auxparam.fc ff0.fc \
	fff.fc
progmain.o: sys.fc fileio.fc string.fc coord.fc lparam.fc mparam.fc \
	mdata.fc fcntl.fc procs.fc auxparam.fc
readl.o: sys.fc coord.fc fileio.fc string.fc fline.fc lparam.fc mparam.fc \
	ss.fc
readm.o: sys.fc fileio.fc mdata.fc mparam.fc lparam.fc string.fc solar.fc \
	coord.fc auxparam.fc
reads1.o: sys.fc coord.fc solar.fc lparam.fc mparam.fc auxparam.fc ss.fc \
	s1.fc
rotm.o: sys.fc fileio.fc coord.fc lparam.fc auxparam.fc mparam.fc mdata.fc
select.o: sys.fc coord.fc mparam.fc lparam.fc fline.fc ff0.fc fff.fc \
	mdata.fc fileio.fc auxparam.fc
sets12.o: sys.fc coord.fc solar.fc lparam.fc mparam.fc auxparam.fc \
	fileio.fc mdata.fc ss.fc s1.fc s2.fc
spec.o: mdata.fc fileio.fc sys.fc
sphere.o: graph.fc coord.fc fline.fc sys.fc
spllof.o: sys.fc fline.fc lparam.fc ff0.fc fff.fc s3.fc
suncal.o: sys.fc
sunparam.o: sys.fc
sysconf.o: sys.fc
test.o: sys.fc mdata.fc mparam.fc coord.fc fileio.fc fcntl.fc string.fc \
	lparam.fc auxparam.fc
transc.o: coord.fc sys.fc lparam.fc
transm.o: sys.fc fileio.fc coord.fc lparam.fc mparam.fc mdata.fc \
	auxparam.fc
unixio.o: sys.fc fileio.fc fcntl.fc
unixmisc.o: convu.fi sys.fc
vaxio.o: sys.fc fileio.fc fcntl.fc
vaxmisc.o: convv.fi sys.fc
vector.o: sys.fc coord.fc mdata.fc mparam.fc fileio.fc auxparam.fc
version.o: sys.fc update.fi
virial.o: sys.fc coord.fc solar.fc mdata.fc mparam.fc auxparam.fc
vppio.o: sys.fc fileio.fc fcntl.fc
vppmisc.o: convi.fi sys.fc
writel.o: sys.fc string.fc fileio.fc mparam.fc lparam.fc fline.fc ff0.fc \
	ss.fc
writem.o: sys.fc mparam.fc fileio.fc mdata.fc string.fc solar.fc coord.fc
