c  generate a simulated magnetogram based on the non-linear force-free
c  field solution of Low and Lou (Astrophys. J. 352, 343-352, 1990)
c
	include 'sys.fc'
	include 'mdata.fc'
	include 'mparam.fc'
	include 'coord.fc'
	include 'solar.fc'
	include 'fileio.fc'
	include 'fcntl.fc'
	include 'string.fc'
	include 'auxparam.fc'
	integer uin,uout
	dimension xx(3),bb(3),dir(3),xxs(3)
	character*80 fname1,fname2,chr
	logical alloc
c
	keyin  = 5
	stdin  = 10
	lst    = 6
	msg    = msgini()
	errflg = 0
	chr='0'
        if(myargc().eq.1) call myargv(1,chr)
        if(chr.eq.'1') then
          msglvl=1
        else if(chr.eq.'2') then
          msglvl=2
        else if(chr.eq.'3') then
          msglvl=3
        else
          msglvl=0
        endif
c
	include 'sysconf.cfg'
	call initfc
c
	if(mycode.eq.0 .or. myint.eq.0) stop
	nbytem=sign(2,myint)
c
	uin=98
	lrec0=80
	uout=97
c
	mesh=0
	rsp=70.0
c  the following are not set in readff because solar.fc is not included there
	ypos=0.0
	pdeg=0.0
	b0deg=0.0
	cl0deg=0.0
c
	call getfn('NLFFF',fname1)
	call openfl(other1,'NLFFF',uin,fname1,text,rdo,0)
	call readff(uin,iproj,kindm,nx,ny,xs,ys,dx,dy,rsp)
	read(uin,'(a)') fname2
	close (uin)
c
c  kindm =2 - pseudo-vector
c        =3 - real vector
c  bx,by,bz are evaluated on the following meshes
c  iproj=0: plane, face-on
c        1: plane, oblique
c      2,3: sphere
c
c  readff sets kindm etc.
c  xxs and dir passed through auxparam.fc
	do 10 i=1,3
	xxs(i)=xxsaux(i)
	dir(i)=diraux(i)
 10	continue
	b0=b0fff
	al=alfff
	order=odrfff
	isym=symfff
	ak=akfff
	dmu=dmufff
	nmu=nmufff
c
	call openfl(other2,'GMGRAM',uout,fname2,export,wrt,lrec0)
	tsys='GMT+09'
	inst='DUMMY'
	wavel=5250.0
	mfile='NLFFF'
	dateo=' '
	stime=' '
	etime=' '
	ftag=' '
	write(title,100) iproj,xxs,dir,al
 100	format('NLFFF',i2,1p7e10.2)
c
	alloc=.true.
	kz=newk(lspec('BL'), alloc)
	kx=newk(lspec('BX'), alloc)
	ky=newk(lspec('BY'), alloc)
	ka=0
	kj=0
	if(kx.eq.0.0 .or. ky.eq.0 .or. kz.eq.0) then
	  write(msg,*) '(prog.genfff) error kx,ky,kz=',kx,ky,kz
	  stop
	endif
	factm(lspecs(kx))=0.2
	factm(lspecs(ky))=0.2
	factm(lspecs(kz))=0.2
c
	call umake(kindm,m5,m4,m3,m2,m1)
	if(m4.eq.2) then
	  ka=newk(lspec('ALPHA'),alloc)
	  if(ka.eq.0) then
	    write(msg,*) '(prog.genfff) error ka=',ka
	    stop
	  endif
	  factm(lspecs(ka))=0.2
	endif
	if(m2.eq.2) then
	  kj=newk(lspec('JN'),alloc)
	  if(kj.eq.0) then
	    write(msg,*) '(prog.genfff) error kj=',kj
	    stop
	  endif
	  factm(lspecs(kj))=0.2
	endif
c
	if(iproj.ge.1) then
	  call ctot(xxs,xx)
	  depth=-xx(3)
	else
	  depth=-xxs(3)
	endif
c
	do 20 j=1,ny
	do 20 i=1,nx
	xx(1)=xs+dx*float(i-1)
	xx(2)=ys+dy*float(j-1)
	xx(3)=0.0
	if(iproj.eq.1) then
	  call projt(xx,xt,yt)
	else if(iproj.ge.2) then
	  call projsr(xx,rsp)
	endif
c
	call nlfff(xx,bb,alpha,xxs,dir,b0,al,
     &	     order,isym,ak,dmu,nmu,pval,pdval)
c
	rdata(i,j,kx)=bb(1)
	rdata(i,j,ky)=bb(2)
	rdata(i,j,kz)=bb(3)
	if(ka.gt.0) rdata(i,j,ka)=alpha
	if(kj.gt.0) rdata(i,j,kj)=alpha*bb(3)
c! incomplete
 20	continue
c
	kindmo = kindm
	call writem (uout)
	write(lst,*) 'non-linear force-free field of Low and Lou:'
	write(lst,*) '  source location=',xxs
	write(lst,*) '  b0=',b0
	write(lst,*) '  direction=',dir
	if(iproj.eq.1) 
     &write(lst,*) '  xtod,ytod=',xtod,ytod
	write(lst,*) '  nx,ny=',nx,ny
	write(lst,*) '  xs,ys=',xs,ys
	write(lst,*) '  dx,dy=',dx,dy
	write(lst,*) '  data projection: iproj=',iproj
	write(lst,*) '  kindmo=',kindmo
c
	write(lst,*) 'ak=',ak
	write(lst,*) 'pval='
	write(lst,*) (pval(i),i=1,nmu)
cx	write(lst,*) 'pdval='
cx	write(lst,*) (pdval(i),i=1,nmu)
	stop
	end
