c
c  This program gives an example of generating a magnetogram file
c  with the standard format of MAGPACK2 package.
c  Edit this file and compile it by:
c     make genmag
c
	include 'sys.fc'
	include 'mparam.fc'
	include 'fileio.fc'
	include 'mdata.fc'
	include 'string.fc'
	include 'solar.fc'
	include 'coord.fc'
c
	integer iunit,ounit
	character ifile*40,ofile*40
c
	include 'sysconf.cfg'
	call initfc
	if(mycode.eq.0 .or. myint.eq.0) stop
	lrec0=80
	iunit=90
	ounit=91
	nbytem=sign(2,myint)
c
c******************* Edit the Following Lines ************************
c
c  Number of observed points in x and y. For example,
c
	nx=50
	ny=50
c
c  Coordinates of the lower-left corner (South-East corner) of the
c  magnetogram on the sun. The unit of length is 10^4 km. For example,
c
	xs = -5.0
	ys = -5.0
c
c  Mesh size of observation in 10^4 km. For example,
c
	dx = 0.5
	dy = 0.5
c
c  Suppose the longitudinal magnetogram is stored in a file '../data/ifile'.
c  The file format is supposed to be unformatted, sequential.
c
	ifile = '../data/ifile'
c  The name of the file should be changed to an appropriate name.
c
c  Open the file.
	open(unit=iunit,file=ifile,form='unformatted')
c
c  Read data
	read(iunit) ((rdata(i,j,1),i=1,nx),j=1,ny)
c
c  The data, which are the longitudinal magnetic field 'BL', are
c  now stored in rdata(*,*,1). This situation is represented
c  by the following setting. These need not be altered.
	ll = lspec('BL')
	kloc(ll)=1
	lspecs(1)=ll
	nframe=1
	kmem(1)=1
	kindm=1
c
c  Suppose the unit of data is gauss. The output will be converted into
c  an integer array by:
c      real data = factor * integer data.
c  Here we set the factor to 1.0.
	factm(ll) = 1.0
c
c  If the unit of data is not gauss, set an appropriate value to
c  factm(ll) above so that the integer data do not overflow nor become
c  too small.
c
c---------------------------------------------------------------------
c  If the input file contains other data, e.g. 'BX' and 'BY',
c  these can be read as follows. Uncomment the following lines if needed.
c	read(iunit) ((rdata(i,j,2),i=1,nx),j=1,ny)
c	lx = lspec('BX')
c	kloc(lx)=2
c	lspecs(2)=lx
c	kmem(2)=2
c	factm(lx) = 1.0
c	read(iunit) ((rdata(i,j,3),i=1,nx),j=1,ny)
c	ly = lspec('BY')
c	kloc(ly)=3
c	lspecs(3)=ly
c	kmem(3)=3
c	factm(ly) = 1.0
c	nframe=3
c	kindm=2
c  'kindm=2' means a vector magnetogram.
c---------------------------------------------------------------------
c
c  Set other parameters which are written to the header of the output file.
c
c  Direction of the solar pole, the so-called P-angle, in degree.
c  For example,
	pdeg = 2.0
c
c  Heliographic latitude of the disk center, the so-called B0-angle, 
c  in degree. For example,
	b0deg = -3.0
c
c  Direction of the y-axis of the data in the celestial coordinates.
c  If the y-axis is directed to the celestial north, this should be 0.
	ydeg = 0.0
c
c  Time zone which is used to describe the observing times.
c  You may give 'UT+11' etc., up to 6 characters.
	tsys='UT'
c
c  Name of the instrument used to obtain the magnetogram, up to 8 
c  characters.
	inst='my VMG'
c
c  Wavelength of observation.
	wavel=5250.0
c
c  File identifier, usually the name of the input file. However, this 
c  can be anything, up to 8 characters.
	mfile='SAMPLE'
c
c  Date of observation 'yy/mm/dd'
	dateo='96/12/01'
c
c  Start and end times of observation 'hh:mm:ss:
	stime='00:00:00'
	etime='00:00:10'
c
c  Comment of up to 80 characters, if any.
	title = 'comment...'
c
c  Give the name of the output file here.
	ofile = '../data/ofile'
c  The name of the file should be changed to an appropriate name.
c
c  Creation date and time of the output file in the format of
c  'yy/mm/ddhh:mm:ss'. ftag is a string of 16 characters.
	ftag = '97/01/0100:00:00'
c
c************************** End of Edit ******************************
c
c  Open the output file
	call openfl(other1,'GMGRAM',ounit,ofile,export,wrt,lrec0)
c
	xtod=xs+0.5*dx*float(nx-1)
	ytod=ys+0.5*dy*float(ny-1)
	kindmo=kindm
	mesh=0
	nframo=nframe
	cl0deg=0.0
	rsp=69.6
c
c  Write the header
	call outmh(ounit)
c  Print out the parameters.
	write(lst,*)
     &	'(prog.genmag) kindm,mesh,nframe=',kindm,mesh,nframe
	call printm(nbytem,nx,ny,xs,ys,dx,dy,ydeg,
     &	          rsp,pdeg,b0deg,cl0deg,xtod,ytod,
     &	          tsys,inst,wavel,mfile,dateo,stime,etime,
     &	          ftag,title)
c
c  Write the data (nframe-frames).
	do 1000 k=1,nframe
 1000	call outm(ounit,k)
c
	stop
	end

