c  integration of lines of force
c  by Adams-Bashforth-Moulton predictor-corrector procedure
c  First 4-points are by Runge-Kutta method.
c  Number of field lines to be integrated is nf,
c  nitial stepsize=ds0
c  Integration limit is checked by subr.range.
c  Cancellation of footpoints is made if cancel=true.
c  Result is written in file(unit='ounit') unless wfile=false.
c
	subroutine integ(ounit)
c
	include 'sys.fc'
	include 'fline.fc'
	include 'coord.fc'
	include 'lparam.fc'
	include 'mparam.fc'
	include 'auxparam.fc'
	include 'ss.fc'
	include 's1.fc'
	include 's2.fc'
c
	external dxds,range,modulo
c
	logical range,ldummy
	integer ounit
	dimension xyz1(8,ML)
c  ML is defined in fline.fc
c  xyz1 is a dummy argument.
	dimension xx(8),xx0(8),xx1(8),dxx(8),aps(2),
     &          axx(8,2),adxx(8,4),wk(8,2),foot1(3)
c
	eps=1.0e-4
	mode=0
	if(wfile) call writlh(ounit)
	go to 10
c
	entry integ1(foot1,np1,xyz1)
c  this entry is to integrate one field line
c  output is not written to a file
c
	mode=1
	wfile=.false.
	cancel=.false.
	nf=1
	do 20 m=1,3
 20	foot(m,1)=foot1(m)
c
 10	continue
	ipmax=ML
	big=1.0e20
	if(ds0.eq.0.0) then
	  write(msg,*) '(subr.integ ) error: ds0=',ds0
	  errflg=2
	  return
	endif
c
c  set integration limits
	call setlim(mesh)
c
c  for binary star model, z1lim and z2lim specify the box size in
c  the z-direction. The limit for the height from the stellar surface
c  is fixed to 0.0.
c
c  for normal solar models with spherical geometry, z1lim and z2lim refer
c  to the height limits from the solar surface.
c
	if(nsp.gt.1) then
	  h1lim=0.0
	  h2lim=big
	else
	  h1lim=z1lim
	  h2lim=z2lim
	endif
c
	l=0
c
	if(nf.le.0) go to 9000
	if( (x1lim.ge.x2lim .and. mesh.ne.4)
     &	  .or. y1lim.ge.y2lim .or. z2lim.le.z1lim) then
	  write(msg,*) 
     &	    '(subr.integ ) error: integration limit=',
     &	     x1lim,x2lim,y1lim,y2lim,z1lim,z2lim
	  errflg=2
	  return
	endif
c
	if(msglvl.gt.1) write(lst,100)
 100	  format(/2x, t15,'x',t30,'y',t45,'z',t60,'b')
	if(mode.eq.0 .and. msglvl.eq.1) call dots(msg,1)
c
	do 1000 ll=1,nf
c
c  determine the sign of ds
c
	ds=ds0
	np0=0
	go to 3
c
c  reduce ds when necessary
c
 2	ds=0.2*ds
	if(abs(ds).lt.0.1*ds0) then
	  if(msglvl.gt.1) write(lst,400) ll,(foot(m,ll),m=1,3)
 400	    format(/2x, i5,1p3e15.5,'    too short')
	  go to 1000
	endif
c
c  skip cancelled footpoints
c
 3	continue
	if(foot(1,ll).gt.big) go to 1000
c
c  footpoint on the solar surface -- ld=1
c            above the surface    -- ld=2 (two-way integration)
c
	ld=1
	ldummy=range(foot(1,ll),h0)
	if(h0.gt.h1lim+eps .and. h0.lt.h2lim-eps) ld=2
c
	if(ld.eq.2.and.msglvl.gt.2) 
     &	write(lst,*) '(subr.integ ) two-way integration for ll=',ll
c
	do 1100 ld1=1,ld
c
	do 1200 m=1,3
 1200	xx(m)=foot(m,ll)
	do 1210 m=4,8
 1210	xx(m)=0.0
c
c  determine the sign of ds
c
	call dxds(8,xx,dxx)
	do 1220 m=1,4
 1220	adxx(m,1)=dxx(m)
c
	if(ld1.eq.2) then
	  ds=-ds
	else
	  do 1300 m=1,3
 1300	  xx0(m)=xx(m)+ds*dxx(m)
	  if(.not.range(xx0,h)) ds=-ds
	endif
c
c  initial three points by Runge-Kutta method
c
	h=h0
	do 2000 ip=1,3
	do 2200 m=1,4
	xyzp(m,ip)=xx(m)
 2200	xyzp(m+4,ip)=dxx(m+4)
	hb=h
	if(ip.gt.1) then
	  if(.not.range(xx,h)) go to 4000
	endif
	call rk4(dxds,8,xx,ds,2,8,axx,adxx(1,ip),wk)
	do 2300 m=1,4
 	dxx(m+4)=adxx(m+4,ip+1)
 2300	xx(m)=axx(m,2)
 2000	continue
	ip=3
c  adxx(m,1-4) had been set
	go to 3200
c
c  Adams-Bashforth-Moulton predictor-corrector procedure
c
 3000	continue
	do 3100 ip1=1,3
	do 3100 m=1,4
 3100	adxx(m,ip1)=adxx(m,ip1+1)
	call dxds(8,xx,adxx(1,4))
c
c  this is the entry point after the initial steps by Runge-Kutta method
c
 3200	ip=ip+1
	do 3400 m=1,4
	xyzp(m,ip)=xx(m)
 3400	xyzp(m+4,ip)=adxx(m+4,4)
	hb=h
	if(.not.range(xx,h)) go to 4000
	if(ld.eq.1.and.ip.ge.ipmax) go to 4000
	if(ld.eq.2.and.ld1.eq.1.and.ip.ge.ipmax-4) go to 4000
c  ipmax-4 means that first 4-points for Runge-Kutta procedure are
c  reserved
	if(ld1.eq.2.and.np0+ip.ge.ipmax) go to 4000
c
c  predictor
c
	do 3500 m=1,4
 3500	xx1(m)=xx(m)+(55.0*adxx(m,4)-59.0*adxx(m,3)+37.0*adxx(m,2)
     &	      -9.0*adxx(m,1))*ds/24.0
c
c  corrector
c
	call dxds(8,xx1,dxx)
	do 3600 m=1,4
 3600	xx(m)=xx(m)+
     &	   (9.0*dxx(m)+19.0*adxx(m,4)-5.0*adxx(m,3)+adxx(m,2))*ds
     &	   /24.0
	go to 3000
c
 4000	np=ip
c
c  try again with smaller step size
c
	if(np.le.3) go to 2
c
c  precise position of endpoints
c
	if(ld1.eq.1) l=l+1
	hh=1.0
	if((h-h1lim)*(hb-h1lim).le.0.0) hh=(hb-h1lim)/(hb-h)
	if((h-h2lim)*(hb-h2lim).le.0.0) hh=(hb-h2lim)/(h-hb)
	do 4300 m=1,3
	xx0(m)=xyzp(m,np-1)+hh*(xyzp(m,np)-xyzp(m,np-1))
	if(ld.eq.1) foote(m,ll)=xx0(m)
 4300	xyzp(m,np)=xx0(m)
	xyzp(4,np)=xyzp(4,np-1)+hh*(xyzp(4,np)-xyzp(4,np-1))
	call dxds(8,xx0,dxx)
	do 4400 m=1,4
 4400	xyzp(m+4,np)=dxx(m+4)
c
c  rearrangement of data for two-way integration
c
	if(ld.eq.2) then
	  if(ld1.eq.1) then
	    np0=np
	    do 6000 ip=1,np
	    ii=np+1-ip
	    do 6100 m=1,8
 6100	    xyz2(m,ii)=xyzp(m,ip)
 6000	    continue
	  else
	    if(msglvl.eq.2) 
     &	    write(lst,350) l,(xyz2(m,  1),m=1,3),xyz2(8,  1),
     &                       (xyz2(m,np0),m=1,3),xyz2(8,np0),
     &                       (xyzp(m, np),m=1,3),xyzp(8, np)
 350	    format(/2x, i5,1p4e15.5,/(2x, 5x,4e15.5))
	    if(np+np0-1.le.3) go to 2
	    do 6200 ip=2,np
	    ii=np0+ip-1
	    do 6300 m=1,8
 6300	    xyz2(m,ii)=xyzp(m,ip)
 6200	    continue
	  endif
	else
	  if(msglvl.eq.2) 
     &	  write(lst,350) l,(xyzp(m, 1),m=1,3),xyzp(8, 1),
     &	                   (xyzp(m,np),m=1,3),xyzp(8,np)
	endif
c
 1100	continue
c
	if (ld.eq.2) then
	  np=np+np0-1
	  do 6400 ip=1,np
	  do 6400 m=1,8
 6400	  xyzp(m,ip)=xyz2(m,ip)
	  go to 7000
	endif
	if(.not.cancel) go to 7000
	if(ll.eq.nf) go to 7000
	if(h.gt.eps) go to 7000
c
c  cancellation of footpoints
c
	nn=0
	rr2=1.0e10
	ll1=ll+1
c
	do 5000 n=ll1,nf
	if(foot(1,n).gt.big) go to 5000
	r2=0.0
	do 5100 m=1,3
 5100	r2=r2+(foot(m,n)-xx0(m))**2
	if(r2.gt.rr2) go to 5000
	rr2=r2
	nn=n
 5000	continue
c
	if(nn.eq.0) go to 7000
	if(ns1.gt.0) then
	  do 5200 n=1,ns1
	  r1=0.0
	  r2=0.0
	  do 5300 m=1,3
	  r1=r1+(xx0(m)-xxc1(m,n))**2
 5300	  r2=r2+(foot(m,nn)-xxc1(m,n))**2
	  if(r1.lt.aa1(n)**2.and.r2.lt.aa1(n)**2) go to 5900
 5200	  continue
	endif
c
	n=0
	if(nx*ny.eq.0) go to 7000
	if(mod(kindl,10).le.1) then
	  dxdy=abs(dx*dy)
	else
	  dxdy=abs(dx*dy)*rsp**2
	endif
c
cx	if(msglvl.gt.2) write(lst,*) 'closest footpoint distance=',
cx     &	  sqrt(rr2),dxdy
c
	if(rr2.gt.dxdy) go to 7000
 5900	continue
c
	if(msglvl.gt.1) write(lst,500) l,(foot(m,nn),m=1,3)
 500	  format(/2x, i10,'-th lof deleted foot-point at ',1p3e12.3)
	foot(1,nn)=10.0*big
 7000	continue
c
	if(msglvl.gt.2) 
     &	  write(lst,350) l,((xyzp(m,n),m=1,3),xyzp(8,n),n=1,np)
c
cx	if(msglvl.gt.2) 
cx     &	  write(lst,350) l,((xyzp(m,n),m=5,7),xyzp(4,n),n=1,np)
c
c  output to file
c
	if(wfile) then
	  call writel(ounit,np,xyzp)
c  writel internally uses ixyz2/4
	  if(errflg.gt.1) then
	    l=l-1
	    errflg=0
	  endif
	endif
c
	if(mode.eq.0 .and. msglvl.eq.1) call dots(msg,0)
 1000	continue
c
c  end of loop
c
	if(mode.eq.1) then
	  np1=np
	  do 7100 ip=1,np1
	  do 7100 m=1,8
 7100	  xyz1(m,ip)=xyzp(m,ip)
	endif
c
	if(.not.wfile) return
	call writel(ounit,0,xyzp)
c  end of file by np=0
c
c  hard end of file causes trouble in the case of SUNos
c  because pointer reset to 1 (top of file) does not work
c  after hard EOF is encountered
c
	nf=l
 9000	continue
	if(mode.eq.0 .and. msglvl.eq.1) call dots(msg,-1)
c
	if( msglvl.gt.0) then
	  write(lst,*) '(subr.integ )'
	  write(lst,*) '  nf=',nf,', ds0=',ds0
	  write(lst,*) '  integration limits=',
     &	            x1lim,x2lim,y1lim,y2lim,z1lim,z2lim
	  write(lst,*) '  cancel=',cancel
	endif
	return
	end
c
c  integration limit check
c
	logical function range(xx,h)
	include 'sys.fc'
	include 'mparam.fc'
	include 'lparam.fc'
	include 'coord.fc'
c
	integer*4 kindl1
	dimension xx(8),xxt(8)
	data pi/3.141592/
c
	mode = 0
	go to 1
c
c  range0 forces xx to lie on the boundary if h <0
	entry range0(xx,h)
	mode = 1
 1	continue
c
	if (nsp.lt.2) then
c
	eps = 1.0e-4
	kindl1=mod(kindl,1000)
	range=.false.
	if (kindl1.eq.0) then
	  if(xx(1).ge.x1lim .and. xx(1).le.x2lim.and.
     &	     xx(2).ge.y1lim .and. xx(2).le.y2lim.and.
     &	     xx(3).ge.z1lim .and. xx(3).le.z2lim) range=.true.
	  h=xx(3)-z1lim
	  if(.not.range .and. msglvl.gt.2) then
	    write(lst,*) 'x ,limits=',xx(1),x1lim,x2lim
	    write(lst,*) 'y ,limits=',xx(2),y1lim,y2lim
	    write(lst,*) 'z ,limits=',xx(3),z1lim,z2lim
	  endif
	  if(h.lt.eps.and.mode.eq.1) then
	    xx(3)=0.0
	  endif
	else if (mod(kindl1,10).eq.1) then
	  call ctot(xx,xxt)
	  if(xxt(1).ge.x1lim .and. xxt(1).le.x2lim.and.
     &	     xxt(2).ge.y1lim .and. xxt(2).le.y2lim.and.
     &	     xxt(3).ge.z1lim .and. xxt(3).le.z2lim) range=.true.
	  h=xxt(3)
	  if(.not.range .and. msglvl.gt.2) then
	    write(lst,*) 'x,xl,limits=',xx(1),xxt(1),x1lim,x2lim
	    write(lst,*) 'y,yl,limits=',xx(2),xxt(2),y1lim,y2lim
	    write(lst,*) 'z,zl,limits=',xx(3),xxt(3),z1lim,z2lim
	  endif
	  if(h.lt.eps.and.mode.eq.1) then
	    xxt(3)=0.0
	    call ttoc(xxt,xx)
	  endif
	else if (mod(kindl1,10).eq.2 ) then
	  call ctos(xx,ap,al,h)
	  if(ap.ge.x1lim .and. ap.le.x2lim.and.
     &	     al.ge.y1lim .and. al.le.y2lim.and.
     &	     h .ge.z1lim .and. h .le.z2lim) range=.true.
	  if(.not.range .and. msglvl.gt.2) then
	    write(lst,*) 'x,ap,limits=',xx(1),ap,x1lim,x2lim
	    write(lst,*) 'y,al,limits=',xx(2),al,y1lim,y2lim
	    write(lst,*) 'z,h ,limits=',xx(3),h ,z1lim,z2lim
	  endif
	  if(h.lt.eps.and.mode.eq.1) then
	    call stoc(ap,al,0.0,xx)
	  endif
	else if (kindl1.eq.103) then
	  call ctor(xx,apr,alr,h)
	  if(x2lim.gt.x1lim) then
	    if(alr.ge.y1lim .and. alr.le.y2lim.and.
     &       h  .ge.z1lim .and. h  .le.z2lim) then
	      if(apr.ge.x1lim .and. apr.le.x2lim .or.
     &	   apr+2.0*pi.ge.x1lim .and. apr+2.0*pi.le.x2lim .or.
     &	   apr-2.0*pi.ge.x1lim .and. apr-2.0*pi.le.x2lim)
     &	   range=.true.
	    endif
	  else
	    if((apr.ge.0.0  .and. apr.le.x1lim .or.
     &	       apr.ge.x2lim .and. apr.le.2.0*pi) .and.
     &	       alr.ge.y1lim .and. alr.le.y2lim.and.
     &	       h  .ge.z1lim .and. h  .le.z2lim) range=.true.
	  endif
	  if(.not.range .and. msglvl.gt.2) then
	    write(lst,*) 'x,apr,limits=',xx(1),apr,x1lim,x2lim
	    write(lst,*) 'y,alr,limits=',xx(2),alr,y1lim,y2lim
	    write(lst,*) 'z,h  ,limits=',xx(3),h  ,z1lim,z2lim
	  endif
	  if(h.lt.eps.and.mode.eq.1) then
	    call rtoc(apr,alr,0.0,xx)
	  endif
	endif
c
c  extention for binary star model
c
	else
c
	  if(xx(1).ge.x1lim .and. xx(1).le.x2lim.and.
     &	     xx(2).ge.y1lim .and. xx(2).le.y2lim.and.
     &	     xx(3).ge.z1lim .and. xx(3).le.z2lim) then
	    range=.true.
	  else
	    range=.false.
	  endif
	  if(.not.range .and. msglvl.gt.2) then
	    write(lst,*) 'x ,limits=',xx(1),x1lim,x2lim
	    write(lst,*) 'y ,limits=',xx(2),y1lim,y2lim
	    write(lst,*) 'z ,limits=',xx(3),z1lim,z2lim
	  endif
	  h=1.0e20
	  h1lim=0.0
	  do 1000 n=1,nsp
	  h1 = sqrt((xx(1)-xxspm(1,n))**2
     &	           +(xx(2)-xxspm(2,n))**2
     &	           +(xx(3)-xxspm(3,n))**2) - rspm(n)
	  h = min(h,h1)
 1000	  continue
	  range = range .and. (h.ge.h1lim)
c  z1lim and z2lim give box depth and not the height range
	  return
	endif
	end
c
c  set integration limits
c
	subroutine setlim(mesh1)
c
	include 'sys.fc'
	include 'coord.fc'
	include 'lparam.fc'
	include 'auxparam.fc'
	dimension aps(2)
c
       if( iabs(mesh1).le.1) then
          x1lim=x1lim0+x1del
          x2lim=x2lim0+x2del
          y1lim=y1lim0+y1del
          y2lim=y2lim0+y2del
          if (z1lim.eq.0.0) z1lim=z1lim0
          if (z2lim.eq.0.0) z2lim=z2lim0+z2del
        else
          aps(1)=x1lim0
          aps(2)=x2lim0
          call modulo(aps,mesh1,2)
          x1lim=aps(1)+x1del/rsp
          x2lim=aps(2)+x2del/rsp
          y1lim=y1lim0+y1del/rsp
          y2lim=y2lim0+y2del/rsp
          if (z1lim.eq.0.0) z1lim=z1lim0
          if (z2lim.eq.0.0) z2lim=z2lim0+z2del
        endif
c
	return
	end
