c  calculate linear (constant-alpha) force-free fields on the mesh points
c
c  linear (constant-alpha) force-free field by FFT
c  input:  bz given in rdata(*,*,kz), and amat matrix
c  output: bxp and byp given in rdata(*,*,kxp),rdata(*,*,kyp)
c
	subroutine lff1(kz,amat,kxp,kyp,alpha1,tight)
c
	include 'sys.fc'
	include 'coord.fc'
	include 'mdata.fc'
	include 'mparam.fc'
	include 'fileio.fc'
	parameter (MM1D=8*MX*MY)
c
	complex cx0,cy0,cx,cy,cdx,cdy
	logical tight
	dimension ndims(2),work(MMX,2)
	dimension amat(3,3)
	dimension datax(MM1D),datay(MM1D),dataz(MM1D)
	equivalence (qdata(1,1,1),datax(1)),
     &	      (qdata(1,1,9),datay(1),dataz(1))
c
	data pi/3.141592/
c
	call allocm(1,16,'lff1')
	if(errflg.gt.1) then
	  write(msg,*) '(subr.lff1  ) error'
	  return
	endif
	mode=1
	go to 1
c
	entry mxalph(amat,alpha1,tight)
	mode=2
 1	continue
c
c  set bz into data for Fourier transform
c  calculate flux
c
	if(tight) then 
	  nxh = nx/2
	  nyh = ny/2
	  nx2 = nx
	  ny2 = ny
	else
	  nxh = nx
	  nyh = ny
	  nx2 = 2*nx
	  ny2 = 2*ny
	endif
c
c  calculate maximum possible value of alpha
	alx=dx*float(nx2)/(2.0*pi)
	aly=dy*float(ny2)/(2.0*pi)
	alpham=1.0e10
	do 1100 kwx=-1,1
	do 1100 kwy=-1,1
	if (kwx.eq.0 .and. kwy.eq.0) go to 1100
	akwx=float(kwx)/alx
	akwy=float(kwy)/aly
	akx=akwx*amat(1,1)
	aky=akwx*amat(1,2)+akwy*amat(2,2)
	alpham=min(alpham,sqrt(akx*akx+aky*aky))
 1100 continue
	if(mode.eq.2) then
	  alpha1=alpham
	  return
	endif
c
	if(nx2.gt.MMX .or. ny2.gt.MMY) then
	  write(msg,*) '(subr.lff1  ) error: data size nx2,ny2=',nx2,ny2
	  errflg=2
	  return
	endif
c
	ndims(1)=nx2
	ndims(2)=ny2
	nn=nx2*ny2
	do 1200 j=1,ny2
	do 1200 i=1,nx2
	l=i+nx2*(j-1)
	dataz(2*l-1)=0.0
 1200	dataz(2*l)=0.0
c  DC component is not allowed for fff
c
	do 1300 j=1,ny
	do 1300 i=1,nx
	l=i+nx2*(j-1)
	dataz(2*l-1)=rdata(i,j,kz)
 1300	continue
c
c  Fourier transform of bz
c
	if(msglvl.gt.2) write(lst,*)
     &  '(subr.lff1  ) FFT start (BZ)',ndims
	call fourt(dataz,ndims,2,-1,0,work)
	if(msglvl.gt.2) write(lst,*)
     &  '              FFT end'
c
c  amat=unit matrix if (mesh,mesho)=(1,1), (0,0) or (-1,0)
c  amat=atmat if (mesh,mesho)=(0,-1) or (-1,-1)
c  (if mesh=-1, FFT is done as if mesh=0)
c
c  linear force-free field bxp,byp by fourier method
c
	do 2000 j=1,ny2
	do 2000 i=1,nx2
	l=i+nx2*(j-1)
	dataz(2*l-1)=dataz(2*l-1)/float(nn)
	dataz(2*l)  =dataz(2*l)  /float(nn)
	if(i.eq.1.and.j.eq.1) go to 2000
	kwx=i-1
	if(kwx.gt.nxh) kwx=kwx-nx2
	kwy=j-1
	if(kwy.gt.nyh) kwy=kwy-ny2
c
	call setbtc(kwx,kwy,alx,aly,cx0,cy0,amat,alpha1)
	cx=cx0
	cy=cy0
	if(kwx.eq.nxh) then
	  af=0.5
	  if(kwy.eq.nyh) af=0.25
	  call setbtc(-kwx,kwy,alx,aly,cdx,cdy,amat,alpha1)
	  cx=cx+af*(cdx-cx0)
	  cy=cy+af*(cdy-cy0)
	endif
	if(kwy.eq.nyh) then
	  af=0.5
	  if(kwx.eq.nxh) af=0.25
	  call setbtc(kwx,-kwy,alx,aly,cdx,cdy,amat,alpha1)
	  cx=cx+af*(cdx-cx0)
	  cy=cy+af*(cdy-cy0)
	endif
	if(kwx.eq.nxh.and.kwy.eq.nyh) then
	  call setbtc(-kwx,-kwy,alx,aly,cdx,cdy,amat,alpha1)
	  cx=cx+0.25*(cdx-cx0)
	  cy=cy+0.25*(cdy-cy0)
	endif
	cx=cx*cmplx(dataz(2*l-1),dataz(2*l))
	cy=cy*cmplx(dataz(2*l-1),dataz(2*l))
	datax(2*l-1)=real(cx)
	datax(2*l)=aimag(cx)
	datay(2*l-1)=real(cy)
	datay(2*l)=aimag(cy)
c  now dataz(2*l-1) and datax(2*l) were destroyed
c
 2000	continue
	datax(1)=0.0
	datax(2)=0.0
	datay(1)=0.0
	datay(2)=0.0
c
	if(msglvl.gt.2) write(lst,*)
     &   '(subr.lff1  ) rev-FFT start (BX)',ndims
	call fourt(datax,ndims,2,1,1,work)
	if(msglvl.gt.2) write(lst,*)
     &   '              rev-FFT end'
c
	if(msglvl.gt.2) write(lst,*)
     &   '(subr.lff1  ) rev-FFT start (BY)',ndims
	call fourt(datay,ndims,2,1,1,work)
	if(msglvl.gt.2) write(lst,*)
     &   '              rev-FFT end'
c
	errx=0.0
	erry=0.0
	do 2100 j=1,ny
	do 2100 i=1,nx
	l=i+nx2*(j-1)
	errx=max(errx,abs(datax(2*l)))
	erry=max(erry,abs(datay(2*l)))
	rdata(i,j,kxp)=datax(2*l-1)
	rdata(i,j,kyp)=datay(2*l-1)
 2100	continue
c  imaginary parts of datax&y are eliminated
c
	if(msglvl.gt.2) write(lst,*) '(subr.lff1  ) errx,y=',errx,erry
	call freem(1,16)
	return
	end
c
c  factor in front of the fourier coefficients for bt of 
c  linear force-free field
c
	subroutine setbtc(kwx,kwy,alx,aly,cx,cy,amat,alpha)
c
	complex ckz,cx,cy,ci,ckxy
	dimension amat(3,3)
c
	ci=(0.0,1.0)
	akwx=float(kwx)/alx
	akwy=float(kwy)/aly
	akx=akwx*amat(1,1)
	aky=akwx*amat(1,2)+akwy*amat(2,2)
	akz=sqrt(abs(akx**2+aky**2-alpha*alpha))
	ckz=ci*akz-akwx*amat(1,3)-akwy*amat(2,3)
	ckxy= alpha*ci* (akwx*amat(2,3) - akwy*amat(1,3))
     &	 + alpha*alpha*amat(3,3)
c
	cx= ( akwx + ckz*amat(1,3) )*ci*akz - 
     &	alpha* (ci*akwy*amat(3,3) + alpha*amat(1,3) )
     &		/( ckz*amat(3,3)*ci*akz - ckxy )
	cy= ( akwy + ckz*amat(2,3) )*ci*akz + 
     &	alpha* (ci*akwx*amat(3,3) - alpha*amat(2,3) ) 
     &		/( ckz*amat(3,3)*ci*akz - ckxy )
	return
	end
