c  modified newton method to solve nonlinear simultaneous equations
c
	subroutine mnnl(funct,dfunct,n,x,f,g,eps1,dx1,ss1,
     &	                limit,ipr1,h)
c
c  funct(n,x,ff,g): equation to be solved is g(x)=0
c                   ff = g*g
c  dfunct(i,x,fd,g): g(x) when differentiating with x(i)
c                    part of g that does not change with x(i) can be
c                    omitted from the computation of g
c                    if i<0, x(-i) is changed, but the value of g
c                    is not calculated in dfunct.
c 
c  n = dimension of vectors x and g
c  f = sqrt(g*g)
c  eps1 = convergence parameter
c  dx1  = step size for the computation of derivatives of g
c  ss1  = step size of minimum search
c  limit = maximum number of iteration
c  ipr1  = parameter for message level
c  h     = area of size (n,n+2)
c          h(*,1) contains next increment of x-vector
c
c  return value of limit
c  limit = -999: parameter error
c          -kount: minimum not found, or minimization error
c          0: iteration count exceeded limit
c          kount: converged after 'kount' iterations
c
	external funct,dfunct
	include 'sys.fc'
	include 'ff0.fc'
	include 'fff.fc'
c
	dimension x(n),g(n),h(n,n+2)
	dimension delx(NVAR0),delg(NVAR0)
cc	dimension x0(300),g0(300)
c
	eps=eps1
	dx=dx1
	ss=ss1
	if(eps.lt.0.0) eps=1.0e-6
	if(dx.le.0.0) dx=0.001
	if(ss.le.0.0) ss=1.0
	if(n.le.0) go to 9200
	ipr=ipr1
	if(ipr1.gt.900) ipr=1
	call funct(n,x,ff,g)
	f=sqrt(abs(ff))
	kount=0
	kall=1
	hnrm=0.0
	if(limit.le.0) go to 1900
 1000	continue
c
cc	do 6000 j=1,n
cc	x0(j)=x(j)
c6000	g0(j)=g(j)
cc
	do 1100 j=1,n
	xx=x(j)
	x(j)=xx  +dx
	call dfunct(j,x,fd,h(1,1))
cc
cc	if(j.eq.1) write(lst,*) 'dfunct+',h(1,1)
cc
	x(j)=xx  -dx
	call dfunct(j,x,fd,h(1,2))
cc
cc	if(j.eq.1) write(lst,*) 'dfunct-',h(1,2)
cc
c
	do 1200 i=1,n
	h(i,j+2)=(h(i,1)-h(i,2))/(2.0*dx)
 1200	h(i,1)=0.0
cc
cc	if(j.eq.1) write(lst,*) 'diff  ',h(1,3)
cc
	x(j)=xx
	call dfunct(-j,x,fd,h(1,2))
cc	call dfunct(j,x,fd,delg)
c
	if(errfff.gt.0) return
 1100	continue
c
	if(msglvl.gt.2) then
	  do 5000 j=1,n
	  xx=x(j)
	  ddx=dx
	  x(j)=xx  +ddx
	  call  funct(j,x,fd,delx)
cc
cc	  if(j.eq.1) write(lst,*) 'funct+',delx(1)
cc
	  x(j)=xx  -ddx
	  call  funct(j,x,fd,delg)
cc
cc	  if(j.eq.1) write(lst,*) 'funct-',delg(1)
cc
          do 5100 i=1,n
 5100	  delx(i)=(delx(i)-delg(i))/(2.0*ddx)
cc
cc	  if(j.eq.1) write(lst,*) 'diff  ',delx(1)
cc
	  x(j)=xx
	  do 5200 i=1,n
 5200	  delg(i)=(delx(i)-h(i,j+2))
cc   &	     /(abs(h(i,j+2))+abs(delx(i))+1.0e-30)
	  write(lst,*) 'j=',j
	  write(lst,5300) 'delg1=',(h(i,j+2),i=1,n)
	  write(lst,5300) 'delg0=',(delx(i),i=1,n)
	  write(lst,5300) 'del(delg)=',(delg(i),i=1,n)
 5300	  format(1x,a/(1x ,1p5e15.6))
 5000	  continue
	  call funct(j,x,fd,delg)
	endif
c
 1900	continue
	if(ipr.gt.0 .and. mod(kount,ipr).eq.0) then
	  write(lst,100) kount,kall,(x(i),i=1,n)
	  write(lst,110) f,(g(i),i=1,n)
  100	  format(/1x,'iteration count=',i5,5x,'call=',i5,
     &	          5x,'x='/(1x,1p10e8.1))
  110	  format(/1x,'gnorm=',1pe12.3,5x,'g='/
     &	         (1x,1p10e8.1))
	endif
c
	if(limit.le.0) go to 9000
c
c  regularize the matrix in row direction
	do 2000 i=1,n
	hmax=0.0
	do 2100 j=1,n
	if(abs(h(i,j+2)).gt.hmax) hmax=abs(h(i,j+2))
 2100	continue
 2000	g(i)=g(i)/hmax
c
c  regularize the matrix in column direction
	do 2200 j=1,n
	h(i,j+2)=h(i,j+2)/hmax
 2200	continue
	do 2300 j=1,n
	hmax=0.0
	do 2400 i=1,n
	if(abs(h(i,j+2)).gt.hmax) hmax=abs(h(i,j+2))
 2400	continue
	h(j,2)=hmax
	do 2300 i=1,n
	h(i,j+2)=h(i,j+2)/hmax
 2300	continue
c
	call lins1(h(1,3),n,n,g,h,nstop,1.0e-15)
c
c  divide by the normalizing factors to get the solution
	do 2500 i=1,n
 2500	g(i)=g(i)/h(i,2)
c
	hnrm=0.
	do 3000 i=1,n
	h(i,1)=-g(i)
c3000	hnrm=hnrm+h(i,1)**2
 3000	hnrm=hnrm+abs(h(i,1))
	oldf=f
cc	hnrm=sqrt(hnrm)
c  h**2 may cause underflow
c
	if(ipr.eq.0) write(lst,120) kount,f,hnrm
	if(ipr.gt.0 .and. mod(kount,ipr).eq.0) then
	  if(msglvl.gt.1) write(lst,130) hnrm,(h(i,1),i=1,n)
	  if(msglvl.gt.2) write(lst,140) (h(i,2),i=1,n)
	endif
 120	  format(1x,'iteration count=',i5,5x,'gnorm=',1pe12.3,
     &	         5x,'hnorm=',e12.3)
 130	  format(1x,'hnorm=',1pe12.3,5x,'h='/(1x,1p10e8.1))
 140	  format(1x,'norm of derivatives='/(1x,1p10e8.1))
c
	kount=kount+1
	kall=0
	if(hnrm.lt.1.e-30) go to 9100
c*  skip the following statement temporarily
c*  if(hnrm.lt.eps*f) go to 9100
c
	do 3100 i=1,n
 3100	h(i,2)=x(i)
c
	icode=1
cx	icode=-1
	call min2(funct,n,x,h,ff,g,ss,kall,icode,ipr)
c
cc	do 6100 i=1,n
cc	delx(i)=x(i)-x0(i)
c6100	delg(i)=g(i)-g0(i)
cc	do 6200 i=1,n
cc	aux=0.0
cc	do 6300 j=1,n
c6300	aux=aux+hsave(i,j)*delx(j)
c6200	g0(i)=aux
cc	if(factj.gt.0.5) write(lst,6400) (g0(i),delg(i),i=1,n)
c6400	format(1h0,'h*delx,delg='/(1h ,1p10e13.4))
c
	f=sqrt(abs(ff))
	if(icode.lt.0) then
	  go to 9300
	else if(icode.gt.0) then
	  go to 9100
	else if(f-oldf.gt.eps) then
	  go to 9100
	endif
c
	t=0.0
	do 3200 i=1,n
c3200	t=t+(x(i)-h(i,2))**2
 3200	t=t+abs(x(i)-h(i,2))
cc	t=sqrt(t)
	if((hnrm.gt.eps .or. t.gt.eps) .and. kount.lt.limit) go to 1000
	if( hnrm.lt.eps .and. t.lt.eps) go to 8000
	go to 9000
c
 8000	if(ipr.ge.0) write(lst,800) eps
 800	format(1x,'(subr.mnnl  ) converged : eps=',1pe10.3)
	limit=kount
	if(ipr.le.0) return
 8100	write(lst,810) kount,(x(i),i=1,n)
 810	format(1x,'number of iterations=',i5,5x,'x='/(1x,1p10e8.1))
	write(lst,110) f,(g(i),i=1,n)
	return
c
 9000	if(ipr.ge.-1) write(lst,900)
 900	format(1x,'(subr.mnnl  ) iteration count exceeded limit')
	if(limit.lt.0) limit=0
	if(ipr.gt.0) go to 8100
	return
c
 9100	do 9110 i=1,n
 9110	x(i)=h(i,2)
	call funct(n,x,ff,g)
	f=sqrt(abs(ff))
	kall=kall+1
	if(f.lt.eps) go to 8000
	if(ipr.ge.-1) write(lst,910)
 910	format(1x,'(subr.mnnl  ) residual minimization failed')
	limit=-kount
	if(ipr.gt.0) go to 8100
	if(ipr.eq.0) write(lst,130) hnrm,(h(i,1),i=1,n)
	return
c
 9200	write(lst,920) n
 920	format(1x,'(subr.mnnl  ) error: n=',i5,4x,
     &	       'must be positive.')
	limit=-999
	return
c
 9300	if(ipr.ge.-1) write(lst,930)
 930	format(1x,'(subr.mnnl  ) minimum not found')
	limit=-kount
	return
	end
c
	subroutine min2(func,n,x,h,f,g,ss,kall,icode,ipr)
c
c  quadratic minimization
c
c  entry  icode>0  : complete minimization
c              <0  : return if f<f20
c  return icode=-1 : no minimum (s>100000*ss)
c                0 : success
c               +1 : error in minimization
c                    (s<0.01*ss but f>f20)
c
	external func
	include 'sys.fc'
	include 'ff0.fc'
	include 'fff.fc'
	dimension x(1),h(1),g(1)
c
	f20=f
	dd=0.0
	f2=f
	s=ss
	t=0.0
	ifs=0
c  ifs=0: initial two points
c      2: step toward decreasing f
c      3: f increased in both directions
c      1: estimated minimum location
c
	if(ipr.ge.0) write(lst,100) ifs,dd,f
 100	format(5x,'search mode=',i2,5x,'d=',1pe12.3,5x,'g*g=',e12.3)
c
 1	continue
	d=t+s
 2	continue
	do 10 i=1,n
 10	x(i)=x(i)+d*h(i)
	dd=dd+d
	errfff=0
	call func(n,x,f,g)
c
	if(ipr.ge.0) write(lst,110) ifs,dd,f
 110	format(5x,'search mode=',i2,5x,'d=',1pe12.3,5x,'g*g=',e12.3)
	kall=kall+1
c
	if(errfff.gt.0) then
        do 90 i=1,n
 90     x(i)=x(i)-d*h(i)
	  if(ifs.ne.1 .and. ifs.ne.2) then   
          dd=dd-d
	  d=-d
          s=0.1*s
          if(abs(s).gt.0.001*abs(ss)) go to 1
	  endif
	  errfff=0
	  call func(n,x,f,g)
        if(f.gt.f20) go to 1002
	  go to 1001
	endif
c
	if(ifs.eq.1) go to 3
	if(abs(s).gt.1.0e5*abs(ss)) go to 1002
	if(f.lt.f2) if(ifs-1) 11,12,12
	            if(ifs-1) 13,14,14
c
 11	f1=f2
	f2=f
	ifs=2
	if(icode.lt.0) go to 1001
	go to 1
 12	f2=f
	s=2.0*s
	t=0.0
	go to 1
 13	f1=f
	s=-s
	t=s
	ifs=3
	go to 1
 14	f3=f
	den=(f1-f2)+(f3-f2)
	if(den.eq.0.0) if(f-f20) 1001,1001,1003
	d=-s+0.5*s*(f1-f3)/den
	ifs=1
	go to 2
c
 3	continue
	if(f.lt.f2) go to 1001
	if(abs(s).lt.0.01*abs(ss)) if(f-f20) 1001,1001,1003
	f1=f
	s=-(s+d)
	t=s
	ifs=2
	go to 1
c
 1001	icode=0
	return
 1002	icode=-1
	return
 1003	icode=1
	return
	end
