c  data input program for:
c
c  Solar Flare Telescope Video Magnetograph/Dopplergraph and
c  Full-Disk Video Magnetograph
c  (National Astronomical Observatory at Mitaka)
c
c  mtktob.f
c
c  use direct access formatted i/o
c	 input  - recl=128
c	 output - recl=80
c  form of input file is specified by 'kindm'
c  kindm=0 means that kindm=kindmi and all the data will be stored
c
	subroutine obstob( iunit, ifile )
c
	include 'sys.fc'
	include 'coord.fc'
	include 'fileio.fc'
	include 'fcntl.fc'
	include 'string.fc'
	include 'mdata.fc'
	include 'mparam.fc'
	include 'lparam.fc'
	include 'auxparam.fc'
	include 'solar.fc'
c
	logical unique
	integer iunit,fmode1,fstat1,kdest(NSPEC0)
	character ifile*(*)
	rad(deg)=3.141592/180.0*deg
c
	fmode1=export
	fstat1=rdo
	lrec0=128
c
	call openfl(rawdat,'RAWDATA(MTK)',
     &	            iunit, ifile, fmode1, fstat1, lrec0 )
c
c  subr.ilist determines where the data should be stored
c  data specified by lspec will be stored in rdata(*,*,k),
c  where k=kdest(lspec). k runs from 1 to kmax.
c
	unique=.false.
	kmax=MZ
	call ilist(kindm,kdest,kmax,NSPEC0,unique)
	if (kmax.gt.MZ) then
	  write(msg,*)
     &	 '(subr.mtktob) error: insufficient space in rdata, kmax=',
     &	  kmax
	  errflg=2
	  return
	endif
c
	do 10 n=1,NSPEC0
 10	kloc(n)=0
	do 20 k=1,MZ
	kmem(k)=0
	lspecs(k)=0
 20	continue
	nframe=0
c
c  set filename without preceding directories to 'title'
	lc=locstr(ifile,'/',-1)
	if (lc.gt.0 .and. lc.lt.lenstr(ifile) ) then
	  title=ifile(lc+1:)
	else
	  title=ifile
	endif
c
	call inmtkh(iunit)
	if(errflg.gt.1) return
c
      if(msglvl.gt.1) then
        write(lst,*) '(subr.mtktob) kindm=',kindm
        write(lst,*) '  kindmi,mesh,nframei=',kindmi,mesh,nframi
        call printm(nbytmi,nx,ny,xs,ys,dx,dy,ydeg,
     &        rsp,pdeg,b0deg,cl0deg,xtod,ytod,
     &        tsys,inst,wavel,mfile,dateo,stime,etime,ftag,title)
	endif
c
	do 1000 iframe=1,nframi
	call inmtk(iunit,kdest)
	if(errflg.gt.1) return
 1000 continue
c
	if(kindmi.lt.0) call poltob
c
	xxsp(1)=0.0
	xxsp(2)=0.0
	xxsp(3)=-rsp
	prad=rad(pdeg)
	b0rad=rad(b0deg)
c  set xdu and ydu from ypos
	ypos=rad(ydeg)
	call setdu
c
	xtod=xs+0.5*dx*float(nx-1)
	ytod=ys+0.5*dy*float(ny-1)
	xxtod(1)=xtod
	xxtod(2)=ytod
	call projsr(xxtod,rsp)
	call dtoc(xxtod,xxto)
	call settu
	call setas
	call setar
c
c  set integration limits
c
	x1lim0=xs
	x2lim0=xs+dx*float(nx-1)
	z1lim0=0.0
	y1lim0=ys
	y2lim0=ys+dy*float(ny-1)
	z2lim0=0.5*sqrt((x2lim-x1lim)**2+(y2lim-y1lim)**2)
	ds0=sqrt(abs(dx*dy))*0.25
c
	errflg=0
	do 30 lspec1=1,NSPEC0
	k=kdest(lspec1)
	if(k.gt.0) then
	  if(lspecs(k).eq.0) then
	    write(msg,*)
     &	    '(subr.mtktob) error: requested data were not found, spec=',
     &	     specst(lspec1)
	    errflg=2
	  endif
	endif
 30   continue
c
	if(kindm.eq.0) kindm=kindmi
c
	if(msglvl.gt.0) write(lst,*)
     &	 '(subr.mtktob) kindm,mesh=',kindm,mesh
c
	return
	end
c
c  read Mitaka+Norikura format file from i/o unit number 'iunit'
c  'inmtkh' reads the header, 'inmtk' reads the data
c  data will be stored in rdata(*,*,k), where k runs from 1 up to MZ
c  the calling program should specify k=kdest(lspec) if a particular
c  data of lspec is to be read
c  when read operation is successful, kloc(lspec)=k and
c  lspec(k)=lspec is set
c
c  read magnetogram header
c
      subroutine inmtkh(iunit)
c
      include 'sys.fc'
      include 'fileio.fc'
      include 'mparam.fc'
      include 'mdata.fc'
      include 'string.fc'
      include 'solar.fc'
      include 'coord.fc'
c
      logical first,alloc
      integer iunit, kdest(NSPEC0)
      character spec*8
      integer*2 ihdr(256)
      character  hdr(4)*128
      save icode,loc,first
c
c  Mitaka & Norikura standard header
c
c  byte loc  name       length       comment
c  decimai  hex
c  ----  header & file informaiton  -----
c     0     000   hdr_id:     [8]   header identifier 'HDR:0100'
c     8     008   one:        [2]   one in this system
c     10    010   offset      [2]   header offset 0 or 512
c     12    012   file_name   [12]  file name
c     24    018   file_form:  [4]   file format 'STDR'
c  ----  data type information  -----
c     28    01c   nset	      [2]   number of frame set in file
c     30    01e   nbyt        [2]   byte of data (1,2,4 or 8)
c     32    020   data_type   [2]   1.signed, 2.unsigned, 3.real or
c                                  byte-split (low-byte image, high-byte image)
c     34    022   nx          [2]   size of data in x-direc.
c     36    024   ny          [2]        "       in y-direc.
c     38    026   nf          [2]   frame number in the frame set
c  -----  observational basic parameters---
c     40    028   observatory [4]   "MTK ", "NSO ",,,
c     44    02c   telescope   [4]   telescope "ft1", "ft4",,,
c     48    030   icam        [2]   camera no.
c     50    032   obs_mode    [2]   1.data, 2.dark, 3.flat
c     52    034   time_sys    [4]   "JST", "UT",,,
c     56    038   date1       [8]   date yy/mm/dd
c     64    040   time1       [12]  start time hh:mm:ss.sss
c     76    04c   date2       [8]   date yy/mm/dd
c     84    054   time2       [12]  end time hh:mm:ss.sss
c     96    060   seqno       [2]   sequence no.
c     98    062   exp         [2]   exposuer time (msec)
c     100   064   integ       [2]   total integration no.
c     102   066   integ1      [2]   integration no. for 1set
c     104   068   prog        [10]  program name & version
c     114   072   obs_id      [10]  observation id
c     124   07c   o_dummy     [6]   dummy
c  -----  explanation of the data  -----
c     130   082   correct     [2]   status of data correction
c     132   084   value       [4]   physical quantity
c     136   086   scale       [14]  scale   (formatted real)
c     150   096   zero        [14]  ( " ) true=scale*data+zero
c     164   0a4   unit        [10]  unit of above value
c     174   0ae   x_ax        [10]  quantity of x-axis
c     184   0b8   x1          [14]  x-value at i=1 (formatted real)
c     198   0c6   dx          [14]  x-increment
c     212   0d4   y_ax        [10]  quantity of y-axis
c     222   0de   y1          [14]  y-value at j=1 (formatted real)
c     236   0ec   dy          [14]  y-increment
c     250   0fa   com         [50]  comment about the data
c  -----  device dependent parameters  ---
c     300   12c   pix_x1      [2]   starting x-pixel on CCD
c     302   12e   pix_y1      [2]   starting y-pixel on CCD
c     304   130   d_parm      [2*23]      parameters for devices
c                                         d_parm[10]=dwl
c                                         d_parm[11]=scatter
c                                         d_parm[12]=100*kdpfact
c      (only on the fourth file for BL)   d_parm[15]=1000*blfact
c  -----  telescope dependent parameters ---
c     350   15e   hangle      [4]    hour angle
c     354   162   delta       [4]   declination
c     358   164   l0          [2]   central brightness of sun
c     360   168   g_on        [2]   guide on/off  0;off, 1;on
c     362   16a   g_balx      [2]   guide balance x
c     364   16c   g_baly      [2]   guide balance y
c     366   16e   g_x         [2]   guide table x-position
c     368   170   g_y:        [2]   guide table y-position
c     370   172   t_parm      [2*10]      telescope parameter
c                                         t_parm[7]=FOV x-shift
c                                         t_parm[8]=FOV y-shift
c     390   186   t_dummy     [10]  dummy
c  -----  parameters for multi frames in frame set  -----
c     400   190   z_ax        [10]  frame step (z) "TIME", "X"
c     410   19a   z1          [14]  z-value of first frame
c     424   1a8   dz          [14]  z-increment
c     438   1b6   fs_parm     [2*6] frame set parameters
c     450   1c2   fs_com      [50]  frame set comment
c     500   1f4   dummy       [8]   dummy
c     508   1fc   eoh         [4]   end of file 'EOH:'
c     512   200
c
c "frame" means an image data whose size is nx*ny*nbyt
c "frame set" may contain one or many frames, but each frame has
c        common nx, ny, data_type etc. Each frame set has a header
c       at the top.
c "file" may contain one or many frame set(s)
c
c one:    1 in the computer system which writes this header.
c       If one=256, byte reversal is necessary.
c offset: If the header is over written on the top of the data,
c       offset=HLENGTH (512),  else offset=0.
c nset:   It has meaning only when it appears in the first header
c       in the file
c data_type:      1. signed int/char,  2. unsigned int/char  3. real
c       If data_type=-1 or -2 and nbyt=2, byte reversal is necessary
c       for the data
c correct: 1. raw data, 2. dark level is subtracted,
c        3. after flat field correction
c
c sample dump list
c    000  48 44 52 3A 30 31 30 30  00 01 00 00 66 31 62 00  HDR:0100....f1b.
c    010  00 00 00 00 00 00 00 00  66 72 61 6D 00 02 00 02  ........fram....
c    020  00 01 01 00 00 F0 00 01  4D 54 4B 00 66 74 31 00  ........MTK.ft1.
c    030  00 00 00 01 55 54 20 00  39 33 2F 30 34 2F 30 36  ....UT .93/04/06
c    040  30 34 3A 33 30 3A 31 35  00 00 00 00 39 33 2F 30  04:30:15....93/0
c    050  34 2F 30 36 30 34 3A 33  30 3A 33 32 00 00 00 00  4/0604:30:32....
c    060  00 0E 00 03 00 80 00 04  66 31 6F 62 73 00 00 00  ........f1obs...
c    070  00 00 66 39 33 30 34 30  36 62 00 00 20 20 20 20  ..f930406b..
c    080  20 20 00 65 49 00 00 00  20 20 20 20 20 20 20 20    .eI...
c    090  20 20 20 20 20 20 30 00  00 00 00 00 00 00 00 00    0.........
c    0A0  00 00 00 00 20 20 20 20  20 20 20 20 20 20 20 20  ....
c    0B0  20 20 20 20 20 20 20 20  20 20 20 20 20 20 20 20
c    0C0  20 20 20 20 20 20 31 2E  33 32 30 30 00 00 00 00    1.3200....
c    0D0  00 00 00 00 20 20 20 20  20 20 20 20 20 20 20 20  ....
c    0E0  20 20 20 20 20 20 20 20  20 20 20 20 31 2E 33 32          1.32
c    0F0  30 30 00 00 00 00 00 00  00 00 66 31 63 61 6C 69  00........f1cali
c    100  62 2C 20 64 77 6C 3D 20  2D 38 30 20 20 41 70 72  b, dwl= -80  Apr
c    110  20 20 36 20 31 34 3A 30  39 3A 31 30 20 31 39 39    6 14:09:10 199
c    120  33 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00  3...............
c    130  00 03 06 40 F9 C0 00 00  00 00 00 00 00 00 00 32  ...@..........2
c    140  00 00 00 00 FF B0 0B 14  00 68 00 00 00 00 00 00  ........h......
c    150  00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00  ................
c    160  00 00 00 00 00 00 03 57  00 01 00 47 FF CD 01 55  .......W...G..U
c    170  FF 04 2D A2 2D 9F 06 22  05 F0 11 94 11 8A 00 00  ..--.."........
c    180  00 00 00 00 00 00 20 20  20 20 20 20 20 20 20 20  ......
c    190  20 20 20 20 20 20 20 20  20 20 20 20 20 20 20 20
c    ...
c    1B0  20 20 20 20 20 20 00 00  00 00 00 00 00 00 00 00    ..........
c    1C0  00 00 20 20 20 20 20 20  20 20 20 20 20 20 20 20  ..
c    1D0  20 20 20 20 20 20 20 20  20 20 20 20 20 20 20 20
c    ...
c    1F0  20 20 20 20 20 20 20 20  20 20 20 20 45 4F 48 3A          EOH:
c
c  nset is not correctly recorded in the data
c  scale is left as blank in the header
c
c  read header record
c  read both as integer*2 and as strings
c  character code of the header is ascii
c
	lrec0=128
	icode=0
	if (mycode.eq.2) icode=1
	loc=1
	first=.true.
c
	do 10 i=1,4
	call ins( iunit, loc, hdr(i), lrec0, iostat )
	if(iostat.ne.0) go to 9
 10	continue
	do 11 j=1,64
	jj1=2*j-1
	jj2=2*j
c  mod() is used in case ichar may give negative values 
 	ihdr(j    )=256*mod(ichar(hdr(1)(jj1:jj1))+256,256)
     &	      +mod(ichar(hdr(1)(jj2:jj2))+256,256)
 	ihdr(j+ 64)=256*mod(ichar(hdr(2)(jj1:jj1))+256,256)
     &	      +mod(ichar(hdr(2)(jj2:jj2))+256,256)
 	ihdr(j+128)=256*mod(ichar(hdr(3)(jj1:jj1))+256,256)
     &	      +mod(ichar(hdr(3)(jj2:jj2))+256,256)
 	ihdr(j+192)=256*mod(ichar(hdr(4)(jj1:jj1))+256,256)
     &	      +mod(ichar(hdr(4)(jj2:jj2))+256,256)
 11	continue
	do 12 i=1,4
	call validc(hdr(i))
	if (icode.ne.0) call cconv(icode, hdr(i))
 12	continue
	loc=1
c  rewind
	if (ihdr(5).eq.256) call swap2(ihdr,256,1,256)
c
	mfile=hdr(1)(13:24)
	nframi=ihdr(15)
	nbytmi=ihdr(16)
	ntype=ihdr(17)
	nx0=ihdr(18)
	ny0=ihdr(19)
	nf1=ihdr(20)
	inst=hdr(1)(41:48)
	tsys=hdr(1)(53:56)
	icam=ihdr(25)
	dateo=hdr(1)(57:64)
	stime=hdr(1)(65:76)
c  ignore hdr(77:84)=date2
	etime=hdr(1)(85:96)
	nseq=ihdr(49)
	write(mfile(4:),'(i3.3)') nseq
c  mfile has a form 'f1a001' etc.
	msexp=ihdr(50)
	integ=ihdr(51)
	integ1=ihdr(52)
c  the filename is set to 'title' in subr.mtktob
cc	title=hdr(1)(115:124)
	icorr=ihdr(66)
cc	spec=hdr(2)(5:8)
cc	read(hdr(2)(9:22),100) factor
 100	format(f14.0)
	read(hdr(2)(71:84),100) dx0
	read(hdr(2)(109:122),100) dy0
	xs0=ihdr(184) + ihdr(186+7)
	ys0=ihdr(185) + ihdr(186+8)
	dwl=ihdr(153+10)
c
	write(lst,*) 'FOV offset in header=',ihdr(186+7),ihdr(186+8)
c
c  nframi is not correctly recorded in the data
c  therefore set nframi depending on whether the instrument is t1 or t4
c  factm's are left as blank in the header, so they are supplied here
c
	if (inst(5:7).eq.'ft1') then
	  nframi=4
	  factm(lspec('I'))=0.0001
	  factm(lspec('Q'))=0.00005
	  factm(lspec('U'))=0.00005
	  factm(lspec('V'))=0.00005
	  factm(lspec('BL'))=0.1
	  factm(lspec('BX'))=0.1
	  factm(lspec('BY'))=-0.1
	  if(dx0.eq.0.0) dx0= 0.66*(512/nx0)
	  if(dy0.eq.0.0) dy0= 0.66*(480/ny0)
	  dy0 = -abs(dy0)
c  the data are stored from the bottom toward the top row
c  change sign of BY
	  if(icorr.gt.100) then
	    kindmi=1002
	  else
	    kindmi=-3033
cc
	write(6,*) 'icorr=',icorr
	kindmi=1002
cc
	  endif
	  wavel=6302.5
c  passband position
	  mpint(1)=dwl
	else if(inst(5:7).eq.'ft4') then
	  nframi=3
	  factm(lspec('I'))=0.0001
	  factm(lspec('V'))=0.00005
	  factm(lspec('W'))=0.00005
	  factm(lspec('BL'))=0.1
	  factm(lspec('VL'))=1.0
	  if(dx0.eq.0.0) dx0= 0.66*(512/nx0)
	  if(dy0.eq.0.0) dy0= 0.66*(480/ny0)
	  dy0 = -abs(dy0)
c  the data are stored from the bottom toward the top row
	  if(icorr.gt.100) then
	    kindmi=1101
	  else
	    kindmi=-4404
	  endif
	  wavel=6337.2
	else if(inst(5:7).eq.'st2') then
	  nframi=3
	  kindmi=1101
	  factm(lspec('I'))=0.0001
	  factm(lspec('V'))=0.00005
	  factm(lspec('W'))=0.00005
	  factm(lspec('BL'))=0.1
	  factm(lspec('VL'))=1.0
	  if(dx0.eq.0.0) dx0= 5.0*(512/nx0)
	  if(dy0.eq.0.0) dy0= 5.0*(480/ny0)
	  dy0 = -abs(dy0)
c  the data are stored from the bottom toward the top row
c  pixel sizes are very approximate
	  if(icorr.gt.100) then
	    kindmi=1101
	  else
	    kindmi=-5505
	  endif
	  wavel=5324.2
	endif
c
	dx0 = dx0*arcsec
	dy0 = dy0*arcsec
	xs0   = xs0*arcsec - 0.5*nx0*dx0
	ys0   = ys0*arcsec - 0.5*ny0*dy0
	ftag=' '
	ydeg=0.0
	mesh=0
	xtod=0.0
	ytod=0.0
	if(tsys.ne.' '.and.dateo.ne.' '.and.stime.ne.' ') then
	  call suncal(tsys,dateo,stime,sdist,rspc,pc,b0c,cl0c,
     &	              doy,irotno)
	  if(inst(5:7).eq.'st2') pc=0.0
c  artificially set pdeg=0 for st2 because solar north is up
	  rsp=arcsec*rspc
	  pdeg=pc
	  b0deg=b0c
	  cl0deg=cl0c
	endif
c
	nx=nx0
	ny=ny0
	xs=xs0
	ys=ys0
	dx=dx0
	dy=dy0
	if(ixs.eq.0) ixs=1
	if(ixe.eq.0) ixe=nx
	if(iys.eq.0) iys=1
	if(iye.eq.0) iye=ny
c
cc	if(icorr.lt.100) then
cc	  write(msg,*)
cc     & '(subr.mtktob) error: the data are not a magnetogram,',
cc     &	'h.correct=',icorr
cc	  errflg=2
cc	  return
cc	endif
c
	if(nx.gt.MMX .or. ny.gt.MMY) then
	  write(msg,*)
     &	'(subr.mtktob) error: nx or ny too large, nx,ny=',nx,ny
	  errflg=2
	  return
	endif
c
	if(nx/nbinx.gt.nxmax .or. ny/nbiny.gt.nymax) then
          nbinx=(nx-1)/nxmax +1
          nbiny=(ny-1)/nymax +1
	  if(msglvl.gt.0) then
	    write(lst,*)
     &	 '(subr.mtktob) warning: nx or ny too large, nx,ny=',nx,ny
            write(lst,*)
     &	 '              binning will be applied, nbinx,y=',nbinx,nbiny
	  endif
	endif
c
	return
c
 9	if(iostat.eq.1) then
	  write(msg,*) '(subr.inmtkh) error: endfile during read'
	else if(iostat.eq.2) then
	  write(msg,*) '(subr.inmtkh) read error'
	endif
	errflg=2
	return
c
c  read magnetogram data
c
	entry inmtk(iunit,kdest)
c
	lrec0=128
	zlevel=0.0
c
	do 20 i=1,4
	call ins( iunit, loc, hdr(i), lrec0, iostat )
	if(iostat.ne.0) go to 9
 20	continue
	do 21 j=1,64
	jj1=2*j-1
	jj2=2*j
c  mod() is used in case ichar may give negative values 
 	ihdr(j    )=256*mod(ichar(hdr(1)(jj1:jj1))+256,256)
     &	      +mod(ichar(hdr(1)(jj2:jj2))+256,256)
 	ihdr(j+ 64)=256*mod(ichar(hdr(2)(jj1:jj1))+256,256)
     &	      +mod(ichar(hdr(2)(jj2:jj2))+256,256)
 	ihdr(j+128)=256*mod(ichar(hdr(3)(jj1:jj1))+256,256)
     &	      +mod(ichar(hdr(3)(jj2:jj2))+256,256)
 	ihdr(j+192)=256*mod(ichar(hdr(4)(jj1:jj1))+256,256)
     &	      +mod(ichar(hdr(4)(jj2:jj2))+256,256)
 21	continue
	do 22 i=1,4
	call validc(hdr(i))
	if (icode.ne.0) call cconv(icode, hdr(i))
 22	continue
	if (ihdr(5).eq.256) call swap2(ihdr,256,1,256)
c
	spec=hdr(2)(5:8)
cc	read(hdr(2)(9:22),100) factor
cc100	format(f14.0)
	call upper(spec)
	if(spec.eq.'S') spec='W'
c
	lspec1=lspec(spec)
	if(lspec1.le.0) then
	  write(lst,*)
     &	'(subr.inmtk ) warning: invalid spec in file, spec=',spec
	  k=0
	else
cc	  factm(lspec1)=factor
c  factor which was read from the file is not used
	  factor=factm(lspec1)
c
	  factm(lspec1)=abs(factm(lspec1))
c  after read, sign of factm is made positive and this will be used
c  when the data are written to a file (2000/09/28)
	  if(msglvl.gt.1)
     &	    write(lst,*) '(subr.inmtk ) spec=',spec,', factor=',factor
c
c  read multiplication factor for BL
	if (spec.eq.'BL') then
	  blfact=0.001*float(ihdr(153+15))
	  if(blfact.eq.0.0) blfact=1.0
	  mpreal(1)=blfact
cx	  factor=factor/blfact
cx  restore original data scale
c   03/01/11 keep blfact correction except for mtkbk.f
c   not compiled yet
	endif
c
	  k=kdest(lspec1)
	  alloc=.false.
c  alloc=false : therefore the following setups will be done later
c       nframe=nframe+1
c       kloc(lspec1)=k
c       kmem(nframe)=k
c       lspecs(k)=lspec1
c  this is in order to enable data overwriting
c
	  if(k.eq.0) k=newk(lspec1, alloc)
	  if(k.le.0) then
	    if(msglvl.gt.0)
     &    write(lst,*) '(subr.inmtk ) data were skipped, spec=',spec
	  endif
	endif
c
	if(msglvl.gt.1)
     &	  write(lst,*) '(subr.mtktob) lspec=',lspec1,',  kdest=',k
c
	if (k.gt.MZ) then
	  write(msg,*) '(subr.inmtk ) error: k exceeded limit'
	  errflg=2
	  return
	endif
c
	call in2( iunit, loc, idata2, lrec0, nx0, ny0, MMX, iostat )
	if(k.gt.0) then
	  if(iostat.ne.0) go to 19
	  if (nbytmi*myint .lt.0) call swap2(idata2,nx0,ny0,MMX)
c
c  apply binning if required, and store idata2 into rdata
c
	  zlevel=0.0
	  call bin2(idata2,MMX,ixs,ixe,iys,iye,
     &	              nbinx,nbiny,factor,zlevel,
     &	              rdata(1,1,k),MX,
     &	              xs0,ys0,dx0,dy0,
     &	              nx, ny, xs, ys, dx, dy)
c
	  if(msglvl.gt.2) write(lst,*) 'data first,last=',
     &	       idata2(1,1),idata2(nx0,ny0),
     &	       rdata(1,1,k),rdata(nx,ny,k)
	endif
c
	if(k.gt.0) then
	  if(lspecs(k).le.0) then
	    nframe=nframe+1
	    kmem(nframe)=k
	    lspecs(k)=lspec1
	    kloc(lspec1)=k
	  else
c  old data are overwritten
	    lspecs(k)=lspec1
	    call kpack
c  kpack recalculates nframe, kmem and kloc
	  endif
	endif
c
	if(msglvl.gt.0 .and. first ) then
	  write(lst,*) '(subr.mtktob)'
	  if(nbinx.ne.1.or.nbiny.ne.1)
     &  write(lst,*)
     &    '  binning: nbinx,nbiny=',nbinx,nbiny
	  if(ixs.ne.1 .or. iys.ne.1 .or. ixe.ne.nx0 .or. iye.ne.ny0)
     &  write(lst,*)
     &    '  sub-area selection: ixs,ixe,iys,iye=',
     &    ixs,ixe,iys,iye
	  write(lst,*) '  nx,ny=',nx,ny
	  write(lst,*) '  xs,ys=',xs,ys
	  write(lst,*) '  dx,dy=',dx,dy
	endif
	first=.false.
	return
c
 19   if(iostat.eq.1) then
        write(msg,*) '(subr.inmtk ) error: endfile during read'
      else if(iostat.eq.2) then
        write(msg,*) '(subr.inmtk ) read error'
      endif
      errflg=2
      return
      end
c
c  replace non-printable ascii characters with blanks
c
	subroutine validc( str)
	character*(*) str
c
	n=len(str)
	do 10 i=1,n
	ic=ichar(str(i:i))
	if(ic.lt.32 .or. ic.gt.126) str(i:i)=' '
 10	continue
	return
	end
c
c  find a character in a string
c  mode= 1: search from the top of the string
c  mode=-1: search from the end of the string
c
	function locstr(str,chr,mode)
c
	character str*(*), chr*1
	l=len(str)
	if(mode.ge.0) then
	  do 10 i=1,l
	  if(str(i:i).eq.chr) go to 20
 10	  continue
	  i=0
 20	  locstr=i
	else
	  do 30 i=l,1,-1
	  if(str(i:i).eq.chr) go to 40
 30	  continue
	  i=0
 40	  locstr=i
	endif
	return
	end

