c  program oaobook.f
c  okayama observatory vector magnetogram
c
c  this program produces magnetograms in data-book format
c
	include 'sys.fc'
	include 'fileio.fc'
	include 'string.fc'
	include 'coord.fc'
	include 'lparam.fc'
	include 'mparam.fc'
	include 'mdata.fc'
	include 'fcntl.fc'
	include 'procs.fc'
	include 'auxparam.fc'
	include 'graph.fc'
c
	character cfile*80,chr*1
	character dummy*1
	dimension multi(20)
	logical first
	data multi/20*0/
	data first/.true./
c
	keyin  = 5
	stdin  = 10
	lst    = 6
	msg    = 0
	errflg = 0
	msglvl = 1
c
c  character code should either be ASCII or EBCDIC
c  binary integer should either be big endian or
c  full-byte-swap little endian
        include 'sysconf.cfg'
c
        if(mycode.eq.0 .or. myint.eq.0) stop
        if(errflg.gt.0) stop
        if(nbytem.ne.0) then
          nbytem=sign(nbytem,myint)
        else
          nbytem=myint
        endif
        if(nbytel.ne.0) then
          nbytel=sign(nbytel,myint)
        else
          nbytel=myint
        endif
c
c  initialize commons /fcntl/ and /procs/
        call initfc
c
c  open list file
	lst = lstini()
c
c  print version number
	call vsn
c
c  common variables should be static
        call static(ist)
        if(ist.gt.1) stop
c
      if(myargc().eq.1) then
        call myargv(1,cfile)
      else
	  cfile=' '
	  write(msg,*,err=2)
     &	'enter control file name (default=bcontrol.dat)'
	  go to 3
 2	  msg=lst
	  write(msg,*) 'enter control file name (default=bcontrol.dat)'
 3	  read(keyin,'(a)',end=4) cfile
 4	  if(cfile.eq.' ') cfile='bcontrol.dat'
	endif
c
        call openfl(cntrol,lname(cntrol),stdin,cfile,text,rdo,0)
	read(stdin,'(a1)') chr
	if(chr.eq.'0') then
	  msglvl=0
	else if(chr.eq.'1') then
	  msglvl=1
	else if(chr.eq.'2') then
	  msglvl=2
	else if(chr.eq.'3') then
	  msglvl=3
	endif
c
c  initialize common variables and read auxdata
	write(lst,*) '(proc.oaobk ) start'
	multi1=0
	call assign( init,multi1 )
	if(errflg.gt.0) stop
	if(multi1.ne.0) stop
	call initc( funit(auxdat) )
	call clsfl(auxdat)
c
 1000	continue
	errflg=0
c
c  proc.obs2p
c  conversion from observed raw data taken with the vector magnetograph of
c  the Okayama Astrophysical Observatory to polarization degrees.
c  'rawdat' file is opened in 'obstop', not in 'assign', 
c  because the data format depends on the instruments.
c
c  subr.assign reads msglvl
	if(errflg.gt.1) go to 1900
	call assign( obs2p,multi(1) )
	if(errflg.gt.1) go to 1001
	if(multi(1).lt.0) stop
c
	if(errflg.eq.0) then
	  kindm=0
	  kindmo=0
	  nbinx=1
	  nbiny=1
	  ixs=0
	  ixe=0
	  iys=0
	  iye=0
        read(stdin,*) kindm, kindmo
	  read(stdin,*) nbinx,nbiny,ixs,ixe,iys,iye
c  this read is dummy; binning is not applied in obstop for oao data
	  call obstop( funit(rawdat), fname(rawdat) )
	  if(errflg.gt.1) go to 1001
cx	  call clsfl (rawdat)
c
	  dxnew=dx
	  dynew=dy
	  read(stdin,*) dxnew,dynew
	  call rebin(dxnew,dynew)
	  if(errflg.gt.1) go to 1001
c
        call writem( funit(poldeg) )
        if(errflg.gt.1) go to 1001
cx	  call clsfl (poldeg)
	endif
c
 1001	continue
	call clsfl (rawdat)
	call clsfl (poldeg)
	if(errflg.gt.1) go to 1900
c
c  proc.pol2b
c  conversion from polarization degrees (poldeg) to 
c  a raw magnetogram (rmgram)
c  for the vector magnetograph at Okayama
c
c  subr.assign reads msglvl,mkimap,mkdmap
	if(errflg.gt.1) go to 1900
	call assign( pol2b,multi(2) )
	if(errflg.gt.1) go to 1002
	if(multi(2).lt.0) stop
c
	if(errflg.eq.0) then
	  kindm=0
	  kindmo=0
	  read(stdin,*) kindm, kindmo
	  call loadp(poldeg)
	  if(errflg.gt.1) go to 1002
c
	  call poltob
	  if(errflg.gt.1) go to 1002
	  call writem( funit(rmgram) )
	  if(errflg.gt.1) go to 1002
cx	  call clsfl (rmgram)
	endif
c
 1002	continue
	call clsfl (poldeg)
	call clsfl (rmgram)
	if(errflg.gt.1) go to 1900
c
c  proc.databk
c  make plot and write log
c
c	subr.assign reads msglvl
	if(errflg.gt.1) go to 1900
	call assign( databk,multi(3) )
 	if(errflg.gt.1) go to 1003
	if(multi(3).lt.0) stop
c
 	if(errflg.eq.0) then
	  mesho=-2
	  kindm=0
	  call load(mgram)
	  if(errflg.gt.1) go to 1003
	  wddump=fcmnt
c  fcmnt is the filename read by subr.assign in multi-file processing 
c
	  if(first) then
	    lpdev='/dev/lp0'
	    lpname='lp0'
	    ipmode=-5
	    ipage=1
	    scale=1.0
	    read(stdin,*) lpdev,lpname
	    read(stdin,*) ipmode, scale
	    read(stdin,*) ipage
	    xsplot=3.0
	    ysplot=0.0
	    call plots(xsplot,ysplot, ipmode )
	    call factor(scale)
c  read logfile till the end
 10	    read(funit(logfl),'(a)',end=20) dummy
	    go to 10
 20	    continue
	    first=.false.
	  endif
c
    	  call oaobk(ipage,funit(logfl))
	  if(errflg.gt.1) go to 1003
c  ipage is incremented in subr.oaobk
c
 	endif
c
 1003	continue
	if(rds2) call clsfl (mgram)
	if(errflg.gt.1) go to 1900
c
 1900	continue
      do 1100 i=1,20
      if(multi(i).ne.0) go to 1000
 1100 continue
c  first=false if proc.databook has been activated
	if(.not.first) then
	  call clsfl (logfl)
	  call plote()
	endif
c
	write(lst,*) '(proc.oaobook ) end'
	stop
	end
c
	subroutine oaobk(ipage,ulog)
c
      include 'sys.fc'
      include 'mdata.fc'
      include 'coord.fc'
      include 'solar.fc'
      include 'mparam.fc'
      include 'string.fc'
      include 'fileio.fc'
      include 'graph.fc'
c
	character pageno*5
      character tstr1*7,tstr2*7
	real loglvl
	integer ulog
      dimension idot(5,5,3)
      dimension xx(5),yy(5)
      dimension blp(20),blm(20)
      data nb/6/,blp/10.,20.,50.,100.,200.,500.,14*0./,
     *           blm/-10.,-20.,-50.,-100.,-200.,-500.,14*0./
      data idot/12*0,1,12*0,
     *          5*0,0,1,0,1,0,5*0,0,1,0,1,0,5*0,
     *          1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1/

c
c  plot magnetograms
c  size of figure = span in centimeter
c
      idx=dx/0.07 +0.5
      idy=dy/0.07 +0.5
      ixs=xs/0.07 +0.5
      iys=ys/0.07 +0.5
      irsp=rsp/0.07 +0.5
c
	if(ny.lt.nx/2.or.idx.eq.0.or.idy.eq.0
     &     .or.(int(wavel).ne.5250.and.int(wavelen).ne.0)) then
            write(lst,*) '(subr.oaobk ) ',mfile,' skip'
	  return
	endif
c
      write(lst,*) '(subr.oaobk ) processing ',mfile
	kx = kxloc()
	ky = kyloc()
	kz = kzloc()
	ki = kiloc()
c
      span=14.0
      if(dx*float(nx-1).lt.dy*float(ny-1)) go to 20
        dd=span/float(nx-1)
        fact=dd/dx
        go to 21
 20   continue
        dd=span/float(ny-1)
        fact=dd/dy
 21   continue
	ddx=dx*fact
	ddy=dy*fact
      xx(1)=0
      yy(1)=0
      xx(2)=ddx*float(nx-1)
      yy(2)=0
      xx(3)=xx(2)
      yy(3)=ddy*float(ny-1)
      xx(4)=0
      yy(4)=yy(3)
      xx(5)=0
      yy(5)=0
c
      call newpen(white)
      call dataid(ixs,iys,idx,idy,nx,ny,pdeg,b0deg,irsp,
     &       dateo,stime,etime,mfile,
     &	 it1,it2,tstr1,tstr2)
c
      call plot(xx(1),yy(1),3)
      do 30 k=2,5
 30   call plot(xx(k),yy(k),2)
c
c  contours for longitudinal field
c
      dds=0.3*dd
      call newpen(red)
	mkdot=1
	nodot=0
c
      call cntr(nx,ny,0.,0.,ddx,ddy,MX,rdata(1,1,kz),
     &	     nb,blp,nodot,dds,idata4)
      call newpen(blue)
      call cntr(nx,ny,0.,0.,ddx,ddy,MX,rdata(1,1,kz),
     &	     nb,blm,mkdot,dds,idata4)
c
c  transverse fields
c
c     btmax=400.0
c     btmin=40.0
	btmax=1000.0
	btmin=100.0
c
      ww=0.02
      call newpen(green+10)
c  thick green line
c
      do 40 j=2,ny-1
      do 40 i=2,nx-1
      bt=sqrt(rdata(i,j,kx)**2+rdata(i,j,ky)**2)
      btlvl= loglvl(bt,btmin,btmax)
      ds=btlvl*dd
      if(ds.lt.ww .or. bt.eq.0.0 .or.
     &	rdata(i,j,ki)*1000.0.le.10.0) go to 40
      dxx=ds*rdata(i,j,kx)/bt
      dyy=ds*rdata(i,j,ky)/bt
      xx1=dd*float(i-1)-0.5*dxx
      xx2=dd*float(i-1)+0.5*dxx
      yy1=dd*float(j-1)-0.5*dyy
      yy2=dd*float(j-1)+0.5*dyy
      call plot(xx1,yy1,3)
      call plot(xx2,yy2,2)
c     call plot(xx1+ww,yy1,3)
c     call plot(xx2+ww,yy2,2)
c     call plot(xx1,yy1+ww,3)
c     call plot(xx2,yy2+ww,2)
 40   continue
c
c  intensity map
c
      call newpen(yellow)
      ndx=ddx/0.25
      if(ndx.lt.2) ndx=2
	ndy=ddy/0.25
	if(ndy.lt.2) ndy=2
      dddx=ddx/float(ndx)
	dddy=ddy/float(ndy)
      intstp=100
	int0=1000.0*frmean(rdata(1,1,ki),nx,ny,MX)
      int0=int0-intstp/2
      do 50 j=2,ny
      do 50 i=2,nx
      do 50 jj=1,ndy
	do 50 ii=1,ndx
      sx=(float(ii)-0.5)/float(ndx)
      sy=(float(jj)-0.5)/float(ndy)
      a00=rdata(i-1,j-1,ki)
      a10=rdata(i  ,j-1,ki)
      a01=rdata(i-1,j  ,ki)
      a11=rdata(i  ,j  ,ki)
      iint=1000.0*(
     &	a00+(a10-a00)*sx+(a01-a00)*sy+(a00+a11-a10-a01)*sx*sy)
      xx1=ddx*float(i-2)+dddx*float(ii-1)
      yy1=ddy*float(j-2)+dddy*float(jj-1)
      if(iint.gt.int0) go to 50
      if(iint.le.10) go to 50
      intlvl=min(3,(int0-iint)/intstp+1)
      do 55 iii=1,5
      do 55 jjj=1,5
      if(idot(iii,jjj,intlvl).eq.0) go to 55
      call plot(xx1+dddx*float(iii-1)/5.0,yy1+dddy*float(jjj-1)/5.0,3)
      call plot(xx1+dddx*float(iii-1)/5.0,yy1+dddy*float(jjj-1)/5.0,2)
 55   continue
 50   continue
c
	call newpen(white)
	pageno=' '
	if(ipage.gt.0) then
	  if(ipage.lt.10) then
		write(pageno,'(1h-,i1,1h-)') ipage
	  else if(ipage.lt.100) then
		write(pageno,'(1h-,i2,1h-)') ipage
	  else if(ipage.lt.1000) then
		write(pageno,'(1h-,i3,1h-)') ipage
	  else
		write(6,*) 'error. ipage too large.'
	      errflg=2
	      return
	  endif
	  call symbol(span/2.0-0.5,-1.0,0.14,pageno, 0.0, 5)
	  endif
c  print out
      call plot(0.0, 0.0, 999)
c  erase
      call plot(0.0, 0.0, 1000)
c
c  create log 
c  'title' created by setmg or old databook program did not retain
c  original comment (bug to be fixed)
c
	write(ulog,800) ipage,mfile,dateo,stime,etime,
     &	it1,tstr1,it2,tstr2,
     &	nx,ny,ixs,iys,idx,idy,mpstr(1)(1:2),title(1:32)
 800  format(i4,1x,a8,1x,a8,3x,a8,' -',a8,
     &       2x,i2.2,a7,'-',i2.2,a7,
     &       2x,2i4,1x,2i5,1x,2i3,
     &       3x,a2,3x,a32)
	if(ipage.gt.0) ipage=ipage+1
c
	return
      end
c
c     data information
c
      subroutine dataid(ixs,iys,idx,idy,nx,ny,pdeg,b0deg,irsp,
     &	            dateo,stime,etime,mfile,
     &	            it1,it2,tstr1,tstr2)
c
	include 'graph.fc'
c
      character dateo*(*),stime*(*),etime*(*),mfile*(*),line*35
      character tstr1*7,tstr2*7,s1*38,s2*38
      logical l1,l2
      dimension xp(5),yp(5),a(3,3)
c
c     data description
c
	call newpen(white)
	top=21.0
	size=0.14
c
	s1='nj`x`l`@nardqu`snqx'
	s2='@udbsnq@l`fmdsnfq`l'
      call symbol(0.5, top, 0.14, s1//s2, 0.0, 38+38)
      write(line,500) mfile
 500  format('FILE-ID :     ',a6)
      call symbol(0.0, top-2.0, size, line, 0.0, 20)
	write(line,510) dateo
 510  format('DATE :      ',a8)
      call symbol(0.0, top-2.5, size, line, 0.0, 20)
      write(line,520) stime,etime
 520  format('TIME(JST) : ',a8,' -',a8)
      call symbol(0.0, top-3.0 ,size, line, 0.0, 30)
c
      read(stime,592) it1
 592  format(i2,6x)
      read(etime,592) it2
      ist1=0
      it1=it1-9
      if(it1.ge.0) then
	  write(tstr1,591) stime(3:8),' '
	else
        it1=it1+24
	  write(tstr1,591) stime(3:8),'*'
 591	  format(a6,a1)
	  endif
      ist1=1
      ist2=0
      it2=it2-9
      if(it2.ge.0) then   
	  write(tstr2,591) etime(3:8),' '
	else
        it2=it2+24
	  write(tstr2,591) etime(3:8),'*'
	  endif
      write(line,590) it1,tstr1,it2,tstr2
 590  format('    (UT)  : ',i2.2,a7,'-',i2.2,a7)
      call symbol(0.0, top-3.5, size, line, 0.0, 31)
      write(line,530) ixs,iys
 530  format('XS=',i7,5x,'YS=',i7,5x)
      call symbol(0.0, top-4.0, size, line, 0.0, 30)
      write(line,540) idx,idy
 540  format('DX=',i7,5x,'DY=',i7,5x)
      call symbol(0.0, top-4.5 ,size, line, 0.0, 30)
      write(line,550) nx,ny
 550  format('NX=',i7,5x,'NY=',i7,5x)
      call symbol(0.0, top-5.0, size, line, 0.0, 30)
      write(line,560) pdeg
 560  format('P =',f7.1)
      call symbol(0.0, top-5.5, size, line, 0.0, 10)
      write(line,570) b0deg
 570  format('B0=',f7.1)
      call symbol(0.0, top-6.0, size, line, 0.0, 10)
      write(line,580) irsp
 580  format('R =',i7)
      call symbol(0.0, top-6.5, size, line, 0.0, 10)
c
      rsp1=2.25
      xsp=11.0
      ysp=top-4.0
	call newpen(yellow)
c
c     markers
c
      call plot(xsp-0.1,ysp,3)
      call plot(xsp+0.1,ysp,2)
      call plot(xsp,ysp-0.1,3)
      call plot(xsp,ysp+0.1,2)
c
      call symbol(xsp-0.1,ysp+rsp1+0.5,size,'N',0.,1)
      call plot(xsp,ysp+rsp1+0.4,3)
      call plot(xsp,ysp+rsp1+0.1,2)
c
      call symbol(xsp-0.1,ysp-rsp1-0.78,size,'S',0.,1)
      call plot(xsp,ysp-rsp1-0.4,3)
      call plot(xsp,ysp-rsp1-0.1,2)
c
      call symbol(xsp-rsp1-0.70,ysp-0.14,size,'E',0.,1)
      call plot(xsp-rsp1-0.4,ysp,3)
      call plot(xsp-rsp1-0.1,ysp,2)
c
      call symbol(xsp+rsp1+0.5,ysp-0.14,size,'W',0.,1)
      call plot(xsp+rsp1+0.4,ysp,3)
      call plot(xsp+rsp1+0.1,ysp,2)
c
c     sphere
c
      prad1=3.141592/180.0*pdeg
      b0rad1=3.141592/180.0*b0deg
      a(1,1)=cos(prad1)
      a(1,2)=-sin(prad1)*cos(b0rad1)
      a(1,3)=sin(prad1)*sin(b0rad1)
      a(2,1)=sin(prad1)
      a(2,2)=cos(prad1)*cos(b0rad1)
      a(2,3)=-cos(prad1)*sin(b0rad1)
      a(3,1)=0.0
      a(3,2)=sin(b0rad1)
      a(3,3)=cos(b0rad1)
c
      nc=500
      nc=50
      do 10 i=1,nc
      xx=rsp1*cos(6.283184*float(i-1)/float(nc-1))+xsp
      yy=rsp1*sin(6.283184*float(i-1)/float(nc-1))+ysp
      if(i.eq.1) call plot(xx,yy,3)
      call plot(xx,yy,2)
 10   continue
c
      do 2000 i=1,17
      th=10.0/180.0*3.141592*float(i)
      np=float(nc)*sin(th)
      if(np.lt.6) np=6
      l1=.false.
      do 2100 j=1,np
      ph=2.0*3.141592*float(j-1)/float(np-1)
      ux1=rsp1*sin(th)*cos(ph)
      ux2=rsp1*cos(th)
      ux3=rsp1*sin(th)*sin(ph)
      xx=a(1,1)*ux1+a(1,2)*ux2+a(1,3)*ux3+xsp
      yy=a(2,1)*ux1+a(2,2)*ux2+a(2,3)*ux3+ysp
      z= a(3,1)*ux1+a(3,2)*ux2+a(3,3)*ux3
      l2=z.ge.0.0
      if(.not.l2) go to 2110
      if(l1) call plot(xx,yy,2)
      if(.not.l1) call plot(xx,yy,3)
 2110 l1=l2
 2100 continue
 2000 continue
c
      do 3000 i=1,36
      ph=10.0/180.0*3.141592*float(i)
      dth=1.0
      if(mod(i,9).eq.0) dth=0.0
      wth=18.0-2.0*dth
      nt=nc/2
      l1=.false.
      do 3100 j=1,nt
      th=3.141592*(wth*float(j-1)/float(nt-1)+dth)/18.0
      ux1=rsp1*sin(th)*cos(ph)
      ux2=rsp1*cos(th)
      ux3=rsp1*sin(th)*sin(ph)
      xx=a(1,1)*ux1+a(1,2)*ux2+a(1,3)*ux3+xsp
      yy=a(2,1)*ux1+a(2,2)*ux2+a(2,3)*ux3+ysp
      z= a(3,1)*ux1+a(3,2)*ux2+a(3,3)*ux3
      l2=z.ge.0.0
      if(.not.l2) go to 3110
      if(l1) then
        call plot(xx,yy,2)
      else
        call plot(xx,yy,3)
        endif
 3110 l1=l2
 3100 continue
 3000 continue
c
c     observed area
c
	call newpen(white)
      f=rsp1/float(irsp)
      xp(1)=f*float(ixs)+xsp
      yp(1)=f*float(iys)+ysp
      xp(2)=f*float(ixs+idx*(nx-1))+xsp
      yp(2)=f*float(iys)+ysp
      xp(3)=f*float(ixs+idx*(nx-1))+xsp
      yp(3)=f*float(iys+idy*(ny-1))+ysp
      xp(4)=f*float(ixs)+xsp
      yp(4)=f*float(iys+idy*(ny-1))+ysp
      xp(5)=xp(1)
      yp(5)=yp(1)
      call plot(xp(1),yp(1),3)
      do 3200 i=2,5
 3200 call plot(xp(i),yp(i),2)
      return
      end
