c  Okayama Astrophysical Observatory Vector Magnetograph
c  first-stage data reduction program
c
c  oaotop.f
c
c  based on setmg/zs on MELCOM 70 at OAO : 86/07/20
c
c  use direct access formatted i/o
c       input  - recl=42
c       output - recl=80
c
	subroutine obstop( iunit, ifile )
c
	include 'sys.fc'
	include 'coord.fc'
	include 'fileio.fc'
	include 'fcntl.fc'
	include 'string.fc'
	include 'mdata.fc'
	include 'mparam.fc'
	include 'lparam.fc'
	include 'auxparam.fc'
	include 'solar.fc'
c
	logical unique
	integer iunit,fmode1,fstat1,kdest(NSPEC0)
	character ifile*(*)
	rad(deg)=3.141592/180.0*deg
c
	fmode1=export
	fstat1=rdo
	lrec0=42
	nrec1=21
c
	call openfl(rawdat,'RAWDATA(OAO)',
     &	            iunit,ifile,fmode1,fstat1,lrec0 )
c
c  subr.ilist determines where the data should be stored
c  data specified by lspec will be stored in rdata(*,*,k),
c  where k=kdest(lspec). k runs from 1 to kmax.
c
	unique=.false.
	kmax=MZ
	kindm = -2111
c  read all the data because I,Q,U,V,VL are not separated but
c  are contained in each record
c
	call ilist(kindm,kdest,kmax,NSPEC0,unique)
	if (kmax.gt.MZ) then
	  write(lst,*)
     &          '(subr.oaotop) insufficient space in rdata, kmax=',kmax
	  errflg=2
	  return
	endif
c
	do 10 n=1,NSPEC0
 10	kloc(n)=0
	do 20 k=1,MZ
	kmem(k)=0
	lspecs(k)=0
 20	continue
	nframe=0
c
	call setmg(iunit,kdest)
c
	if(msglvl.gt.1) then
	  write(lst,*) '(subr.oaotop) kindm=',kindm
	  write(lst,*) '  kindmi,mesh,nframei=',kindmi,mesh,nframi
	  call printm(nbytmi,nx,ny,xs,ys,dx,dy,ydeg,
     &        rsp,pdeg,b0deg,cl0deg,xtod,ytod,
     &        tsys,inst,wavel,mfile,dateo,stime,etime,ftag,title)
	endif
c
	xxsp(1)=0.0
	xxsp(2)=0.0
	xxsp(3)=-rsp
	prad=rad(pdeg)
	b0rad=rad(b0deg)
c  set xdu and ydu from ypos
	ypos=rad(ydeg)
	call setdu
c
	xtod=xs+0.5*dx*float(nx-1)
	ytod=ys+0.5*dy*float(ny-1)
	xxtod(1)=xtod
	xxtod(2)=ytod
	call projsr(xxtod,rsp)
	call dtoc(xxtod,xxto)
	call settu
	call setas
	call setar
c
c  set integration limits
c
	x1lim0=xs
	x2lim0=xs+dx*float(nx-1)
	z1lim0=0.0
	y1lim0=ys
	y2lim0=ys+dy*float(ny-1)
	z2lim0=0.5*sqrt((x2lim-x1lim)**2+(y2lim-y1lim)**2)
	ds0=sqrt(abs(dx*dy))*0.25
c
	nx0=nx
	ny0=ny
	xs0=xs
	ys0=ys
	dx0=dx
	dy0=dy
	if(ixs.eq.0) ixs=1
	if(ixe.eq.0) ixe=nx
	if(iys.eq.0) iys=1
	if(iye.eq.0) iye=ny
c
	errflg=0
	do 30 lspec1=1,NSPEC0
	k=kdest(lspec1)
	if(k.gt.0) then
	  if(lspecs(k).eq.0) then
	    write(lst,*)
     &          '(subr.oaotop) requested data were not found, spec=',
     &           specst(lspec1)
	    errflg=2
	  endif
	endif
 30	continue
c
	if(kindm.eq.0) kindm=kindmi
c
	if(msglvl.gt.0) write(lst,*)
     &       '(subr.oaotop) kindm,mesh=',kindm,mesh
c
	return
	end
c
c  based on setmg/zs on MELCOM 70 at OAO : 86/07/20
c  (setmg is a modified version of 'idisp' program written by M.Makita)
c  drifts in wq,wu and vel are corrected : 86/11/24
c  spc check is included                 : 89/03/09
c  sign of q in 1+q/2i                   : 89/04/25
c
	subroutine setmg(iunit,kdest)
c
	include 'sys.fc'
	include 'coord.fc'
	include 'fileio.fc'
	include 'fcntl.fc'
	include 'string.fc'
	include 'mdata.fc'
	include 'mparam.fc'
	include 'solar.fc'
c
	character buf*42
	integer*2 id(21)
	dimension xa(MX,MY),xi(MX,MY),xq(MX,MY),xu(MX,MY),
     &	          xv(MX,MY),xrv(MX,MY),
     &	          wi(MX,MY),wq(MX,MY),wu(MX,MY),wv(MX,MY),
     &	          r2(MX,MY),
     &	          nt(3),iut(6),
     &	          p(11),idat(3),itb(3),ite(3)
      integer iunit, kdest(NSPEC0)
c
c  declared in mdata.fc: qdata(MX,MY,MD), MD>=16
c
	equivalence (qdata(1,1,1),xa(1,1)),  (qdata(1,1,2),xi(1,1)),
     &            (qdata(1,1,3),xq(1,1)),  (qdata(1,1,4),xu(1,1)),
     &            (qdata(1,1,5),xv(1,1))
	equivalence (qdata(1,1,6),xrv(1,1)), (qdata(1,1,7),wi(1,1)),
     &            (qdata(1,1,8),wq(1,1)),  (qdata(1,1,9),wu(1,1)),
     &            (qdata(1,1,10),wv(1,1)),  (qdata(1,1,11),r2(1,1))
c
	call allocm(1,11,'setmg')
	if(errflg.gt.1) return
c
	wpv=6.1
	wpl=3.4
	xpv=12.4
	xpl=6.1
	xvel=5.5
c
c  character code of the header is ascii
c  integer is 2 byte long, big endian
c
	lrec0=42
	nrec1=21
	icode=0
	if (mycode.eq.2) icode=1
	nbytmi=2
c
c  read first record
c
c  id(1)=year
c  id(2)=month
c  id(3)=day
c  id(4)=observer's initial (2 char)
c  id(5)=slot size (mm)
c  id(6)=slit width (micron)
c  id(7)=wavelength (angstrom)
c  id(8)=mask aperture (micron)
c  id(9)=prism slit width (micron)
c  id(10)=high voltage (volt)
c  id(11)=disk center x-position (roughly arcsec)
c  id(12)=disk center y-position (roughly arcsec)
c  id(13)=max. gain of qu amplifiers
c  id(14)=max. gain of vr amp.
c  id(15)=max. gain of q'u' amp.
c  id(16)=max. gain of v'r' amp.
c  id(17)=number of integration
c  id(18)=x-step
c  id(19)=y-step
c  id(20)=nx
c  id(21)=ny
c
	loc=1
	call in2( iunit, loc, id, lrec0, nrec1, 1, nrec1, iostat )
	if(iostat.ne.0) go to 9
	loc=1
	call ins( iunit, loc, buf, lrec0, iostat )
	if(iostat.ne.0) go to 9
	if (myint*nbytmi.lt.0) call swap2(id,nrec1,1,nrec1)
	if (icode.ne.0) call cconv(icode, buf)
c
	if(msglvl.gt.1) write(lst,110) (id(i),i=1,3)
 110	format(2x,'date=',3i2)
	idcx=id(11)
	idcy=id(12)
	do 10 i=1,3
 10	idat(i)=id(i)
	do 20 i=1,3
	iut(i)=idat(i)
 20	iut(i+3)=0
	call koyomi(iut,r,sd,b0deg,cl0deg,pdeg,tmt)
	kr=ifix(sd/1.01+.5)
c  sd is true solar radius, kr corresponds to encoder readout
	nt(1)=ifix(tmt)
	tn=(tmt-float(nt(1)))*60.
	nt(2)=ifix(tn)
	nt(1)=nt(1)+9
	nt(3)=ifix((tn-float(nt(2)))*60.+.5)
	kdx=id(18)
	kdy=id(19)
	nx=id(20)
c  added 95/01/08
	write(mpstr(1),'(a2,78x)') id(4)
	wavel=id(7)
	mpint(1)=id(5)
	mpint(2)=id(6)
	mpint(3)=id(8)
	mpint(4)=id(9)
	mpint(5)=id(10)
	mpint(6)=id(17)
c  mpint(1)=slot size
c  mpint(2)=slit width
c  mpint(3)=mask aperture
c  mpint(4)=prism slit width
c  mpint(5)=high voltage
c  mpint(6)=number of integration
c
c  read second record
c
c  id(1)=last record number
c  id(2)=program version numner
c  id(3-21)=comment
c
      loc=2
      call in2( iunit, loc, id, lrec0, nrec1, 1, nrec1, iostat )
      if(iostat.ne.0) go to 9
      loc=2
      call ins( iunit, loc, buf, lrec0, iostat )
      if(iostat.ne.0) go to 9
      if (myint*nbytmi.lt.0) call swap2(id,nrec1,1,nrec1)
      if (icode.ne.0) call cconv(icode, buf)
c
	mpint(7)=id(2)
c  mpint(7)=version number
c
	lrn=id(1)
	if(msglvl.gt.1) write(lst,120) lrn,(id(i),i=3,21)
 120	format(2x,'lrn=',i5,2x,'comment ',19a2)
c
	write(mfile,210) (id(i),i=3,5)
 210	format(3a2,2x)
c *** the form of title is chaned as follows *** 93/12/26
	write(title,220) (id(i),i=3,21)
 220	format(19a2)
c
	nhdr=2
	ny=(lrn-nhdr)/nx
c
c  read (last+1)-th record (comment)
c
	loc=lrn+1
	call ins( iunit, loc, buf, lrec0, iostat )
	if(iostat.ne.0) go to 9
	if (icode.ne.0) call cconv(icode, buf)
	title(39:80)=buf
c
	if(msglvl.gt.1) write(lst,130) buf
 130	format(2x,a)
	if(msglvl.gt.1) write(lst,140) nx,kdx,ny,kdy
 140	format(2x,'nx=',i3,'(step=',i2,')  ','ny=',
     &	         i3,'(step=',i2,')')
c
	last=nx*ny+nhdr
	iagc=0
c  for checking spc phase and max.intensity
	sumxv2=0.0
	sumxr2=0.0
	sumxvr=0.0
c
	sigma=0.0
	nmbr=0
	q1=0.0
	q2=0.0
	q3=0.0
	q4=0.0
c
	loc=3
	do 1000 k=nhdr+1,last
	i=mod(k-nhdr-1,nx)+1
	j=(k-nhdr-1)/nx+1
	if(mod(j,2).eq.0) i=nx-i+1
c
c  read data
c
c  id(1)=hour
c  id(2)=min
c  id(3)=sec
c  id(4)=x-position (roughly arcsec)
c  id(5)=y-position (roughly arcsec)
c  id(6)=gain of qu amplifiers
c  id(7)=gain of vr amp.
c  id(8)=gain of q'u' amp
c  id(9)=gain of v'r' amp
c  id(10)=i signal
c       :average of a/d converter output (-10 to 10 volts) *100
c  id(11)=q
c  id(12)=u
c  id(13)=v
c  id(14)=r
c  id(15)=q'
c  id(16)=u'
c  id(17)=v'
c  id(18)=r'
c  id(19)=vel
c  id(20)=i'
c  id(21)=agc (disk center brightness)
c
	call in2( iunit, loc, id, lrec0, nrec1, 1, nrec1, iostat )
	if (myint*nbytmi.lt.0) call swap2(id,nrec1,1,nrec1)
c
	d1=float(id(1)-nt(1))
	d2=float(id(2)-nt(2))
	d3=float(id(3)-nt(3))
	p(6)=(15.*d1+d2/4.+d3/240.)*3.14159/180.
	p(5)=float(id(19))*xvel
	kx=id(4)-idcx
	ky=id(5)-idcy
	p(11)=(float(kx)**2+float(ky)**2)/float(kr)**2
	d1=float(id(10))
	d2=float(id(11))
	d3=float(id(12))
	d4=float(id(13))
	d5=float(id(20))
	d6=float(id(15))
	d7=float(id(16))
	d8=float(id(17))
c  d9=xr
	d9=float(id(14))
c
	if(id(21).le.14) then
	  agc=300. 
	  iagc=iagc+1
	else
	  agc=float(id(21)-14)
	endif
	if(d1.le.4.) then
	  p12=0.0
	  p(1)=0.0
	  p(2)=0.0
	  p(3)=0.0
	  p(4)=0.0
	else
	  p1=d1/agc
	  p2=d2/d1/10.0**id(6)/xpl
	  p(2)=p2/(1.0+p2/2.0)
	  qi=1.0-p(2)/2.0
	  p(1)=p1/qi
	  p(3)=d3/d1/10.0**id(6)*qi/xpl
	  p(4)=d4/d1/10.0**id(7)*qi/xpv
c  p12=xr-signal
	  p12=d9/d1/10.0**id(7)*qi/xpv
	endif
	if(d5.le.8.) then
	  p(7)=0.0
	  p(8)=0.0
	  p(9)=0.0
	  p(10)=0.0
	else
	  p(7)=d5/agc
	  p(8)=d6/d5/10.0**id(8)/wpl
	  p(9)=d7/d5/10.0**id(8)/wpl
	  p(10)=d8/d5/10.0**id(9)/wpv
	endif
	if(k.eq.nhdr+1) then
	  do 30 ii=1,3
 30	  itb(ii)=id(ii)
	else if(k.eq.last) then
	  do 40 ii=1,3
 40	  ite(ii)=id(ii)
	  if(msglvl.gt.1) 
     &	    write(lst,150) (itb(ii),ii=1,3),(ite(ii),ii=1,3)
 150	  format(2x,'obs.time=',3i2,'-',3i2)
	endif
c  the first index is for y-coordinates in makita program
	xa(j,i)=p(6)
	xi(j,i)=p(1)
	xq(j,i)=p(2)
	xu(j,i)=p(3)
	xv(j,i)=-p(4)
	xrv(j,i)=p(5)
	wi(j,i)=p(7)
	wq(j,i)=p(8)
	wu(j,i)=p(9)
	wv(j,i)=p(10)
	r2(j,i)=p(11)
c
	sumxv2=sumxv2+p(4)**2
	sumxr2=sumxr2+p12**2
	sumxvr=sumxvr+p12*p(4)
	if(i.eq.1.and.j.eq.1) then
	  kx1=kx
	  ky1=ky
	else if(i.eq.nx.and.j.eq.ny) then
	  kxm=kx
	  kym=ky
	endif
c
	if(id(20).gt.40) then
	  q1=q1+p(6)
	  q2=q2+p(6)**2
	  q3=q3+p(6)*p(5)
	  q4=q4+p(5)
	  nmbr=nmbr+1
	endif
 1000	continue
c
	if(iagc.ne.0) then
	  write(lst,*) 'agc=0 on',iagc,'-points'
	endif
	close(iunit)
c  spc check
	cormax=0.9
	corr=sumxvr/(sqrt(sumxv2)*sqrt(sumxr2))
	if(abs(corr).gt.cormax) then
	  write(lst,*) 'spc phase error:corr(v,r)=',corr,char(7)
	  if(msg.ne.lst)  then
	    write(msg,*) 'spc phase error:corr(v,r)=',corr,char(7)
	    write(msg,*) 'hit cr-key to continue'
	    read(keyin,*) dummy
	  endif
	endif
c
	dd=float(nmbr)
	ddd=dd*q2-q1**2
	pp=(dd*q3-q4*q1)/ddd
	qq=-(q3*q1-q4*q2)/ddd
	call iavrg(xi,r2,nx,ny,MX,xi0,xi1,xi2)
	call iavrg(wi,r2,nx,ny,MX,wi0,wi1,wi2)
	call wpdrft(xi,xa,wq,nx,ny,wq0,wq1,wq2,sigwq,MX)
	call wpdrft(xi,xa,wu,nx,ny,wu0,wu1,wu2,sigwu,MX)
	call wpdrft(xi,xa,wv,nx,ny,wv0,wv1,wv2,sigwv,MX)
	call vavrg(wq,wi,nx,ny,MX,wqa,wqb,2)
	call vavrg(wu,wi,nx,ny,MX,wua,wub,2)
	call vavrg(wv,wi,nx,ny,MX,wva,wvb,2)
	do 1100 j=1,ny
	q=wqa*j+wqb
	u=wua*j+wub
	do 1100 i=1,nx
	xq(j,i)=xq(j,i)-(q+wq0+wq1*cos(2.*xa(j,i))+wq2*sin(2.*xa(j,i)))
	xu(j,i)=xu(j,i)-(u+wu0+wu1*cos(2.*xa(j,i))+wu2*sin(2.*xa(j,i)))
	xrv(j,i)=xrv(j,i)-pp*xa(j,i)-qq
	sigma=sigma+xrv(j,i)**2/dd
 1100	continue
	sigma=sqrt(sigma)
	call vavrg(xrv,xi,nx,ny,MX,va,vb,1)
	call vavrg(xrv,xi,nx,ny,MX,dva,dvb,2)
	if(msglvl.gt.1) then 
	  write(lst,160)
 160	  format(2x,'limb darkening coefficients')
	  write(lst,165) xi0,xi1,xi2,wi0,wi1,wi2
 165	  format(4x,'xi=',1pe12.4,'+',e12.4,'*r**2+',e12.4,'*r**4'/
     &	   4x,'wi=',e12.4,'+',e12.4,'*r**2+',e12.4,'*r**4')
	  write(lst,170) wq0,wq1,wq2,sigwq
	  write(lst,171) wu0,wu1,wu2,sigwu
	  write(lst,172) wv0,wv1,wv2,sigwv
 170	  format(4x,'wq=',1pe10.3,'+',e10.3,'*cos(2ha)+',e10.3,
     &	             '*sin(2ha)',5x,'sigma=',e10.3)
 171	  format(4x,'wu=',1pe10.3,'+',e10.3,'*cos(2ha)+',e10.3,
     &	             '*sin(2ha)',5x,'sigma=',e10.3)
 172	  format(4x,'wv=',1pe10.3,'+',e10.3,'*cos(2ha)+',e10.3,
     &	             '*sin(2ha)',5x,'sigma=',e10.3)
	  write(lst,180) qq,pp,sigma
 180	  format(2x,'rvel=',1pe10.3,'+',e10.3,'*time(radian)',5x,
     &	             'sigma=',e10.3)
	  write(lst,190)
 190	  format(2x,'formulae of linear approximation')
	  write(lst,195) wqa,wqb,wua,wub,wva,wvb,va,vb,dva,dvb
 195	  format(4x,'wq=',1pe12.4,'*y+',e12.4/
     &         4x,'wu=',e12.4,'*y+',e12.4/4x,'wv=',e12.4,'*y+',e12.4/
     &         4x,'rv=',e12.4,'*x+',e12.4/4x,'rv=',e12.4,'*y+',e12.4)
	endif
c
c  original idisp/mm ends here
c
c   restore sign of raw v-data
	do 2100 j=1,ny
	do 2100 i=1,nx
 2100 xv(j,i)=-xv(j,i)
c
c   inversion of data when scan direction is not N,W
c
      if(kx1.gt.kxm) then
        call xinv(wi , nx,ny,MX)
        call xinv(xq , nx,ny,MX)
        call xinv(xu , nx,ny,MX)
        call xinv(xv , nx,ny,MX)
        call xinv(xrv,nx,ny,MX)
      endif
      if(ky1.gt.kym) then
        call yinv(wi , nx,ny,MX)
        call yinv(xq , nx,ny,MX)
        call yinv(xu , nx,ny,MX)
        call yinv(xv , nx,ny,MX)
        call yinv(xrv,nx,ny,MX)
      endif
      kxs=min(kx1,kxm)
      kys=min(ky1,kym)
c
	factv=1.0/10000.0
	factq=1.0/10000.0
	factu=1.0/10000.0
	factd=1.0
	facti=1.0/1000.0
c  rdata(*,*,ki) = I'/AGC = O(1)
	lspeci=lspec('I')
	lspecq=lspec('Q')
	lspecu=lspec('U')
	lspecv=lspec('V')
	lspecd=lspec('VL')
	factm(lspeci)=facti
	factm(lspecq)=factq
	factm(lspecu)=factu
	factm(lspecv)=factv
	factm(lspecd)=factd
	ki=kdest(lspeci)
	kq=kdest(lspecq)
	ku=kdest(lspecu)
	kv=kdest(lspecv)
	kd=kdest(lspecd)
c
	do 2000 j=1,ny
	do 2000 i=1,nx
	rdata(i,j,ki)=wi(j,i)
	rdata(i,j,kv)=xv(j,i)
	rdata(i,j,kd)=xrv(j,i)
	al=sqrt(xq(j,i)**2+xu(j,i)**2)
	az=0.0
	if(al.ne.0.0) az=atan2(xu(j,i),xq(j,i))/2.0 + xa(j,i)
	rdata(i,j,kq)=al*cos(2.0*az)
	rdata(i,j,ku)=al*sin(2.0*az)
 2000	continue
c
c   subscripts i and j have been exchanged
c
	dx=arcsec*float(kdx)
	dy=arcsec*float(kdy)
	xs=arcsec*float(kxs)
	ys=arcsec*float(kys)
	rsp=arcsec*float(kr)
c
c *** change the form of title *** 93/12/26
c *** because the old form below does not keep the original comment ***
cx	write(title,200) idat,itb,ite,mfile 
c200	  format('DATE: ',3i3.2,'  TIME(JST): ',3i3.2,' - ',3i3.2,
cx     &	         1x,'(',a6,')',22x)
cx	if(msglvl.gt.0) write(lst,'(2x, a)') title
c
c  set creation date/time of the output file
c
	ftag=mktag()
	tsys='JST'
	inst='OAO 65CM'
c
	nframi=5
	nframe=5
	kmem(1)=ki
	kmem(2)=kq
	kmem(3)=ku
	kmem(4)=kv
	kmem(5)=kd
	lspecs(ki)=lspeci
	lspecs(kq)=lspecq
	lspecs(ku)=lspecu
	lspecs(kv)=lspecv
	lspecs(kd)=lspecd
	kloc(lspeci)=ki
	kloc(lspecq)=kq
	kloc(lspecu)=ku
	kloc(lspecv)=kv
	kloc(lspecd)=kd
c
	write(dateo,230) idat
 230	format(i2.2,'/',i2.2,'/',i2.2)
	write(stime,235) itb
 235	format(i2.2,':',i2.2,':',i2.2)
	write(etime,235) ite
c
	if(msglvl.gt.2) then
	  write(lst,*) 'mfile=',mfile
	  write(lst,*) 'ftag=',ftag
	  write(lst,*) 'dateo=',dateo
	  write(lst,*) 'stime=',stime
	  write(lst,*) 'etime=',etime
	  write(lst,*) 'wavelength=',wavel
	endif
	ydeg=0.0
	mesh=0
	xtod=0.0
	ytod=0.0
c
	call freem(1,11)
	return
c
 9	if(iostat.eq.1) then
	  write(lst,*) '(subr.setmg ) endfile during read'
	else if(iostat.eq.2) then
	  write(lst,*) '(subr.setmg ) read error'
	endif
	errflg=2
	call freem(1,11)
	return
	end
c
	subroutine xinv(a,nx,ny,ndim)
	dimension a(ndim,1)
	do 10 j=1,ny
	do 10 i=1,nx/2
	aux=a(i,j)
	a(i,j)=a(nx+1-i,j)
	a(nx+1-i,j)=aux
 10	continue
	return
	entry yinv(a,nx,ny,ndim)
	do 20 j=1,ny/2
	do 20 i=1,nx
	aux=a(i,j)
	a(i,j)=a(i,ny+1-j)
	a(i,ny+1-j)=aux
 20	continue
	return
	end
c
	include 'oaolib.fi'
