c  conversion from polarization degrees to magnetic fields
c  NAOJ Magnetograms
c
	subroutine poltob
c
	include 'sys.fc'
	include 'string.fc'
	include 'mdata.fc'
	include 'fileio.fc'
	include 'mparam.fc'
	include 'auxparam.fc'
	dimension cbl1s(3,6),cbl3s(3,6),cblms(3,6),
     &	          cbt1s(3,6),cbt3s(3,6),cvels(3,6)
c
c  indices are (formula number, waveband number)
c
c  qu = sqrt(q**2+u**2)
c  bl = cbl1*v + cbl3*v**3 + cblm*v*qu
c  bt = sqrt(qu) *( cbt1 + cbt3*qu )
c  vl = cvel*w
c
c  band 1: 5250A (OAO line)
c  band 2: 5282-5297A (OAO wide band)
c  band 3: 6303A (Flare Telescope T1)
c  band 4: 6337A (Flare Telescope T4)
c  band 5: 5324A (STEP full-disk magnetograph)
c  band 6: 5896A (Na D1-line MOF tachometer)
c
	do 1 j=1,6
	do 1 i=1,3
	cbl1s(i,j)=0.0
	cbl3s(i,j)=0.0
	cblms(i,j)=0.0
	cbt1s(i,j)=0.0
	cbt3s(i,j)=0.0
	cvels(i,j)=1.0
 1	continue	
c
c  OAO
c  formula #1 cubic formula based on a model atmosphere
	cbl1s(1,1)= -4500.0
	cbl3s(1,1)=-15000.0
	cbt1s(1,1)=  2300.0
c  formula #2 linear, the same scale as OAO databook
	cbl1s(2,1)= -2480.0
	cbt1s(2,1)=  1235.0
c  formula #3 linear, the same scale as OAO gdisp
	cbl1s(3,1)= -4960.0
	cbt1s(3,1)=  2470.0
c  Flare Telescope T1
	cbl1s(1,3)= -4000.0
	cbt1s(1,3)=  3780.0
c  Flare Telescope T4
	cbl1s(1,4)=  8000.0
	cvels(1,4)=  15000.0
c  STEP full-disk magnetograph
	cbl1s(1,5)=  25000.0
	cvels(1,5)=  250000.0
c  MOF Na D-line Doppler
	cvels(1,6)=  0.0
c
	iband = mod(-kindm,10)
	iform1= iform
	if(iband.ne.1) iform1=1
	cbl1=cbl1s(iform1,iband)
	cbl3=cbl3s(iform1,iband)
	cblm=cblms(iform1,iband)
	cbt1=cbt1s(iform1,iband)
	cbt3=cbt3s(iform1,iband)
	cvel=cvels(iform1,iband)
c
	lspcq=lspec('Q')
	lspcu=lspec('U')
	lspcv=lspec('V')
	lspci=lspec('I')
	lspcw=lspec('W')
	lspcd=lspec('VL')
	lspcx=lspec('BX')
	lspcy=lspec('BY')
	lspcz=lspec('BL')
c
	kq=kloc(lspcq)
	ku=kloc(lspcu)
	kv=kloc(lspcv)
	ki=kloc(lspci)
	kw=kloc(lspcw)
	kd=kloc(lspcd)
	kx=kloc(lspcx)
	ky=kloc(lspcy)
	kz=kloc(lspcz)
c
	if(msglvl.gt.2) write(lst,*) '(subr.poltob) kq,u,v,x,y,z=',
     &	kq,ku,kv,kx,ky,kz
c
	if( (kq.eq.0 .and. ku.eq.0 .and. kv.eq.0) .and.
     &	    (kx.gt.0 .or.  ky.gt.0 .or.  kz.gt.0) ) then
	  if(msglvl.gt.0) write(lst,*)
     &	'(subr.poltob) warning: q,u,v are not found:',
     &	' maybe already converted'
	  return
	endif
c
c  circular polarization to BL
c
	if(kv.gt.0) then
	  kz=kv
	  do 10 j=1,ny
	  do 10 i=1,nx
	  if(kq.gt.0 .and. ku.gt.0) then
	    plin=sqrt(rdata(i,j,kq)**2+rdata(i,j,ku)**2)
	  else
	    plin=0.0
	  endif
	  bz=rdata(i,j,kv)*(cbl1+cbl3*rdata(i,j,kv)**2 +cblm*plin)
	  rdata(i,j,kz)=bz
c
 10	  continue
	  factm(lspcz)=1.0
	  lspecs(kz)=lspcz
	  kloc(lspcz)=kz
	  kloc(lspcv)=0
	endif
c
c  linear polarization to Bt
c
	if(kq.gt.0 .and. ku.gt.0) then
	  kx=kq
	  ky=ku
	  do 20 j=1,ny
	  do 20 i=1,nx
	  plin=sqrt(rdata(i,j,kq)**2+rdata(i,j,ku)**2)
	  bt=sqrt(plin)*(cbt1 +cbt3*plin)
	  if(plin.eq.0.0) then
	    az=0.0
	  else
	    az=0.5*atan2(rdata(i,j,ku),rdata(i,j,kq))
	  endif
	  bx=bt*cos(az)
	  by=bt*sin(az)
	  rdata(i,j,kx)=bx
	  rdata(i,j,ky)=by
 20	  continue
	  factm(lspcx)=1.0
	  factm(lspcy)=1.0
	  lspecs(kx)=lspcx
	  lspecs(ky)=lspcy
	  kloc(lspcx)=kx
	  kloc(lspcy)=ky
	  kloc(lspcq)=0
	  kloc(lspcu)=0
	endif
c
c  wing intensity difference to VL
c
	if(kw.gt.0) then
	  kd=kw
	  do 30 j=1,ny
	  do 30 i=1,nx
	  vel=cvel*rdata(i,j,kw)
	  rdata(i,j,kd)=vel
 30	  continue
	  factm(lspcd)=1.0
	  lspecs(kd)=lspcd
	  kloc(lspcd)=kd
	  kloc(lspcw)=0
	endif
c
c  set kindm
c
	if(kq.eq.0 .and. ku.eq.0) then
	  kindm=1
	else 
	  kindm=2
	endif
	if(kd.gt.0) kindm=kindm+100
	if(ki.gt.0) kindm=kindm+1000
c
	if(msglvl.gt.0) write(lst,*)
     &	   '(subr.poltob) conversion completed: waveband=',iband,
     &	   ', formula=',iform
	if(msglvl.gt.1) 
     &	  write(lst,*)   '  kindm,nframe=',kindm,nframe
	if(msglvl.gt.2) then
	  write(lst,100) '  kq,u,v,i,d=',kq,ku,kv,ki,kd
	  write(lst,100) '  lspecq,u,v,i,d=',lspcq,lspcu,lspcv,
     &	                  lspci,lspcd
	  write(lst,100) '  kx,y,z=',kx,ky,kz
	  write(lst,100) '  lspecx,y,z=',lspcx,lspcy,lspcz
	  write(lst,100) '  lspecs=',lspecs
	  write(lst,100) '  kloc=',kloc
	endif
 100	format(2x,a,(20i3/))
	return
	end
c
c  conversion from (Bt,azimuth) to (bx,by)
c
	subroutine btob
c
	include 'sys.fc'
	include 'string.fc'
	include 'mdata.fc'
	include 'fileio.fc'
	include 'mparam.fc'
	include 'auxparam.fc'
c
	lspct=lspec('BT')
	lspca=lspec('AZ')
	lspcx=lspec('BX')
	lspcy=lspec('BY')
c
	kt=kloc(lspct)
	ka=kloc(lspca)
	kx=kloc(lspcx)
	ky=kloc(lspcy)
c
	if(msglvl.gt.2) write(lst,*) '(subr.btob  ) kt,a,x,y=',
     &	kt,ka,kx,ky
c
	if( (kt.eq.0 .and. ka.eq.0) .and.
     &	    (kx.gt.0 .or.  ky.gt.0) ) then
	  if(msglvl.gt.0) write(lst,*)
     &	    '(subr.btob  ) data are already converted'
	  return
	endif
c
	if(kt.gt.0 .and. ka.gt.0) then
	  kx=kt
	  ky=ka
	  do 20 j=1,ny
	  do 20 i=1,nx
	  bt=rdata(i,j,kt)
	  az=rdata(i,i,ka)
	  rdata(i,j,kx)=bt*cos(az)
	  rdata(i,j,ky)=bt*sin(az)
 20	  continue
	  factm(lspcx)=1.0
	  factm(lspcy)=1.0
	  lspecs(kx)=lspcx
	  lspecs(ky)=lspcy
	  kloc(lspcx)=kx
	  kloc(lspcy)=ky
	  kloc(lspct)=0
	  kloc(lspca)=0
	endif
c
	if(msglvl.gt.0) write(lst,*)
     &	   '(subr.btob  ) conversion completed'
	return
	end
