c     quasi-newton method to solve nonlinear simultaneous equations
c
c     entry  n     : number of unknown
c            x     : initial point
c            eps   : convergence parameter
c            dx    : differentiation increment
c            limit : maximum number of iteration
c            ss    : step of minimum search
c            ipr   : control list of iteration
c                     >0 -- full listing once in ipr-iteration
c                     =0 -- simple listing in every iteration
c                     <0 -- no listing
c            mode  : control update formula
c                    h = h + (dx)(ut)/(u*dg) - (h*dg)(vt)/(v*dg)
c                     abs(mode)=1 -- u=dx,vt=dg*h  (dfp)
c                               2 -- ut=vt=dg*h    (broyden)
c                               3 -- ut=vt=dx      (secant)
c                               4 -- no update
c                     mode>0 -- complete minimization
c                         <0 -- only making decrease
c            ortho : orthogonalize u and v or not
c
c     return limit=0     : no iteration
c                 =kount : success or count over
c                 =-kount: error in minimization
c                 =-998  : error in matrix inversion
c                 =-999  : parameter error
c
c     actual size : h(n,n+5), ho(n,limit)(if ortho=true)
c
      subroutine qnnl(funct,dfunct,n,x,f,g,eps,dx,ss,mode,
     *                limit,ipr,h,ortho,ho)
      external funct
	include 'sys.fc'
      logical ortho
      dimension x(1),g(1),h(n,1),ho(n,1)
c
      if(n.le.0) go to 7800
      if(eps.lt.0.0) eps=1.0e-6
      if(dx.le.0.0) dx=0.01
      if(ss.le.0.0) ss=1.0
      if(limit.lt.0) limit=0
      if(mode.eq.0.or.iabs(mode).gt.4) mode=1
      call clockm(itime)
      time=0.001*float(itime)
      write(lst,300) time,mode,ortho,dx,ss
 300  format('0subr. qnnl start. *** time=',f10.5,2x,'mode=',
     *        i2,2x,'ortho=',l1,2x,'dx=',f10.4,
     *        2x,'ss=',f10.4)
      call funct(n,x,ff,g)
      f=sqrt(abs(ff))
      dxnrm=0.0
      ier=0
      kount=0
      kall=0
      if(limit.eq.0) go to 2000
c
 1000 continue
c    set jacobian
      do 1100 j=1,n
      x(j)=x(j)+dx
      call dfunct(j,x,fd,h(1,j+5))
      x(j)=x(j)-2.0*dx
      call dfunct(j,x,fd,h)
      x(j)=x(j)+dx
      do 1100 i=1,n
 1100 h(i,j+5)=(h(i,j+5)-h(i,1))/(2.0*dx)
      do 1200 i=1,n
      hmax=0.0
      do 1250 j=1,n
      if(abs(h(i,j+5)).gt.hmax) hmax=abs(h(i,j+5))
 1250 continue
      if(hmax.ne.0.0) go to 1201
      write(lst,*) 'matrix singular i=',i
      limit=-999
      return
 1201 continue
      h(i,3)=hmax
      do 1200 j=1,n
 1200 h(i,j+5)=h(i,j+5)/hmax
c
      call clockm(itime)
      time=0.001*float(itime)
      write(lst,330) time
 330  format('0(subr.qnnl  ) time=',f10.3,' after matrix setting')
      eps1=1.0e-5
      call inv2(h(1,6),n,0,n,n,h(1,1),h(1,2),det,nstop,eps1)
c
      if(nstop.ne.1) go to 8000
      call clockm(itime)
      time=0.001*float(itime)
      write(lst,340) time,eps1
 340  format('0(subr.qnnl  ) time=',f10.3,
     *' after matrix inversion.  min.pivot=', 1pe12.4)
      do 1500 j=1,n
      hmax=h(j,3)
      do 1500 i=1,n
 1500 h(i,j+5)=h(i,j+5)/hmax
c      top of iteration loop
 2000 continue
      call clockm(itime)
      time=0.001*float(itime)
      if(ipr) 2300,2200,2100
 2100 if(mod(kount,ipr).ne.0) go to 2300
      write(lst,100) kount,time,kall,dxnrm,(x(i),i=1,n)
      write(lst,110) f,(g(i),i=1,n)
 100  format('0iteration count=',i5,5x,'time=',f10.5,5x,'call=',i5,5x,
     *       'dxnorm=',1pe15.6,5x,'x='/(1h ,10e13.5))
 110  format(1h0,'gnorm=',1pe15.6,5x,'g='/(1h ,1p10e13.5))
      kall=0
      go to 2300
 2200 write(lst,120) kount,time,f,kall,dxnrm
 120  format('0iteration count=',i5,5x,'time=',f10.5,5x,'gnorm=',
     *       1pe15.5,5x,'call=',i5,5x,'dxnorm=',e15.6)
      kall=0
 2300 kount=kount+1
      if(limit.eq.0) return
c    h(*,1)=-h*g, h(*,2)=g, h(*,3)=x
      do 2400 i=1,n
      h(i,2)=g(i)
      h(i,3)=x(i)
      aux=0.0
      do 2500 j=1,n
 2500 aux=aux-g(j)*h(i,j+5)
 2400 h(i,1)=aux
      hnrm=0.
      do 2600 i=1,n
 2600 hnrm=hnrm+h(i,1)**2
      hnrm=sqrt(hnrm)
      if(ipr.gt.0.and.mod(kount,ipr).eq.0)
     *     write(lst,130) hnrm,(h(i,1),i=1,n)
  130 format(1h0,'hnorm=',1pe15.6,5x,'h='/(1h ,10e13.5))
      if(hnrm.lt.eps*f) go to 7100
      icode=mode
c
      call min2(funct,n,x,h,ff,g,ss,kall,icode,ipr,check)
c
      f=sqrt(abs(ff))
c    h(*,1)=dx, h(*,2)=dg
      do 3100 i=1,n
      h(i,2)=g(i)-h(i,2)
 3100 h(i,1)=x(i)-h(i,3)
      dxnrm=0.0
      do 3200 i=1,n
 3200 dxnrm=dxnrm+h(i,1)**2
      dxnrm=sqrt(dxnrm)
      if(icode) 7900,3000,7100
 3000 continue
      ier=0
      if(hnrm.gt.eps) go to 3300
      if(dxnrm.le.eps) go to 7400
 3300 if(kount.ge.limit) go to 7000
      if(iabs(mode).eq.4) go to 2000
      if(iabs(mode)-2) 4000,4100,4200
c    dfp formula
 4000 continue
      do 4010 i=1,n
      h(i,3)=h(i,1)
      aux=0.0
      do 4020 j=1,n
 4020 aux=aux+h(j,2)*h(j,i+5)
 4010 h(i,4)=aux
      go to 4300
c    broyden's formula
 4100 continue
      do 4110 i=1,n
      aux=0.0
      do 4120 j=1,n
 4120 aux=aux+h(j,2)*h(j,i+5)
      h(i,3)=aux
 4110 h(i,4)=aux
      go to 4300
c    secant formula
 4200 continue
      do 4210 i=1,n
      h(i,3)=h(i,1)
 4210 h(i,4)=h(i,1)
 4300 continue
c    now h(*,3)=u, h(*,4)=v
      if(.not.ortho) go to 6000
c    orthogonalization of u and v
      do 5000 i=1,n
 5000 ho(i,kount)=h(i,2)
      if(kount.eq.1) go to 5300
      do 5100 k=2,kount
      udg=0.0
      vdg=0.0
      aux=0.0
      do 5200 i=1,n
      udg=udg+h(i,3)*ho(i,k-1)
      vdg=vdg+h(i,4)*ho(i,k-1)
 5200 aux=aux+h(i,2)*ho(i,k-1)
      do 5100 i=1,n
      h(i,3)=h(i,3)-udg*ho(i,k-1)
      h(i,4)=h(i,4)-vdg*ho(i,k-1)
 5100 ho(i,kount)=ho(i,kount)-aux*ho(i,k-1)
 5300 dgnrm=0.0
      do 5400 i=1,n
 5400 dgnrm=dgnrm+ho(i,kount)**2
      dgnrm=sqrt(dgnrm)
      do 5500 i=1,n
 5500 ho(i,kount)=ho(i,kount)/dgnrm
 6000 continue
c    update h, after h(*,5)=h*dg
      udg=0.0
      vdg=0.0
      do 6100 i=1,n
      aux=0.0
      do 6200 j=1,n
 6200 aux=aux+h(i,j+5)*h(j,2)
      h(i,5)=aux
      udg=udg+h(i,3)*h(i,2)
      vdg=vdg+h(i,4)*h(i,2)
 6100 continue
      if(udg.eq.0.0.or.vdg.eq.0.0) go to 7250
      do 6300 j=1,n
      do 6300 i=1,n
 6300 h(i,j+5)=h(i,j+5)+h(i,1)*h(j,3)/udg-h(i,5)*h(j,4)/vdg
      go to 2000
c    end of iteration loop
 7000 write(lst,140)
 140  format('0subr. qnnl return. *** iteration count over')
      go to 7450
 7100 do 7200 i=1,n
 7200 x(i)=h(i,3)
      call funct(n,x,ff,g)
      f=sqrt(abs(ff))
      kall=kall+1
      if(f.le.eps) go to 7400
      if(ier.lt.0) go to 7300
      ier=-1
 7250 write(lst,150)
 150  format('0trying to reset jacobian')
      go to 1000
 7300 write(lst,160)
 160  format('0subr. qnnl return. *** minimization error')
      limit=-kount
      go to 7450
 7400 write(lst,170) eps
 170  format('0subr. qnnl return. *** converged : eps=',1pe10.3)
      limit=kount
 7450 call clockm(itime)
      time=0.001*float(itime)
      if(ipr) 7700,7600,7500
 7500 write(lst,100) kount,time,kall,dxnrm,(x(i),i=1,n)
      write(lst,110) f,(g(i),i=1,n)
      go to 7700
 7600 write(lst,120) kount,time,f,kall,dxnrm
 7700 return
 7800 write(lst,180) n
 180  format('0subr. qnnl terminated by errors.     n=',i5,5x,
     *       'must be positive')
      limit=-999
      return
 7900 write(lst,190)
 190  format('0subr. qnnl return. *** minimum not found')
      limit=-kount
      go to 7450
 8000 write(lst,200)
 200  format('0subr. qnnl return. *** jacobian is singular')
      limit=-998
      return
      end
      subroutine min2(func,n,x,h,f,g,ss,kall,icode,ipr,check)
c
c     quadratic minimization
c
c     entry  icode>0  : complete minimization
c                 <0  : return if f<f20
c     return icode=-1 : no minimum (s>100000*ss)
c                   0 : success
c                  +1 : error in minimization
c                       (s<0.01*ss but f>f20)
c
	include 'sys.fc'
      dimension x(1),h(1),g(1)
      dimension gg(1000)
      logical check
      f20=f
      dd=0.0
      f2=f
      s=ss
      t=0.0
      ifs=0
      if(ipr.ge.0) write(lst,100) ifs,dd,f
 100  format(1h0,'ifs=',i2,5x,'d=',1pe15.6,5x,'f=',e15.6)
c
    1 continue
      d=t+s
    2 continue
      do 10 i=1,n
   10 x(i)=x(i)+d*h(i)
      dd=dd+d
      do 90 i=1,n
 90   gg(i)=g(i)
      call func(n,x,f,g)
      if(ipr.ge.0) write(lst,110) ifs,dd,f
 110  format(1h ,'ifs=',i2,5x,'d=',1pe15.6,5x,'f=',e15.6)
      if(check) write(lst,910) (g(i),i=1,n)
 910  format('0g='/(1p10e13.4))
      do 91 i=1,n
      gg(i)=(g(i)-gg(i))/d
      if(abs(gg(i)+g(i)).lt.0.1*abs(g(i))) gg(i)=0.0
 91   continue
      if(check) write(lst,900) (gg(i),i=1,n)
 900  format('0dg='/(1p10e13.4))
      kall=kall+1
      if(ifs.eq.1) go to 3
      if(abs(s).gt.1.0e5*abs(ss)) go to 1002
      if(f.lt.f2) if(ifs-1) 11,12,12
                  if(ifs-1) 13,14,14
c
   11 f1=f2
      f2=f
      ifs=2
      if(icode.lt.0) go to 1001
      go to 1
   12 f2=f
      s=2.0*s
      t=0.0
      go to 1
   13 f1=f
      s=-s
      t=s
      ifs=2
      go to 1
   14 f3=f
      d=-s+0.5*s*(f1-f3)/((f1-f2)+(f3-f2))
      ifs=1
      go to 2
c
    3 continue
      if(f.lt.f2) go to 1001
      if(abs(s).lt.0.01*abs(ss)) if(f-f20) 1001,1001,1003
      f1=f
      s=-(s+d)
      t=s
      ifs=2
      go to 1
c
 1001 icode=0
      return
 1002 icode=-1
      return
 1003 icode=1
      return
      end
c
      subroutine  inv2  ( a,n,m,n1,m1,nans,x,det,nstop,eps )
	include 'sys.fc'
      dimension  a(n1,1), nans(1), x(1)
      mm = n + m
      if( n )  910, 910, 101
  101 if( n-n1 )  102, 102, 910
  102 if( m )  910, 103, 103
  103 if( mm-m1 )  104, 104, 910
  104 if(eps)   106, 107, 107
  106 eps = 1.0e-5
  107 epss = 0.01 * eps
      pivmn=1.0d30
      det = 1.0
      do  110  i = 1, n
      nans(i) = 0
  110 continue
c
      do  300  k = 1, n
c                                       pivot search
      bigst = -1.0
      do  140  i = 1, n
      if( nans(i) )  120, 120, 140
  120 continue
      if( bigst - abs(a(i,k)) )  130, 140, 140
  130 continue
      bigst = abs(a(i,k))
      l = i
  140 continue
      if( bigst - eps )  920, 920, 150
  150 pivot = a(l,k)
      det = det * pivot
      if(abs(pivot).lt.pivmn) pivmn=abs(pivot)
      nans(l) = k
c                                       sweep out
      pivi = -1.0/ pivot
      do  200  j = 1, mm
      if( j - k )  160, 200, 160
  160 continue
      w = a(l,j) * pivi
      if(abs(w)-epss) 190, 190, 170
  170 continue
      do  180  i = 1,n
      a(i,j) = a(i,j) + a(i,k) *w
  180 continue
  190 continue
      a(l,j) = -w
  200 continue
      do  250  i = 1, n
      a(i,k) = a(i,k) * pivi
  250 continue
      a(l,k) = -pivi
  300 continue
c
c                                       exchange  root
      do  350  j = 1, mm
      do  320  i = 1, n
      x(i) = a(i,j)
  320 continue
      do  330  i = 1, n
      a( nans(i), j) = x(i)
  330 continue
  350 continue
      if(n-1) 500, 500, 360
  360 continue
      do  430 i = 1, n
      do  410 j = 1, n
      x(j) = a(i,j)
  410 continue
      do  420 j = 1, n
      a(i,j) = x( nans(j) )
  420 continue
  430 continue
c                                       determinant
      do 390 i=2,n
      do 380 j=i,n
      if(nans(i-1)-nans(j)) 380,380,370
  370 continue
      det = -det
  380 continue
  390 continue
c
  500 continue
      nstop = 1
      eps=pivmn
      return
c                                       argument error
  910 continue
      write(lst,2001)  n, m, n1, m1
      nstop = 3
      return
c                                       matrix  singular
  920 continue
      write(lst,2002)  bigst, eps, k
      det = 0.0
      nstop = 2
      do  930  j = 1, n
      do  930  i = 1, n
      a(i,j) = 1.0e30
  930 continue
      return
 2001 format(1h0, '(subr. inv2) error: n=',i4,', m=',i4,', n1=',i4,
     *', m1=',i4,
     *       /15x,'n and m should satisfy the following inequalities.',
     *       /15x,'n.gt.0, n.le.n1, m.ge.0, m+n.le.m1.' ,
     *       /15x,'return with no calculation.'  )
 2002 format(1h0, '(subr. inv2) warning: matrix is nearly singular.',
     *       /15x,'the pivot ',e15.7,' is less than or equal to eps ',
     *       e15.7,'. (',i4,'-th eliminating process) ' )
      end
      subroutine clockm(itime)
      itime=0
      return
      end
