c  read magnetogram data of standard format
c  form of input file is specified by 'kindm'
c  kindm=0 means that kindm=kindmi and all the data will be stored
c
      subroutine readm(iunit)
c
	include 'sys.fc'
	include 'fileio.fc'
	include 'mdata.fc' 
	include 'mparam.fc'
	include 'lparam.fc'
	include 'string.fc'
	include 'solar.fc'
	include 'coord.fc'
	include 'auxparam.fc'
c
	logical update,unique
	integer iunit, kdest(NSPEC0)
	dimension xx(3)
	rad(deg)=3.141592/180.0*deg
c
c  subr.ilist determines where the data should be stored
c  data specified by lspec will be stored in rdata(*,*,k),
c  where k=kdest(lspec). k runs from 1 to kmax.
c
	unique=.false.
	kmax=MZ
	call ilist(kindm,kdest,kmax,NSPEC0,unique)
	if (kmax.gt.MZ) then
	  write(msg,*) 
     &	'(subr.readm ) error: insufficient space in rdata, kmax=',
     &	kmax
	  errflg=2
	  return
	endif
c
	do 10 n=1,NSPEC0
 10	kloc(n)=0
	do 20 k=1,MZ
	kmem(k)=0
	lspecs(k)=0
 20	continue
	nframe=0
c  these are also done in initc.f, but
c  kmem etc are cleared every time readm is called
c  therefore, passing of data between procedures is not
c  by memory but by files
c
	update=.true.
	call inmh(iunit,update)
	if(errflg.gt.1) return
c
	if(msglvl.gt.0) then
   	  write(lst,*) '(subr.readm ) kindm=',kindm
	  write(lst,*) '      kindmi,mesh,nframi=',kindmi,mesh,nframi
	endif
	if(msglvl.gt.1) 
     &	call printm(nbytmi,nx,ny,xs,ys,dx,dy,ydeg,
     &              rsp,pdeg,b0deg,cl0deg,xtod,ytod,
     &              tsys,inst,wavel,mfile,dateo,stime,etime,ftag,title)
c
	do 30 iframe=1,nframi
 	call inm(iunit,kdest)
	if(errflg.gt.1) return
 30	continue
c  nframe is incremented by inm
c
	xxsp(1)=0.0
	xxsp(2)=0.0
	xxsp(3)=-rsp
	prad=rad(pdeg)
	b0rad=rad(b0deg)
	cl0=rad(cl0deg)
c  set xdu and ydu from ypos
c
	ypos=rad(ydeg)
c
c  if ydeg=pdeg, the data are heliocentric (solar north up)
c  in this case, in order to avoid coordinate transform, 
c  set ypos=0
c  the z-axis of the C-coordinates is then toward solar north
c
	if(ydeg.eq.pdeg) then
	  ypos=0.0
	  prad=0.0
	endif
	call setdu
c
	if(mesh.eq.0) then
	  xtod=xs+0.5*dx*float(nx-1)
	  ytod=ys+0.5*dy*float(ny-1)
	  xxtod(1)=xtod
	  xxtod(2)=ytod
	  call projsr(xxtod,rsp)
	  call dtoc(xxtod,xxto)
	else
	  xxtod(1)=xtod
	  xxtod(2)=ytod
	  call projsr(xxtod,rsp)
	  call dtoc(xxtod,xxto)
	endif
	if(mesh.eq.4) then
	  cl0eff=cl0
	else
	  cl0eff=0.0
	endif
	call settu
	call setas
	call setar
c
c  set default integration limits
c
	x1lim0=xs
	x2lim0=xs+dx*float(nx-1)
	z1lim0=0.0
	if( iabs(mesh).le.1) then
	  y1lim0=ys
	  y2lim0=ys+dy*float(ny-1)
	  z2lim0=0.5*sqrt((x2lim0-x1lim0)**2+(y2lim0-y1lim0)**2)
	  ds0=sqrt(abs(dx*dy))*0.25
	else
	  y1lim0=alofy(ys)
	  y2lim0=alofy(ys+dy*float(ny-1))
	  if(rrss.lt.1.0.or.mod(kindl,1000).lt.100) then
	    z2lim0=rsp*0.5*sqrt((x2lim0-x1lim0)**2+(y2lim0-y1lim0)**2)
	  else
	    z2lim0=rsp*rrss -rsp
	  endif
	  ds0=sqrt(abs(dx*alofy(dy)))*0.25*rsp
	endif
c
c  for mesh=0
c  clear data outside the solar disk (r> rsp)
c
	if (mesh.eq.0) then
	  do 40 iframe=1,nframe
	  do 40 j=1,ny
	  do 40 i=1,nx
	  xx(1)= xs + dx*float(i-1)
	  xx(2)= ys + dy*float(j-1)
	  if( (xx(1)-xxsp(1))**2+(xx(2)-xxsp(2))**2 .gt.
     &	    rsp*rsp) then
	    rdata(i,j,iframe) = ufl
	  endif
 40	  continue
	endif
c
	errflg=0
	do 50 lspec1=1,NSPEC0
	k=kdest(lspec1)
	if(k.gt.0) then
	  if(lspecs(k).eq.0) then
	    write(msg,*)
     &	    '(subr.readm ) error: requested data were not found, spec=',
     &	     specst(lspec1)
	    errflg=2
	  endif
	endif
 50	continue
c
	if (kindm.eq.0) kindm=kindmi
	if(msglvl.gt.0) write(lst,*)
     &	 '(subr.readm ) kindm,mesh,nframe=',kindm,mesh,nframe
c
	return
	end
c
c  read intensity/velocity data of standard format
c
	subroutine readiv(lspciv,iunit)
c
	include 'sys.fc'
	include 'mdata.fc'
	include 'fileio.fc'
	include 'mparam.fc'
	logical update
	integer iunit, kdest(NSPEC0)
c
	if(nframe.eq.MZ) then
	  kmax=MZ
	  write(msg,*)
     &	'(subr.readiv) error: insufficient space in rdata, kmax=',
     &	kmax
	  errflg=2
	  return
	endif
c
	do 30 lspec1=1,NSPEC0
 30	kdest(lspec1)=-1
c
	kdest(lspciv)=nframe+1
	kloc(lspciv)=0
	update=.false.
c
	if (msglvl.gt.0) 
     &  write(lst,*) '(subr.readiv) for ',specst(lspciv),' (no update)'
c  the rest of the messages will be issued by call inmh
	call inmh(iunit,update)
	if(errflg.gt.1) return
	call inm(iunit,kdest)
	if(errflg.gt.1) return
c
	errflg=0
	if (kloc(lspeciv).eq.0) then
	  write(msg,*)
     &	    '(subr.readiv) error: requested data were not found, spec=',
     &	     specst(lspeciv)
	  errflg=2
	endif
c
	return
	end
c
c  read standard data file from i/o unit number 'iunit'
c  'inmh' reads the header, 'inm' reads the data
c  data will be stored in rdata(*,*,k), where k runs from 1 up to MZ
c  the calling program should specify k=kdest(lspec) if a particular 
c  data of lspec is to be read
c  when read operation is successful, kloc(lspec)=k and
c  lspec(k)=lspec is set
c  if update=.false., (which is the case when this routine is called
c  from readiv), parameters in commons /fileio/ etc. are not updated
c  by the read-in data
c
c  read magnetogram header
c
	subroutine inmh(iunit,update)
c
	include 'sys.fc'
	include 'fileio.fc'
	include 'mparam.fc'
	include 'mdata.fc'
	include 'string.fc'
	include 'solar.fc'
	include 'coord.fc'
c
	logical update,first,alloc
	integer iunit, kdest(NSPEC0)
	character buf*80,spec*8
	character tsys1*6,inst1*8,mfile1*8,ftag1*16,title1*80,
     &	dateo1*8,stime1*8,etime1*8
	save icode,loc,first
c
	lrec0=80
	icode = 0
	loc=1
	first=.true.
	call ins( iunit, loc, buf, lrec0, iostat )
	if( iostat.ne.0) go to 9
c
c  mycode:0=unknown, 1=ascii, 2=ebcdic
c  icode = target code if icode.ne.0
c
	if (mycode.ne.0) then
	  if(buf(17:18).eq.blank(3-mycode)) icode=mycode
	endif
	if (icode.ne.0) call cconv(icode, buf)
c
	if(update) then
	  read(buf,110) kindmi,nbytmi,nframi,tsys,inst,mfile,dateo,
     &	                stime,etime,ftag
	else
	  read(buf,110) kindm1,nbyte1,nfram1,tsys1,inst1,mfile1,dateo1,
     &	                stime1,etime1,ftag1
	endif
 110	format(i8,2i4,2x,a6,5a8,a16)
c
	call ins( iunit, loc, buf, lrec0, iostat )
	if( iostat.ne.0) go to 9
	if (icode.ne.0) call cconv(icode, buf)
	if(update) then
	  read(buf,120) nx0,ny0,xs0,ys0,dx0,dy0,ydeg,mesh,wavel
	  nx=nx0
	  ny=ny0
	  xs=xs0
	  ys=ys0
	  dx=dx0
	  dy=dy0
	  if(ixs.eq.0) ixs=1
	  if(ixe.eq.0) ixe=nx
	  if(iys.eq.0) iys=1
	  if(iye.eq.0) iye=ny
	else
	  read(buf,120) nx1,ny1,xs1,ys1,dx1,dy1,ydeg1,mesh1,wavel1
	endif
 120	format(2i4,5f12.5,i4,f8.2)
c
	call ins( iunit, loc, buf, lrec0, iostat )
	if( iostat.ne.0) go to 9
	if (icode.ne.0) call cconv(icode, buf)
	if(update) then
	  read(buf,130) rsp,pdeg,b0deg,cl0deg,xtod,ytod
	else
	  read(buf,130) rsp1,pdeg1,b0deg1,cl0dg1,xtod1,ytod1
	endif
 130	format(6f12.5,8x)
c
	call ins( iunit, loc, buf, lrec0, iostat )
	if( iostat.ne.0) go to 9
	if (icode.ne.0) call cconv(icode, buf)
	if(update) then
	  read(buf,140) title
	else
	  read(buf,140) title1
	endif
 140	format(a80)
c
	if(msglvl.gt.0 .and. .not.update)
     &  write(lst,*) '  kindmi,mesh,nframi=',kindm1,mesh1,nfram1
	if(msglvl.gt.1 .and. .not.update)
     &  call printm(nbyte1,nx1,ny1,
     &          xs1,ys1,dx1,dy1,ydeg1,
     &          rsp1,pdeg1,b0deg1,cl0dg1,xtod1,ytod1,
     &          tsys1,inst1,wavel1,mfile1,dateo1,stime1,etime1,
     &	    ftag1,title1)
c
c  check data size
c
        if (nx*ny*nbytmi.gt.MMX*MMY*4) then
          write(msg,*)
     &    '(subr.inmh  ) error: data size is too large'
          write(msg,*)
     &	  '     nx,ny,nbytmi=',nx,ny,nbytmi
          write(msg,*)
     &    '     input data size (bytes)=',nx*ny*nbytmi
          write(msg,*)
     &    '     internal buffer size (bytes)=',MMX*MMY*4
          write(msg,*)
     &    ' increase MX and MY in mdata.fc so that',
     &    ' 16*MX*MY >= nx*ny*nbytmi'
          errflg=2
          return
        endif
c
	nx1=ixe-ixs+1
	ny1=iye-iys+1
	if(nx1/nbinx.gt.MX .or.
     &	   ny1/nbiny.gt.MY) then
	  nbinx=(nx1-1)/nxmax +1
	  nbiny=(ny1-1)/nymax +1
	  if(msglvl.gt.0) then
	    write(lst,*)
     &	'(subr.inmh  ) warning: nx or ny too large, nx,ny=',nx1,ny1
	    write(lst,*)
     &	'      binning will be applied, nbinx,y=',nbinx,nbiny
	    write(lst,*)
     &	'      in order to avoid binning, increase MX and MY in'
	    write(lst,*)
     &	'      mdata.fc so that MX>=nx and MY>=ny'
	  endif
	else if(nx1/nbinx.gt.nxmax .or.
     &	        ny1/nbiny.gt.nymax) then
	  nbinx=(nx1-1)/nxmax +1
	  nbiny=(ny1-1)/nymax +1
	  if(msglvl.gt.0) then
	    write(lst,*)
     &	'(subr.inmh  ) warning: nx or ny too large, nx,ny=',nx1,ny1
	    write(lst,*)
     &	'      binning will be applied, nbinx,y=',nbinx,nbiny
	    write(lst,*)
     &	'      in order to avoid binning, increase nxmax,nymax in'
	    write(lst,*)
     &	'      auxdata.dat so that nxmax>=nx and nymax>=ny'
	  endif
	endif
c
	return
c
 9	if(iostat.eq.1) then
	  write(msg,*) '(subr.inmh  ) error: endfile during read'
	else if(iostat.eq.2) then
	  write(msg,*) '(subr.inmh  ) read error'
	endif
	errflg=2
	return
c
c  read magnetogram data
c
	entry inm(iunit,kdest)
c
	lrec0=80
c
	call ins( iunit, loc, buf, lrec0, iostat )
	if(iostat.ne.0) go to 19
	if(icode.ne.0) call cconv(icode, buf)
	read(buf,150) spec,factor,calpha
 150	format(a8,2e12.5,48x)
	call upper(spec)
c
	if(msglvl.gt.1)
     &	  write(lst,*) '(subr.inm   ) spec=',spec,', factor=',factor
c
	lspec1=lspec(spec)
	if(lspec1.le.0) then
	  write(lst,*)
     &	'(subr.inm   ) warning: invalid spec in file, spec=',spec
	  k=0
	else
	  factm(lspec1)=factor
	  k=kdest(lspec1)
	  alloc=.false.
	  if(k.eq.0) k=newk(lspec1, alloc)
	  if(k.gt.0) then
	    if(lspecs(k).gt.lspec1) k=0
c
c  lspec(BX) .gt. lspec(BXP)
c  so that when both BX and BXP exist in the file, and read request
c  is made for BX/BXP, BX is stored while BXP is discarded
c
	  else
	    if(msglvl.gt.0)
     &	    write(lst,*) '(subr.inm   ) data were skipped, spec=',spec
	  endif
	endif
c
	if(msglvl.gt.1)
     &	  write(lst,*) '(subr.inm   ) lspec=',lspec1,',  kdest=',k
c
	if (k.gt.MZ) then
	  write(msg,*) '(subr.inm   ) error: k exceeded limit'
	  errflg=2
	  return
	endif
c
	call allocm(1,16,'inm')
	if (nbytmi.eq.1) then
	  if(errflg.gt.1) then
	    write(msg,*) '(subr.inm   ) error'
	    return
	  endif
	  call in1( iunit, loc, idata1, lrec0, nx0, ny0, MMX, iostat )
	  if(k.gt.0) then
	    if(iostat.ne.0) go to 19
c
c  apply binning if required, and store idata1 into rdata
c
	    zlevel=0.0
	    call bin1(idata1,MMX,ixs,ixe,iys,iye,
     &	              nbinx,nbiny,factor,zlevel,
     &	              rdata(1,1,k),MX,
     &	              xs0,ys0,dx0,dy0,
     &	              nx, ny, xs, ys, dx, dy)
c
	    if(msglvl.gt.2) write(lst,*) 'data first,last=',
     &	       ichar(idata1(1,1)),ichar(idata1(nx0,ny0)),
     &	       rdata(1,1,k),rdata(nx,ny,k)
	  endif
c
	else if (iabs(nbytmi).eq.2) then
	  if(errflg.gt.1) then
	    write(msg,*) '(subr.inm   ) error'
	    return
	  endif
	  call in2( iunit, loc, idata2, lrec0, nx0, ny0, MMX, iostat )
	  if(k.gt.0) then
	    if(iostat.ne.0) go to 19
	    if (nbytmi*myint .lt.0) call swap2(idata2,nx0,ny0,MMX)
c
c  apply binning if required, and store idata2 into rdata
c
	    zlevel=0.0
	    call bin2(idata2,MMX,ixs,ixe,iys,iye,
     &	              nbinx,nbiny,factor,zlevel,
     &	              rdata(1,1,k),MX,
     &	              xs0,ys0,dx0,dy0,
     &	              nx, ny, xs, ys, dx, dy)
c
	    if(msglvl.gt.2) write(lst,*) 'data first,last=',
     &	       idata2(1,1),idata2(nx0,ny0),
     &	       rdata(1,1,k),rdata(nx,ny,k)
	  endif
c
	else if (iabs(nbytmi).eq.4) then
	  if(errflg.gt.1) then
	    write(msg,*) '(subr.inm   ) error'
	    return
	  endif
	  call in4( iunit, loc, idata4, lrec0, nx0, ny0, MMX, iostat )
c
	  if(k.gt.0) then
	    if( iostat.ne.0) go to 19
	    if (nbytmi*myint .lt.0) call swap4(idata4,nx0,ny0,MMX)
c
c  apply binning if required, and store idata4 into rdata
c
	    zlevel=0.0
	    call bin4(idata4,MMX,ixs,ixe,iys,iye,
     &	              nbinx,nbiny,factor,zlevel,
     &	              rdata(1,1,k),MX,
     &	              xs0,ys0,dx0,dy0,
     &	              nx, ny, xs, ys, dx, dy)
c
	    if(msglvl.gt.2)
     &	      write(lst,*) 'data first,last=',
     &	                   idata4(1,1),idata4(nx0,ny0),
     &	                   rdata(1,1,k),rdata(nx,ny,k)
	  endif
c
	endif
	call freem(1,16)
c
	if(k.gt.0) then
	  if(lspecs(k).le.0) then
	    nframe=nframe+1
	    kmem(nframe)=k
	    lspecs(k)=lspec1
	    kloc(lspec1)=k
	  else
c  old data are overwritten
	    lspecs(k)=lspec1
	    call kpack
c  kpack recalculates nframe, kmem and kloc
	  endif
	endif
c
	xe=xs+dx*float(nx-1)
	ye=ys+dy*float(ny-1)
c
	if(msglvl.gt.0 .and. first ) then
	  write(lst,*) '(subr.inm   )'
	  if(nbinx.ne.1.or.nbiny.ne.1)
     &  write(lst,*) 
     &    '  binning: nbinx,nbiny=',nbinx,nbiny
	  if(ixs.ne.1 .or. iys.ne.1 .or. ixe.ne.nx0 .or. iye.ne.ny0)
     &  write(lst,*)
     &    '  sub-area selection: ixs,ixe,iys,iye=',
     &    ixs,ixe,iys,iye
	  write(lst,*) '  nx,ny=',nx,ny
	  write(lst,*) '  xs,ys=',xs,ys
	  write(lst,*) '  xe,ye=',xe,ye
	  write(lst,*) '  dx,dy=',dx,dy
	endif
	first=.false.
	return
c
 19	if(iostat.eq.1) then
	  write(msg,*) '(subr.inm   ) error: endfile during read'
	else if(iostat.eq.2) then
	  write(msg,*) '(subr.inm   ) read error'
	endif
	errflg=2
	return
	end
c
c  shift of origin and binning
c
	subroutine bin1(idata1,ndim,ixs,ixe,iys,iye,
     &	                nbinx,nbiny,factor,zlevel,
     &	                rdata,ndimr,
     &	                xs0,ys0,dx0,dy0,
     &	                nx, ny, xs, ys, dx, dy)
	character*1 idata1(ndim,ndim)
	dimension rdata(ndimr,ndimr)
	include 'sys.fc'
c
	if(nbinx.gt.1 .or. nbiny.gt.1) go to 1
c
c  case of no binning
c
	do 1000 j=iys,iye
	if(dy0.ge.0.0) then
	  jb=j+1-iys
	else
	  jb=iye+1-j
	endif
c
	do 1000 i=ixs,ixe
	if(dx0.ge.0.0) then
	  ib=i+1-ixs
	else
	  ib=ixe+1-i
	endif
c
	rdata(ib,jb)=factor*float(mod(ichar(idata1(i,j))+256,256))
     &	      +zlevel
 1000	continue
c
	nx=ixe+1-ixs
	ny=iye+1-iys
	if(dx0.ge.0.0) then
	  xs=xs0+dx0*float(ixs-1)
	else
	  xs=xs0+dx0*float(ixe-1)
	  dx=abs(dx0)
	endif
	if(dy0.ge.0.0) then
	  ys=ys0+dy0*float(iys-1)
	else
	  ys=ys0+dy0*float(iye-1)
	  dy=abs(dy0)
	endif  
	return
c
c  binning
c
 1	continue
	ixee=ixs-1+((ixe+1-ixs)/nbinx)*nbinx
	iyee=iys-1+((iye+1-iys)/nbiny)*nbiny
	do 2000 j=iys,iyee,nbiny
	if(dy0.ge.0.0) then
	  jb=(j-iys)/nbiny +1
	else
	  jb=(iyee-j)/nbiny +1
	endif
c
	do 2000 i=ixs,ixee,nbinx
	if(dx0.ge.0.0) then
	  ib=(i-ixs)/nbinx +1
	else
	  ib=(ixee-i)/nbinx +1
	endif
c
	d=0.0
	do 2100 jj=1,nbiny
	do 2100 ii=1,nbinx
 2100	d=d+factor*float(mod(ichar(idata1(i+ii-1,j+jj-1))+256,256))
     &	+zlevel
 	rdata(ib,jb)=d/float(nbinx*nbiny)
 2000	continue
c
	nx=(ixee+1-ixs)/nbinx
	ny=(iyee+1-iys)/nbiny
	if(dx0.ge.0.0) then
	  xs=xs0+dx0*float(ixs-1) + (dx0/2.0)*float(nbinx-1)
	  dx=dx0*float(nbinx)
	else
	  xs=xs0+dx0*float(ixee-1) + abs(dx0/2.0)*float(nbinx-1)
	  dx=abs(dx0)*float(nbinx)
	endif
	if(dy0.ge.0.0) then
	  ys=ys0+dy0*float(iys-1) + (dy0/2.0)*float(nbiny-1)
	  dy=dy0*float(nbiny)
	else
	  ys=ys0+dy0*float(iyee-1) + abs(dy0/2.0)*float(nbiny-1)
	  dy=abs(dy0)*float(nbiny)
	endif  
	return
	end
c
	subroutine bin2(idata2,ndim,ixs,ixe,iys,iye,
     &	                nbinx,nbiny,factor,zlevel,
     &	                rdata,ndimr,
     &	                xs0,ys0,dx0,dy0,
     &	                nx, ny, xs, ys, dx, dy)
	integer*2 idata2(ndim,ndim)
	dimension rdata(ndimr,ndimr)
	include 'sys.fc'
c  iufl2=-32000
c  ufl=1.0e-15
c  ufl and iufl2 are in sys.fc and defined by initfc.f
	nufl=0
c
	if(nbinx.gt.1 .or. nbiny.gt.1) go to 1
c
c  case of no binning
c
	do 1000 j=iys,iye
	if(dy0.ge.0.0) then
	  jb=j+1-iys
	else
	  jb=iye+1-j
	endif
c
	do 1000 i=ixs,ixe
	if(dx0.ge.0.0) then
	  ib=i+1-ixs
	else
	  ib=ixe+1-i
	endif
c
	if(idata2(i,j).ne.iufl2) then
	  rdata(ib,jb)=factor*float(idata2(i,j))+zlevel
	else
	  rdata(ib,jb)=ufl
	  nufl=nufl+1
	endif
 1000	continue
c
	nx=ixe+1-ixs
	ny=iye+1-iys
	if(dx0.ge.0.0) then
	  xs=xs0+dx0*float(ixs-1)
	else
	  xs=xs0+dx0*float(ixe-1)
	  dx=abs(dx0)
	endif
	if(dy0.ge.0.0) then
	  ys=ys0+dy0*float(iys-1)
	else
	  ys=ys0+dy0*float(iye-1)
	  dy=abs(dy0)
	endif  
	if(nufl.gt.0 .and. msglvl.gt.1) 
     &	write(lst,*) '(subr.bin2  ) unfilled points=',nufl
	return
c
c  binning
c
 1	continue
	ixee=ixs-1+((ixe+1-ixs)/nbinx)*nbinx
	iyee=iys-1+((iye+1-iys)/nbiny)*nbiny
	do 2000 j=iys,iyee,nbiny
	if(dy0.ge.0.0) then
	  jb=(j-iys)/nbiny +1
	else
	  jb=(iyee-j)/nbiny +1
	endif
c
	do 2000 i=ixs,ixee,nbinx
	if(dx0.ge.0.0) then
	  ib=(i-ixs)/nbinx +1
	else
	  ib=(ixee-i)/nbinx +1
	endif
c
	d=0.0
	do 2100 jj=1,nbiny
	do 2100 ii=1,nbinx
	if(idata2(i+ii-1,j+jj-1).ne.iufl2) then
	  d=d+factor*float(idata2(i+ii-1,j+jj-1))+zlevel
	else
	  rdata(ib,jb)=ufl
	  nufl=nufl+1
	  go to 2000
	endif
 2100	continue
 	rdata(ib,jb)=d/float(nbinx*nbiny)
 2000	continue
c
	nx=(ixee+1-ixs)/nbinx
	ny=(iyee+1-iys)/nbiny
	if(dx0.ge.0.0) then
	  xs=xs0+dx0*float(ixs-1) + (dx0/2.0)*float(nbinx-1)
	  dx=dx0*float(nbinx)
	else
	  xs=xs0+dx0*float(ixee-1) + abs(dx0/2.0)*float(nbinx-1)
	  dx=abs(dx0)*float(nbinx)
	endif
	if(dy0.ge.0.0) then
	  ys=ys0+dy0*float(iys-1) + (dy0/2.0)*float(nbiny-1)
	  dy=dy0*float(nbiny)
	else
	  ys=ys0+dy0*float(iyee-1) + abs(dy0/2.0)*float(nbiny-1)
	  dy=abs(dy0)*float(nbiny)
	endif  
	if(nufl.gt.0 .and. msglvl.gt.1) 
     &	write(lst,*) '(subr.bin2  ) unfilled points=',nufl
	return
	end
c
	subroutine bin4(idata4,ndim,ixs,ixe,iys,iye,
     &	                nbinx,nbiny,factor,zlevel,
     &	                rdata,ndimr,
     &	                xs0,ys0,dx0,dy0,
     &	                nx, ny, xs, ys, dx, dy)
c
	include 'sys.fc'
	integer*4 idata4(ndim,ndim)
	dimension rdata(ndimr,ndimr)
c  iufl4=-2 140 000 000
c  ufl=1.0e-15
c  ufl and iufl4 are in sys.fc and defined by initfc.f
c
	nufl=0
c
	if(nbinx.gt.1 .or. nbiny.gt.1) go to 1
c
c  case of no binning
c
	do 1000 j=iys,iye
	if(dy0.ge.0.0) then
	  jb=j+1-iys
	else
	  jb=iye+1-j
	endif
c
	do 1000 i=ixs,ixe
	if(dx0.ge.0.0) then
	  ib=i+1-ixs
	else
	  ib=ixe+1-i
	endif
c
	if(idata4(i,j).ne.iufl4) then
	  rdata(ib,jb)=factor*float(idata4(i,j))+zlevel
	else
	  rdata(ib,jb)=ufl
	  nufl=nufl+1
	endif
 1000	continue
	nx=ixe+1-ixs
	ny=iye+1-iys
	if(dx0.ge.0.0) then
	  xs=xs0+dx0*float(ixs-1)
	else
	  xs=xs0+dx0*float(ixe-1)
	  dx=abs(dx0)
	endif
	if(dy0.ge.0.0) then
	  ys=ys0+dy0*float(iys-1)
	else
	  ys=ys0+dy0*float(iye-1)
	  dy=abs(dy0)
	endif
	if(nufl.gt.0 .and. msglvl.gt.1) 
     &	write(lst,*) '(subr.bin4  ) unfilled points=',nufl
	return
c
c  binning
c
 1	continue
	ixee=ixs-1+((ixe+1-ixs)/nbinx)*nbinx
	iyee=iys-1+((iye+1-iys)/nbiny)*nbiny
	do 2000 j=iys,iyee,nbiny
	if(dy0.ge.0.0) then
	  jb=(j-iys)/nbiny +1
	else
	  jb=(iyee-j)/nbiny +1
	endif
c
	do 2000 i=ixs,ixee,nbinx
	if(dx0.ge.0.0) then
	  ib=(i-ixs)/nbinx +1
	else
	  ib=(ixee-i)/nbinx +1
	endif
c
	d=0.0
	do 2100 jj=1,nbiny
	do 2100 ii=1,nbinx
	if(idata4(i+ii-1,j+jj-1).ne.iufl4) then
	  d=d+factor*float(idata4(i+ii-1,j+jj-1))+zlevel
	else
	  rdata(ib,jb)=ufl
	  nufl=nufl+1
	  go to 2000
	endif
 2100	continue
 	rdata(ib,jb)=d/float(nbinx*nbiny)
 2000	continue
c
	nx=(ixee+1-ixs)/nbinx
	ny=(iyee+1-iys)/nbiny
	if(dx0.ge.0.0) then
	  xs=xs0+dx0*float(ixs-1) + (dx0/2.0)*float(nbinx-1)
	  dx=dx0*float(nbinx)
	else
	  xs=xs0+dx0*float(ixee-1) + abs(dx0/2.0)*float(nbinx-1)
	  dx=abs(dx0)*float(nbinx)
	endif
	if(dy0.ge.0.0) then
	  ys=ys0+dy0*float(iys-1) + (dy0/2.0)*float(nbiny-1)
	  dy=dy0*float(nbiny)
	else
	  ys=ys0+dy0*float(iyee-1) + abs(dy0/2.0)*float(nbiny-1)
	  dy=abs(dy0)*float(nbiny)
	endif
	if(nufl.gt.0 .and. msglvl.gt.1) 
     &	write(lst,*) '(subr.bin4  ) unfilled points=',nufl
	return
	end
c
	subroutine bin4f(fdata4,ndim,ixs,ixe,iys,iye,
     &	                nbinx,nbiny,factor,zlevel,
     &	                rdata,ndimr,
     &	                xs0,ys0,dx0,dy0,
     &	                nx, ny, xs, ys, dx, dy)
c
	include 'sys.fc'
	real*4 fdata4(ndim,ndim)
	dimension rdata(ndimr,ndimr)
c
	if(nbinx.gt.1 .or. nbiny.gt.1) go to 1
c
c  case of no binning
c
	do 1000 j=iys,iye
	if(dy0.ge.0.0) then
	  jb=j+1-iys
	else
	  jb=iye+1-j
	endif
c
	do 1000 i=ixs,ixe
	if(dx0.ge.0.0) then
	  ib=i+1-ixs
	else
	  ib=ixe+1-i
	endif
c
	rdata(ib,jb)=factor*fdata4(i,j)+zlevel
 1000	continue
	nx=ixe+1-ixs
	ny=iye+1-iys
	if(dx0.ge.0.0) then
	  xs=xs0+dx0*float(ixs-1)
	else
	  xs=xs0+dx0*float(ixe-1)
	  dx=abs(dx0)
	endif
	if(dy0.ge.0.0) then
	  ys=ys0+dy0*float(iys-1)
	else
	  ys=ys0+dy0*float(iye-1)
	  dy=abs(dy0)
	endif
	if(nufl.gt.0 .and. msglvl.gt.1) 
     &	write(lst,*) '(subr.bin4  ) unfilled points=',nufl
	return
c
c  binning
c
 1	continue
	ixee=ixs-1+((ixe+1-ixs)/nbinx)*nbinx
	iyee=iys-1+((iye+1-iys)/nbiny)*nbiny
	do 2000 j=iys,iyee,nbiny
	if(dy0.ge.0.0) then
	  jb=(j-iys)/nbiny +1
	else
	  jb=(iyee-j)/nbiny +1
	endif
c
	do 2000 i=ixs,ixee,nbinx
	if(dx0.ge.0.0) then
	  ib=(i-ixs)/nbinx +1
	else
	  ib=(ixee-i)/nbinx +1
	endif
c
	d=0.0
	do 2100 jj=1,nbiny
	do 2100 ii=1,nbinx
	d=d+factor*fdata4(i+ii-1,j+jj-1)+zlevel
 2100	continue
 	rdata(ib,jb)=d/float(nbinx*nbiny)
 2000	continue
c
	nx=(ixee+1-ixs)/nbinx
	ny=(iyee+1-iys)/nbiny
	if(dx0.ge.0.0) then
	  xs=xs0+dx0*float(ixs-1) + (dx0/2.0)*float(nbinx-1)
	  dx=dx0*float(nbinx)
	else
	  xs=xs0+dx0*float(ixee-1) + abs(dx0/2.0)*float(nbinx-1)
	  dx=abs(dx0)*float(nbinx)
	endif
	if(dy0.ge.0.0) then
	  ys=ys0+dy0*float(iys-1) + (dy0/2.0)*float(nbiny-1)
	  dy=dy0*float(nbiny)
	else
	  ys=ys0+dy0*float(iyee-1) + abs(dy0/2.0)*float(nbiny-1)
	  dy=abs(dy0)*float(nbiny)
	endif
c
	return
	end
c
	subroutine printm(nbytmi,nx,ny,xs,ys,dx,dy,ydeg,
     &	            rsp,pdeg,b0deg,cl0deg,xtod,ytod,
     &	            tsys,inst,wavel,mfile,dateo,stime,etime,
     &	            ftag,title)
	character*(*) tsys,inst,mfile,dateo,stime,etime,ftag,title
c
	include 'sys.fc'
c
	xe=xs+dx*float(nx-1)
	ye=ys+dy*float(ny-1)
	write(lst,*) '  nbyte       =',nbytmi
	write(lst,*) '  instrument  =',inst
	write(lst,*) '  wavelength  =',wavel
	write(lst,*) '  fileid      =',mfile
	write(lst,*) '  systime     =',tsys
	write(lst,*) '  dateo,stime,etime=',dateo,',',stime,',',etime
	write(lst,*) '  ftag        =',ftag
	write(lst,*) '  nx,ny       =',nx,ny
	write(lst,*) '  xs,ys       =',xs,ys
	write(lst,*) '  dx,dy,ypos  =',dx,dy,ydeg
	write(lst,*) '  xe,ye       =',xe,ye
	write(lst,*) '  rsp,p,b0,cl0=',rsp,pdeg,b0deg,cl0deg
	write(lst,*) '  xtod,ytod   =',xtod,ytod
	write(lst,*) '  title =',title(1:lenstr(title))
	return
	end
c
c  rebinning of magnetogram given on D-grid
c
	subroutine rebin(dxnew,dynew)
c
	include 'sys.fc'
	include 'fileio.fc'
	include 'coord.fc'
	include 'lparam.fc'
	include 'mparam.fc'
	include 'mdata.fc'
	include 'auxparam.fc'
c
c  ufl=1.0e-15
c  ufl is in sys.fc and defined by initfc.f
c
	eps=1.0e-5
	if(abs(dxnew-dx).lt.eps .and. abs(dynew-dy).lt.eps ) return
c
	if(mesh.gt.0) then
	  write(msg,*) '(subr.rebin ) error: mesh=',mesh
	  errflg=2
	  return
	endif
	if(kindm.eq.1 .or. kindm.eq.6) then
	  write(msg,*) '(subr.rebin ) error: kindm=',kindm
	  errflg=2
	  return
	endif
c
	xe=xs+dx*float(nx-1)
	ye=ys+dy*float(ny-1)
	nxnew = ( xe - xs )/dxnew +1.5
	nynew = ( ye - ys )/dynew +1.5
	if( nxnew.gt.MX .or. nynew.gt.MY ) then
	  write(msg,*)
     &   '(subr.rebin ) error: mesh size too large, new nx,ny=',
     &   nxnew,nynew
	  errflg=2
	  return
	endif
c
	do 1500 iframe=1,nframe
	k=kmem(iframe)
	do 1200 jnew=1,nynew
	do 1200 inew=1,nxnew
	x=xs+dxnew*float(inew-1)
	y=ys+dynew*float(jnew-1)
	call getij(nx,ny,xs,ys,dx,dy,x,y,i,j,xr,yr)
	wdata(inew,jnew)=ufl
	if(i.ge.1.and.i.le.nx-1.and.j.ge.1.and.j.le.ny-1) then
	  b1=rdata(i,  j,  k)
	  b2=rdata(i+1,j,  k)
	  b3=rdata(i+1,j+1,k)
	  b4=rdata(i,  j+1,k)
	  if(b1.ne.ufl .and. b2.ne.ufl .and. b3.ne.ufl .and. b4.ne.ufl)
     &     wdata(inew,jnew)=
     &     b1+(b2-b1)*xr+(b4-b1)*yr+(b3-b4-b2+b1)*xr*yr
	endif
 1200	continue
c
	do 1300 jnew=1,nynew
	do 1300 inew=1,nxnew
 1300	rdata(inew,jnew,k)=wdata(inew,jnew)
 1500	continue

	nx=nxnew
	ny=nynew
	dx=dxnew
	dy=dynew
	xenew=xs+dxnew*float(nxnew-1)
	yenew=ys+dynew*float(nynew-1)
	x1lim0=xs
	x2lim0=xenew
	y1lim0=ys
	y2lim0=yenew
	x1lim=x1lim0+x1del
	x2lim=x2lim0+x2del
	y1lim=y1lim0+y1del
	y2lim=y2lim0+y2del
c  z1lim0 and z2lim0 are not altered
c
	if(msglvl.gt.0) write(lst,*) 
     &  '(subr.rebin ) new dx,dy,nx,ny=',dx,dy,nx,ny
	return
	end
