c  How to prepare data: a sample program
c
c  This sample program describes how to prepare data in the standard
c  format of the MAGPACK2 package. Although the format is described
c  in detail in the document (doc/data.doc), to count the number of
c  bytes and precisely format the output is a cumbersome task.
c  In the following example, the data that are stored in a file
c  as a simple unformatted array is read, and the parameter values
c  are given in this program explicitly. Then the result is written
c  to a file with the standard format.
c
	include 'sys.fc'
	include 'mdata.fc'
	include 'mparam.fc'
	include 'coord.fc'
	include 'solar.fc'
	include 'fileio.fc'
	include 'fcntl.fc'
	include 'string.fc'
	include 'auxparam.fc'
	integer uin,uout
	character*80 fname1,fname2
	dimension ibz(50,50),ibx(50,50),iby(50,50)
c
	logical alloc
c
	keyin  = 5
	stdin  = 10
	lst    = 6
	msg    = 6
	errflg = 0
	msglvl=1
	uin=11
	uout=12
	lrec0=80
	alloc=.true.
c
	include 'sysconf.cfg'
	call initfc
	if(mycode.eq.0 .or. myint.eq.0) stop
	nbytem=sign(2,myint)
c
c   ***************************************************
c   * You may customize the program below this line.  *
c   ***************************************************
c
c  Firstly, the data are read. The sample is supposed to be created by
c  the program 'gendata' included in this package. You may change
c  this part according to your own form of data.
c
c  First, specify the name of the data file from the keyboard.
c
	write(lst,*) 'enter input file name '
	read(keyin,'(a)') fname1
c
c  Open the file and read the data
c
	open(uin, form='unformatted', file=fname1)
	read(uin) title
	read(uin) nx,ny,xs,ys,dx,dy,rsp,p,b0
	read(uin) ((ibz(i,j),i=1,nx),j=1,ny)
c
c  Here, 'title' is a string of 80 characters describing the contents
c  of this file. Two integers (nx,ny) specify the number of mesh points,
c  (xs,ys) is the coordinate of the lower-left corner of the observed
c  region, and (dx,dy) is the grid spacing. The length is usually 
c  given in units of 10**4 km, but any units can be used. 'rsp' is the
c  radius of the sun in the specified unit. 'p' and `b0' designate the
c  orientation of the rotation axis of the sun under the usual definition.
c  The units of 'p' and 'b0' are degree. Finally, an integer array
c  ibz is read, which contains the measured longitudinal component of
c  the magnetic field, in units of Gauss.
c
c  If your data also contains transverse fields, you may read them
c  for example,
c
c*	read(uin) ((ibx(i,j),i=1,nx),j=1,ny)
c*	read(uin) ((iby(i,j),i=1,nx),j=1,ny)
c
c  If you want, you may change the parameter values.
c
c*	title='New Title'
c*	nx=50
c*	ny=50
c*	dx=0.5
c*	dy=0.5
c*	xs=-dx*0.5*float(nx-1)
c*	ys=-dy*0.5*float(ny-1)
c*	rsp=69.5
c*	pdeg=0.0
c*	b0deg=0.0
c
c  Here, you specify the parameters.
c
c  'mesh' specifies the type of mesh used to describe the data.
c  Here the data are supposed to be an observed magnetogram.
c  In such a case, mesh=0.
c
	mesh=0
c
c  'kindm' describes the type of the magnetogram. For a longitudinal
c  magnetogram, kindm=1. For a vector magnetogram, kindm=2.
c
	kindm=1
c
c  'ydeg' is the position angle of the y-axis of the data. If the 
c  y-axis is directed toward north, ydeg=0
c
	ydeg=0.0
c
c  'cl0deg' is the Carringron longitude of the central meridian.
c  This value is not used (currently) in the field-line computation,
c  so that simply set it to zero.
c
	cl0deg=0.0
c
c  'tsys' (up to 6 characters) designates the time zone that is used
c  to describe the following observing times.
c  
	tsys='GMT+09'
c
c  'inst' (up to 8 characters) is the name of the instrument that
c  produced the data.
c
	inst='DUMMY'
c
c  'wavel' is a floating point number indicating the observing
c  wavelength.
c
	wavel=6302.5
c
c  'mfile' (up to 8 characters) gives a brief description of the data
c  contents.
c
	mfile='TESTDATA'
c
c  'dateo' is the observing data, 'stime' and 'etime' are starting
c  and ending times of the observation, respectively.
c
	dateo='97/01/01'
	stime='12:00:00'
	etime='12:02:00'
c
c  Enter the name of the output file from the keyboard.
c
	write(msg,*) 'enter the name of output file '
	read(keyin,'(a)') fname2  
c
	call openfl(mgram,'MGRAM',uout,fname2,export,wrt,lrec0)
c
	xxsp(1)=0.0
	xxsp(2)=0.0
	xxsp(3)=-rsp
	call setdu
	xtod=0.0
	ytod=0.0
c
c  The output data will be prepared on an array:
c       rdata(i,j,k) (i=1-nx, j=1-ny).
c  To which k-number the data are to be stored is automatically
c  determined by the program.
c 
c  Obtain the k-value for BL (longitudinal magnetogram)
c
	kz=newk(lspec('BL'), alloc)
c
c  In the present case, BL is the only data so that the actual value
c  of kz would be 1.
c
c  If you have other data, for example transverse fields, you may
c  also obtain k-values for BX and BY.
c
c*	kx=newk(lspec('BX'), alloc)
c*	ky=newk(lspec('BY'), alloc)
c
c  Specify the scaling factor. The data written to a file are in
c  integer, and the real value is real=factor*integer. The factor
c  is given here. 
c
	factm(lspecs(kz))=1.0
c
c  If you have BX and BY, also give the following.
c
c*	factm(lspecs(kx))=1.0
c*	factm(lspecs(ky))=1.0
c
c  generate magnetogram
c
	do 20 j=1,ny
	do 20 i=1,nx
	rdata(i,j,kz)=float(ibz(i,j))/factm(lspecs(kz))
c*	rdata(i,j,kx)=float(ibx(i,j))/factm(lspecs(kx))
c*	rdata(i,j,ky)=float(iby(i,j))/factm(lspecs(ky))
 20	continue
c
	kindmo = kindm
	call writem (uout)
c
	stop
	end
