c  subroutine to calculate solar parameters by
c  Almanac for Computors (U.S.Naval Observatory)
c  originally written by K.Shibasaki (NRO/NAOJ)
c
      subroutine suncal(tsys,dateo,timeo,dist,rsun,p,b0,cl0,doy,
     &	                carrot)
c
c  input:
c  tsys = time zone
c  dateo= date of observation
c  timeo= time of observation
c
c  output:
c  dist  = distance in au
c  rsun  = semidiameter in arcsec
c  p     = position angle of sun's pole in deg.
c  b0    = heliographic latitude of disk center in deg.
c  cl0   = Carrington longitude of the meridian in deg.
c  doy   = day of year
c  carrot= Carrington rotation number
c
	include 'sys.fc'
c
	real*8 rd,ut,t1,t,tt,gml,g,tgl,rlg,e,ai,omg,
     &	       cbclm,cbslm,xlm,dcl0
	character tsys*6,dateo*8,timeo*8,tntbl(25)*6,tdtbl(25)*6,
     &	tsysu*6
	integer carrot
	dimension iut(6)
c
	data tntbl/'GMT-12', 'GMT-11', 'HST', 'YST', 'PST',
     &	    'MST', 'CST', 'EST', 'AST', 'GMT-3',
     &	    'GMT-2', 'GMT-1', 'GMT', 'MET', 'EET',
     &	    'GMT+3', 'GMT+4', 'GMT+5', 'GMT+6', 'GMT+7',
     &	    'WST', 'JST', 'EST', 'GMT+11', 'NZST' /
c  daylight saving time
	data tdtbl/'***-12', '***-11', 'HDT', 'YDT', 'PDT',
     &	     'MDT', 'CDT', 'EDT', 'ADT', '***-3',
     &	     '***-2', '***-1', 'BST', 'METDST', 'EETDST',
     &	     '***+3', '***+4', '***+5', '***+6', '***+7',
     &	     '***+8', '***+9', '***+10', '***+11', 'NZDT'/
	if(dateo(3:3).eq.'/') then
	  read(dateo,'(i2,1x,i2,1x,i2)') iut(1),iut(2),iut(3)
	  if(iut(1).ge.50.and.iut(1).lt.100) then
	    iut(1)=iut(1)+1900
	  else if(iut(1).lt.50) then
	    iut(1)=iut(1)+2000
	  endif
	else
	  read(dateo,'(i4,i2,i2)') iut(1),iut(2),iut(3)
	endif
	read(timeo,'(i2,1x,i2,1x,i2)') iut(4),iut(5),iut(6)
c
	idif=0
	tsysu=tsys
	call upper(tsysu)
	if(tsysu(1:2).eq.'UT') then
	  read(tsysu,'(2x,i4)') idif
	else if(tsysu(1:3).eq.'GMT') then
	  read(tsysu,'(3x,i3)') idif
	else
	  do 10 idif=-12,12
	  if(tsysu.eq.tntbl(idif+13)) go to 1
 10	  continue
	  do 20 idif=-11,13
	  if(tsysu.eq.tdtbl(idif+12)) go to 1
 20	  continue
	  write(lst,*)
     &	'(subr.suncal) warning: unknown timezone(',tsys,'), assume UT'
	  idif=0
	endif
 1	continue
c
	rd=3.14159265358979d0/180.0d0
	iyear=iut(1)
	month=iut(2)
	iday =iut(3)
	ihour=iut(4)-idif
	min =iut(5)
	isec =iut(6)
	ut=(dble(isec)/60.0d0+dble(min))/60.0+dble(ihour)
	it1=367*(iyear-1900)
	it2=((month+9)/12+iyear)*7/4
	it3=275*month/9
	t1=dble(it1-it2+it3+iday)+ut/24.0d0
c  t1 = JD -2418313.5
	t =(t1+3293.5d0)/36525.0d0
c  t = (JD -2415020)/36525
	tt=t1+20093.5d0
c  tt = JD -2398220
	gml=(279.691d0 + 36000.769d0*t)*rd
	g =(358.476d0 + 35999.050d0*t)*rd
	tgl=gml+(1.919d0-0.0048d0*t)*rd*dsin(g)
     &	 +0.020d0*rd*dsin(2.0d0*g)
	rlg=0.00003d0+(0.000018d0*t-0.00727d0)*dcos(g)
     &	 -0.00009d0*dcos(2.0d0*g)
	dist =10.0d0**(rlg)
	rsun =961.18d0/dist
	e =23.441d0*rd
	ai =7.25d0*rd
	omg=(73.739167d0 + 0.013958d0*tt/365.25d0)*rd
	b0 =dasin(dsin(tgl-omg)*dsin(ai))
	cbclm=-dcos(tgl-omg)
	cbslm=-dsin(tgl-omg)*dcos(ai)
	xlm=datan2(cbslm,cbclm)
	itgl=int( ( (tgl-omg)/rd) /360.0d0)
	dcl0=(360.0d0-360.0d0/25.38d0*tt)*rd+xlm+itgl*360.0d0*rd
	p =-datan(dcos(tgl)*dtan(e))-datan(dcos(tgl-omg)*dtan(ai))
	b0=b0/rd
	dcl0=dcl0/rd
	crot=dcl0/360.0d0
	cl0=dcl0-360.0d0*float(int(crot)-1)
	carrot=-crot-42
	p =p /rd
	idoy1=(month+9)/12*(1+(mod(iyear,4)+2)/3)
	doy =float(it3-idoy1+iday-30)+ut/24.0
c
	return
	end
c
	subroutine carsub(fyear,rotno,dcl0)
c
c  copied from suncal.f
c  fyear is the year and date expressed in 365.25-day year
c  rotno = Carrington rotation number
c  dcl0 = Carrington longitude of the central meridian
c
	integer rotno
	real*8 fyear,rd,t1,t,tt,gml,g,tgl,ai,omg,
     &	       cbclm,cbslm,xlm,dcl0
c
	rd=3.14159265358979d0/180.0d0
	t1 = (fyear-1900.0d0)*365.25d0 - 3293.5d0
c t1 = JD -2418313.5
        t =(t1+3293.5d0)/36525.0d0
c t = (JD -2415020)/36525
	tt=t1+20093.5d0
c tt = JD -2398220
	gml=(279.691d0 + 36000.769d0*t)*rd
	g =(358.476d0 + 35999.050d0*t)*rd
	tgl=gml+(1.919d0-0.0048d0*t)*rd*dsin(g)+
     &	    0.020d0*rd*dsin(2.0d0*g)
	ai =7.250d0*rd
	omg=(73.739167d0+0.013958d0*tt/365.25d0)*rd
	cbclm=-dcos(tgl-omg)
	cbslm=-dsin(tgl-omg)*cos(ai)
	xlm=datan2(cbslm,cbclm)
	itgl=int( ((tgl-omg)/rd) /360.0d0)
        dcl0=(360.0d0-360.0d0/25.38d0*tt)*rd+xlm+itgl*360.0d0*rd
	dcl0=dcl0/rd
	lc=int(dcl0/360.0d0)
	dcl0=dcl0-360.0d0*(lc-1)
	rotno=-lc-42
	return
	end
c
	subroutine carcmp(rotno, date, time)
c
	include 'sys.fc'
	integer rotno,nod(12),bom(12)
	real*8 fyear1,fyear2,fyear,cl1,cl2,cl0
	character date*8,time*8
	data nod/31,28,31,30,31,30,31,31,30,31,30,31/
c
c  guess
	cl0=180.0d0
	fyear1 = dble(rotno)*0.0744663d0 + 1854.1628d0
 1	call carsub(fyear1,no1,cl1)
	if(no1.eq.rotno) then
  	  if(cl1.lt.cl0) then
	    fyear2=fyear1-1.0d0/365.0d0
	    call carsub(fyear2,no2,cl2)
	    if(no2.ne.no1) then
	      write(msg,*) '(subr.carcmp) error'
	      errflg=2
	      return
	    endif
	    fyear=fyear1-(fyear2-fyear1)/(cl2-cl1)*(cl1-cl0)
	  else
	    fyear2=fyear1+1.0d0/365.0d0
	    call carsub(fyear2,no2,cl2)
	    if(no2.ne.no1) then
	      write(msg,*) '(subr.carcmp) error'
	      errflg=2
	      return
	    endif
	  fyear=fyear1-(fyear2-fyear1)/(cl2-cl1)*(cl1-cl0)
	  endif
	else
  	  fyear1 = fyear1 - dble(no1-rotno)*0.0744663d0
 	  go to 1
	endif
c
	iyear=int(fyear)
	doy=365.25d0*(fyear-dble(iyear))+1.0d0
	idoy=int(doy)
	nod(2)=28
	if (mod(iyear,100).eq.0.and.mod(iyear,400).ne.0) then
	 nod(2)=28
	else if (mod(iyear,4).eq.0) then
	  nod(2) = 29
	endif
c
	bom(1)=1
	do 10 m=2,12
 10	bom(m)=bom(m-1)+nod(m)
c
	do 20 month=12,1,-1
	if(idoy.ge.bom(month)) go to 21
 20	continue
	month=1
 21	continue
	day = doy - bom(month) +1
	iday = int(day)
	ihour = int(24.0*(day-float(iday)))
c
	write(date,'(i4,i2.2,i2.2)') iyear,month,iday
	write(time,'(i2.2,6h:00:00)') ihour
	return
	end
