c
c  vectorization of pseudo-vector magnetograms
c  in referring to the potential field vectors (mode=1) or
c  to the linear (constant-alpha) force-free fields (mode=2,3)
c
c  this routine performs mesh conversion but
c  does not perform vector component conversion
c
c  this routine should be called with:
c
c  (1) mesh=0,-1 and mesho=0,-1
c
c  (1a) kindm =0???a (a>=2)
c              BL,(pseudo-)BT
c       kindmo=0??bc (c=3,      7,           )
c                       BL,BT   BL,BT,BLP,BTP
c       (b=0, 1,  4)
c            JL,JL&FL
c
c  (1b) kindm =0???a (a>=1)
c              BL (at least)
c       kindmo=0??0c (c=5,      6)
c                       BL,BTP  BL,pseudo-BT,BTP
c
c  (2) mesh=1, mesho=1
c
c  (2a) kindm =1???a (a>=3)
c              BN,BH
c       kindmo=1??bc (c=3,      7            )
c                       BN,BH   BN,BH,BNP,BHP
c       (b=0, 2, 5)
c            JN,JN&FN
c
c  (2b) kindm =1???a (a>=1)
c              BN (at least)
c       kindmo=1??05
c              BN,BHP
c
c  (3) mesh=0,-1 and mesho=-2  --- in preparation
c
c  (3a) kindm =0???a (a>=2)
c              BL,(pseudo-)BT
c       kindmo=0??bc (c=3,      7,           )
c                       BL,BT   BL,BT,BLP,BTP
c       (b=0, 3,  6)
c            JR, JR&FR
c
c  (3b) kindm =0???a (a>=1)
c              BL (at least)
c       kindmo=0??0c (c=5,      6)
c                       BL,BTP  BL,pseudo-BT,BTP
c
	subroutine vectr1(mode)
c
c  mode=1: use current-free field
c  mode=2: use linear (constant-alpha) force-free field
c          optimum alpha is determined and stored as calpha in mparam.fc
c  mode=3: use linear force-free field with given value of calpha
c
	parameter (NALPHA=20)
c
	include 'sys.fc'
	include 'coord.fc'
	include 'mdata.fc'
	include 'mparam.fc'
	include 'fileio.fc'
	include 'auxparam.fc'
c
	dimension amat(3,3)
	dimension bxs(MX,MY),bys(MX,MY),bzs(MX,MY),
     &	badfit(MX,MY),unsmth(MX,MY),flips(MX,MY)
	dimension cosb2(NALPHA),alphas(NALPHA)
	logical alloc,tight
c
	equivalence (bxs(1,1), qdata(1,1,1)),
     &	      (bys(1,1), qdata(1,1,2)),
     &	      (bzs(1,1), qdata(1,1,3)),
     &	      (badfit(1,1),qdata(1,1,4)),
     &	      (unsmth(1,1),qdata(1,1,5)),
     &	      (flips (1,1),qdata(1,1,6))
c
	if(msglvl.gt.2) write(lst,*) '(subr.vectr1) start'
c
c  check parameters
c
	if(kindmo.eq.0) kindmo=kindm
	call umake( kindm,  m5   ,m4   ,m3   ,m2   ,m1   )
	call umake( kindmo, m5new,m4new,m3new,m2new,m1new)
	if(m2new.gt.3) then
	  m2new=4+iabs(mesho)
	else if(m2new.gt.0) then
	  m2new=1+iabs(mesho)
	endif
	kindmo=make(m5,m4new,m3new,m2new,m1new)
c
	if(mesh.gt.1) then
	  write(msg,*) '(subr.vectr1) error: mesh=',mesh
	  errflg=2
	  return
	endif
c
	if(mesho.gt.1) then
	  write(msg,*) '(subr.vectr1) error: mesho=',mesh
	  errflg=2
	  return
	endif
c
	if(mesh.eq.1 .and. mesho.ne.1) then
	  write(msg,*)
     &	   '(subr.vectr1) error: mesho=',mesho,' :should be 1'
	  errflg=2
	endif
c
	if(mesh.le.0 .and. mesho.le.0 .and. m5.ne.0) then
	  write(msg,*) '(subr.vectr1) error: kindm=',kindm
	  errflg=2
	  return
	endif
c
	if(m1.eq.1 .and. 
     &	((m1new.ne.5 .and. m1new.ne.6) .or. m2new.ne.0)) then
	  write(msg,*) '(subr.vectr1) error: kindm,kindmo=',
     &	    kindm,kindmo
	  errflg=2
	  return
	endif
c
	if(mesh.eq.1 .and. mesho.eq.1 .and. m5.ne.1) then
	  write(msg,*) '(subr.vectr1) error: kindm=',kindm
	  errflg=2
	  return
	endif
c
	if(msglvl.gt.2) write(lst,*) '(subr.vectr1) end of param.check'
c
	kx=kxloc()
	ky=kyloc()
	kz=kzloc()
	kxp=0
	kyp=0
	kzp=0
	kj=kjloc()
	kf=kfloc()
	alloc=.true.
c
	if(kz.eq.0 .or.
     &	  (kx.eq.0 .or. ky.eq.0) .and. m1.ne.1) then
	  write(msg,*) '(subr.vectr1) error: kx,ky,kz=',kx,ky,kz
	  errflg=2
	  return
	endif
c  if kx=ky=0 and m1=1, only the energy of the potential field is
c  evaluated.
c
	if((kx.eq.0 .or. ky.eq.0) .and. mode.eq.2) then
	  write(msg,*) '(subr.vectr1) mode changed to 1 because kx=ky=0'
	  mode=1
	endif
c
	if(m5.eq.0) then
	  kxp=newk(lspec('BXP'), alloc)
	  kyp=newk(lspec('BYP'), alloc)
	  if(kxp.eq.0 .or. kyp.eq.0) then
	    write(msg,*) '(subr.vectr1) error: kxp,kyp=',kxp,kyp
	    write(msg,*) '                     nframe,mz=',nframe,MZ   
	    errflg=2
	    return
	  endif
	endif
c
	if(m5.eq.1) then 
	  kxp=newk(lspec('BHXP'), alloc)
	  kyp=newk(lspec('BHYP'), alloc)
	  if(kxp.eq.0 .or. kyp.eq.0) then
	    write(msg,*) '(subr.vectr1) error: kxp,kyp=',kxp,kyp
	    write(msg,*) '                     nframe,mz=',nframe,MZ   
	    errflg=2
	    return
	  endif
	endif
c
	if((m2new.eq.1 .or. m2new.eq.4) .and. 
     &	  (m2.ne.1 .and. m2.ne.4) ) 
     &	    kj=newk(lspec('JL'), alloc)
c
	if( m2new.eq.4 .and. m2.ne.4 ) 
     &	    kf=newk(lspec('FL'), alloc)
c
	if((m2new.eq.2 .or. m2new.eq.5 ) .and.
     &	  (m2.ne.2 .and. m2.ne.5) ) 
     &	    kj=newk(lspec('JN'), alloc)
c
	if( m2new.eq.5 .and. m2.ne.5 ) 
     &	    kf=newk(lspec('FN'), alloc)
c
	if((m2new.eq.3 .or. m2new.eq.6) .and.
     &	  (m2.ne.3 .and. m2.ne.6) ) 
     &	    kj=newk(lspec('JR'), alloc)
c
	if( m2new.eq.6 .and. m2.ne.6 ) 
     &	    kf=newk(lspec('FR'), alloc)
c
	kzp=kz
c
	if(m5.eq.0. .and. (m1new.eq.7 .or. m1new.eq.8) .and.
     &	m1.ne.7 .and. m1.ne.8 ) then
	  kzp=newk(lspec('BLP'), alloc)
	  if(kzp.eq.0) then
	    write(msg,*) '(subr.vectr1) error: kzp=',kzp
	    write(msg,*) '                     nframe,mz=',nframe,MZ   
	    errflg=2
	    return
	  endif
	endif
c
	if(m5.eq.1 .and.
     &	(m1new.eq.5 .or. m1new.eq.7 .or. m1new.eq.8) .and.
     &	(m1.ne.5 .and. m1.ne.7 .and. m1.ne.8) ) then
	  kzp=newk(lspec('BNP'), alloc)
	  if(kzp.eq.0) then
	    write(msg,*) '(subr.vectr1) error: kzp=',kzp
	    write(msg,*) '                     nframe,mz=',nframe,MZ   
	    errflg=2
	    return
	  endif
	endif
c
	if(kzp.eq.kz) then
	  if(m5.eq.0) then
	    kloc(lspec('BLP'))=kz
	  else if(m5.eq.1) then
	    kloc(lspec('BNP'))=kz
	  endif
	endif
c
c  scalar maps: k1=div2d(B), k2=unsmoothness, k3=flips
c
	k1=0
	k2=0
	k3=0
	if( m4new.eq.4 .or. m4new.eq.5 .or. 
     &    m4new.eq.6 .or. m4new.eq.7 ) then
	  k1=newk(lspec('1S'), alloc)
	  k2=newk(lspec('2S'), alloc)
	  k3=newk(lspec('3S'), alloc)
	  if( k1.eq.0. or. k2.eq.0 .or. k3.eq.0) then
	    write(msg,*) '(subr.vectr1) error: k1,k2,k3=',k1,k2,k3
	    errflg=2
	    return
	  endif
	endif
c
	if(msglvl.gt.2) then
	  write(lst,*) '(subr.vectr1) end of alloc'
	  write(lst,*) '  kx,ky,kz,kxp,kyp=',kx,ky,kz,kxp,kyp
	  write(lst,*) '  kj,kf,k1,k2,k3=',kj,kf,k1,k2,k3
	endif
c
	if(btlim.le.0.0) then
	  write(msg,*) '(subr.vectr1) error: btlim=',btlim
	  errflg=2
	  return
	endif
c
	nx2=2*nx
	ny2=2*ny
	call setdu
	call settu
c  subr.settu sets at-matrix
c
	if(mesho.eq.-1) then
	  do 1400 jj=1,3
	  do 1400 ii=1,3
 1400	  amat(ii,jj)=atmat(ii,jj)
	else
	  do 1500 jj=1,3
	  do 1600 ii=1,3
 1600	  amat(ii,jj)=0.0
 1500	  amat(jj,jj)=1.0
	endif
c
c  amat=unit matrix if (mesh,mesho)=(1,1), (0,0) or (-1,0)
c  amat=atmat if (mesh,mesho)=(0,-1) or (-1,-1)
c  (if mesh=-1, FFT is done as if mesh=0)
c
	if(mode.eq.1) then
c  potential field bxp,byp by Fourier method
c  results are returned in rdata(*,*,kxp) and rdata(*,*,kyp)
c
 4000	  call cff1(kz,amat,kxp,kyp)
	  if(errflg.gt.1) return
	  if(kx.eq.0 .or. ky.eq.0) go to 3000
	else
c
c  linear (constant-alpha) force-free field bxp,byp by Fourier method
c  results are returned in rdata(*,*,kxp) and rdata(*,*,kyp)
c
c  get maximum allowable value of alpha
	  tight=.true.
	  call mxalph(amat,alpham,tight)
	  if(mode.eq.3 .and. abs(calpha).gt.alpham) then
	    write(msg,*) '(subr.vectr1) given alpha exceeded limit'
	    write(msg,*) '     alpha=',calpha,'  max=',alpham
	    errflg=1
	    return
	  endif
c
	  if(mode.eq.3) then
	    call lff1(kz,amat,kxp,kyp,calpha,tight)
	  else
	    do 4100 j=1,ny
	    do 4100 i=1,nx
	    if(rdata(i,j,kx)**2+rdata(i,j,ky)**2.gt.btlim**2) go to 4200 
 4100	    continue
	    write(msg,*) 
     &	 '(subr.vectr1) no datapoints with bt>btlim, use cff1'
	    go to 4000
 4200	    continue
c
	    na=5
	    do 4300 n=1,2*na+1
	    calpha=alpham*float(n-na-1)/float(na+1)
	    alphas(n)=calpha
	    call lff1(kz,amat,kxp,kyp,calpha,tight)
	    aux=0.0
	    auxm=1.0e10
	    m=0
	    do 4400 j=1,ny
	    do 4400 i=1,nx
	    if(rdata(i,j,kx)**2+rdata(i,j,ky)**2.lt.btlim**2) go to 4400 
	    aux=aux+ 
     &	(rdata(i,j,kx)*rdata(i,j,kxp)+
     &	 rdata(i,j,ky)*rdata(i,j,kyp))**2
     &	/(rdata(i,j,kx)**2+rdata(i,j,ky)**2)
     &	/(rdata(i,j,kxp)**2+rdata(i,j,kyp)**2)
	    m=m+1
 4400	    continue
 	    cosb2(n)=aux/float(m)
	    if(cosb2(n).lt.auxm) auxm=cosb2(n)
 4300	    continue
	    aux1=0.0
	    aux2=0.0
	    do 4500 n=1,2*na+1
	    aux1=aux1+alphas(n)*(cosb2(n)-auxm)
	    aux2=aux2+cosb2(n)-auxm
 4500	    continue
	    calpha = aux1/aux2
c  alpha1 is the center of gravity of cosb2 curve
	    if(msglvl.gt.0) then
	      write(lst,*) '(subr.vectr1) best-fit value of alpha=',calpha
	      write(lst,*) '   alpha-max=',alpham
	    endif
	  endif
	endif
c
c  flip the direction of bt according to btp
c
	call allocm(1,6,'vectr1')
	if(errflg.gt.1) then
	  write(msg,*) '(subr.vectr1) error'
	  return
	endif
c
c  set tuning parameters
	methdv = apint(1)
	nitrv  = apint(2)
	nsize  = apint(3)
	apot   = apreal(1)
	wuns   = apreal(2)
	wflp   = apreal(3)
c
	itrc=1
	if(m1.eq.4 .or. m1.eq.8 .or. nitrv.eq.0) itrc=0
c  no flipping made if itrc=0
c
	call flip(kx,ky,kxp,kyp,bxs,bys,badfit,unsmth,flips, MX, 
     &	    itrc,nflip,nflip1,uns)
	if(msglvl.gt.1)
     &	write(lst,100) nflip,nflip1,uns
 100	format(1x,'(subr.vectr1) nflip(total,max),unsmoothness=',
     &	 i6,i4,1pe11.3)
	if(nitrv.lt.2) go to 2900
	uns0=uns
c
c  flip the direction of bt according to the surrounding bt-vectors
c
	itrmin=5
	do 2400 itrc=2,nitrv
	if(nflip.eq.0) go to 2900
	call flip(kx,ky,kxp,kyp,bxs,bys,badfit,unsmth,flips, MX,
     &	    itrc,nflip,nflip1,uns)
	if(msglvl.gt.1) write(lst,110) nflip,nflip1,uns
 110	format(1x,'              nflip(total,max),unsmoothness=',
     &	 i6,i4,1pe11.3)
	if(uns.gt.uns0 .and. itrc.gt.itrmin ) then
	  if(msglvl.gt.1) write(lst,'(1x,a)')
     &	    '              unsmoothness increased'
	  go to 2500
	endif
	uns0=uns
 2400	continue
	if(msglvl.gt.0) write(lst,*) 
     &	'(subr.vectr1) iteration not converged: nitrv=', nitrv
 2500	continue
	if(msglvl.gt.0) then
	  write(lst,110) nflip,nflip1,uns
	  write(lst,*) 'method_v,nsize=',methdv,nsize
	  write(lst,*) 'apot,wuns,wflp=',apot,wuns,wflp
	endif
 2900	continue
c
c  vectorization completed
c
c  store unsmth, and flips if necessary
c
	if(k2.gt.0 .and. k3.gt.0) then
	  do 3200 j=1,ny
	  do 3200 i=1,nx
	  rdata(i,j,k2)=unsmth(i,j)
	  rdata(i,j,k3)=flips (i,j)
 3200	  continue
	endif
c
 3000 continue
c
c  store bzp in rdata if necessary
c
	if(kzp.gt.0 .and. kzp.ne.kz) then
	  do 3300 j=1,ny
	  do 3300 i=1,nx
	  l=i+nx2*(j-1)
	  rdata(i,j,kzp)=rdata(i,j,kz)
 3300	  continue
	endif
c
	mesh=mesho
	kindm=make(m5,m4new,m3,m2,m1new)
c
c  calculation of electric currents
c
	if(m2new.ne.0 .or. k1.ne.0)
     &	  call curlb1(kx,ky,kz,kxp,kyp, kj,kf,k1,amat,
     &	       bxs,bys,bzs,
     &	       qdata(1,1,5),qdata(1,1,9),qdata(1,1,13), MX)
c  qdata(*,*,5-8), qdata(*,*,9-12), qdata(*,*,13-16)
c  are passed to curlb1 as working space
c
	kindm=make(m5,m4new,m3,m2new,m1new)
 9000	continue
c
	if(msglvl.gt.0) write(lst,*) 
     &  '(subr.vectr1) mesho,kindmo=',mesho,kindmo
	call freem(1,6)
	return
	end
c
c  swall =t -- search all points for flipping
c        =f -- search points with badfit>1
c
	subroutine flip(kx,ky,kxp,kyp,refx,refy,
     &	                badfit,unsmth,flips,ndim,
     &	                itrc,nflip,nflip1,uns)
c
	include 'sys.fc'
	include 'mdata.fc'
	include 'mparam.fc'
	include 'auxparam.fc'
	parameter (NDIMW=10)
c
	dimension refx(ndim,ndim),refy(ndim,ndim),
     &	    badfit(ndim,ndim),unsmth(ndim,ndim),flips(ndim,ndim)
	dimension w(NDIMW,NDIMW)
	dimension val(NDIMW,NDIMW,2),coeff(6,2)
c
	methdv = apint(1)
	nsize  = apint(3)
	apot   = apreal(1)
	wuns   = apreal(2)
	wflp   = apreal(3)
	if(nsize.gt.NDIMW) then
	  write(msg,*) 'error: nsize too large',nsize
	  errflg=2
	  return
	endif
c
	mid=(nsize+1)/2
c
	if(itrc.le.1) then
	  mid=1
	  do 10 j=1,ny
	  do 10 i=1,nx
	  refx(i,j)=rdata(i,j,kxp)
	  refy(i,j)=rdata(i,j,kyp)
	  unsmth(i,j)=0.0
	  badfit(i,j)=2.0
c  all points may be flipped in the first iteration
 10	  continue
	endif
	if(itrc.le.2) then
	  do 20 j=1,ny
	  do 20 i=1,nx
 20	  flips(i,j)=0.0
	endif
c
	nflip=0
	if(itrc.lt.1) go to 1
c
c  refx,y are only defined in intervals i=mid, nx+1-mid
c                                       j=mid, ny+1-mid
	do 1000 j=mid,ny+1-mid
	do 1000 i=mid,nx+1-mid
	if(i.gt.15.and.i.lt.40.and.j.gt.15.and.j.lt.35.and.
     &	mod(i,5).eq.0.and.mod(j,5).eq.0.and.msglvl.gt.2)
     &	then
     	  write(lst,100) i,j,'x',rdata(i,j,kx),refx(i,j),rdata(i,j,kxp)
	  write(lst,100) i,j,'y',rdata(i,j,ky),refy(i,j),rdata(i,j,kyp)
	  write(lst,100) i,j,'-',badfit(i,j),unsmth(i,j),flips(i,j)
 100	format(1x,2i4,a3,1p2e12.4,2x,2e12.4) 
	endif
c
	if(rdata(i,j,kx)*refx(i,j)+rdata(i,j,ky)*refy(i,j) .lt. 0.0
     &	  .and. ( swall .or. badfit(i,j).gt.1.0)  ) then
	  rdata(i,j,kx)=-rdata(i,j,kx)
	  rdata(i,j,ky)=-rdata(i,j,ky)
	  flips(i,j)=flips(i,j)+1.0
	  nflip=nflip+1
	endif
 1000	continue
c
 1	continue
	nflip1=framax(flips, nx,ny,ndim )
c
c  calculate new refx and refy
c
	wp=1.0/(1.0+apot*float(itrc)**2)
c  weight of potential field
c  reduce weight for large unsmoothness or large flip counts
c
	do 2000 j=mid,ny+1-mid
	do 2000 i=mid,nx+1-mid
	wsum=0.0
	do 2100 jj=1,nsize
	do 2100 ii=1,nsize
	sep=(ii-mid)**2+(jj-mid)**2
	w(ii,jj)=1.0/(1.0 + wuns*unsmth(i+ii-mid,j+jj-mid) 
     &	            + wflp*flips(i,j)**2 )
     &	  /max(1.0, sep)
	val(ii,jj,1)=rdata(i+ii-mid,j+jj-mid,kx)
	val(ii,jj,2)=rdata(i+ii-mid,j+jj-mid,ky)
	wsum=wsum+w(ii,jj)
 2100	continue
	wsum=wsum-w(mid,mid)
 	w(mid,mid)=0.0
c
	bt=sqrt(rdata(i,j,kx)**2+rdata(i,j,ky)**2)
	if(bt.lt.btlim) then
	  method=1
	else
	  method=methdv
	endif
c
	if(method.eq.1) then
	  xsum=0.0
	  ysum=0.0
	  do 2200 jj=1,nsize
	  do 2200 ii=1,nsize
	  xsum=xsum+rdata(i+ii-mid,j+jj-mid,kx)*w(ii,jj)
	  ysum=ysum+rdata(i+ii-mid,j+jj-mid,ky)*w(ii,jj)
 2200   continue
	  refx1=xsum/wsum
	  refy1=ysum/wsum
c  refx1,y1=average of bx,by on the surrounding nsize**2 points
c  with weight w
c
	else if(method.eq.2) then
	  call lfit(val,w,nsize,nsize,NDIMW,coeff)
	  refx1=val(mid,mid,1)
	  refy1=val(mid,mid,2)
c
	else if(method.eq.3) then
	  call qfit(val,w,nsize,nsize,NDIMW,coeff)
	  refx1=val(mid,mid,1)
	  refy1=val(mid,mid,2)
	endif
c
	refx(i,j)=wp*rdata(i,j,kxp)+(1.0-wp)*refx1
	refy(i,j)=wp*rdata(i,j,kyp)+(1.0-wp)*refy1 
 2000	continue
c
c  adjust scale
c
      btrsum=0.0
      btsum=0.0
      do 2500 j=1,ny
      do 2500 i=1,nx
      btsum=btsum+rdata(i,j,kx)**2+rdata(i,j,ky)**2
      btrsum=btrsum+refx(i,j)**2+refy(i,j)**2
 2500 continue
      scale=sqrt(btsum/btrsum)
      if(itrc.eq.1 .and. msglvl.gt.2) write(lst,*)
     &    '(subr.vectr1) scale bt/btp = ',scale
c
c  calculate unsmoothness
c
	sum=0.0
	count=0.0
	do 3000 j=mid,ny+1-mid
	do 3000 i=mid,nx+1-mid
cx	unsmth(i,j)=( (rdata(i,j,kx)-scale*refx(i,j))**2
cx   &	       +(rdata(i,j,ky)-scale*refy(i,j))**2 )*
cx   &	       ( rdata(i,j,kx)**2+rdata(i,j,ky)**2 )/btlim**4
	unsmth(i,j)=( (rdata(i,j,kx)-scale*refx(i,j))**2
     &	       +(rdata(i,j,ky)-scale*refy(i,j))**2 )
     &	       /btlim**2
cx	sum=sum+unsmth(i,j)*(rdata(i,j,kx)**2+rdata(i,j,ky)**2)
	sum=sum+unsmth(i,j)
	count=count+1.0
 3000	continue
cx	uns=sqrt(sum/btsum)
	uns=sqrt(sum/count)
c
c  calculate badness of potential field fitting
c
	do 3100 j=1,ny
	do 3100 i=1,nx
cx	badfit(i,j)=( (rdata(i,j,kx)-rdata(i,j,kxp))**2
cx   &	       +(rdata(i,j,ky)-rdata(i,j,kyp))**2 )*
cx   &	       ( rdata(i,j,kx)**2+rdata(i,j,ky)**2 )/btlim**4
	badfit(i,j)=( (rdata(i,j,kx)-rdata(i,j,kxp))**2
     &	       +(rdata(i,j,ky)-rdata(i,j,kyp))**2 )
     &	       /btlim**2
 3100	  continue
c
	return
	end
c
c  linear fit to data
c  val(i,j) = coeff(1) + coeff(2)*i + coeff(3)*j
c  weight   = weight(i,j)
c
      subroutine lfit(val,w,ix,iy,ndim,coeff)
c
      dimension val(ndim,ndim,2),w(ndim,ndim),sum(6,6),sum0(6,6),
     &      coeff(6,2),iwk(6),wk(6),pivot(6)
c
      do 10 jj=1,3
      coeff(jj,1)=0.0
      coeff(jj,2)=0.0
      do 10 ii=1,3
 10   sum(ii,jj)=0.0
c
      do 20 j=1,iy
      do 20 i=1,ix
      wk(1)=1
      wk(2)=i
      wk(3)=j
c
      do 20 jj=1,3
      coeff(jj,1)=coeff(jj,1)+wk(jj)*w(i,j)*val(i,j,1)
      coeff(jj,2)=coeff(jj,2)+wk(jj)*w(i,j)*val(i,j,2)
      do 20 ii=1,3
 20   sum(ii,jj)=sum(ii,jj) + wk(ii)*wk(jj)*w(i,j)
c
	do 30 jj=1,3
	do 30 ii=1,3
 30	sum0(ii,jj)=sum(ii,jj)
c
      eps=1.0e-5
      call lisn2(sum,3,6,coeff,2,2,iwk,pivot,det,nstop,eps)
c
      do 40 j=1,iy
      do 40 i=1,ix
      resd=
     &  (coeff(1,1)+coeff(2,1)*float(i)+coeff(3,1)*float(j))
     &  *val(i,j,1) + 
     &  (coeff(1,2)+coeff(2,2)*float(i)+coeff(3,2)*float(j))
     &  *val(i,j,2)
	if(resd.lt.0.0) then
	  val(i,j,1)=-val(i,j,1)
	  val(i,j,2)=-val(i,j,2)
	endif
 40   continue
c
      do 50 jj=1,3
      coeff(jj,1)=0.0
      coeff(jj,2)=0.0
 50	continue
c
      do 60 j=1,iy
      do 60 i=1,ix
      wk(1)=1
      wk(2)=i
      wk(3)=j
      do 60 jj=1,3
      coeff(jj,1)=coeff(jj,1)+wk(jj)*w(i,j)*val(i,j,1)
      coeff(jj,2)=coeff(jj,2)+wk(jj)*w(i,j)*val(i,j,2)
 60   continue
c
      call lisn2(sum0,3,6,coeff,2,2,iwk,pivot,det,nstop,eps)
c
	do 70 ixy=1,2
      do 70 j=1,iy
      do 70 i=1,ix
 70	val(i,j,ixy)=
     &  coeff(1,ixy)+coeff(2,ixy)*float(i)+coeff(3,ixy)*float(j)
c
      return
      end
c
c  quadratic fit to data
c  val(i,j) = coeff(1) + coeff(2)*i + coeff(3)*j
c           + coeff(4)*i**2 + coeff(5)*i*j + coeff(6)*j**2
c  weight   = weight(i,j)
c  
	subroutine qfit(val,w,ix,iy,ndim,coeff)
c
	dimension val(ndim,ndim,2),w(ndim,ndim),sum(6,6),sum0(6,6),
     &	coeff(6,2),iwk(6),wk(6),pivot(6)
c
	do 10 jj=1,6
	coeff(jj,1)=0.0
	coeff(jj,2)=0.0
	do 10 ii=1,6
 10	sum(ii,jj)=0.0 
c
	do 20 j=1,iy
	do 20 i=1,ix
	wk(1)=1
	wk(2)=i
	wk(3)=j
	wk(4)=i*i
	wk(5)=i*j
	wk(6)=j*j
c
	do 20 jj=1,6
	coeff(jj,1)=coeff(jj,1)+wk(jj)*w(i,j)*val(i,j,1)
	coeff(jj,2)=coeff(jj,2)+wk(jj)*w(i,j)*val(i,j,2)
	do 20 ii=1,6
 20	sum(ii,jj)=sum(ii,jj) + wk(ii)*wk(jj)*w(i,j)
c
	do 30 jj=1,6
	do 30 ii=1,6
 30	sum0(ii,jj)=sum(ii,jj)
c
	eps=1.0e-5
	call lisn2(sum,6,6,coeff,2,2,iwk,wk,det,nstop,eps)
c
	do 40 j=1,iy
      do 40 i=1,ix
      resd=
     &  (coeff(1,1)+coeff(2,1)*float(i)+coeff(3,1)*float(j)
     &  +coeff(4,1)*float(i*i) 
     &  +coeff(5,1)*float(i*j)
     &  +coeff(6,1)*float(j*j) )
     &  *val(i,j,1) +
     &  (coeff(1,2)+coeff(2,2)*float(i)+coeff(3,2)*float(j)
     &  +coeff(4,2)*float(i*i) 
     &  +coeff(5,2)*float(i*j)
     &  +coeff(6,2)*float(j*j) )
     &  *val(i,j,2)
      if(resd.lt.0.0) then
        val(i,j,1)=-val(i,j,1)
        val(i,j,2)=-val(i,j,2)
      endif
 40   continue
c
      do 50 jj=1,6
      coeff(jj,1)=0.0
      coeff(jj,2)=0.0
 50   continue
c
      do 60 j=1,iy
      do 60 i=1,ix
	wk(1)=1
	wk(2)=i
	wk(3)=j
	wk(4)=i*i
	wk(5)=i*j
	wk(6)=j*j
      do 60 jj=1,6
      coeff(jj,1)=coeff(jj,1)+wk(jj)*w(i,j)*val(i,j,1)
      coeff(jj,2)=coeff(jj,2)+wk(jj)*w(i,j)*val(i,j,2)
 60   continue
c
      call lisn2(sum0,6,6,coeff,2,2,iwk,pivot,det,nstop,eps)
c
	do 70 ixy=1,2
	do 70 j=1,iy
	do 70 i=1,ix
 	val(i,j,ixy)=coeff(1,ixy)
     &	    +coeff(2,ixy)*float(i) + coeff(3,ixy)*float(j)
     &	    +coeff(4,ixy)*float(i*i) 
     &	    +coeff(5,ixy)*float(i*j)
     &	    +coeff(6,ixy)*float(j*j)
 70	continue
c
	return
	end
