;+
; NAME :	convft
; PURPOSE :
;	convert flare telescope T1/T4  data into MAGPACK2's standard format
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	.run convft
; INPUTS :
;	necessary data such as
;	h(0,1,2,3) = nkrheader
;		img,q,u,v (T1, raw data)
;		img,bx,by,bl (T1, calibrated data)
;		img,s,v (T1, raw data)
;		img,vel,bl (T4, calibrated data)
;	should exist on memory
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	none
; OUTPUTS :
;	creates arrays mparam and rdata
;	mparam = magpack2 parameters
;	rdata = i,q,u,v (T1, raw), i,bx,by,bl (T1, calibrated), or
;	        i,s,v (T4, raw), or i,vel,bl (T4, calibrated)
; COMMON BLOCKS : 	none
; SIDE EFFECTS :
;	data will be binned so that new x-size is nxnew (=128 by default)
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1993.12
;-
;------------------------------------------------------------------------
;
	nxnew = 128
	if ( h(0).telescope = 'ft1' ) then begin
		if ( h(1).correct gt 100 and n_elements(bx) gt 0 ) then $
			convft1, h,img,bx,by,bl, mparam,rdata, nxnew=nxnew $
		else $
			convft1, h,img,q,u,v, mparam,rdata, nxnew=nxnew
	endif else if ( h(0).telescope = 'ft4' ) then begin
		if ( h(1).correct gt 100 and n_elements(bx) gt 0 ) then $
			convft4, h,img,vel,bl, mparam,rdata, nxnew=nxnew $
		else $
			convft4, h,img,s,v, mparam,rdata, nxnew=nxnew
	endif else print, '(proc.convft) data are not from the flare telescope'

end
