;  decode header information of a fits file

; remove quotation marks
function trimq, str
	buf = str
	p = strpos(buf,"'")
	while p ne -1 do begin
		strput,buf,' ',p
		p = strpos(buf,"'")
	endwhile
	buf = strtrim(buf,2)
	return, buf
end

; return 1(true) if str contains an integer
function isint,str,pos,len,val
	buf = strmid(str,pos,len)
	for i=0,len-1 do begin
		chr=strmid(buf,i,1)
		if not (chr ge '0' and chr le '9') and chr ne ' ' $
			then goto, noint
	endfor	
	if strtrim(buf,2) eq '' then val=0 else val=fix(buf)
	return,1
noint:
	return,0
end

pro dechdr, header, mparam, speca=speca,factor=factor,zlevel=zlevel, $
	mode=mode, skylvl=skylvl, $
	list=list

;  speca is a string array, and specf is a comma-separated list
;  of file specification words.
;
;  the parameter given by CTYPE3 keyword is read into specf
;  e.g. CTYPE3='I,BL' --> specf='I,BL', speca(0)='I', speca(1)='BL'
;

	rd = 3.141592/180.0
	stdate=0
	sttime=0
	stp=0
	stb0=0
	stcl0=0
	arcsec=0.07
	specf=''
      tsys ='GMT'
      inst ='        '
      mfile='        '
      dateo='  /  /  '
      stime='  :  :  '
      etime='  :  :  '
      title=''
	rsun=69.6
      pdeg=0.0
      b0deg=0.0
      cl0deg=0.0
      ydeg=0.0
	wavel=0.0

	nframe=1
	naxis=2
	xs=0.0
	ys=0.0
	dx=1.0
	dy=1.0
	rspdef=1.0e20
	rsp=rspdef
	xc=rspdef
	yc=rspdef
;  default value of rsp,xc,yc are practically infinity
	apdeg=0.0
	aldeg=0.0
;  set default sky level to be a large negative value
	skylvl= -2140000000L
	ncarr = 0

	kindm=0
	mesh=0

;  assume FITS header is in ASCII code
;  assume FITS data are big endian

	if not keyword_set(mode) then mode=0

	if keyword_set(list) then print,'(proc.rdfits) FITS HEADER'

	n = n_elements(header)

;  top of keyword search loop
	for i=0,n-1 do begin

	keywd=strtrim(strmid(header(i),0,8),2)
	str=strmid(header(i),10,70)
	value=strmid(header(i),10,20)
	setto=''

	case keywd of

;  general keywords

	'END': goto, endscan

	'SIMPLE': begin
		if not strcmp(strtrim(value,2),'T',1,/fold_case) then begin
			print,'(proc.rdfits) error: simple=',value
			return
		endif
		end

	'BITPIX': begin
		nbitm = fix(value)
	  	nbytem=nbitm/8
	  	setto='nbytem'
		end
;  assume FITS data are big endian: nbytem>0

	'NAXIS': begin
		naxis = fix(value)
		if ( naxis eq 2) then begin
			nframe=1
			factor=1.0 + fltarr(nframe)
			zlevel=0.0
			image=''
		endif
		end

	'NAXIS1': begin
		nx = fix(value)
	  	setto='nx'
		end

	'NAXIS2': begin
	  	ny = fix(value)
	  	setto='ny'
		end

	'NAXIS3': begin
	  	nframe = fix(value)
	  	setto='nframe'
		factor=1.0 + fltarr(nframe)
		zlevel=fltarr(nframe)
		image=strarr(nframe)
		end

	'BSCALE': begin
		factor = replicate(float(value),nframe)
	  	setto='factor(*)'
		end

	'BZERO': begin
	 	zlevel=replicate(float(value),nframe)
	  	setto='zlevel(*)'
		end

	'OBJECT': begin
	  	mfile = trimq(strmid(str,1,8))
	  	title=mfile
	  	setto='mfile,title'
		end

	'CRVAL1': begin
	  	xs = float(value)
	  	setto='xs'
		end

	'CRVAL2': begin
	  	ys = float(value)
	  	setto='ys'
		end

	'CDELT1': begin
	  	dx = float(value)
	  	setto='dx'
		end

	'CDELT2': begin
	  	dy = float(value)
		setto='dy'
		end

	'CTYPE3': begin
		specf=strtrim(strmid(str,1,18),2)
	  	setto='specf'
		end

	'DATEOBS': begin
	  	dateo = strmid(str,1,8)		; 'yy/mm/dd'
	  	setto='dateo'
	  	stdate=1
		end

	'TIMEOBS': begin
		stime=strmid(str,1,8)
		etime=strmid(str,10,8)
;	str = 'hh:mm:ss-hh:mm:ss'
	  	setto='stime,etime'
	  	sttime=1
		end

	'INSTRUME': begin
	  	buf = trimq(strmid(str,1,8))
;  set mode=6
		if mode eq 0 and strcmp(buf,'MDI',3,/fold_case) then mode=6
		if strtrim(inst,2) eq '' then $
			inst=buf $
		else inst=strmid(inst+buf,0,8)
	  	setto='inst'
		end

	'TELESCOP': begin
		buf = trimq(strmid(str,1,8))
		if strtrim(inst,2) eq '' then $
			inst=buf else inst=strmid(inst+buf,0,8)
	  	setto='inst'
		end

	'ORIGIN': begin
		origin = trimq(str)
		setto='origin'
		if mode eq 0 then begin
;  set mode (1,2,3,4,7) depending on origin
		  	if strcmp(origin,'MAGPACK2',8,/fold_case) then mode=1
	  		if strcmp(origin,'MEES',4,/fold_case) then mode=2
	  		if strcmp(origin,'BBSO',4,/fold_case) then mode=3
	  		if strcmp(origin,'KPNO',4,/fold_case) then mode=4
;           could be mode=5(synoptic)
	  		if strcmp(origin,'MT. WILSON',10,/fold_case) or $
			   strcmp(origin,'MT.WILSON',9,/fold_case) then mode=7
		endif
		end

;  keywords specific to magpack2 package (mode=1)

	'KINDM': begin
		kindm = fix(value)
	  	setto='kindm'
		end

	'MESH': begin
		mesh = fix(value)
	  	setto='mesh'
		end

	'P': begin
		pdeg = float(value)
	  	setto='pdeg'
	  	stp=1
		end

	'B0': begin
		b0deg = float(value)
	  	setto='b0deg'
	  	stb0=1
		end

	'CL0': begin
		cl0deg = float(value)
		setto='cl0deg'
		stcl0=1
		end

	'RSUN': begin
		rsp = float(value)
		setto='rsp'
		end

	'YDEG': begin
		ydeg = float(value)
		setto='ydeg'
		end

	'SYSTIME': begin
		tsys = strmid(str,1,6)
		setto='tsys'
		end

	'WAVELEN': begin
		wavel = float(value)
		setto='wavel'
		end

;  keywords specific to U.Hawaii (Mees) magnetograms (mode=2)

	'T_START': begin
		if mode eq 2 then begin
			dateo = strmid(str,1,2)+'/'+strmid(str,3,2)+'/' $
					+strmid(str,5,2)
			stime = strmid(str,8,2)+':'+strmid(str,10,2)+':' $
					+strmid(str,12,2)
;                  'yymmdd.hhmmssss'
	  		setto='dateo,stime'
	  		stdate=1
	  		sttime=1
		endif
		end

	'T_STOP': begin
		if mode eq 2 then begin
			etime = strmid(str,8,2)+':'+strmid(str,10,2)+':' $
					+strmid(str,12,2)
;                  'yymmdd.hhmmssss'
	  		setto='etime'
		endif
		end

	'POS-LAT': begin
		aldeg = float(value)
	  	setto='aldeg'
		end

	'POS-CMD': begin
		apdeg = float(value)
	  	setto='apdeg'
		end

	'L0': begin
		cl0deg = float(value)
	  	setto='cl0deg'
	 	stcl0=1
		end

	'RADIUS': begin
		rsp = float(value)
	  	setto='rsp'
		end

;  DATE-OBS, TIME-OBS keywords (BBSO, Mt.Wilson, SOHO/MDI)

	'DATE-OBS': begin
		buf = trimq(str)
;                  'dd/mm/yy' or 'yyyy/mm/dd' or 'yy/mm/dd'
		if isint(buf,0,4,iyr) then begin
;                 yyyy/mm/dd
			dateo = strmid(buf,0,4)+strmid(buf,5,2)+strmid(buf,8,2)
		endif else if isint(buf,6,2,iyr) then begin
			if iyr gt 31 then begin
;             dd/mm/yy
				dateo = strmid(buf,0,2)+'/'+strmid(buf,3,2)+'/'+ $
						strmid(buf,6,2)
	    		endif else begin
;             yy/mm/dd
				dateo = strmid(buf,0,8)
			endelse
		endif
	  	setto='dateo'
	  	stdate=1
		end

	'TIME-OBS': begin
		stime = strmid(str,1,8)		; 'hh:mm:ss.ss'
	  	setto='stime'
	  	sttime=1
		end

;  BBSO

	'WAVELNTH': begin
		wavel = float(strmid(value,1,4))
;  WAVELNTH='xxxxA'
	  	setto='wavel'
	  	if mode eq 0 then mode=3
		end

;  keywords specific to KPNO magnetograms (mode=4)

	'UTDATE': begin
		if mode eq 4  then begin
			dateo=strmid(str,7,2)+'/'+strmid(str,1,2)+'/'+strmid(str,4,2)
;	str='mm/dd/yy'
		  	setto='dateo'
	  		stdate=1
		endif
		end

	'OBS_DATE': begin
		if mode eq 4 then begin
			dateo = strmid(str,14,2)+'/'+strmid(str,16,2)+'/'+strmid(str,18,2)
;	str =  mmddyy (integer)
	  		setto='dateo'
	  		stdate=1
		endif
		end

;  DATEOBS is not given
;  UTDATE='mm/dd/yy' for data of 1996 and later
;  OBS_DATE=mmddyy (integer) for 512ch magnetograph (-1991)
;  IRAFNAME='yymmdd.....' can be used, but sometimes there 
;  appears the second IRAFNAME keyword with other format

;  tested on data of: 74/07/01, 91/07/01 (512ch magnetograph)
;                     92/07/01, 93/07/02, 94/07/01, 95/07/01,
;                     96/07/01, 97/07/01, 98/07/01, 99/05/29,
;                     00/01/03 (spectromagnetograph, full res.)
;                     00/01/03 (spectromagnetograph, med. res.)
 
	'UTSTART': begin
		stime = strmid(str,1,8)		; 'hh:mm:ss' or hh:mm
	  	if strmid(stime,0,2) eq '  ' then stime=strmid(str,15,5)+':00'
	  	setto='stime'
	  	sttime=1
		end

	'UTSTOP': begin
		etime = strmid(str,1,8)		; 'hh:mm:ss'
	  	setto='etime'
		end

	'UT_START': begin
		stime = strmid(str,13,5)+':00'	; hh:mm
	  	setto='stime'
	  	sttime=1
		end

	'SCALE': begin
		dy = float(value)
	  	dx=dy
	  	setto='dx,dy'
		end

	'E_XCEN': begin
		xc = float(value)
	  	setto='xc'
		end

	'E_YCEN': begin
		yc = float(value)
	  	setto='yc'
		end

	'E_XSMD': begin
		rx = float(value)
	  	setto='rx'
		end

	'E_YSMD': begin
		rsp = float(value)
	  	dx = dy*rx/rsp
	  	setto='rsp, dx=dy*rx/rsp'
		end

	'WAVE': begin
		wavel = float(value)
	  	setto='wavel'
		end

	'WV_LNGTH': begin
		wavel = float(value)
	  	setto='wavel'
		end

	'B_ZERO': begin
		b0deg = float(value)
	  	setto='b0deg'
	  	stb0=1
		end

	'L_ZERO': begin
		cl0deg = float(value)
	  	setto='cl0deg'
	  	stcl0=1
		end

	'IRAFNAME': begin
		mfile = trimq(strmid(str,1,8))
	  	title=mfile
		if strcmp(dateo,'  ',2) and isint(mfile,0,6,ival) then begin
;  assume IRAFNAME includes obsdate
     	    		dateo=strmid(mfile,0,2)+'/'+strmid(mfile,2,2)+'/'+ $
				strmid(mfile,4,2)
			setto='mfile,title,dateo'
			stdate=1
		endif else begin
;  synoptic data: get Carrington rotation number from IRAFNAME
			if not isint(mfile,1,4,ncarr) then $ 
				dummy=isint(mfile,0,4,ncarr)
	  	endelse
		end

;  keywords specific to KPNO synoptic magnetograms (mode=5)
;  in the file containing data(i,j), longitude increases with i
;  (i=1 <==> longitude=0 deg, i=360 <==> longitude=359 deg)

;  tested on data of: Carrington rotation 1893 (year 1962)

	'AV_BZERO': begin
		b0deg = float(value)
	  	setto='b0deg'
	  	stb0=1
	  	if mode eq 0 then mode=5
		end

	'CAR_ROT': begin
		ncarr = fix(value)
		if mode eq 0 then mode=5
		end

;  keywords specific to SOHO/MDI magnetograms (mode=6)

;  1024*1024, 1.98"
;  tested on data of 97/11/06

	'IM_SCALE': begin
		dx = float(value)
	  	dy=dx
	  	setto='dx,dy'
	  	if mode eq 0 then mode=6
		end

	'X0': begin
		xs = float(value)
	  	xs=-xs
	  	setto='-xs'
		end

	'BLANK': begin
		skylvl = fix(value)
	  	setto='skylvl'
		end

;  X0, Y0 also used in Mt.Wilson
	'Y0': begin
		ys = float(value)
	  	ys=-ys
	  	setto='-ys'
		end

	'R_SUN': begin
		rsp = float(value)
	  	setto='rsp'
		end

	'P_ANGLE': begin
		pdeg = float(value)
	  	setto='pdeg'
	  	stp=1
		end

	'DTYPE': begin
		spedf = trimq(str)
;  DTYPE=V(Doppler), B(magnetogram), C(continuum), M(modulation)
		if specf eq 'B' then specf='BL'
	  	setto='specf'
		end

;  keywords specific to Mt.Wilson magnetograms (mode=7)

;  512*512 pixels, 3.75"
;  tested on data of 95/04/04, 99/07/01

	'DXB_IMG': begin
		dx = float(value)
	  	dy=dx
	 	setto='dx,dy'
	  	if mode eq 0 then mode=7
		end

	'DYB_IMG': begin
		dy = float(value)
	  	setto='dy'
		end

	'TAVG-UT': begin
		avgut = float(value)
	  	ih=fix(avgut)
	  	im=fix(60.0*(avgut-ih))
	  	stime = string(ih,format='(i2.2)')+':'+string(im,format='(i2.2)') $
				+':00'
	  	etime=stime
	  	setto='stime,etime'
	  	sttime=1
		end

	'CENLON': begin
		cl0deg = float(value)
	  	setto='cl0deg'
	  	stcl0=1
		end

	'R0': begin
		rsp = float(value)
	  	setto='rsp'
		end

	'LAMBDA': begin
		wavel = float(value)
	  	setto='wavel'
		end

	else:
	endcase

;  keywords specific to U.Hawaii (Mees) magnetograms (mode=2)

;  tested on data of 93/08/10 (IVM)

	if strcmp(keywd,'IMAGE',5) then begin
		iframe = fix(strmid(keywd,5,1))-1
		image(iframe)=trimq(str)
	  	if strcmp(image(iframe),'I_CONT',6) then begin
			image(iframe)='I'
		endif else if strcmp(image(iframe),'B_LONG',6) then begin
	    		image(iframe)='BL'
		endif else if strcmp(image(iframe),'B_TRANS',6) then begin
	    		image(iframe)='TB'	; 'BT' was assigned to Btheta
		endif else if strcmp(image(iframe),'B_AZIM',5) then begin
	    		image(iframe)='AZ'
		endif else if strcmp(image(iframe),'CE',2) then begin
	    		image(iframe)='VL'
		endif else begin
	  		image(iframe)='S1'
		endelse
		if specf eq '' then specf=image(iframe) else $
	    	specf=specf+','+image(iframe)
	  	setto='image('+strmid(keywd,5,1)+')'
	  	if mode eq 0 then mode=2
	endif
	if strcmp(keywd,'ISCALE',6) then begin
		iframe=fix(strmid(keywd,6,1))-1
	  	factor(iframe) = float(value)
	  	if strcmp(image(iframe),'AZ',2) then begin
	    		factor(iframe)=factor(iframe)*3.141592/180.0
;	unit of azimuth is now radian
	  	endif
		  setto='factor('+strmid(keywd,6,1)+')'
	endif
endscan:

	if keyword_set(list) and strtrim(keywd,2) ne '' then begin
		print,strtrim(header(i))
	  	if setto ne '' then print,'set to: ',setto
	endif
	endfor

;  end of keyword search loop

;  calculate solar parameters from date

	if ncarr ne 0 then begin
		cardate, ncarr, dateo, stime
		stdate=1
		sttime=1
	endif

	if stdate and sttime then begin 
		sunparam, dateo,stime,rspcal,pcal,b0cal,cl0cal,doy,irotno, $
			tsys=tsys
		if not stp then begin
			pdeg=pcal
			if keyword_set(list) then print,'pdeg from ephemeris'
		endif
	  	if not stb0 then begin
			b0deg=b0cal
			if keyword_set(list) then print,'b0deg from ephemeris'
		endif
	  	if not stcl0 then begin
			cl0deg=cl0cal
			if keyword_set(list) then print,'cl0deg from ephemeris'
		endif
		if rsp eq rspdef then begin
			rsp = rspcal
			if keyword_set(list) then print,'rsp from ephemeris'
		endif
	endif else begin
		if rsp eq rspdef then begin
			rsp = rsun/arcsec
			if keyword_set(list) then print,'rsp set to rsun'
		endif
	endelse

;  Mees

	if mode eq 2 then begin
		kindm=1102
	endif

;  BBSO

	if mode eq 3 then begin

;  tested on data of 93/08/11, 96/05/19
;  512*481, pixel size uncertain

		dx=-0.5
		dy=0.5
;  to be confirmed
	endif

;  KPNO

	if mode eq 4 then begin
		if xc eq rspdef then begin
			xs=-rsp
			ys=-rsp
;  medium resolution data are binned but have the original pixel size
		endif else begin
			rsp=rsp*dy
			xs=-xc*dx
			ys=-yc*dy
		endelse
		dx=dx*(2048/nx)
		dy=dy*(2048/ny)
;  spectromagnetograph 1788*1788 pixels, 1.14"
		if inst eq 'SPECTROM' then begin
			if nx eq 1788 then skylvl=-10000
			if nx eq 894  then skylvl=-750
			if abs(wavel-8688.0) lt 1.0 then begin
	      		factor(0)=factor(0)*1.46
	      		zlevel(0)=zlevel(0)*1.46
			endif else if abs(wavel-8542.0) lt 1.0 then begin
	      		factor(0)=factor(0)*2.16
	      		zlevel(0)=zlevel(0)*2.16
			endif else if abs(wavel-5507.0) lt 1.0 then begin
	      		factor(0)=factor(0)*1.26
	      		zlevel(0)=zlevel(0)*1.26
			endif
		endif else begin
;  512ch magnetograph 2048*2048 pixels, 1"
			skylvl=-2048
		endelse
		if keyword_set(list) then print,'KPNO data: skylevel, factor=', $
     			skylvl,factor(0),zlevel(0)

;  data are heliocentric
		ydeg = pdeg
	endif

;  KPNO synoptic data
;  x = longitude from 0 deg to 359 deg 
; (the Carrington longitude for each x-value is given in header)
;  y = sin(latitude) from -1 to +1
;  data are 'BR' (BL divived by cosine of angle between LOS and radius vector

	if mode eq 5 then begin
		rsp = rsun/arcsec
		xs=1.0e-4
		ys=-1.0
		dx=2.0*3.141592/float(nx)
		dy=2.0/float(ny-1)
		specf='BR'
		kindm=30006
		mesh=4
		cl0deg=180.0
;  data are heliocentric
		ydeg = pdeg
	endif

;  MDI

	if mode eq 6 then begin
;  data are heliocentric, but p-angle is set to 0 in the header
		if stdate and sttime then pdeg = pcal
		ydeg=pdeg
	endif

;  Mt.Wilson

	if mode eq 7 then begin
		xs=xs*dx
		ys=ys*dy
		rsp=rsp*dx
		skylvl=-32768
;  B0 was given in radian in the header
		b0deg = 180.0*b0deg/3.141592

;  data are geocentric, but p-angle is not given in the header
;  if the date and time were decoded, pdeg is supplied from ephemeris
	endif

;  change length scale from arcsec to ulen (approx.10^4 km)
        dx=dx*arcsec
        dy=dy*arcsec
        xs=xs*arcsec
        ys=ys*arcsec
        rsp=rsp*arcsec

;  assume kindm=1 (BL) if only one data set is present
	if nframe eq 1 and (specf eq '' or specf eq 'BL') then begin
		specf='BL'
		kindm=1
	endif

	if rsp eq rspdef then begin
		print, '(proc.rdfits) warning: rsp undefined'
	endif

	if specf ne '' then speca=split(specf,delim=',') 
	if nframe gt 1 then begin
		if strpos(specf,'BL') then begin
			if strpos(specf,'BX') and strpos(specf,'BY') then kindm=2 $
			else kindm=1
		endif
		if strpos(specf,'VL') then kindm=kindm+100
		if strpos(specf,'I') then kindm=kindm+1000
	endif

;	if apdeg ne.0.0 or aldeg ne 0.0 then begin
;		ap=rd*apdeg
;		al=rd*aldeg
;		rtoc,ap,al,0.0, xx
;		xs=xx(1)
;		ys=xx(2)
;	endif

	xtod=0.0
	ytod=0.0
	if mesh eq 0 then begin
		xtod=xs+0.5*dx*(nx-1)
		ytod=ys+0.5*dy*(ny-1)
	endif

	mparam = defmp()

	mparam.kindm  = kindm
	mparam.nbytem = nbytem
	mparam.nframe = nframe
	mparam.tsys   =tsys
	mparam.inst   =inst
	mparam.mfile  =mfile
	mparam.dateo  =dateo
	mparam.stime  =stime
	mparam.etime  =etime
	mparam.ftag   =mktag()
	mparam.nx     =nx
	mparam.ny     =ny
	mparam.xs     =xs
	mparam.ys     =ys
	mparam.dx     =dx
	mparam.dy     =dy
	mparam.ypos   =rd*ydeg
	mparam.mesh   =mesh
	mparam.wavel  =wavel
	mparam.rsp    =rsp
	mparam.prad   =rd*pdeg
	mparam.b0rad  =rd*b0deg
	mparam.cl0rad =rd*cl0deg
	mparam.xxtod(0)  =xtod
	mparam.xxtod(1)  =ytod
	mparam.title   = title

	return
end
