;+
; NAME :	defldat
; PURPOSE :
;	define field-line data structure
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	ldata = defldata(ncomp,nsize,nfinit)
; INPUTS :
;	ncomp = number of components in field line data (3 or 8)
;	nsize = size of heap area :heap is declared as fltarr(ncomp,nsize)
;	nfinit = initial number of field lines
;	         the size of np is set to 2*nfinit
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	none
; OUTPUTS :
;	ldata (structure)
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 2000 August
;-
;------------------------------------------------------------------------
;
function defldat, ncomp,nsize,nfinit
;
; definition of MAGPACK2 field line data structure
;
	str8=string(' ',format='(a8)')
	str16=string(' ',format='(a16)')

	lofd={ldata1, $
		nf:   0, $
		np:   intarr(min([2*nfinit,200])), $
		loc:	intarr(min([2*nfinit,200])), $
		heap:	fltarr(nsize,ncomp), $
		nex:  0, $
		nfinit:	0, $
		ns1:	0, $
		ns2:	0, $
		ns3:	0, $
		bmin1:	0.0, $
		bmin2:	0.0, $
		flxpwr:	0.0, $
		kindl:	0L, $
		nbytel:	0, $
		ncomp:	0, $
		xfactor:	0.0, $
		bfactor:	0.0, $
		lfile:	str8, $
		mtag:		str16, $
		ltag:		str16, $
		rrss:		0.0, $
		chksum:	0L } 

	return, lofd
end
