;+
; NAME :	defmh2
; PURPOSE :
;	define magnetogram header part-2
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	mhdr2 = defmh2()
; INPUTS :
;	none
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	spec = data specification string
;	factor = conversion factor (real=factor*integer)
;	alpha = alpha of linear force-free field
; OUTPUTS :
;	mhdr2 (structure)
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	alpha added : 2000 July
;-
;------------------------------------------------------------------------
;
function defmh2, spec=spec, factor=factor, alpha=alpha
;
	mag2={ mhdr2, $
		spec:  string(' ', format='(a8)'), $
		factor:string(' ', format='(a12)'), $
		alpha:string(' ', format='(a12)'), $
		dummy: string(' ', format='(a48)') }

	if keyword_set(spec) then $
		mag2.spec= strmid( strtrim( spec+'        ',1), 0, 8) $
	else $
		mag2.spec = string(' ', format='(a8)')

	if keyword_set(factor) then $
;		mag2.factor= string( factor, format='(f12.5)') $
		mag2.factor= string( factor, format='(g12.5)') $
	else $
;		mag2.factor= string( 0.0, format='(f12.5)')
		mag2.factor= string( 0.0, format='(g12.5)')

	if keyword_set(alpha) then $
		mag2.alpha= string( alpha, format='(g12.5)') $
	else $
		mag2.alpha= string( 0.0, format='(g12.5)')

	return, mag2
end
