;+
; NAME :	defmp
; PURPOSE :
;	define magnetogram parameters
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	mparam = defmp()
; INPUTS :
;	none
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	none
; OUTPUTS :
;	mparam (structure)
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	TS 1993/5 rev angles are in radian
;	TS 95/10/31: add wavel
;	TS 2000/07/23: add alpha
;	TS 2000/10/24: ydeg,pdeg,b0deg,cl0deg added
;	TS 2002/05/18: mpstr,mpint,mpreal added
;-
;------------------------------------------------------------------------
;
function defmp
;
	str4=string(' ',format='(a4)')
	str8=string(' ',format='(a8)')
	str12=string(' ',format='(a12)')
	str16=string(' ',format='(a16)')

	status = defstats()
	factors = deffactr()

	mparam={mparam1, kindm:0L, $
		nbytem:0, $
		nframe:0, $
		tsys:'GMT+09', $
		inst:str8, $
		mfile:str8, $
		dateo:str8, $ 
		stime:str8, $
		etime:str8, $
		ftag:str16, $
		nx:0, $
		ny:0, $
		xs:0.0, $
		ys:0.0, $
		dx:0.0, $
		dy:0.0, $
		ypos:0.0, $
		ydeg:0.0, $
		mesh:0, $
		wavel:0.0, $
		alpha:0.0, $
		rsp:69.6, $
		prad:0.0, $
		b0rad:0.0, $
		cl0rad:0.0, $
		pdeg:0.0, $
		b0deg:0.0, $
		cl0deg:0.0, $
		xxtod:[0.0, 0.0, 0.0], $
		title:string(' ',format='(a80)'), $
		status: status, $
		factors: factors, $
		mpstr:	strarr(10), $
		mpint:	intarr(10), $
		mpreal:	fltarr(10) }

	return,mparam
end
