;+
; NAME :	defspecs
; PURPOSE :
;	define data specification strings
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	specs = defspecs( nspec )
; INPUTS :
;	none
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	none
; OUTPUTS :
;	specs (array of strings)
;	nspec = number of strings in specs
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	TS 2000/10 DIVB added
;	TS 2002/05 BT,AZ added, units added
;-
;------------------------------------------------------------------------
;
function defspecs, nspec, units=units
;
        specs=[ 'I',  'BT',  'AZ',   $
                'V',  'Q',   'U',   $
                'V/I','Q/I', 'U/I', $
                'W',  '*',   '*',   $
                'W/I','*',   '*',   $
                'BLP','BXP', 'BYP', $
                'BNP','BHXP','BHYP',$
                'BRP','BPHP','BTHP',$
                'BL', 'BX',  'BY',  $
                'BN', 'BHX', 'BHY', $
                'BR', 'BPH', 'BTH', $
                'VL', 'VX',  'VY',  $
                'VN', 'VHX', 'VHY', $
                'VR', 'VPH', 'VTH', $
                'JL', '*',   '*',   $
                'JN', '*',   '*',   $
                'JR', '*',   '*',   $
                'FL', '*',   '*',   $
                'FN', '*',   '*',   $
                'FR', '*',   '*',   $
		'ALPHA','*', '*',   $
		'DIVB', '*', '*',   $
		'1S', '1X',  '1Y',  $
		'2S', '2X',  '2Y',  $
		'3S', '3X',  '3Y',  $
		'4S', '4X',  '4Y'  ]

	units = ['ADU', 'GAUSS', 'DEG', $
		'ADU',  'ADU', 'ADU', $
		' ',' ',' ', $
		'ADU',  ' ',   ' ', $
		' ',' ',  ' ', $
		'GAUSS','GAUSS','GAUSS', $
		'GAUSS','GAUSS','GAUSS', $
		'GAUSS','GAUSS','GAUSS', $
		'GAUSS','GAUSS','GAUSS', $
		'GAUSS','GAUSS','GAUSS', $
		'GAUSS','GAUSS','GAUSS', $
		'M/S', 'M/S', 'M/S', $
		'M/S', 'M/S', 'M/S', $
		'M/S', 'M/S', 'M/S', $
		'G/10**4KM',' ',' ', $
		'G/10**4KM',' ',' ', $
		'G/10**4KM',' ',' ', $
		'G**2/10**4KM',' ',' ', $
		'G**2/10**4KM',' ',' ', $
		'G**2/10**4KM',' ',' ', $
		'/10**4KM',' ',' ', $
		'G/10**4KM',' ',' ', $
		' ', ' ', ' ', $
		' ', ' ', ' ', $
		' ', ' ', ' ', $
		' ', ' ',' ' ]

        size_specs = size(specs)
        nspec = fix(size_specs(1))
	return, specs
;	the tags should be consistent with structures status and factors
end
