;+
; NAME :	dipole
; PURPOSE :
;	generate a simulated magnetogram of a dipole field
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	.run dipole
; INPUTS :
;	none (parameters are specified in the program)
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	none
; OUTPUTS :
;	data are kept in mparam and rdata
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	TS 2000/10/09 modification
;-
;------------------------------------------------------------------------
;
;  dipole field
;  (x,y,z): points where the field (bx,by,bz) is evaluated
;  xxs: source point
;  dmom: dipole moment
;  dir: direction of dipole moment (unit vector)
;
	pro dipole,  x,y,z,bx,by,bz,dmom=dmom,xxs=xxs,dir=dir

	dp1 = fltarr(3)
	dp2 = fltarr(3)

	if abs(dir(0)) lt abs(dir(1))  and $ 
		abs(dir(0)) lt abs(dir(2))  then begin
		dp1(0)=1.0
	endif else $
	if abs(dir(1)) lt abs(dir(0))  and $ 
		abs(dir(1)) lt abs(dir(2))  then begin
		dp1(1)=1.0
	endif else begin
		dp1(2)=1.0
	endelse

	dp2(0)=dir(1)*dp1(2)-dir(2)*dp1(1)
	dp2(1)=dir(2)*dp1(0)-dir(0)*dp1(2)
	dp2(2)=dir(0)*dp1(1)-dir(1)*dp1(0)

	dp2 = dp2/ sqrt( total(dp2*dp2) )

	dp1(0)=dp2(1)*dir(2)-dp2(2)*dir(1)
	dp1(1)=dp2(2)*dir(0)-dp2(0)*dir(2)
	dp1(2)=dp2(0)*dir(1)-dp2(1)*dir(0)

	x0 = x - xxs(0)
	y0 = y - xxs(1)
	z0 = z - xxs(2)
	r=sqrt(x0*x0 +y0*y0 + z0*z0)
	xx=x0*dp1 (0)+y0*dp1 (1)+z0*dp1 (2)
	yy=x0*dp2 (0)+y0*dp2 (1)+z0*dp2 (2)
	zz=x0*dir(0)+y0*dir(1)+z0*dir(2)
	bxx=dmom*3.0*xx*zz/r^5
	byy=dmom*3.0*yy*zz/r^5
	bzz=dmom*(2.0*zz*zz-xx*xx-yy*yy)/r^5

	bx = bxx*dp1(0) + byy*dp2(0) + bzz*dir(0)
	by = bxx*dp1(1) + byy*dp2(1) + bzz*dir(1)
	bz = bxx*dp1(2) + byy*dp2(2) + bzz*dir(2)

	return
	end

;	generate a simulated magnetogram based on a dipole field
;	main program

	common blkd, coord_d
	common blkt, coord_t

	nx = 41
	ny = 41
	xc = 0.0
	yc = 0.0
	dx = 0.5
	dy = 0.5
	rsp = 70.0
	bmax = 1000.0
	dir = [1.0, 0.0, 0.0]
	dir = dir/sqrt(total(dir*dir))
	iproj = 1

;	xxs = [35.0, 0.0, -12.0]
;	xs = 25.0
;	ys = -10.0

;	xxs = [0.0, 35.0, -12.0]
;	xs = -10.0
;	ys = 25.0

;	dx = 2.0
;	dy = 2.0
;	xxs = [0.0, 0.0, -20.0]
;	xs = -40
;	ys = -40

	xxs = [0.0, 0.0, -2.0]
	xs=-10.0
	ys=-10.0

	xc = xs + dx*(nx/2)
	yc = ys + dy*(nx/2)

	mparam = defmp()

;	mparam.kindm = 2L	; pseudo vector
	mparam.kindm = 3L	; vector
	mparam.nbytem = 2
	mparam.nframe = 3
	mparam.tsys='GMT+09'
	mparam.inst='DUMMY'
	mparam.mfile='DIPOLE'
	mparam.nx = nx
	mparam.ny = ny
	mparam.xs = xs
	mparam.ys = ys
	mparam.dx = dx
	mparam.dy = dy
	mparam.xxtod=[xc,yc]
	mparam.rsp = rsp
	mparam.mesh = 0
	mparam.title='DIPOLE'

	rdata = fltarr(nx,ny,8)
	kz=0
	kx=1
	ky=2
	mparam.status.bl = kz
	mparam.status.bx = kx
	mparam.status.by = ky
	mparam.factors.bl=0.2
	mparam.factors.bx=0.2
	mparam.factors.by=0.2

	dmom=1.0

	setdu, mparam
	settu, mparam

	x = fltarr(nx,ny)
	y = x
	z = x


	for j=0,ny-1 do x(*,j) = xs + dx*indgen(nx)
	for i=0,nx-1 do y(i,*) = ys + dy*indgen(ny)

	if iproj eq 1 then begin
		projt, x,y,z
		ctot, xxs(0),xxs(1),xxs(2), xx,yy,zz
		depth = -zz
		dv = total(coord_t.ztu*dir)
	endif else $
	if iproj ge 2 then begin
		projs, x,y,z
		projt, x,y,z
		ctot, xxs(0),xxs(1),xxs(2), xx,yy,zz
		depth = -zz
		dv = total(coord_t.ztu*dir)
	endif else begin
		depth = -xxs(2)
		dv = total(coord_d.zdu*dir)
	endelse
	dh = sqrt(1.0-dv*dv)

	dipole, x,y,z,bx,by,bz,dmom=dmom,xxs=xxs,dir=dir

;	mirror source
	if iproj eq 2 then begin
		xxsm=[ xxs(0),xxs(1), -2*rsp -xxs(2)]
		dirm=[dir(0), dir(1), -dir(2)]
		dipole, x,y,z,bx2,by2,bz2,dmom=dmom,xxs=xxsm,dir=dirm
		bx = bx+bx2
		by = by+by2
		bz = bz+bz2
	endif

	bmax1 = sqrt( max( bx*bx+by*by+bz*bz) )
	dmom=bmax/bmax1
	rdata(*,*,kx)=bx*dmom
	rdata(*,*,ky)=by*dmom
	rdata(*,*,kz)=bz*dmom

        ulen=1.0e9
        engy=dmom^2*(dh^2+2.0*dv^2)/(32.0*depth^3)*ulen^3
        print, 'dipole:'
        print, '  location=',xxs
        print, '  depth=',depth
        print, '  moment=',dmom
        print, '  direction=',dir
        print, '  dv,dh=',dv,dh
        if iproj eq 1 then $
		print, '  xtod,ytod=',mparam.xxtod(0:1)
        print, '  nx,ny=',nx,ny
        print, '  xs,ys=',xs,ys,'  rsp=',rsp
        print, '  dx,dy=',dx,dy
        print, '  data projection: iproj=',iproj
        print, '  kindmo=',mparam.kindm
        print, '  energy=',engy,' erg (for plane boundary)'

end

