;+
; NAME :	displ3
; PURPOSE :
;	plot magnetic field lines
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	displ3, mparam, ldata [,/whitebg]
; INPUTS :
;	mparam : magnetogram parameters
;	ldata : data of field lines
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	whitebg = white background (default is dark background)
; OUTPUTS :
;	field lines are drawn
; COMMON BLOCKS : 	gparam as an include file @gparam.pro
; SIDE EFFECTS :
;	none
; RESTRICTIONS :	X-window environment is assumed
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 2000/11/02
;-
;------------------------------------------------------------------------
;
pro displ3, mparam, ldata, whitebg=whitebg
@gparam

	setcolor, bandwidth, w_start, rb_start, g_start, color8, $
			whitebg=whitebg

	rsp = mparam.rsp
	height0 = 50.0
	bright0 = 0.3

	nf = ldata.nf
	kindl = ldata.kindl

	for l = 0, nf-1 do begin
		np = ldata.np(l)
		loc = ldata.loc(l)
		xx = ldata.heap(loc:loc+np-1,0)
		yy = ldata.heap(loc:loc+np-1,1)
		zz = ldata.heap(loc:loc+np-1,2)
		if ( kindl mod 10 lt 2 ) then $
			height = zz $
		else $
			height = sqrt( xx^2 + yy^2 + (zz + rsp )^2 ) -rsp
		maxheight = max(height)

		for i=1, np-1 do begin

			if ( maxheight gt height0 ) then $
				bright = height(i) / height0 $
			else $
				bright = ( height(i)+height0-maxheight) /height0
			bright = bright < 1.0
			bright = bright > 0.0
			bright = bright0+ bright*(1.0- bright0)
			pline3, xx(i-1:i), yy(i-1:i), zz(i-1:i), $
				color = g_start + bright *(bandwidth-1), $
				thick=2, /device
		endfor
	endfor
	return
end
