;+
; NAME :	dumpl
; PURPOSE :
;	read magnetic field line data and print them
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	dumpl, filename=filename
; INPUTS :
;
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	filename = name of field line file
; OUTPUTS :
;	none
; COMMON BLOCKS : 	none
; SIDE EFFECTS :
;	if filename is not supplied, it has to be typed in from keyboard
; RESTRICTIONS :
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 2000 September
;-
;------------------------------------------------------------------------
;
pro dumpl, filename=filename
;
	if not keyword_set(filename) then begin
		filename = ' '
		read,'enter input file name > ',filename
	endif

	listfile=''
	fout=0
	read,'enter listing file name (default=screen) > ',listfile
	if strtrim(listfile,2) ne '' then begin
		openw,lst,listfile,/get_lun
		fout=1
	endif

	readl, ldata,filename=filename,/list

	nf = ldata.nf
	ncomp = ldata.ncomp
	
	print,'number of field lines in the file =',nf
	npmax=100000
	npstep=1
	mode=9

while 1 do begin
	print, 'enter 1(simple), 2(full listing), 9(end)'
	read, mode
	if mode eq 9 then goto, fin

	if mode eq 2 then begin
		print,'specify field line numbers'
		line=''
		read, line
		lno = fix(split(line,delim=','))
		print, 'enter npstep (do n=0,np-1,npstep)'
		read, npmax,npstep
	endif else lno=indgen(nf)

	n = n_elements(lno)
	for i=0, n-1 do begin
		l = lno(i)
		np = ldata.np(l)
		loc = ldata.loc(l)
		x = ldata.heap(loc:loc+np-1,0)
		y = ldata.heap(loc:loc+np-1,1)
		z = ldata.heap(loc:loc+np-1,2)
		if ncomp eq 8 then begin
			bx = ldata.heap(loc:loc+np-1,4)
			by = ldata.heap(loc:loc+np-1,5)
			bz = ldata.heap(loc:loc+np-1,6)
			bs = ldata.heap(loc:loc+np-1,7)
		endif

		print,l,'-th field line,',np,' points'
		if mode eq 1 then nstep=np-1

		if ncomp eq 8 then begin
			print,''
			print,string('', $
			format="(/2x, t8,'x',t19,'y',t30,'z',t41,'bx',t52,'by',t63,'bz',t74,'bs')")
			for i=0,np-1,nstep do begin
				print,x(i),y(i),z(i),bx(i),by(i),bz(i),bs(i), $
				format='(2x, 7e11.3)'
			endfor
			if fout then begin
				printf,lun,''
				printf,lun,string('', $
				format="(/2x, t8,'x',t19,'y',t30,'z',t41,'bx',t52,'by',t63,'bz',t74,'bs')")
				for i=0,np-1,nstep do begin
					printf,lun, $
					x(i),y(i),z(i),bx(i),by(i),bz(i),bs(i), $
					format='(2x, 7e11.3)'
				endfor
			endif
		endif else begin
    			print,''
			print,string('', $
 			format="(/2x, t8,'x',t19,'y',t30,'z')")
			for i=0,np-1,nstep do begin
				print,x(i),y(i),z(i),$
				format='(2x, 3e11.3)'
			endfor
			if fount then begin
  				printf,lun,''
				printf,lun,string('', $
 				format="(/2x, t8,'x',t19,'y',t30,'z')")
				for i=0,np-1,nstep do begin
					printf,lun, $
					x(i),y(i),z(i), $
					format='(2x, 3e11.3)'
				endfor
			endif
		endelse
	endfor
endwhile
fin:
	if fout then begin
		close,lun
		free_lun,lun
	endif
	return
end
