;	Green's function monopole, kindL=11

pro field1, x,y,z,bx,by,bz, psi, xxs=xxs, lvec=lvec, a0=a0, b0=b0, $
	npoint=npoint

;	when x,y,z are 1-D vectors, xxs should be a 3-element vector
;	bx,by,bz are 1-D vectors representing field vectors at (x,y,z)

;	when xxs is a 2-D vector [xxs(0:2,*)], x,y,z should be scalars
;	bx,by,bz are sums of contributions from xxs, and are scalars

	ss = size(xxs)
	if ss(0) eq 2 then begin
		xsrc=xxs(0,*)
		ysrc=xxs(1,*)
		zsrc=xxs(2,*)
		bx = 0.0*xsrc
	endif else begin
		xsrc=xxs(0)
		ysrc=xxs(1)
		zsrc=xxs(2)
		bx = 0.0*x
	endelse
	by = bx
	bz = bx
	psi = bx

	pi=3.141592
	flx = b0*a0*a0*pi

	nvec = [0.0, 0.0, 1.0] 	; n-vector
	dotnl = total(lvec*nvec)
	amu = nvec-dotnl*lvec

	am = [7.699525, -20.40468, 17.55048, -4.345321, 1.0,-0.5, 0.5]
	dm = [2.200483,  1.741699, 1.353484,  1.123245, 0.5, 1.0, 0.0]

	if not keyword_set(npoint) then npoint=4
	if npoint eq 1 then begin
		n1=6
		n2=6
	endif else $
	if npoint eq 2 then begin
		n1=4
		n2=5
	endif else begin
		n1=0
		n2=3
	endelse


;  sum(m)=0.5
;  sum(m/d**2)=1.0
;  normalized bb = roughly gaussian
;	 flux(z>0)=3.141592
;	 radius   =1.0
;	 bn at center=1.0
;  b=sum(bb*flux/area)

	zsrc=0.0*zsrc
	bbx=0.0*bx
	bby=0.0*bx
	bbz=0.0*bx
	pp=0.0*bx

	for n=n1,n2 do begin
		x1=x-xsrc + a0*dm(n)*nvec(0)
		y1=y-ysrc + a0*dm(n)*nvec(1)
		z1=z-zsrc + a0*dm(n)*nvec(2)
		r2=x1^2+y1^2+z1^2
		r=sqrt(r2)
		r3=r2*r
        	dr=lvec(0)*x1+lvec(1)*y1+lvec(2)*z1
        	den=r+dr
        	rm=amu(0)*x1+amu(1)*y1+amu(2)*z1
        	bbx=bbx+am(n)*(-amu(0)/(r*den)+rm*lvec(0)/(r*den*den) $
			+rm*(2.0*r+dr)*x1/(r3*den*den)+dotnl*x1/r3)
		bby=bby+am(n)*(-amu(1)/(r*den)+rm*lvec(1)/(r*den*den) $
			+rm*(2.0*r+dr)*y1/(r3*den*den)+dotnl*y1/r3)
		bbz=bbz+am(n)*(-amu(2)/(r*den)+rm*lvec(2)/(r*den*den) $
			+rm*(2.0*r+dr)*z1/(r3*den*den)+dotnl*z1/r3)
		pp=pp+am(n)*(rm/den+dotnl)/r
	endfor
	bx=bx+bbx*flx/pi
	by=by+bby*flx/pi
	bz=bz+bbz*flx/pi
	psi=psi+pp*flx/pi

	if ss(0) eq 2 then begin
		bx = total(bx)
		by = total(by)
		bz = total(bz)
		psi = total(psi)
	endif

	return
end

;	generate a simulated magnetogram based on a
;	Green function monopole source

	common blkd, coord_d
	common blkt, coord_t

	nx = 81
	ny = 81
	xc = 0.0
	yc = 0.0
	dx = 0.2
	dy = 0.2
	rsp = 70.0
	bmax = 1000.0
	lvec = [0.0, 0.0, 1.0]
	lvec = lvec/sqrt(total(lvec*lvec))

	xxs = [0.0, 0.0, 0.0]
	xs=-8.0
	ys=-8.0

	xc = xs + dx*(nx/2)
	yc = ys + dy*(nx/2)

	mparam = defmp()

;	mparam.kindm = 2L	; pseudo vector
	mparam.kindm = 3L	; vector
	mparam.nbytem = 2
	mparam.nframe = 3
	mparam.tsys='GMT+09'
	mparam.inst='DUMMY'
	mparam.mfile='MONOPL'
	mparam.nx = nx
	mparam.ny = ny
	mparam.xs = xs
	mparam.ys = ys
	mparam.dx = dx
	mparam.dy = dy
	mparam.xxtod=[xc,yc]
	mparam.rsp = rsp
	mparam.mesh = 0
	mparam.title='MONOPOLE(PLANAR:BL)'

	rdata = fltarr(nx,ny,8)
	kz=0
	kx=1
	ky=2
	mparam.status.bl = kz
	mparam.status.bx = kx
	mparam.status.by = ky
	mparam.factors.bl=0.2
	mparam.factors.bx=0.2
	mparam.factors.by=0.2

	setdu, mparam
	settu, mparam

	x = fltarr(nx,ny)
	y = x
	z = x


	for j=0,ny-1 do x(*,j) = xs + dx*indgen(nx)
	for i=0,nx-1 do y(i,*) = ys + dy*indgen(ny)

	a0=1.0
	b0=1000.0
	field1, x,y,z,bx,by,bz,psi, a0=a0,b0=b0,xxs=xxs,lvec=lvec

	rdata(*,*,kx)=bx
	rdata(*,*,ky)=by
	rdata(*,*,kz)=bz
	bl = bx*lvec(0)+by*lvec(1)+bz*lvec(2)

        print, 'monopole (kindl=11):'
        print, '  location=',xxs
        print, '  a0,b0=',a0,b0
        print, '  lvec=',lvec
        print, '  nx,ny=',nx,ny
        print, '  xs,ys=',xs,ys,'  rsp=',rsp
        print, '  dx,dy=',dx,dy
        print, '  kindmo=',mparam.kindm

	xxs = [ [1.,0.,0.], [-1.,0.,0.] ]
	a0 = [1.0,1.0]
	b0=[1.0,1.0]
	for j=0,ny-1 do for i=0,nx-1 do begin
		field1, x(i,j),y(i,j),z(i,j),bx1,by1,bz1,psi1, $
			a0=a0,b0=b0,xxs=xxs,lvec=lvec
		bx(i,j)=bx1
		by(i,j)=by1
		bz(i,j)=bz1
		psi(i,j)=psi1
	endfor
end

