;	Green's funtion monopole, kindL=2

pro field2, x,y,z,bx,by,bz, psi, xxs=xxs, a0=a0, b0=b0, $
	npoint=npoint, rsp=rsp, rrss=rrss

	common crssn, rrsssave, ain,aout,amin,amout

;       when x,y,z are 1-D vectors, xxs should be a 3-element vector
;       bx,by,bz are 1-D vectors representing field vectors at (x,y,z)

;       when xxs is a 2-D vector [xxs(0:2,*)], x,y,z should be scalars
;       bx,by,bz are sums of contributions from xxs, and are scalars

	ss = size(xxs)
	if ss(0) eq 2 then begin
		xsrc=xxs(0,*)
		ysrc=xxs(1,*)
		zsrc=xxs(2,*)
		bx = 0.0*xsrc
	endif else begin
		xsrc=xxs(0)
		ysrc=xxs(1)
		zsrc=xxs(2)
		bx = 0.0*x
	endelse
	by = bx
	bz = bx
	psi = bx

	pi=3.141592
	flx = b0*a0*a0*pi
	xxsp = [0.0, 0.0, -rsp]

	if not keyword_set(rrss) then rrss=0.0
	if not keyword_set(rrsssave) then rrsssave=0.0

	if rrss gt 1.0 and rrss ne rrsssave then begin
;  source surface model when bn is specified
;  rsstbl is a table of relative source surface radii rrss = rss/rsp
;  amtbl = am/rrss, am=monopole strength
;  aoutbl = ao/rrss**2, ao=normalized outer radius of monopole
;  aout  = ao*rsp, ain=rsp**2/aout

		rsstbl = [0.10, 0.2, 0.3, 0.4, 0.5, 0.6, $
			0.7, 0.8, 0.85, 0.90, 0.95] 
		amtbl  = [1.210556, 1.189272, 1.139110, 1.052349, 0.941330, $
			0.855617, 0.834568, 0.855196, 0.881043, 0.930295, $
			0.982045 ]
		aoutbl = [0.898693, 0.892683, 0.878500, 0.853382, 0.822118, $
			0.808789, 0.837233, 0.877156, 0.906698, 1.006620, $
			1.121181 ]

		rssinv = 1.0/rrss
      	if rssinv gt rsstbl(10) then begin
			print, 'error: rrss too small',rrss
			stop
		endif
		if rssinv lt rsstbl(0) then begin
			print, 'warning: rrss is too large: set rrss=infinity'
			rrss=0.0
		endif else begin
			loc = where(rssinv lt rsstbl)
			n = loc(0)
			f=(rssinv-rsstbl(n-1))/(rsstbl(n)-rsstbl(n-1))
			ao=aoutbl(n-1)+f*(aoutbl(n)-aoutbl(n-1))
			ao=ao*rrss^2
			if ao le rrss then begin
				print, 'error: rrss too small',rrss
				stop
			endif

			aout=ao*rsp
			ain=rsp^2/aout
			amout=amtbl(n-1)+f*(amtbl(n)-amtbl(n-1))
			amout=amout*rrss/4.0
			amin=amout*rsp/aout

;  amtbl was calculated for the source am=2 on the solar surface.
;  to make emergent flux=pi, am should be am=0.5.
;  therefore amout is divided by 4.

		endelse
	endif

	am = [7.699525, -20.40468, 17.55048, -4.345321, 1.0,-0.5, 0.5]
	dm = [2.200483,  1.741699, 1.353484,  1.123245, 0.5, 1.0, 0.0]

	if not keyword_set(npoint) then npoint=4
	if npoint eq 1 then begin
		n1=6
		n2=6
	endif else $
	if npoint eq 2 then begin
		n1=4
		n2=5
	endif else begin
		n1=0
		n2=3
	endelse


;  sum(m)=0.5
;  sum(m/d**2)=1.0
;  normalized bb = roughly gaussian
;	 flux(z>0)=3.141592
;	 radius   =1.0
;	 bn at center=1.0
;  b=sum(bb*flux/area)

	x0 = x -xxsp(0)
	y0 = y -xxsp(1)
	z0 = z -xxsp(2)
	r0 = sqrt(x0*x0 +y0*y0 +z0*z0)

	zsrc = rsp - sqrt(rsp^2 - xsrc^2 -ysrc^2)
	nvec = [xsrc-xxsp(0),ysrc-xxsp(1),zsrc-xxsp(2)]/rsp

	bbx=0.0*bx
	bby=0.0*bx
	bbz=0.0*bx
	pp=0.0*bx

	for n=n1,n2 do begin
		x1=x-xsrc + a0*dm(n)*nvec(0)
		y1=y-ysrc + a0*dm(n)*nvec(1)
		z1=z-zsrc + a0*dm(n)*nvec(2)
		rn = x0*nvec(0)+y0*nvec(1)+z0*nvec(2)
		r1 =sqrt(x1*x1+y1*y1+z1*z1)
		sig = r1*0 + 1.0
		den0=r0+rn
		den =r1+rn-(rsp-a0*dm(n))
		loc = where(rn lt 0.0, count)
		if count gt 0 then begin
			sig(loc) =-1.0
			den0(loc)=r0(loc)-rn(loc)
			den(loc) =r1(loc)-rn(loc)+(rsp-a0*dm(n))
		endif

        	bbx=bbx+am(n)*( x1/r1^3 $
			+0.5*(sig*x0/r0+nvec(0))/(rsp*den0) $
			-0.5*(sig*x1/r1+nvec(0))/(rsp*den ) ) 
        	bby=bby+am(n)*( y1/r1^3 $
			+0.5*(sig*y0/r0+nvec(1))/(rsp*den0) $ 
			-0.5*(sig*y1/r1+nvec(1))/(rsp*den ) )
        	bbz=bbz+am(n)*( z1/r1^3 $
			+0.5*(sig*z0/r0+nvec(2))/(rsp*den0) $
			-0.5*(sig*z1/r1+nvec(2))/(rsp*den ) )
		pp=pp+am(n)*(1.0/r1-0.5*sig/rsp*alog(den0/den) )

	endfor

;  inclusion of source surface

	if rrss gt 1.0 then begin
		xout=x0- aout*nvec(0)
		yout=y0- aout*nvec(1)
		zout=z0- aout*nvec(2)
		xin =x0- ain *nvec(0)
		yin =y0- ain *nvec(1)
		zin =z0- ain *nvec(2)
		rout=sqrt(xout^2+yout^2+zout^2)
		rin =sqrt(xin^2+yin^2+zin^2)
		deni=rin+rn-ain
		loc = where(rn lt 0.0, count)
		if count gt 0 then $
			deni(loc)=rin(loc)-rn(loc)+ain

		bbx=bbx -amin*xin/rin^3 $
			-amout*( xout/rout^3 $
			+(sig*x0/r0 +nvec(0))/(rsp*den0) $
			-(sig*xin/rin+nvec(0))/(rsp*deni) )
		bby=bby -amin*yin/rin^3 $
			-amout*( yout/rout^3 $
			+(sig*y0/r0 +nvec(1))/(rsp*den0) $
			-(sig*yin/rin+nvec(1))/(rsp*deni) )
		bbz=bbz -amin*zin/rin^3 $
			-amout*( zout/rout^3 $
			+(sig*z0/r0 +nvec(2))/(rsp*den0) $
			-(sig*zin/rin+nvec(2))/(rsp*deni) )
		pp=pp -amin/rin $
			-amout*(1.0/rout-1.0/rsp*alog(den0/deni) )
	endif

	bx=bx+bbx*flx/pi
	by=by+bby*flx/pi
	bz=bz+bbz*flx/pi
	psi=psi+pp*flx/pi

	if ss(0) eq 2 then begin
		bx = total(bx)
		by = total(by)
		bz = total(bz)
		psi = total(psi)
	endif

	return
end

;	generate a simulated magnetogram based on a
;	Green function monopole source

	common blkd, coord_d
	common blkt, coord_t

	nx = 81
	ny = 81
	xc = 0.0
	yc = 0.0
	dx = 0.2
	dy = 0.2
	rsp = 70.0
	bmax = 1000.0

	xxs = [0.0, 0.0, 0.0]
	xs=-8.0
	ys=-8.0

	xc = xs + dx*(nx/2)
	yc = ys + dy*(nx/2)

	mparam = defmp()

;	mparam.kindm = 2L	; pseudo vector
	mparam.kindm = 3L	; vector
	mparam.nbytem = 2
	mparam.nframe = 3
	mparam.tsys='GMT+09'
	mparam.inst='DUMMY'
	mparam.mfile='MONOPL'
	mparam.nx = nx
	mparam.ny = ny
	mparam.xs = xs
	mparam.ys = ys
	mparam.dx = dx
	mparam.dy = dy
	mparam.xxtod=[xc,yc]
	mparam.rsp = rsp
	mparam.mesh = 0
	mparam.title='MONOPOLE(SPHERICAL:BN)'

	rdata = fltarr(nx,ny,8)
	kz=0
	kx=1
	ky=2
	mparam.status.bl = kz
	mparam.status.bx = kx
	mparam.status.by = ky
	mparam.factors.bl=0.2
	mparam.factors.bx=0.2
	mparam.factors.by=0.2

	setdu, mparam
	settu, mparam
	setas, mparam

	xxsp = [0.0, 0.0, -rsp]

	x = fltarr(nx,ny)
	y = x
	z = x

	for j=0,ny-1 do x(*,j) = xs + dx*indgen(nx)
	for i=0,nx-1 do y(i,*) = ys + dy*indgen(ny)

	projs, x,y,z
	vnx = (x-xxsp(0))/rsp
	vny = (y-xxsp(1))/rsp
	vnz = (z-xxsp(2))/rsp

	a0=1.0
	b0=1000.0
	rrss=2.5
	field2, x,y,z,bx,by,bz,psi, a0=a0,b0=b0,xxs=xxs,rsp=rsp, rrss=rrss

	rdata(*,*,kx)=bx
	rdata(*,*,ky)=by
	rdata(*,*,kz)=bz
	br = bx*vnx+by*vny+bz*vnz

	pi=3.141592
	dx1 = 2*pi/(nx-1)
	dy1 = pi/(ny-1)
	ap = fltarr(nx,ny)
	al = ap
	h = 0*ap +rsp*(rrss-1)

	for j=0,ny-1 do ap(*,j) = -pi + dx1*indgen(nx)
	for i=0,nx-1 do al(i,*) = -0.5*pi + dy1*indgen(ny)
	stoc, ap,al,h, x1,y1,z1
	vnx = (x1-xxsp(0))/(rsp*rrss)
	vny = (y1-xxsp(1))/(rsp*rrss)
	vnz = (z1-xxsp(2))/(rsp*rrss)
	field2, x1,y1,z1,bx1,by1,bz1,psi, a0=a0,b0=b0,xxs=xxs,rsp=rsp, rrss=rrss
	br1 = bx1*vnx+by1*vny+bz1*vnz
	bp1 = sqrt( (bx1-br1*vnx)^2+(by1-br1*vny)^2+(bz1-br1*vnz)^2 )

	print, 'monopole (kindl=2) :'
	print, '  location=',xxs
	print, '  a0,b0=',a0,b0
	print, '  rsp=',rsp
	if rrss gt 1.0 then $
	print, '  rrss=',rrss
	print, '  nx,ny=',nx,ny
	print, '  xs,ys=',xs,ys,'  rsp=',rsp
	print, '  dx,dy=',dx,dy
	print, '  kindmo=',mparam.kindm

        xxs = [ [1.,0.,0.], [-1.,0.,0.] ]
        a0 = [1.0,1.0]
        b0=[1.0,1.0]
        for j=0,ny-1 do for i=0,nx-1 do begin
                field2, x(i,j),y(i,j),z(i,j),bx1,by1,bz1,psi1, $
                        a0=a0,b0=b0,xxs=xxs, rsp=rsp
                bx(i,j)=bx1
                by(i,j)=by1
                bz(i,j)=bz1
                psi(i,j)=psi1
        endfor

end

