;+
; NAME :	jnft1
; PURPOSE :
;	show electric current distribution based on 
;	vector magnetograms of the Solar Flare Telescope
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	.run jnft1
; INPUTS :
;	following data should be on the memory
;	h = norikura header
;	img = intensity image
;	q,u,v = Stokes polarization parameters
;	bx,by,bl = magnetic vector components
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
; OUTPUTS :
;	electric currents (Jn) will be displayed
; COMMON BLOCKS : 	none
; SIDE EFFECTS :	arrays mparam and rdata will be created
; RESTRICTIONS :	X-window environment is assumed
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	1995.7 distinction between (q,u,v) and (bx,by,bl)
;-
;------------------------------------------------------------------------
;
; @transc
print,'original data size in x-direction = ',h(0).nx
nxnew=64
repeat begin
	print,'enter new data size in x-direction (64, 128, or 256)'
	print,'smaller data size gives lower resolution but less noise'
	read,nxnew
endrep until nxnew eq 64 or nxnew eq 128 or nxnew eq 256

if h(1).correct gt 100 then $
	convft1, h,img,bx,by,bl, mparam, rdata, nxnew=nxnew $
else begin
	convft1, h,img,q,u,v, mparam, rdata, nxnew=nxnew
	pol2b, mparam, rdata
endelse
curlb1, mparam, rdata, /keep_bp
magplot, mparam, rdata, /nomenu
end

