;+
; NAME :	kloc
; PURPOSE :
;	find the location of data on the memory
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	k = kloc( mparam, specstr=specstr )
; INPUTS :
;	mparam
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	specify data specification string explicitly as specstr or
;	symbolically as /i, /x, etc.
;	for potential fiels, use /px rather than /xp
;
;	keywords /bx and /bxp cannot be used because /bx is regarded
;	as an abbreviation for /bxp and is complained as 'ambiguous'
;
;	for divb, use /bdiv because /divb is ambiguous with /d
;
; OUTPUTS :
;	k = location indicator : specified data are in rdata(*,*,k)
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	TS 2000/09: /alpha added
;	TS 2000/10: /divb added, specstr added
;-
;------------------------------------------------------------------------
;
function kloc, mparam, specstr=specstr, $
	i=i, q=q, u=u, v=v, d=d, $
	x=x, y=y, z=z, px=px, py=py, pz=pz, $
	j=j, alpha=alpha, f=f, bdiv=bdiv

	if keyword_set(specstr) then begin
		sp1 = specstr & sp2 =' ' & sp3 = ' '
	endif

	if keyword_set(i) then begin
		sp1 = 'I'    & sp2 = ' ' & sp3 = ' '
	endif
	if keyword_set(q) then begin
		sp1 = 'Q'    & sp2 = 'Q/I'  & sp3 = ' ' 
	endif
	if keyword_set(u) then begin
		sp1 = 'U'    & sp2 = 'U/I'  & sp3 = ' '
	endif
	if keyword_set(v) then begin
		sp1 = 'V'    & sp2 = 'V/I'  & sp3 = ' '
	endif
	if keyword_set(x) then begin
		sp1 = 'BX'   & sp2 = 'BHX'  & sp3 = 'BPH'
	endif
	if keyword_set(y) then begin
		sp1 = 'BY'   & sp2 = 'BHY'  & sp3 = 'BTH'
	endif
	if keyword_set(z) then begin
		sp1 = 'BL'   & sp2 = 'BN'   & sp3 = 'BR'
	endif
	if keyword_set(px) then begin
		sp1 = 'BXP'  & sp2 = 'BHXP' & sp3 = 'BPHP'
	endif
	if keyword_set(py) then begin
		sp1 = 'BYP'  & sp2 = 'BHYP' & sp3 = 'BTHP'
	endif
	if keyword_set(pz) then begin
		sp1 = 'BLP'  & sp2 = 'BNP'  & sp3 = 'BRP'
	endif
	if keyword_set(d) then begin
		sp1 = 'VL'   & sp2 = 'VN'   & sp3 = 'VR'
	endif
	if keyword_set(j) then begin
		sp1 = 'JL'   & sp2 = 'JN'   & sp3 = 'JR'
	endif
	if keyword_set(f) then begin
		sp1 = 'FL'   & sp2 = 'FN'   & sp3 = 'FR'
	endif
	if keyword_set(alpha) then begin
		sp1 = 'ALPHA'   & sp2 = ' '   & sp3 = ' '
	endif
	if keyword_set(bdiv) then begin
		sp1 = 'DIVB'   & sp2 = ' '   & sp3 = ' '
	endif

	kloc1 = -1
	specs = defspecs(nspec)
	for i=0, nspec-1 do begin
		k = mparam.status.(i)
		if k ne -1 then begin
			spec = specs(i)
			if strtrim(spec,2) eq sp1 or $
			   strtrim(spec,2) eq sp2 or $
			   strtrim(spec,2) eq sp3 then $
				kloc1 = k
		endif
	endfor

	return, kloc1
end
