;+
; NAME :	kmem
; PURPOSE :
;	return an array 'karr' which contains a series of k-values as:
;	the first  valid data in rdata(*,*,karr(0))
;	the second valid data in rdata(*,*,karr(1)),,,
;	the last   valid data in rdata(*,*,karr(nframe-1))
;	the rest is filled with -1, and the size of karr is the
;	same as the third dimension of rdata
;	specarr contains the content of rdata in character string
;	larr contains the logical specification numbers
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	kmem, mparam, rdata, karr=karr,larr=larr,specarr=specarr,
;	                     specf=specf, [/nolist]
; INPUTS :
;	mparam, rdata
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	nolist: if set, suppress message
; OUTPUTS :
;	karr (integer array), larr (integer array), specarr(string array)
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1993
;	TS 2000/09 changed from function to procedure
;-
;------------------------------------------------------------------------
;
pro kmem, mparam, rdata, karr=karr, specarr=specarr, $
	larr=larr, specf=specf, nolist=nolist
;
	specs = defspecs(nspec)	; get nspec
	dim = size (rdata)
	kmax = fix(dim(3)) -1
	karr=intarr(kmax+1)-1
	larr=intarr(kmax+1)-1
	specarr=replicate('empty',kmax+1)
	specf=''

	km=-1
	for k=0, kmax do begin
		for i=0, nspec-1 do begin
			if k eq mparam.status.(i) then begin
				km=km+1
				karr(km)=k
				specarr(km)=specs(i)
				if specf eq '' then specf=specs(i) $
				else specf=specf+','+specs(i)
				larr(km)=i
				if not keyword_set(nolist) then $
					print,'rdata(*,*,'+strtrim(k,2)+') contains ',specs(i)
				goto, nextk
			endif
		endfor
	nextk:
	endfor
	if (km ne mparam.nframe -1) then begin
		print,'(func.kmem) nframe differs from the actual number of valid data in rdata'
		print,'  nframe=',mparam.nframe, ' is reset to', km+1
		mparam.nframe = km+1
	endif
	return
end
