;+
; NAME :	loglevel
; PURPOSE :
;	logarithmic data scaling
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	level = loglevel( bval, bmin, bmax )
; INPUTS :
;	bval, bmin(>0), bmax(>0)
;	abs(bval)=bmin --> return loglevel=0.0
;	bval= bmax --> return loglevel= 1.0
;	bval=-bmax --> return loglevel=-1.0
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	none
; OUTPUTS :
;	level (float)
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;-
;------------------------------------------------------------------------
;
function loglevel, bval, bmin, bmax
;
	logval = alog( ( abs(bval)+1.0e-30 )/bmin )  / alog(bmax/bmin)
	logval = logval > 0.0
	logval = logval < 1.0
	negative = where( bval lt 0, count )
	if ( count gt 0 ) then $
		logval( negative ) = -logval( negative )
	return, logval
end
