;+
; NAME :    lvl125
; PURPOSE :
;  generate contour levels
;  contour levels which are multiples of 1,2,5 are generated
;  nlevel : number of contour levels
;  plevel : positive contour levels, plevel(0)=min,
;           plevel(nlevel-1)=max
;  mlevel = -plevel : negative contour levels
;
; CATEGORY :
;     solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	lvl125, rdata1, nlevel=nlevel, plevel=plevel, mlevel=mlevel
;	here rdata is one frame of data arrays rdata
; OUTPUTS :
;     none
; COMMON BLOCKS :       none
; SIDE EFFECTS :  produce arrays
; RESTRICTIONS :  X-window environment is assumed
; PROCEDURE :
;
; MODIFICATION HISTORY :
;     T.Sakurai, 2000/10/09
;-
;------------------------------------------------------------------------
;

pro lvl125, rdata1, nlevel=nlevel, plevel=plevel, mlevel=mlevel

	clevel = [1.0, 2.0, 5.0]

	valmax=max(abs(rdata1))
	if valmax eq 0.0 then begin
		print,'(proc.lvl125) warning: data are all zero'
		plevel=indgen(nlevel)
		mlevel= -rotate(plevels,2)
		return
	endif

	plevel = fltarr(nlevel)
	f=alog10(valmax)
	dex=10.0^fix(f)
	f=valmax/dex
	if f gt clevel(2) then begin
		ist=2
	endif else if f gt clevel(1) then begin
		ist=1
	endif else begin
		ist=0
	endelse
	
	n1 = nlevel-1

	for i=n1,0,-1 do begin
		plevel(i) = clevel( (3*n1+ist+i-n1) mod 3 )*dex
		if (3*n1+ist+i-n1) mod 3 eq 0 then dex=dex/10.0
	endfor
	mlevel= -rotate(plevel,2)

	return
end
