;+
; NAME :	mklist
; PURPOSE :
;	make a list of scalar and vector data
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	mklist, mparam, nxy,nz,kxlist,kylist,kzlist,strx,stry,strz
; INPUTS :
;	mparam
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	none
; OUTPUTS :
;	scalar (or z-component) data indices will be stored in kzlist
;	from kzlist(0),,,,kzlist(nz-1)   -- total number = nz
;	likewise for vector data
;	strx,y,z,return string arrays
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;-
;------------------------------------------------------------------------
;
pro mklist, mparam, nxy,nz,kxlist,kylist,kzlist,strx,stry,strz
;
	specstr = defspecs(nspec)
	kxlist = intarr(10)
	kylist = intarr(10)
	kzlist = intarr(10)
	strx = strarr(10)
	stry = strarr(10)
	strz = strarr(10)
	nz = 0
	nxy = 0
 
	for lspecz = 0, nspec-1, 3 do begin
		kz = mparam.status.(lspecz)
		lspecx = lspecz +1
		lspecy = lspecz +2
		kx = mparam.status.(lspecx)
		ky = mparam.status.(lspecy)

		if ( kz ge 0 ) then begin
			kzlist(nz) = kz
			strz(nz) = specstr(lspecz)
			nz = nz +1
		endif

		if ( kx ge 0 and ky ge 0 ) then begin
			kxlist(nxy) = kx
			kylist(nxy) = ky
			strx(nxy) = specstr(lspecx)
			stry(nxy) = specstr(lspecy)
			nxy = nxy +1
		endif
	endfor
	return
end
	
