;  non-linear force-free field solution of Low and Lou
;  (Astrophys. J. 352, 343-352, 1990)
;
pro nlfff, x,y,z,bx,by,bz,xxs=xxs, dir=dir, b0=b0, $
	lenunit=al, alpha=alpha

	common cnlfff, order,isym,ak,dmu,nmu,pval,pdval

	x0 = x - xxs(0)
	y0 = y - xxs(1)
	z0 = z - xxs(2)
	r=sqrt(x0*x0+y0*y0+z0*z0)
	erx = x0/r
	ery = y0/r
	erz = z0/r
	efx=dir(1)*erz-dir(2)*ery
	efy=dir(2)*erx-dir(0)*erz
	efz=dir(0)*ery-dir(1)*erx
	sinth=sqrt(efx^2+efy^2+efz^2) < 1.0
	loc = where(sinth gt 0.0, count)
	if count gt 0 then begin
		efx(loc)=efx(loc)/sinth(loc)
		efy(loc)=efy(loc)/sinth(loc)
		efz(loc)=efz(loc)/sinth(loc)
	endif

	etx=efy*erz-efz*ery
	ety=efz*erx-efx*erz
	etz=efx*ery-efy*erx

;  pval(0) = p(costh=-1)
;  pcal(nmu) = p(costh=0)
;  pdval = dp/d(mu)

	costh=sqrt(abs(1.0-sinth^2))
	loc = where(dir(0)*erx+dir(1)*ery+dir(2)*erz lt 0.0, count)
	if count gt 0 then costh(loc)=-costh(loc)
	imu = fix((-abs(costh)+1.0)/dmu) < (nmu-2)
	rmu = (-abs(costh)+1.0)/dmu - imu
	pval1=pval(imu)*(1.0-rmu) + pval(imu+1)*rmu
	pdval1=pdval(imu)*(1.0-rmu) + pdval(imu+1)*rmu
	loc = where(costh ge 0.0, count)
	if count gt 0 then begin
	  if isym eq -1 then $
	    pval1(loc)=-pval1(loc) $
	  else $
	    pdval1(loc)=-pdval1(loc)
	endif

	denom=(r/al)^(2.0+order)
	br=-pdval1/denom
	bt=br*0.0
	bf=br*0.0
	loc = where(sinth gt 0.0, count)
	if count gt 0 then begin
	  bt(loc)= order*pval1(loc)/(denom(loc)*sinth(loc))
	  bf(loc)= ak*abs(pval1(loc))*abs(pval1(loc))^(1.0/order)/ $
		(denom(loc)*sinth(loc))
	endif
 	bx=b0*(br*erx+bt*etx+bf*efx)
 	by=b0*(br*ery+bt*ety+bf*efy)
 	bz=b0*(br*erz+bt*etz+bf*efz)
	alpha=ak*(1.0+1.0/order)/r*pval1*abs(pval1)^(1.0/order)
	return
end

pro rnlfff,x,f,g, dif=dif

	common cnlfff, order,isym,ak,dmu,nmu,pval,pdval

	ak=x(0)
	g = 0*x
	dmu=1.0/float(nmu-1)
;  y(0)=p
;  y(1)=dp/dmu
;  y(2)=mu
;  isym=-1: antisymmeric with respect to mu=0, therefore p(0)=0
;  isym= 1: symmetric with respect to mu=0, therefore dp/dmu(0)=0
	y0=[0.0, 10.0, -1.0]
	rk4, funct='odell', ya,fa, init=y0, stepsize=dmu, nsteps=nmu
	if isym eq -1 then $
	  g(0)=ya(0,nmu-1) $
	else $
	  g(0)=ya(1,nmu-1)

	f=total(g*g)
	pval=ya(0,*)
	pdval=ya(1,*)

;  pval stores the values of p from mu=-1 to 0
;  pdval stores the values of dp/dmu from mu=-1 to 0
	return
	end

;  ordinary differential equation of Low and Lou

pro odell, y,f

	common cnlfff, order,isym,ak,dmu,nmu,pval,pdval

;  y(0)=p
;  y(1)=dp/dmu
;  y(2)=mu(=costh)
;  f=dy/dmu

	f = 0.0*y

	amu=y(2)
	p=y(0)
	dp=y(1)
	f(0)=dp
	f(2)=1.0
	if abs(y(2)) eq 1.0 then $
	  f(1)=order*(order+1.0)*p/(2.0*amu) $
	else $
	  f(1)=-order*(order+1.0)*p* $
     	 (1.0+(ak/order)^2*abs(p)^(2.0/order)) /(1.0-amu*amu)

	return
end

;  generate a simulated magnetogram based on the non-linear force-free
;  field solution of Low and Lou (Astrophys. J. 352, 343-352, 1990)

	common cnlfff, order,isym,ak,dmu,nmu,pval,pdval
	common blkd, coord_d
	common blkt, coord_t

	nx = 41
	ny = 41
	dx = 0.5
	dy = 0.5
	rsp = 70.0
	bmax = 1000.0
	dir = [1.0, 0.0, 1.0]
	dir = dir/sqrt(total(dir*dir))
	iproj = 1

	xxs = [0.0, 0.0, -2.0]
	xs=-10.0
	ys=-10.0

	xc = xs + dx*(nx/2)
	yc = ys + dy*(nx/2)

	order=1
	isym=-1
	ak=sqrt(0.425)
	al=1.0
	b0 = 1000.0

;  kindm =2 - pseudo-vector
;        =3 - real vector
;  bx,by,bz are evaluated on the following meshes
;  iproj=0: plane, face-on
;        1: plane, oblique
;      2,3: sphere

        mparam = defmp()

;       mparam.kindm = 2L       ; pseudo vector
        mparam.kindm = 3L       ; vector
        mparam.nbytem = 2
        mparam.nframe = 3
        mparam.tsys='GMT+09'
        mparam.inst='DUMMY'
        mparam.mfile='DIPOLE'
        mparam.nx = nx
        mparam.ny = ny
        mparam.xs = xs
        mparam.ys = ys
        mparam.dx = dx
        mparam.dy = dy
        mparam.xxtod=[xc,yc]
        mparam.rsp = rsp
        mparam.mesh = 0
        mparam.title='NLFFF'

        rdata = fltarr(nx,ny,8)
        kz=0
        kx=1
        ky=2
        mparam.status.bl = kz
        mparam.status.bx = kx
        mparam.status.by = ky
        mparam.factors.bl=0.2
        mparam.factors.bx=0.2
        mparam.factors.by=0.2

        setdu, mparam
        settu, mparam

        x = fltarr(nx,ny)
        y = x
        z = x

        for j=0,ny-1 do x(*,j) = xs + dx*indgen(nx)
        for i=0,nx-1 do y(i,*) = ys + dy*indgen(ny)

        if iproj eq 1 then begin
                projt, x,y,z
                ctot, xxs(0),xxs(1),xxs(2), xx,yy,zz
                depth = -zz
        endif else $
        if iproj ge 2 then begin
                projs, x,y,z
                projt, x,y,z
                ctot, xxs(0),xxs(1),xxs(2), xx,yy,zz
                depth = -zz
        endif else begin
                depth = -xxs(2)
        endelse

	eps1=1.0e-4
	delx1=0.001
	ss1=0.025
	limit=5
	nmu=101
	pval=fltarr(nmu)
	pdval = fltarr(nmu)

	xvec=[ak]
	mnnl, xvec,f,gvec,funct='rnlfff',eps=eps1,dx=delx1,step=ss1, $
		limit=limit, status=status, prcntl=1
	ak=xvec(0)

	nlfff, x,y,z,bx,by,bz, xxs=xxs,dir=dir,b0=1.0,lenunit=al, alpha=alpha

	bmax1 = sqrt( max( bx*bx+by*by+bz*bz) )
	b00=b0/bmax1

	rdata(*,*,kx)=bx*b00
	rdata(*,*,ky)=by*b00
	rdata(*,*,kz)=bz*b00
;	if ka gt 0 then rdata(*,*,ka)=alpha
;	if kj gt 0 then rdata(*,*,kj)=alpha*bz*b00
;! incomplete

	print, 'non-linear force-free field of Low and Lou:'
	print, '  source location=',xxs
	print, '  b0=',b0
	print, '  direction=',dir
	if iproj eq 1 then $
	print, '  xtod,ytod=', mparam.xxtod(0:1)
	print, '  nx,ny=',nx,ny
	print, '  xs,ys=',xs,ys
	print, '  dx,dy=',dx,dy
	print, '  data projection: iproj=',iproj
	print, '  kindmo=',mparam.kindm
	print, '  ak=',ak
;	print, '  pval=' pval
end
