;+
; NAME :	nonfff
; PURPOSE :
;	generate a simulated magnetogram of a non-force-free field
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	.run nonfff
; INPUTS :
;	none ( parameters are given in the program )
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	none
; OUTPUTS :
;	creates a magnetogram file
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;-
;------------------------------------------------------------------------
;
;  (x,y,z): points where the field (bx,by,bz) is evaluated
;  xxs: reference position of the field
;
pro nonfff,  x,y,z,bx,by,bz,jx,jy,jz, $
	 a0=a0,ax=ax,ay=ay,akz=akz,xxs=xxs
;
	x0 = x - xxs(0)
	y0 = y - xxs(1)
	z0 = z

	e=exp(-ax*x0*x0-ay*y0*y0-akz*z0)

	bx = a0*akz*e
	by = 0.0*bx
	bz =-2.0*a0*ax*x0*e
	jx = 4.0*a0*ax*ay*x0*y0*e
	jy =(2.0*ax-4.0*(ax*x0)^2-akz*akz)*a0*e
	jz = 2.0*a0*ay*akz*y0*e
	return
end
;
;	generate a simulated magnetogram based on a non-force-free field
;	main program
;
	common blkd, coord_d
	common blkt, coord_t

	bmax = 1000.0
	xxs = [0.0, 0.0, -1.0]
	wx = 1.0
	wy = 1.0
	hz = 1.0

	nx = 21
	ny = 21
	xc = 0.0
	yc = 0.0
	dx = 0.5
	dy = 0.5
	xs = -dx*(nx/2)
	ys = -dy*(ny/2)

	mparam = defmp()

;	mparam.kindm = 23L
	mparam.kindm = 3L
	mparam.nbytem = 2
	mparam.nframe = 4
	mparam.tsys='GMT+09'
	mparam.inst='DUMMY'
	mparam.mfile='NONFFF'
	mparam.nx = nx
	mparam.ny = ny
	mparam.xs = xs
	mparam.ys = ys
	mparam.dx = dx
	mparam.dy = dy
	mparam.xxtod = [xc,yc]
	mparam.mesh = 0
	mparam.title='NONFFF'
;
	rdata = fltarr(nx,ny,8)
	kz=0
	kx=1
	ky=2
	kj = 3
	mparam.status.bl = kz
	mparam.status.bx = kx
	mparam.status.by = ky
	mparam.status.jn = kj
	mparam.factors.bl=0.2
	mparam.factors.bx=0.2
	mparam.factors.by=0.2
	mparam.factors.jn=1.0
	if mparam.kindm eq 3L then mparam.status.jn = -1
	if mparam.kindm eq 23L then mparam.status.jn = kj
;
	a0=1.0
	ax=1.0/wx^2
	ay=1.0/wy^2
	akz=1.0/hz
	bmax1=0.0

	x = fltarr(nx,ny)
	y = x
	z = x

	for j=0,ny-1 do x(*,j) = xs + dx*indgen(nx)
	for i=0,nx-1 do y(i,*) = ys + dy*indgen(ny)

	projt, x,y,z

	nonfff,  x,y,z,bx,by,bz,jx,jy,jz, $
		 a0=a0,ax=ax,ay=ay,akz=akz,xxs=xxs

	bmax1 = sqrt( max( bx*bx+by*by+bz*bz) )
	a0 = bmax*a0/bmax1
	rdata(*,*,kx)=bx*a0
	rdata(*,*,ky)=by*a0
	rdata(*,*,kz)=bz*a0

	if mparam.kindm eq 23L then begin
		jn= jx*coord_t.ztu(0) + jy*coord_t.ztu(1) + jz*coord_t.ztu(2)
		rdata(i,j,kj)=jn
	endif

end

