;+
; NAME :	plotl
; PURPOSE :
;	plot magnetic field lines on a line-drawing, b&w device
;	for printer output from disp.pro
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	plotl, mparam, ldata [,/color]
; INPUTS :
;	mparam : magnetogram parameters
;	ldata : data of field lines
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	color : if set, plot output is made in color
; OUTPUTS :
;	creates field line plot
; COMMON BLOCKS : 	none
; SIDE EFFECTS :
;	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1993
;	T.S. 1997 handling of nex
;	TS 2000 August: file I/O is moved to readl.pro
;	                field line data are passed by structure ldata
;-
;------------------------------------------------------------------------
;
pro plotl, mparam, ldata, color=color
;
	if keyword_set(color) then begin
		setcolor, bandwidth, w_start, rb_start, g_start, color8
		thecolor = color8.green
	endif else $
		thecolor = !p.color

	height0 = 50.0
	bright0 = 0.3

	nf = ldata.nf

	for l = 0, nf-1 do begin
		np = ldata.np(l)
		loc = ldata.loc(l)
		xx = ldata.heap(loc:loc+np1-1,0)
		yy = ldata.heap(loc:loc+np1-1,1)
		zz = ldata.heap(loc:loc+np1-1,2)

		for i=1, np-1 do begin
			plots, xx(i-1:i), yy(i-1:i), $
			color = thecolor, $
			thick=2, noclip=0
		endfor
	endfor
	return
end
