;+
; NAME :	pol2b
; PURPOSE :
;	conversion from polarization degree to magnetic field
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	pol2b, mparam, rdata
; INPUTS :
;	mparam
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	none
; OUTPUTS :
;	rdata
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	mparam will be modified
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;-
;------------------------------------------------------------------------
;
pro pol2b, mparam,rdata

	if mparam.kindm gt 0 then begin
		print,'(proc.pol2b) data are already converted into magnetic fields'
		return
	endif

	nx= long(mparam.nx)
	ny= long(mparam.ny)	; changed from fix to long 94/10

	kq = mparam.status.q
	ku = mparam.status.u
	kv = mparam.status.v
	kd = mparam.status.w

	if ( kv lt 0 ) then begin
		print, '(proc.pol2b) no polarization data in rdata array'
		return
	endif

	kindm0 = -mparam.kindm
;	kindm0 = 1000*wavei + 100*waved + 10*wavequ + wavev
	wavei = kindm0/1000 mod 10
	waved = kindm0/100 mod 10
	wavev = kindm0 mod 10
	kindm = 0L
	if wavei gt 0 then kindm = kindm + 1000L
	if waved gt 0 then kindm = kindm + 100L

	cbl1 = 0.0
	cbl3 = 0.0
	cbt1 = 0.0
	cvel = 0.0
	case wavev of
	1:	begin	; 5250A
		cbl1= -4500.0 
		cbl3=-15000.0
		cbt1= 2300.0
		end
;        cbl1= -4960.0 , cbl3=     0.0 , cbt1= 2470.0 -- old values
	3:	begin	; 6303A
		cbl1=  -4000.0 
;		cbt1=  3780.0
;
;	tentative
;	sum[ BL^2 ] = sum[ BT^2 ], i.e.
;	sum[(cbl1*V)^2] = sum[(cbt1*sqrt(LIN))^2]
		cbt1 = abs(cbl1) *sqrt( total(rdata(*,*,kv)^2) $
			/ total( sqrt( rdata(*,*,kq)^2 + rdata(*,*,ku)^2 ) ) )
		print, '(proc.pol2b) cbl,cbt=',cbl1,cbt1
		end
	4:	begin	; 6337A
		cbl1=  8000.0
		cvel = 15000.0
		end
	5:	begin	; 5324A
		cbl1 = -20000.0
		cvel = -22300.0 *2	; ?
		end
	endcase

	rdata(*,*,kv) = rdata(*,*,kv) * (cbl1 + cbl3*rdata(*,*,kv)^2 )
	mparam.status.v = -1
	mparam.status.bl = kv

	if (kd ge 0) then begin
		rdata(*,*,kd) = cvel* rdata(*,*,kd)
		mparam.status.w = -1
		mparam.status.vl = kd
	endif

	if ( kq lt 0 or ku lt 0 ) then begin
		mparam.kindm = kindm + 1L
		goto, exit
	endif

      th = fltarr(nx,ny)
      bt = th

      aux = where( rdata(*,*,kq) ne 0.0 or rdata(*,*,ku) ne 0.0, count )
      if count gt 0 then $
		th(aux) = atan( rdata(aux +nx*ny*ku), rdata(aux +nx*ny*kq) )/2.0
; ? tricky usage of where function

      bt = cbt1 * sqrt( sqrt( rdata(*,*,kq)^2 + rdata(*,*,ku)^2 ) )
      rdata(*,*,kq) = bt*cos(th)
      rdata(*,*,ku) = bt*sin(th)
      mparam.status.bx = kq
      mparam.status.by = ku
      mparam.status.q = -1
      mparam.status.u = -1

      mparam.kindm = kindm + 2L
exit:

	print,'(proc.pol2b) end: kindm=',mparam.kindm
	return
end
