;+
; NAME :	readl
; PURPOSE :
;	read magnetic field line data
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	readl, ldata [,filename=filename] ,[/list]
; INPUTS :
;
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	filename = name of field line file
;	/list : if set, parameter values in the input file are printed
; OUTPUTS :
;	ldata : a structure containing field line parameters and data
; COMMON BLOCKS : 	none
; SIDE EFFECTS :
;	if filename is not supplied, it has to be typed in from keyboard
; RESTRICTIONS :
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 2000 August
;-
;------------------------------------------------------------------------
;
pro readl, ldata, filename=filename, list=list
;
	if keyword_set(filename) then begin
		loffile = filename
	endif else begin
		loffile = ' '
		read,'(proc.readl) enter input file name > ',loffile
	endelse

	openr, u_lof, loffile, /get_lun
	point_lun,u_lof,0
	status = fstat(u_lof)
	nbyte = status.size

	ncomp=8
	nbytel=4
	lofno = 0
	loc = 0

	lhdr1 = deflh1()
	lhdr2 = deflh2()
	lhdr3 = deflh3()
	str4 = '    '
	str8 = '        '
	str12 = '            '

	readu, u_lof, lhdr1
	np = long(lhdr1.np)	; changed from fix to long 94/10

	if(lhdr1.nex eq str4) then $
		nex = 0 $
	else $
		nex = fix(lhdr1.nex)

	readu, u_lof, lhdr2

	if (nex eq 1) then begin
		readu, u_lof, lhdr3
		rrss = float(lhdr3.rrss)
	endif else rrss=0.0

	kindl	=long(lhdr2.kindl)
	nbytel	=fix(lhdr2.nbytel)
	ncomp	=fix(lhdr2.ncomp)
	xfactor	=float(lhdr2.xfactor)
	bfactor	=float(lhdr2.bfactor)
	if lhdr1.nfinit eq str4 then nfinit = 0 $
	else nfinit = fix(lhdr1.nfinit)

	if keyword_set(list) then begin
		print,'parameters in ',loffile
		print,'kind,nbyte,ncomp=',kindl,nbytel,ncomp
		print,'xfactor,bfactor=',xfactor,bfactor
		print,'lfile=',lhdr2.lfile
		print,'mtag=',lhdr2.mtag
		print,'ltag=',lhdr2.ltag
		if nex ne 0 then print,'nex=',nex
		print,'nfinit=',nfinit
		print,'ns1,2,3=',lhdr1.ns1,lhdr1.ns2,lhdr1.ns3
		print,'bmin1,bmin2=',lhdr1.bmin1,lhdr1.bmin2
		print,'flxpwr=',lhdr1.flxpwr
	endif

	nsize = nbyte/abs(nbytel*ncomp)
	ldata = defldat(ncomp,nsize,nfinit)
;	allocate heap of size (nsize,ncomp)
		
	ldata.nex = nex
	ldata.nfinit = nfinit
	ldata.ns1 = fix(lhdr1.ns1)
	ldata.ns2 = fix(lhdr1.ns2)
	ldata.ns3 = fix(lhdr1.ns3)
	ldata.bmin1 = float(lhdr1.bmin1)
	ldata.bmin2 = float(lhdr1.bmin2)
	ldata.flxpwr = float(lhdr1.flxpwr)
	ldata.kindl = long(kindl)
	ldata.nbytel = fix(nbytel)
	ldata.ncomp = ncomp
	ldata.xfactor = xfactor
	ldata.bfactor = bfactor
	ldata.lfile = lhdr2.lfile
	ldata.mtag = lhdr2.mtag
	ldata.ltag = lhdr2.ltag
	ldata.rrss = rrss
	if lhdr3.chksum ne str12 then ldata.chksum = long(lhdr3.chksum)

	while ( not eof(u_lof) ) do begin
		readu, u_lof, lhdr1
		np = long(lhdr1.np)
		if (np le 0) then goto, endlofread

		if ( abs(nbytel) eq 2) then begin
			if (ncomp eq 3 ) then $
				xyzb = intarr(3,np) $
			else $
				xyzb = intarr(8,np)
		endif else begin
			if (ncomp eq 3 ) then $
				xyzb = lonarr(3,np) $
			else $
				xyzb = lonarr(8,np)
		endelse

		readu, u_lof, xyzb
		nlast = ( abs(nbytel)*ncomp*np) mod 80
		if ( nlast ne 0 and not eof(u_lof) ) then begin
			skipstr = replicate(' ', 80 - nlast )
			readu, u_lof, skipstr
		endif

		if ( nbytel lt 0 ) then byteorder,xyzb
		xx = xfactor*xyzb(0,*)
		yy = xfactor*xyzb(1,*)
		zz = xfactor*xyzb(2,*)
		if ncomp eq 8 then begin
			ss = xfactor*xyzb(3,*)
			bx = bfactor*xyzb(4,*)
			by = bfactor*xyzb(5,*)
			bz = bfactor*xyzb(6,*)
			bs = bfactor*xyzb(7,*)
		endif

		ldata.np(lofno) = np
		ldata.loc(lofno) = loc
		ldata.heap(loc:loc+np-1,0) = xx
		ldata.heap(loc:loc+np-1,1) = yy
		ldata.heap(loc:loc+np-1,2) = zz
		if ncomp eq 8 then begin
			ldata.heap(loc:loc+np-1,3) = ss
			ldata.heap(loc:loc+np-1,4) = bx
			ldata.heap(loc:loc+np-1,5) = by
			ldata.heap(loc:loc+np-1,6) = bz
			ldata.heap(loc:loc+np-1,7) = bs
		endif
		lofno = lofno +1
		loc = loc + np

	endwhile
endlofread:

	ldata.nf = lofno

	close, u_lof
	free_lun, u_lof
	return
end
