;+
; NAME :	readm
; PURPOSE :
;	read magnetogram file
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	readm, mparam, rdata [,filename=magfile]
;		[,/list] [,/nolist]
; INPUTS :
;	none
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	filename : magnetogram file name
;	/list : if set, parameter values in the header are printed
;	/nolist: if set, messages are not printed
; OUTPUTS :
;	mparam : magnetogram parameters
;	rdata  : magnetogram data
; COMMON BLOCKS : 	none
; SIDE EFFECTS :
;	if filename is not specified, it should be typed in from keyboard
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	TS 2000/09 call to myint
;-
;------------------------------------------------------------------------
;
pro readm, mparam, rdata, filename=magfile, list=list, nolist=nolist

	if not keyword_set(magfile) then begin
		magfile = ' '
		read,'(proc.readm) enter input file name > ',magfile
	endif

	openr, unitm, magfile, /get_lun
	readmh1, unitm, mparam
	specs = defspecs(nspec)

	if keyword_set(list) then begin
		print,'parameters in ',magfile
		printmp, mparam 
	endif

	point_lun,unitm,320

	mhdr2 = defmh2( )
	nx = long(mparam.nx)
	ny = long(mparam.ny)	; changed from fix to long 94/10
	nbytem = abs(mparam.nbytem)	; take absolute value, 2000/09
	nframe=mparam.nframe
	maxdim = max([ min([8,8*512L*512L/(nx*ny)]),nframe])
	rdata = fltarr(nx,ny,maxdim)
	uflval, ufl, iufl2,iufl4

	k = 0
	blk12=string(' ',format='(a12)')

	for iframe=0, nframe-1 do begin
		readu,unitm, mhdr2
		spec=mhdr2.spec
		factor=float(mhdr2.factor)

		if mhdr2.alpha eq blk12 then $
			alpha=0.0 $
		else $
			alpha= float(mhdr2.alpha)

		if ( nbytem eq 2) then begin
			a=intarr(nx,ny)
			iufl=iufl2
		endif
		if ( nbytem eq 4) then begin
			a=lonarr(nx,ny)
			iufl=iufl4
		endif
		readu, unitm, a
		if ( mparam.nbytem*myint() lt 0 ) then byteorder,a
;			mparam.nbytem <0 for little-endian data in file
		loc = where(a eq iufl, count)
		if count gt 0 then a(loc) = ufl

		nlast = (nbytem*nx*ny) mod 80
		if ( nlast ne 0 and not eof(unitm) ) then begin
			skipstr = replicate(' ', 80 - nlast )
			readu,unitm,skipstr
		endif

		for i=0, nspec-1 do begin
			if strupcase(strtrim(spec)) eq specs(i) then begin
				if k eq maxdim then begin
					print, 'no space left in rdata, ', $
						 specs(i),' ignored'
					goto, escape
				endif

				rdata(*,*,k) = factor*a
				mparam.status.(i) = k
				mparam.factors.(i) = factor
				if not keyword_set(nolist) then $
					print, k, '-th record is ', specs(i),', factor=', factor
				if mparam.alpha eq 0.0 and alpha ne 0.0 then begin
					mparam.alpha = alpha
					if not keyword_set(nolist) then $
						print,'  alpha=',alpha
				endif
				k = k+1
				goto, escape
			endif
		endfor
		print,k,'-th record: invalid spec ',spec
		escape:

	endfor

	close,unitm
	free_lun, unitm
end
