;+
; NAME :	readmh1
; PURPOSE :
;	read magnetogram header part-1 from file and store it into mparam
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	readmh1, unitm, mparam
; INPUTS :
;	unitm = logical i/o unit number for magnetogram input
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	none
; OUTPUTS :
;	creates mparam
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	TS 95/10/31: add wavel
;	TS 97/07/01: if mhdr1.wavel=' ', set mparam.wavel=0
;	TS 2000/10/10: ypos=prad=0 if ydeg=pdeg
;-
;------------------------------------------------------------------------
;
pro readmh1, unitm, mparam
;
	mhdr1 = defmh1()
	mparam = defmp()

	point_lun,unitm,0
	readu, unitm, mhdr1

	xt = float(mhdr1.xt)
	yt = float(mhdr1.yt)
	pi=3.141592
	mparam.kindm   =fix(mhdr1.kindm)
	mparam.nbytem  =fix(mhdr1.nbytem)
	mparam.nframe  =fix(mhdr1.nframe)
	mparam.tsys    =mhdr1.tsys
	mparam.inst    =mhdr1.inst
	mparam.mfile   =mhdr1.mfile
	mparam.dateo   =mhdr1.dateo
	mparam.stime   =mhdr1.stime
	mparam.etime   =mhdr1.etime
	mparam.ftag    =mhdr1.ftag
	mparam.nx      =fix(mhdr1.nx)
	mparam.ny      =fix(mhdr1.ny)
	mparam.xs      =float(mhdr1.xs)
	mparam.ys      =float(mhdr1.ys)
	mparam.dx      =float(mhdr1.dx)
	mparam.dy      =float(mhdr1.dy)
	mparam.ydeg    =mhdr1.ydeg
	mparam.mesh    =fix(mhdr1.mesh)
	if mhdr1.wavel ne '        ' then mparam.wavel =float(mhdr1.wavel)
	mparam.rsp     =float(mhdr1.rsp)
	mparam.pdeg    =mhdr1.pdeg
	mparam.b0deg   =mhdr1.b0deg
	mparam.cl0deg  =mhdr1.cl0deg
	if mparam.pdeg eq mparam.ydeg then begin
		mparam.prad=0.0
		mparam.ypos=0.0
	endif else begin
		mparam.prad    =pi*mparam.pdeg/180.0
		mparam.ypos    =pi*mparam.ydeg/180.0
	endelse
	mparam.b0rad   =pi*mhdr1.b0deg/180.0
	mparam.cl0rad  =pi*mhdr1.cl0deg/180.0
	mparam.xxtod(0)  =float(mhdr1.xt)
	mparam.xxtod(1)  =float(mhdr1.yt)
	mparam.title   =mhdr1.title

	return
end
