;  solar differential rotation

	pro difrot, dday,al,model,rangle

;  dday = time difference in day
;  al   =latitude in radian
;  model=0: Carrington rotation
;        1: Newton and Nunn
;        2: Snodgrass
;  rangle=synodic rotation angle in radian

;  first calculate sidereal rotation speed rotdeg (deg/day)

	if model eq 1 then begin
		rotdeg= 14.368 - 2.69*sin(al)^2
	endif else $
	if model eq 2 then begin
		rotdeg= 14.049 - 1.492*sin(al)^2 - 2.605*sin(al)^4
	endif else begin
		rotdeg= 14.18
	endelse

;  then calculate synodic rotation angle rangle (rad/day)

	rangle=3.141592*(rotdeg-0.9856)*dday/180.0
	return
end

;  rotation of magnetograms around the polar axis
;  novect=0 requires vector transform
;  novect=1 indicates pseudo-scalar transform, ignoring vector properties
;
pro rotm, mparam, rdata, kindmo=kindm2,mesho=mesh2, $
	dday=dday,rotmodel=rotmodel,novect=novect

	common blkt, coord_t

	if n_elements(rotmodel) eq 0 then rotmodel=1	; assume Newton&Nunn
	if n_elements(kindm2) eq 0 then kindm2=mparam.kindm
	if n_elements(mesh2) eq 0 then mesh2=mparam.mesh
	if not keyword_set(dday) then dday=0.0
	if not keyword_set(novect) then novect=0

;  at first, transform into R-coordinates

	ms = umake(kindm2, m5,m4,m3,m2,m1)
	if not keyword_set(novect) then m5=3
	kindm3 = 10000L*m5+1000*m4+100*m3+10*m2+m1
	mesh3=3

	transm, mparam, rdata, kindmo=kindm3,mesho=mesh3
;  kindm and mesh in mparam are updated by transm

;  apply differential rotation

	xs = mparam.xs
	ys = mparam.ys
	dx = mparam.dx
	dy = mparam.dy
	nx = mparam.nx
	ny = mparam.ny

	apold=[xs,xs+dx*(nx-1),xs+dx*(nx-1),xs,xs,xs+dx*(nx-1)]
	almesh=[alofy(ys),alofy(ys),alofy(ys+dy*(ny-1)),alofy(ys+dy*(ny-1)),0,0]
	if almesh(0)*almesh(3) lt 0.0 then begin
		almesh(4)=0.0
	endif else begin
		almesh(4)=almesh(0)
	endelse
	if almesh(1)*almesh(2) lt 0.0 then begin
		almesh(5)=0.0
	endif else begin
		almesh(5)=almesh(1)
	endelse

	difrot,dday,almesh,rotmodel,rangle
	apnew=apold+rangle

	shift0=apnew(0)
	modulo, apnew,mesh3
	shift0=apnew(0)-shift0
	xsnew=min(apnew)
	xenew=max(apnew)
	dxnew=abs(dx)
	nxnew = ( xenew - xsnew )/dxnew + 1.5

	kmem,mparam,rdata,karr=kmemarr,/nolist
	nframe = mparam.nframe
	dimdata = size(rdata)
	if dimdata(0) eq 3 then nframe0 = dimdata(3) else nframe0=1
	newdata=fltarr(nxnew,ny,nframe0)
	uflval, ufl

	for km=0, nframe-1 do begin
		k=kmemarr(km)
		ap = fltarr(nxnew,ny)
		y  = fltarr(nxnew,ny)
		h  = fltarr(nxnew,ny)
		for jnew=0,ny-1 do ap(*,jnew)=xsnew+dxnew*indgen(nxnew)
		for inew=0,nxnew-1 do y(inew,*)=ys+dy*indgen(ny)
		al = alofy(y)

		difrot, dday,al,rotmodel,rangle
		for jnew=0,ny-1 do ap(*,jnew)=xsnew + $
			dxnew*indgen(nxnew) -rangle(*,jnew) -shift0
		loc =  getij (nx,ny,xs,ys,dx,dy,ap,y,ii,jj,xr,yr)
		b1=shift(rdata(*,*,k), 0, 0)
		b2=shift(rdata(*,*,k),-1, 0)
		b3=shift(rdata(*,*,k),-1,-1)
		b4=shift(rdata(*,*,k), 0,-1)
		b0 = ufl+fltarr(nxnew,ny)
		if (loc(0) ge 0) then begin
			i=ii(loc)
			j=jj(loc)
			b0(loc)= b1(i,j)+ $
				(b2(i,j)-b1(i,j))*xr(loc)+ $
				(b4(i,j)-b1(i,j))*yr(loc)+ $
				(b3(i,j)-b4(i,j)-b2(i,j)+b1(i,j))* $
					xr(loc)*yr(loc)
		endif else stop
		newdata(*,*,k)=b0
	endfor

	rdata = newdata
	mparam.xs=xsnew
	mparam.nx=nxnew
	mparam.dx=dxnew

	settu, mparam
	xto = coord_t.xxto(0)
	yto = coord_t.xxto(1)
	zto = coord_t.xxto(2)
	projr, xto,yto,zto, apr,alr
	difrot, dday,alr,rotmodel,rangle
	apr=apr+rangle
	rtoc, apr,alr,0.0, xto,yto,zto
	ctod, xto,yto,zto, xtod,ytod,ztod
	mparam.xxtod=[xtod,ytod]
	settu, mparam

;  finally, transform from R-coordinates to the coordinates specified
;  by mesh2
	transm, mparam,rdata, kindmo=kindm2,mesho=mesh2
;  kindm and mesh in mparam are updated by transm to final values

	print, '(proc.rotm  ) dday,rotmodel,novect=',dday,rotmodel,novect
	return
end

