;+
; NAME :	transc package (setar, projr, rtoc, ctor, rtod, dtor, rvu)
; PURPOSE :
;	coordinate transforms related to R-coordinates
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	see each entry
; INPUTS :
;	see each entry
; OPTIONAL INPUT PARAMETERS : 
; KEYWORD PARAMETERS :
; OUTPUTS :
;	see each entry
; COMMON BLOCKS : 	blkr is created by setar.pro
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	TS 2000/10/07: separated from transc.pro as setar.pro
;-
;------------------------------------------------------------------------
;

pro projr, x,y,z, apr,alr, extend=extend
;  projection of cartesian coordinates onto the spherical surface 

	common blkd, coord_d
	common blkr, coord_r

	rsp = coord_r.rsp
	xxsp = coord_r.xxsp

	z=0*x + xxsp(2)
	apr = x*0.0
	alr = x*0.0
	pi = 3.141592
	eps = 0.01
	uflval, ufl

	r2 = (x - xxsp(0))^2 + (y - xxsp(1))^2

;  points within solar disk
	loc = where (r2 lt rsp*rsp, count)
	if count gt 0 then begin
		z(loc) = sqrt(rsp*rsp-r2(loc)) +xxsp(2)
		ctor, x(loc),y(loc),z(loc), apr1,alr1,h1
		apr(loc)=apr1
		alr(loc)=alr1
	endif

;   points outside solar disk but there exists y-intercept
	loc = where (r2 ge rsp*rsp and abs(y-xxsp(1)) lt rsp, count)
	if count gt 0 and keyword_set(extend) then begin
		x1 = sign(sqrt(rsp*rsp-y(loc)^2), x(loc))
		z(loc) = xxsp(2)
		ctor, x1,y(loc),z(loc),apr1,alr1,h1
		apr(loc)=apr1
		alr(loc)=alr1
	endif
	if count gt 0 and not keyword_set(extend) then begin
		apr(loc)=ufl
		alr(loc)=ufl
	endif

;  y > +rsp
	loc = where(y-xxsp(1) gt rsp, count)
	if count gt 0 and keyword_set(extend) then begin
		if b0 gt 0 then begin
			apr(loc) = cl0eff +sign(pi-eps, x(loc))
			alr(loc) = 0.5*pi
		endif else begin
			apr(loc) = cl0eff +sign(0.5*pi-rps, x(loc))
			alr(loc) = 0.5*pi +b0
		endelse
	endif
	if count gt 0 and not keyword_set(extend) then begin
		apr(loc)=ufl
		alr(loc)=ufl
	endif
;  y < -rsp
	loc = where(y-xxsp(1) lt -rsp, count)
	if count gt 0 then begin
		if b0 gt 0 then begin
			apr(loc) = cl0eff +sign(pi-eps, x(loc))
			alr(loc) = -0.5*pi +b0
		endif else begin
			apr(loc) = cl0eff +sign(0.5*pi-rps, x(loc))
			alr(loc) = -0.5*pi
		endelse
	endif
	if count gt 0 and not keyword_set(extend) then begin
		apr(loc)=ufl
		alr(loc)=ufl
	endif
	
	return
end

pro rtoc, apr,alr,h, x,y,z
	common blkd, coord_d
	common blkr, coord_r

	rsp = coord_r.rsp
	cl0eff = coord_r.cl0eff
	halfpi = 0.5*3.141592	

	convsx, h+rsp, halfpi-alr, apr-cl0eff, zr,xr,yr
;	xr=conventional y
;	yr=conventional z
;	zr=conventional x
	mat=coord_r.armat
;	x = mat(0,0)*xr + mat(0,1)*yr + mat(0,2)*zr + coord_r.xxsp(0)
;	y = mat(1,0)*xr + mat(1,1)*yr + mat(1,2)*zr + coord_r.xxsp(1)
;	z = mat(2,0)*xr + mat(2,1)*yr + mat(2,2)*zr + coord_r.xxsp(2)
	x = mat(0,0)*xr + mat(0,1)*yr + mat(0,2)*zr + mat(0,3)
	y = mat(1,0)*xr + mat(1,1)*yr + mat(1,2)*zr + mat(1,3)
	z = mat(2,0)*xr + mat(2,1)*yr + mat(2,2)*zr + mat(2,3)
	return
end

pro ctor, x,y,z, apr,alr,h
	common blkd, coord_d
	common blkr, coord_r

	rsp = coord_r.rsp
	cl0eff = coord_r.cl0eff
	pi=3.141592
	halfpi = 0.5*3.141592	
	mat=coord_r.arinv
	xr= mat(0,0)*x + mat(0,1)*y + mat(0,2)*z + mat(0,3)
	yr= mat(1,0)*x + mat(1,1)*y + mat(1,2)*z + mat(1,3)
	zr= mat(2,0)*x + mat(2,1)*y + mat(2,2)*z + mat(2,3)
;	xx = x - coord_r.xxsp(0)
;	yy = y - coord_r.xxsp(1)
;	zz = z - coord_r.xxsp(2)
;	xr= xx*armat(0,0) + yy*armat(1,0) + zz*armat(2,0)
;	yr= xx*armat(0,1) + yy*armat(1,1) + zz*armat(2,1)
;	zr= xx*armat(0,2) + yy*armat(1,2) + zz*armat(2,2)
;	conventional x = zr
;	conventional y = xr
; 	conventional z = yr
	convxs, zr,xr,yr, r,theta,phi
	apr=phi+cl0eff
	alr=halfpi-theta
	h=r-rsp
	return
end

pro rtod, apr,alr,h, xd,yd,zd

	rtoc, apr,alr,h, x,y,z
	ctod, x,y,z, xd,yd,zd
	return
end

pro dtor, xd,yd,zd, apr,alr,h

	dtoc, xd,yd,zd, x,y,z
	ctor, x,y,z, apr,alr,h
	return
end

;  unit vectors of R-coordinates

pro rvu, apr,alr, apru,alru,hu

      common blkr, coord_r

      n = n_elements(apr)
      apru0 = fltarr(3,n)
      alru0 = apru0
      hu0 = apru0
	cl0eff = coord_r.cl0eff

      apru0(0,*)= cos(apr-cl0eff)
      apru0(1,*)= 0.0
      apru0(2,*)=-sin(apr-cl0eff)
      alru0(0,*)=-sin(alr)*sin(apr-cl0eff)
      alru0(1,*)= cos(alr)
      alru0(2,*)=-sin(alr)*cos(apr-cl0eff)
      hu0 (0,*)= cos(alr)*sin(apr-cl0eff)
      hu0 (1,*)= sin(alr)
      hu0 (2,*)= cos(alr)*cos(apr-cl0eff)

      mat = coord_r.armat(0:2,0:2)

      apru = mat#apru0
      alru = mat#alru0
      hu  = mat#hu0

      return
end

pro setar, mparam
; set matrix for R-coordinates

	common blkr, coord_r

	rsp = mparam.rsp
	xxsp=[0.0, 0.0, -rsp]
	p = mparam.prad
	b0 = mparam.b0rad
	cl0 = mparam.cl0rad
	if mparam.mesh eq 4 then cl0eff=cl0 else cl0eff=0.0

	armat= [[ cos(p),         sin(p),         0.0,     0.0 ], $
		[-sin(p)*cos(b0), cos(p)*cos(b0), sin(b0), 0.0 ], $
		[ sin(p)*sin(b0),-cos(p)*sin(b0), cos(b0), 0.0 ], $
		[ xxsp(0),        xxsp(1),        xxsp(2), 1.0    ] ]
	arinv = fltarr(4,4)
	arinv(indgen(4)*(4+1)) = 1.0	; unit matirx
	arinv(0:2,3) = -armat(0:2,3)
	arinv = rotate(armat,4)#arinv
	arinv(3,*)=[0,0,0,1]
	
	coord_r = { coord_r1, $
		armat:armat, arinv:arinv, xxsp:xxsp, rsp:rsp, $
		p:p, b0:b0, cl0:cl0, cl0eff:cl0eff }
	return
end

