;+
; NAME :	transc package (setas, projs, stoc, ctos, stod, dtos, svu)
; PURPOSE :
;	coordinate transforms related to S-coordinates
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	see each entry
; INPUTS :
;	see each entry
; OPTIONAL INPUT PARAMETERS : 
; KEYWORD PARAMETERS :
; OUTPUTS :
;	see each entry
; COMMON BLOCKS : 	blks is created by setas.pro
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	TS 2000/10/07: separated from transc.pro as setas.pro
;-
;------------------------------------------------------------------------
;

pro projs, x,y,z, ap,al, extend=extend
;  projection of cartesian coordinates onto the spherical surface 

	common blkd, coord_d
	common blks, coord_s

	rsp = coord_s.rsp
	xxsp = coord_s.xxsp
	halfpi = 1.5707963
	uflval, ufl

	z=0*x + xxsp(2)
	ap = 0.0*x
	al = 0.0*x
	r2 = (x - xxsp(0))^2 + (y - xxsp(1))^2

	loc = where (r2 lt rsp*rsp, count)
	if count gt 0 then begin
		z(loc) = sqrt(rsp*rsp-r2(loc)) + xxsp(2)
		ctos, x(loc),y(loc),z(loc), ap1,al1,h1
		ap(loc) = ap1
		al(loc) = al1
	endif

	loc = where(r2 ge rsp*rsp and abs(y-xxsp(1)) lt rsp, count)
	if count gt 0 and keyword_set(extend) then begin
		x1 = sign(sqrt(abs(rsp^2-y(loc)^2)), x(loc))
		z(loc) = xxsp(2)
		ctos, x1,y(loc),z(loc), ap1,al1,h1
		ap(loc) = ap1
		al(loc) = al1
	endif
	if count gt 0 and not keyword_set(extend) then begin
		ap(loc)=ufl
		al(loc)=ufl
	endif

	loc = where(abs(y-xxsp(1)) ge rsp, count)
	if count gt 0 then begin
		ap(loc) = sign(halfpi,x(loc))
		al(loc) = sign(halfpi,y(loc)-xxsp(1))
	endif
	if count gt 0 and not keyword_set(extend) then begin
		ap(loc)=ufl
		al(loc)=ufl
	endif

	return
end

pro stoc, ap,al,h, x,y,z

	stod, ap,al,h, xd,yd,zd
	dtoc, xd,yd,zd, x,y,z
	return
end

pro ctos, x,y,z, ap,al,h

	ctod, x,y,z, xd,yd,zd
	dtos, xd,yd,zd, ap,al,h
	return
end

pro stod, ap,al,h, xd,yd,zd
	common blkd, coord_d
	common blks, coord_s

	halfpi=1.5707963
	rsp = coord_s.rsp
	xxsp = coord_s.xxsp
	convsx, h+rsp, halfpi-al, ap, zd,xd,yd
;	xd = conventional y + xxsp(0)
;	yd = conventional z + xxsp(1)
;	zd = conventional x + xxsp(2)
	xd = xd +xxsp(0)
	yd = yd +xxsp(1)
	zd = zd +xxsp(2)
	return
end

pro dtos, xd,yd,zd, ap,al,h
	common blkd, coord_d
	common blks, coord_s

	halfpi = 0.5*3.141592
	rsp = coord_s.rsp
	xxsp = coord_s.xxsp

;	conventional x = zd - xxsp(2)
;	conventional y = xd - xxsp(0)
;	conventional z = yd - xxsp(1)
	convxs, zd -xxsp(2), xd-xxsp(0), yd-xxsp(1), r,theta,phi
	ap = phi
	al = halfpi - theta
	h = r-rsp
	return
end

;  unit vectors of S-coordinates

pro svu, ap,al, apu,alu,hu
      common blks, coord_s

      n = n_elements(ap)
      apu0 = fltarr(3,n)
      alu0 = apu0
      hu0 = apu0

      apu0(0,*)= cos(ap)
      apu0(1,*)= 0.0
      apu0(2,*)=-sin(ap)
      alu0(0,*)=-sin(al)*sin(ap)
      alu0(1,*)= cos(al)
      alu0(2,*)=-sin(al)*cos(ap)
      hu0 (0,*)= cos(al)*sin(ap)
      hu0 (1,*)= sin(al)
      hu0 (2,*)= cos(al)*cos(ap)

      mat = coord_s.asmat(0:2,0:2)

      apu = mat#apu0
      alu = mat#alu0
      hu  = mat#hu0

      return
end

pro setas, mparam
; set matrix for S-coordinates

	common blks, coord_s

	rsp = mparam.rsp
	xxsp=[0.0, 0.0, -rsp]
	ypos = mparam.ypos

	asmat=[ [ cos(ypos), sin(ypos), 0.0,    0.0 ], $
		[-sin(ypos), cos(ypos), 0.0,    0.0 ], $
		[ 0.0,       0.0,       1.0,    0.0 ], $
		[ xxsp(0),   xxsp(1),  xxsp(2), 1.0   ] ]
	asinv = fltarr(4,4)
	asinv(indgen(4)*(4+1)) = 1.0	; unit matrix
	asinv(0:2,3) = -asmat(0:2,3)
	asinv = rotate(asmat,4)#asinv
	asinv(3,*)=[0,0,0,1]

	coord_s = { coord_s1, $
		asmat:asmat, asinv:asinv, xxsp:xxsp, rsp:rsp, ypos:ypos }
	return
end
