;+
; NAME :	transc package (setdu, dtoc, ctod, convxs, convsx)
; PURPOSE :
;	coordinate transforms related to D-coodinates
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	see each entry
; INPUTS :
;	see each entry
; OPTIONAL INPUT PARAMETERS : 
; KEYWORD PARAMETERS :
; OUTPUTS :
;	see each entry
; COMMON BLOCKS : 	blkd is created by setdu.pro
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	TS 2000/10/07: separated from transc.pro as setdu.pro 
;-
;------------------------------------------------------------------------
;
function unitmat, n
; now made in-line and not used
	mat = fltarr(n,n)
	mat(indgen(n)*(n+1)) = 1.0
	return, mat
end

pro dtoc, xd,yd,zd, x,y,z
	common blkd, coord_d

	mat = coord_d.admat
	x = mat(0,0)*xd + mat(0,1)*yd + mat(0,2)*zd + mat(0,3)
	y = mat(1,0)*xd + mat(1,1)*yd + mat(1,2)*zd + mat(1,3)
	z = mat(2,0)*xd + mat(2,1)*yd + mat(2,2)*zd + mat(2,3)
;	xdu = coord_d.xdu
;	ydu = coord_d.ydu
;	zdu = coord_d.zdu
;	x = xdu(0)*xd + ydu(0)*yd + coord_d.xxsp(0)
;	y = xdu(1)*xd + ydu(1)*yd + coord_d.xxsp(1)
;	z = zd + coord_d.xxsp(2) + coord_d.rsp
	return
end

pro ctod, x,y,z, xd,yd,zd
	common blkd, coord_d

	mat = coord_d.adinv
	xd = mat(0,0)*x + mat(0,1)*y + mat(0,2)*z + mat(0,3)
	yd = mat(1,0)*x + mat(1,1)*y + mat(1,2)*z + mat(1,3)
	zd = mat(2,0)*x + mat(2,1)*y + mat(2,2)*z + mat(2,3)
;	xdu = coord_d.xdu
;	ydu = coord_d.ydu
;	zdu = coord_d.zdu
;	xx = x - coord_d.xxsp(0)
;	yy = y - coord_d.xxsp(1)
;	xd = xx*xdu(0) + yy*xdu(1)
;	yd = xx*ydu(0) + yy*ydu(1)
;	zd = z - coord_d.xxsp(2) -coord_d.rsp
	return
end

pro convxs, convx,convy,convz, r,theta,phi
;  conversion between (x,y,z) and (r,theta,phi)
;  conventional definition is adopted here, namely
;  the z-axis is the polar axis (theta=0 or pi)
;  the phi-angle is measured from the x-axis toward the y-axis
;  in the xz-plane, phi=0 in x>0 and phi=pi in x<0
;
	r    = sqrt(convx^2+convy^2+convz^2)
	theta= acos(convz/r)
	phi  = atan(convy,convx)
	return
end

pro convsx, r,theta,phi, convx,convy,convz

	convx=r*sin(theta)*cos(phi)
	convy=r*sin(theta)*sin(phi)
	convz=r*cos(theta)
	return
end

pro setdu, mparam
;	set matrix and vectors of data coordinates
;
	common blkd, coord_d

	rsp = mparam.rsp
	xxsp=[0.0, 0.0, -rsp]
	ypos=mparam.ypos
	sinal=sin(ypos)
	cosal=cos(ypos)
	sinap=sin(ypos)
	cosap=cos(ypos)

	admat=[ [ cos(ypos), sin(ypos), 0.0,      0.0 ], $
		[-sin(ypos), cos(ypos), 0.0,      0.0 ], $
		[ 0.0,       0.0,       1.0,      0.0 ], $
		[ xxsp(0),   xxsp(1),   xxsp(2)+rsp,  1.0 ]  ]

; in usual matrix notation admat(i,j) is as follows
;                   ----> j
; | admat = cos(ypos)   -sin(ypos)   0.0   xxsp(0)
; V         sin(pos)     cos(ypos)   0.0   xxsp(1)
;           0.0          0.0         1.0   xxsp(2)+rsp
; i         0.0          0.0         0.0   1.0
;

	adinv = fltarr(4,4)
	adinv(indgen(4)*(4+1)) = 1.0	; unit matrix
	adinv(0:2,3) = -admat(0:2,3)
	adinv = rotate(admat,4)#adinv
	adinv(3,*)=[0,0,0,1]

	xdu= [ cos(ypos), sin(ypos), 0.0]
	ydu= [-sin(ypos), cos(ypos), 0.0]
	zdu= [ 0.0,       0.0,       1.0]

	coord_d = { coord_d1, $
		admat:admat, adinv:adinv, xxsp:xxsp, rsp:rsp, $
		ypos:ypos, xdu:xdu, ydu:ydu, zdu:zdu }
	return
end
