;+
; NAME :	transc package (settu, projt, ttod, dtot, ttoc, ctot)
; PURPOSE :
;	coordinate transforms related to T-coordinates
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	see each entry
; INPUTS :
;	see each entry
; OPTIONAL INPUT PARAMETERS : 
; KEYWORD PARAMETERS :
; OUTPUTS :
;	see each entry
; COMMON BLOCKS : 	blkt is created by settu.pro
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	TS 2000/10/07: separated from transc.pro as settu.pro
;-
;------------------------------------------------------------------------
;

pro projt, x,y,z, xt,yt
;	projection of cartesian coordinates onto a tangential plane

	common blkd, coord_d
	common blkt, coord_t

;     deltax=x-coord_t.xxto(0)
;     deltay=y-coord_t.xxto(1)
      cosals = coord_t.cosals
      cosaps = coord_t.cosaps
      sinals = coord_t.sinals
      sinaps = coord_t.sinaps
      den=cosals*cosaps
	xxto = coord_t.xxto
;     xt=(cosals*deltax + sinals*sinaps*deltay)/den
;     yt= cosaps*deltay/den
;     zd=-(sinaps*cosals*deltax + sinals*deltay)/den
;     z=zd +coord_t.xxto(2)
	mat1= fltarr(4,4)
	mat1(indgen(4)*(4+1)) = 1.0	; unit matrix
	mat1(0,3)= -xxto(0)
	mat1(1,3)= -xxto(1)
	mat2 = [[ cosals/den,        0.0,      -cosals*sinaps/den, 0.0 ], $
		[ sinals*sinaps/den, cosaps/den, -sinals/den,        0.0 ], $
		[ 0.0,               0.0,        1.0,                0.0 ], $
		[ 0.0,               0.0,        xxto(2),            1.0 ] ]
	mat = mat2#mat1
	xt = mat(0,0)*x + mat(0,1)*y + mat(0,3)
	yt = mat(1,0)*x + mat(1,1)*y + mat(1,3)
	z  = mat(2,0)*x + mat(2,1)*y + mat(2,3)
      return
end

pro ttod, xt,yt,zt, xd,yd,zd
	common blkt, coord_t

      mat = coord_t.atmat
      xd = mat(0,0)*xt + mat(0,1)*yt + mat(0,2)*zt + mat(0,3)
      yd = mat(1,0)*xt + mat(1,1)*yt + mat(1,2)*zt + mat(1,3)
      zd = mat(2,0)*xt + mat(2,1)*yt + mat(2,2)*zt + mat(2,3)
;     xd = mat(0,0)*xt + mat(0,1)*yt + mat(0,2)*zt + coord_t.xxto(0)
;     yd = mat(1,0)*xt + mat(1,1)*yt + mat(1,2)*zt + coord_t.xxto(1)
;     zd = mat(2,0)*xt + mat(2,1)*yt + mat(2,2)*zt + coord_t.xxto(2)
      return
end

pro dtot, xd,yd,zd, xt,yt,zt
	common blkt, coord_t

	mat = coord_t.atinv
	xt = mat(0,0)*xd + mat(0,1)*yd + mat(0,2)*zd + mat(0,3)
	yt = mat(1,0)*xd + mat(1,1)*yd + mat(1,2)*zd + mat(1,3)
	zt = mat(2,0)*xd + mat(2,1)*yd + mat(2,2)*zd + mat(2,3)
;	xx = xd - coord_t.xxtod(0)
;	yy = yd - coord_t.xxtod(1)
;	zz = zd - coord_t.xxtod(2)
;	xt = xx*atmat(0,0) + yy*atmat(1,0) + zz*atmat(2,0)
;	yt = xx*atmat(0,1) + yy*atmat(1,1) + zz*atmat(2,1)
;	zt = xx*atmat(0,2) + yy*atmat(1,2) + zz*atmat(2,2)
	return
end

pro ttoc, xt,yt,zt, x,y,z
	common blkd, coord_d
	common blkt, coord_t

;	ttod, xt,yt,zt, xd,yd,zd, coord_t, coord_d
;	dtoc, xd,yd,zc, x,y,z, coord_d
	mat = coord_d.admat # coord_t.atmat
	x = mat(0,0)*xt + mat(0,1)*yt + mat(0,2)*zt + mat(0,3)
	y = mat(1,0)*xt + mat(1,1)*yt + mat(1,2)*zt + mat(1,3)
	z = mat(2,0)*xt + mat(2,1)*yt + mat(2,2)*zt + mat(2,3)
	return
end

pro ctot, x,y,z, xt,yt,zt
	common blkd, coord_d
	common blkt, coord_t

;	ctod, x,y,z, xd,yd,zd, coord_d
;	dtot, xd,yd,zd, xt,yt,zt, coord_d, coord_t
	mat = coord_t.atinv # coord_d.adinv
	xt = mat(0,0)*x + mat(0,1)*y + mat(0,2)*z + mat(0,3)
	yt = mat(1,0)*x + mat(1,1)*y + mat(1,2)*z + mat(1,3)
	zt = mat(2,0)*x + mat(2,1)*y + mat(2,2)*z + mat(2,3)
	return
end

pro settu, mparam
;	set matrix and vectors of tangential coordinates

	common blkt, coord_t

	rsp = mparam.rsp
	xxsp=[0.0, 0.0, -rsp]
	halfpi = 1.5707963

	if mparam.mesh eq 0 then begin
		xtod=mparam.xs+0.5*mparam.dx*(mparam.nx-1)
		ytod=mparam.ys+0.5*mparam.dy*(mparam.ny-1)
		mparam.xxtod(0)=xtod
		mparam.xxtod(1)=ytod
	endif else begin
		xtod = mparam.xxtod(0)
		ytod = mparam.xxtod(1)
	endelse

;  in the fortran version projs is called
;  here, equivalent operations are made in-line
	zt2 = rsp*rsp - xtod*xtod - ytod*ytod
	if zt2 ge 0.0 then ztod = sqrt(zt2)-rsp else begin
		print,'(func.settu ) point xtod outside the sphere'
		stop
	endelse

	dtoc, xtod,ytod,ztod, xto,yto,zto
	xxto = [xto,yto,zto]
	xxtod = [xtod,ytod,ztod]

	convxs, ztod+rsp,xtod,ytod, r,theta,phi
	als = halfpi - theta
	aps = phi
	sinals=sin(als)
	cosals=cos(als)
	sinaps=sin(aps)
	cosaps=cos(aps)

	atmat= [ [ cosaps,        0.0,       -sinaps,        0.0 ], $
		   [-sinaps*sinals, cosals,    -cosaps*sinals, 0.0 ], $
		   [ sinaps*cosals, sinals,     cosaps*cosals, 0.0 ], $
		   [ xtod,          ytod,       ztod,          1.0 ] ]
	atinv = fltarr(4,4)
	atinv(indgen(4)*(4+1)) = 1.0	; unit matrix
	atinv(0:2,3) = -atmat(0:2,3)
	atinv = rotate(atmat,4)#atinv
	atinv(3,*)=[0,0,0,1]

	xtu = [ cosaps, 0.0, -sinaps ]
	ytu = [ -sinals*sinaps, cosals, -sinals*cosals ]
	ztu = [ cosals*sinaps, sinals, cosals*cosaps ]
	ypnorm=sqrt(sinals^2 + (cosals*cosaps)^2)
	ypu = [ 0.0, cosals*cosaps/ypnorm, -sinals/ypnorm ]

	coord_t = { coord_t1, $
		xxto:xxto, xxtod:xxtod, $ 
		atmat:atmat, atinv:atinv, xxsp:xxsp, rsp:rsp, $
		sinals:sinals, cosals:cosals, sinaps:sinaps, cosaps:cosaps, $
		xtu:xtu, ytu:ytu, ztu:ztu, ypu:ypu }

	return
end

