;  plot a sphere of radius rsp1 centered at xxsp1 in C-coodrinates
;  if limb=.true., solar limb is drawn
;  if lonlat=.true., longitude/latitude circles are also drawn
;  clipping limits are given by rectangle spanned by four corner
;  points xc4,yc4

pro sphere, xc4,yc4, dl,lonlat,limb,clip, color=color, $
	device=device, normal=normal

	common blkr, coord_r
@gparam

	pi=3.141593
	xxsp = coord_r.xxsp
	rsp = coord_r.rsp
	armat = coord_r.armat
	cl0eff = coord_r.cl0eff

	eps=-1.0e-2
	ncir=fix(2.0*pi*rsp/dl)

;  draw circle of radius rsp (solar limb)
;  bypass rotpt (gmode=2)

	if not limb then goto, nolimb

	l1=0
	if eyed gt 0.0 then begin
		zl=rsp*rsp/(eyed+rsp)
	endif else begin
		zl=0.0
	endelse
	rsplim=sqrt(abs(rsp^2-zl^2))

	xp=rsplim*cos(2.0*pi*indgen(ncir)/float(ncir-1))+xxsp(0)
	yp=rsplim*sin(2.0*pi*indgen(ncir)/float(ncir-1))+xxsp(1)
	zp=fltarr(ncir) +zl +xxsp(2)
	if not clip then begin
		pline3,xp,yp,zp, $
		device=device, normal=normal, $
		color=color, gmode=2
	endif else $
	if inside(xp,yp,xc4,yc4, loc=loc) then $
		pline3,xp(loc),yp(loc),zp(loc), $
		device=device, normal=normal, $
		color=color, gmode=2

;  plot latitude circles

nolimb:
	if not lonlat then return

	for i=1,17 do begin
		th=10.0/180.0*pi*i
		np=fix(ncir*sin(th))
		if np lt 6 then np=6
		ph = 2.0*pi*indgen(np)/(np-1) - 0.5*pi
		ux1=rsp*sin(th)*cos(ph)
		ux2=rsp*cos(th)
		ux3=rsp*sin(th)*sin(ph)
		xp=armat(0,0)*ux1+armat(0,1)*ux2+armat(0,2)*ux3 +armat(0,3)
		yp=armat(1,0)*ux1+armat(1,1)*ux2+armat(1,2)*ux3 +armat(1,3)
		zp=armat(2,0)*ux1+armat(2,1)*ux2+armat(2,2)*ux3 +armat(2,3)

		if not clip then begin
			pline3,xp,yp,zp, color=color, $
				device=device, normal=normal
		endif else $
		if inside(xp,yp,xc4,yc4, loc=loc) then $
			pline3, xp(loc),yp(loc),zp(loc), color=color, $
				device=device, normal=normal
	endfor

;  plot longitude circles

	for i=1,36 do begin
		ph=10.0/180.0*pi*i -cl0eff
		dth=1.0
		if (i mod 9) eq 0 then dth=0.0
		wth=18.0-2.0*dth
		nt=ncir/2
		th = pi*(wth*indgen(nt)/(nt-1) +dth)/18.0
		ux1=rsp*sin(th)*cos(ph)
		ux2=rsp*cos(th)
		ux3=rsp*sin(th)*sin(ph)
		xp=armat(0,0)*ux1+armat(0,1)*ux2+armat(0,2)*ux3 +armat(0,3)
		yp=armat(1,0)*ux1+armat(1,1)*ux2+armat(1,2)*ux3 +armat(1,3)
		zp=armat(2,0)*ux1+armat(2,1)*ux2+armat(2,2)*ux3 +armat(2,3)

		if not clip then begin
			pline3,xp,yp,zp, color=color, $
				device=device, normal=normal
		endif else $
		if inside(xp,yp,xc4,yc4, loc=loc) then $
			pline3 ,xp(loc),yp(loc),zp(loc), color=color, $
				device=device, normal=normal
	endfor
	return
	end

;  set matrix for rotation and view angle transform

pro setmat
	common blkr, coord_r
@gparam

	rd=3.141592/180.0

	rotang=rd*rotdeg
	xang=rd*xdeg
	zang=rd*zdeg

	armat = coord_r.armat
	arinv = coord_r.arinv
	xxsp = coord_r.xxsp

	roty = [ [cos(rotang), 0.0, -sin(rotang),  0.0], $
		   [0.0,         1.0,  0.0,          0.0], $
		   [sin(rotang), 0.0,  cos(rotang),  0.0], $
		   [0.0,         0.0,  0.0,          1.0] ]
;	roty does not perform origin shift

	subtr = [ [1.0,       0.0,       0.0,    0.0], $
		    [0.0,       1.0,       0.0,    0.0], $
		    [0.0,       0.0,       1.0,    0.0], $
		    [-xxsp(0), -xxsp(1), -xxsp(2), 1.0] ]

	rotx = [ [1.0,     0.0,       0.0,       0.0], $
		   [0.0,     cos(xang), sin(xang), 0.0], $
		   [0.0,    -sin(xang), cos(xang), 0.0], $
		   [xxsp(0), xxsp(1),   xxsp(2),   1.0] ]

	rotz = [ [ cos(zang), sin(zang), 0.0,     0.0], $
		   [-sin(zang), cos(zang), 0.0,     0.0], $
		   [0.0,        0.0,       1.0,     0.0], $
		   [xxsp(0),    xxsp(1),   xxsp(2), 1.0] ]

;  rotr = armat*roty*arinv
;  rotv = rotx*rotz

	rotr = armat#roty#arinv
	rotx = rotx#subtr
	rotz = rotz#subtr
	rotv = rotx#rotz

	return
	end

;  rotation around the polar axis and view angle transform

pro rotpt, x,y,z, xr,yr,zr
	common blkr, coord_r
@gparam 

	xxsp = coord_r.xxsp

;  rotation around the polar axis

	x1=rotr(0,0)*x+rotr(0,1)*y+rotr(0,2)*z +rotr(0,3)
	y1=rotr(1,0)*x+rotr(1,1)*y+rotr(1,2)*z +rotr(1,3)
	z1=rotr(2,0)*x+rotr(2,1)*y+rotr(2,2)*z +rotr(2,3)

;  view angle transform

	xr=rotv(0,0)*x1+rotv(0,1)*y1+rotv(0,2)*z1 +rotv(0,3)
	yr=rotv(1,0)*x1+rotv(1,1)*y1+rotv(1,2)*z1 +rotv(1,3)
	zr=rotv(2,0)*x1+rotv(2,1)*y1+rotv(2,2)*z1 +rotv(2,3)

	if eyed gt 0.0 then begin
		xr=xr*eyed/(eyed-zr)
		yr=yr*eyed/(eyed-zr)
	endif

	return
end

;  this entry only applies the rotion around the z-axis
;  (for N,S,E,W markers)

pro rotptv, x,y,z,xr,yr,zr
	common blkr, coord_r
@gparam 

	xxsp = coord_r.xxsp

	xr=rotz(0,0)*x+rotz(0,1)*y+rotz(0,2)*z +rotz(0,3)
	yr=rotz(1,0)*x+rotz(1,1)*y+rotz(1,2)*z +rotz(1,3)
	zr=rotz(2,0)*x+rotz(2,1)*y+rotz(2,2)*z +rotz(2,3)

	if eyed gt 0.0 then begin
		xr=xr*eyed/(eyed-zr)
		yr=yr*eyed/(eyed-zr)
	endif
	return
end

pro pers, x,y,z, xr,yr,zr
@gparam

	xr=x
	yr=y
	zr=z
	if eyed gt 0.0 then begin
		xr=xr*eyed/(eyed-zr)
		yr=yr*eyed/(eyed-zr)
	endif
	return
end

function visibl, x,y,z, loc=loc
@gparam
	common blkr, coord_r
	rsp = coord_r.rsp
	xxsp = coord_r.xxsp

	eps=0.00001*rsp
	rp=sqrt((x-xxsp(0))^2+(y-xxsp(1))^2)
	zz =z-xxsp(2)-rsp
	rp0=rp
	zl=0.0
	if eyed gt 0.0 then begin
		rp0=rp*(eyed+rsp)/sqrt(rp^2+(eyed-zz)^2)
		zl=rsp*rsp/(eyed+rsp)
	endif
	loc = where(rp0 ge rsp-eps or rsp+z ge zl-eps, count)
	if count gt 0 then return,1 else return,0
end
