;+
; NAME :	splin1
; PURPOSE :
;	1-d spline fit to y(x)
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	splin1, y, dx, dydx
; INPUTS :
;	y = data  y(x)
;	dx = x-increment
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	none
; OUTPUTS :
;	dydx = dy/dx at points corresponding to y(x)
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;-
;------------------------------------------------------------------------
;
pro tdms, a,b,c, x, y
;
;  solve tri-diagonal matrix equation
;
;  matrix / b(1),c(1), 0  ,                             ` * x = y
;         | a(2),b(2),c(2), 0  ,                        |
;         |  0  ,a(3),b(3),c(3),                        |
;         |                                             |
;         |                       ,a(n-1),b(n-1),c(n-1) |
;         `                         0   , a(n) , b(n)  /
;
	dim = size(a)
	n = fix(dim(1))

	x = y

	a(0)=-c(0)/b(0)
	y(0)=y(0)/b(0)

	for i=1, n-2 do begin
		p=-c(i)/(b(i)+a(i)*a(i-1))
		y(i)=(y(i)-a(i)*y(i-1))/(b(i)+a(i)*a(i-1))
        	a(i)=p
	endfor

	x(n-1)=(x(n-1)-a(n-1)*y(n-2))/(a(n-1)*a(n-2)+b(n-1))

	for i=n-2, 0, -1 do begin
		x(i)=a(i)*x(i+1)+y(i)
	endfor

	return
end

pro splin1, y, dx, dydx
;
	dydx = y
	a=y & b=y & c=y & d=y
	dim = size(y)
	n = fix(dim(1))

	a(0)= 0.0
	b(0)= 2.0
	c(0)= 1.0
	d(0)= 3.0*(y(1)-y(0))/dx
	a(n-1)=1.0
	b(n-1)=2.0
	c(n-1)=0.0
	d(n-1)=3.0*(y(n-1)-y(n-2))/dx

	for i=1, n-2 do begin
        	a(i)=1.0
        	b(i)=4.0
        	c(i)=1.0
        	d(i)=3.0*( y(i+1)-y(i-1) )/dx
	endfor

	tdms, a,b,c, dydx, d
	return
end
