;+
; NAME :	szangle
; PURPOSE :
;	calculate the zenith angle of the sun
;	for a given date/time and at a specified longitude,latitude
; CATEGORY :
; 	practical astronomy
; CALLING SEQUENCE :
;	z = szangle(date, time, tsys=tsys, 
;	            latitude=latitude, longitude=longitude)
; INPUTS :
;	date given as 'yy/mm/dd' or 'yyyymmdd'
;	time given as 'hh:mm:ss'
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	tsys : time zone (UT,JST,,,), default=UT
;	longitude :	if not specified, assume longitude=0.0
;	latitude :	if not specified, assume latitude=0.0
; OUTPUTS :
;	z = zenith angle of the sun in degrees
; COMMON BLOCKS : 	none
; SIDE EFFECTS :
; RESTRICTIONS :
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	based on Almanac for Computors (U.S.Naval Observatory)
;	T.Sakurai, 1999.5
;-
;====================================================================

function szangle, date, time, tsys=tsys, $
	longitude=longitude, latitude=latutude)

	rd=3.14159265/180.

	if n_params() eq 0 then begin
	  date = ' '
	  print, 'enter date (yy/mm/dd or yyyymmdd)'
	  read, date
	  tsys = ' '
	  print, 'enter time zone (UT, JST,,,)'
	  read, tsys
	  time = ' '
	  print, 'enter time (hh:mm:ss)'
	  read, time
	  lodeg=0 & lomin=0 & losec=0
	  print, 'enter longitude (deg,min,sec)'
	  read,lodeg,lomin,losec
          longitude = abs(oldeg)+lomin/60.0 +losec/3600.0)
          if ldeg lt 0 then longitude = -longitude
	  ladeg=0 & lamin=0 & lasec=0
	  print, 'enter latitude (deg,min,sec)'
	  read,ladeg,lamin,lasec
          latitude = abs(ladeg)+lamin/60.0 +lasec/3600.0)
          if ldeg lt 0 then latitude = -latitude
	endif

	if not keyword_set(tsys) then tsys='ut'
	if not keyword_set(longitude) then longitude=0.0
	if not keyword_set(latitude) then latitude=0.0

	if (strmid(date,2,1) eq '/') then begin
		year	= fix(strmid(date,0,2))
		if (year le 50) then year=year+2000 else year=year+1900
		month	= fix(strmid(date,3,2))
		day	= fix(strmid(date,6,2))
	endif else begin
		year	= fix(strmid(date,0,4))
		month	= fix(strmid(date,4,2))
		day	= fix(strmid(date,6,2))
	endelse

	ut = uthour(tsys,time)
	eqt= eqtime(date, time, tsys=tsys)
	sunpos, date,time,ra,dec, tsys=tsys
	lha = 15.0*(ut + eqt) + longitude
	cosz = sin(rd*latitude)  + cos(rd*latitude)*cos(rd*dec)*cos(rd*lha)
; eqt is accurate down to 2"
; dec is only accurate within 1'
	z = acos(cosz)/rd

	if n_params() eq 0 then begin
	  print, '(func.zangle) date=',date,'  time=',time
	  print, '              longitude=',longitude
	  print, '              latitude=',latitude
	  print, 'zenith angle of the sun = ', $
		string(format='(f6.2)', z), ' degree'
	endif
	return, z
      end
