;+
; NAME :	timediff
; PURPOSE :
;	calculate time difference 
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	tdif = timediff(date1,time1,date2,time2)
; INPUTS :
;	date1,2 = 'yy/mm/dd', time1,2 = 'hh:mm:ss'
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	/min:		tdif in minutes (float)
;	/hour:	tdif in hours (float)
;	/day:		tdif in days (float)
; OUTPUTS :
;     tdif = time difference (datetime2 - datetime1) in sec (long integer)
;	if /min, /hour, or /day is specified, tdif is converted to
;	respective unit.
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1993
;-
;------------------------------------------------------------------------
;
      function timediff, date1,time1, date2,time2, min=min, hour=hour, day=day

	date12 = [date1,date2]
	time12 = [time1,time2]
	year12 = long(strmid(date12,0,2))
	month12= long(strmid(date12,3,2))
	day12  = long(strmid(date12,6,2))
	hour12 = long(strmid(time12,0,2))
	min12	 = long(strmid(time12,3,2))
	sec12	 = long(strmid(time12,6,2))

	year12= (year12 mod 100)+1900
	ut12 = sec12 + min12*60 + 3600*hour12
	it1=367*(year12-1900)
	it2=((month12+9)/12+year12)*7/4
	it3=275*month12/9
	t=(it1-it2+it3+day12)*24*3600 +ut12
	tdif = t(1)-t(0)
	if (keyword_set(min)) then return, tdif/60.0
	if (keyword_set(hour)) then return, tdif/3600.0
	if (keyword_set(day)) then return, tdif/(24*3600.0)
	return, tdif
end
