;+
; NAME :	transc package
; PURPOSE :
;	coordinate transforms 
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	see each entry
; INPUTS :
;	see each entry
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	none
; OUTPUTS :
;	see each entry
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;-
;------------------------------------------------------------------------
;
function unitmat, n

	mat = fltarr(n,n)
	mat(indgen(n)*(n+1)) = 1.0
	return, mat
end

function setdu, mparam
;	set matrix and vectors of data coordinates
;
	rsp = mparam.rsp
	xxsp=[0.0, 0.0, -rsp]
	ypos=mparam.ypos
	sinal=sin(ypos)
	cosal=cos(ypos)
	sinap=sin(ypos)
	cosap=cos(ypos)

	admat=[ [ cos(ypos), sin(ypos), 0.0,      0.0 ], $
		[-sin(ypos), cos(ypos), 0.0,      0.0 ], $
		[ 0.0,       0.0,       1.0,      0.0 ], $
		[ xxsp(0),   xxsp(1),   xxsp(2),  1.0 ]  ]

; in usual matrix notation admat(i,j) is as follows
;                   ----> j
; | admat = cos(ypos)   -sin(ypos)   0.0   xxsp(0)
; V         sin(pos)     cos(ypos)   0.0   xxsp(1)
;           0.0          0.0         1.0   xxsp(2)
; i         0.0          0.0         0.0   1.0
;

	adinv = unitmat(4)
	adinv(*,3) = -admat(*,3)
	adinv = rotate(admat,4)#adinv
	adinv(3,*)=[0,0,0,1]

	xdu= [ cos(ypos), sin(ypos), 0.0]
	ydu= [-sin(ypos), cos(ypos), 0.0]
	zdu= [ 0.0,       0.0,       1.0]

	coord_d = { coord_d1, $
		admat:admat, adinv:adinv, xxsp:xxsp, rsp:rsp, $
		xdu:xdu, ydu:ydu, zdu:zdu }
	return, coord_d
end

function settu, mparam
;	set matrix and vectors of tangential coordinates
;
	rsp = mparam.rsp
	xxsp=[0.0, 0.0, -rsp]
	halfpi = 0.5*3.141592	
	xtod = mparam.xxtod(0)
	ytod = mparam.xxtod(1)
	ztod = 0
	coord_d = setdu(mparam)
	dtoc, xtod,ytod,ztod, xto,yto,zto, coord_d
	zt2 = rsp*rsp - xto*xto - yto*yto
	if zt2 ge 0.0 then zto = sqrt(zt2) else zto = -rsp
	ztod = zto
	if yto gt rsp then begin
		theta = halfpi
	endif else begin
	if yto lt -rsp then begin
		theta = -halfpi
	endif else begin
		theta = acos(yto/rsp)
	endelse
	endelse
	xxtod=[xtod,ytod,ztod]
	xxto=[xto,yto,zto]

	als = halfpi - theta
	aps = atan( xto, zto )
	sinals=sin(als)
	cosals=cos(als)
	sinaps=sin(aps)
	cosaps=cos(aps)

	atmat= [ [ cosaps,        0.0,       -sinaps,        0.0 ], $
		   [-sinaps*sinals, cosals,    -cosaps*sinals, 0.0 ], $
		   [ sinaps*cosals, sinals,     cosaps*cosals, 0.0 ], $
		   [ xxto(0),       xxto(1),    xxto(2),       1.0 ] ]
	atinv = unitmat(4)
	atinv(*,3) = -atmat(*,3)
	atinv = rotate(atmat,4)#atinv
	atinv(3,*)=[0,0,0,1]

	xtu = [ cosaps, 0.0, -sinaps ]
	ytu = [ -sinals*sinaps, cosals, -sinals*cosals ]
	ztu = [ cosals*sinaps, sinals, cosals*cosaps ]
	ypnorm=sqrt(sinals^2 + (cosals*cosaps)^2)
	ypu = [ 0.0, cosals*cosaps/ypnorm, -sinals/ypnorm ]

	coord_t = { coord_t1, $
		xxto:xxto, xxtod:xxtod, $ 
		atmat:atmat, atinv:atinv, xxsp:xxsp, rsp:rsp, $
		xtu:xtu, ytu:ytu, ztu:ztu, ypu:ypu }
	return, coord_t
end

function setas, mparam
; set matrix for S-coordinates
	rsp = mparam.rsp
	xxsp=[0.0, 0.0, -rsp]
	ypos = mparam.ypos

	asmat=[ [ cos(ypos), sin(ypos), 0.0,    0.0 ], $
		[-sin(ypos), cos(ypos), 0.0,    0.0 ], $
		[ 0.0,       0.0,       1.0,    0.0 ], $
		[ xxsp(0),   xxsp(1),  xxsp(2), 1.0   ] ]
	asinv = unitmat(4)
	asinv(*,3) = -asmat(*,3)
	asinv = rotate(asmat,4)#asinv
	asinv(3,*)=0
	coord_s = { coord_s1, $
		asmat:asmat, asinv:asinv, xxsp:xxsp, rsp:rsp }
	return, coord_s
end

function setar, mparam
; set matrix for R-coordinates
	rsp = mparam.rsp
	xxsp=[0.0, 0.0, -rsp]
	p = mparam.prad
	b0 = mparam.b0rad
	cl0 = mparam.cl0rad
	xs = mparam.xs
	xe = mparam.xs +mparam.dx*(mparam.nx -1)
; this is misleading if mesh is not 3
; to be eliminated

	armat= [[ cos(p),         sin(p),         0.0,     0.0 ], $
		[-sin(p)*cos(b0), cos(p)*cos(b0), sin(b0), 0.0 ], $
		[ sin(p)*sin(b0),-cos(p)*sin(b0), cos(b0), 0.0 ], $
		[ xxsp(0),        xxsp(1),        xxsp(2), 1.0    ] ]
	arinv = unitmat(4)
	arinv(*,3) = -armat(*,3)
	arinv = rotate(armat,4)#arinv
	arinv(3,*)=0
	
	coord_r = { coord_r1, $
		armat:armat, arinv:arinv, xxsp:xxsp, rsp:rsp, $
		cl0:cl0, xs:mparam.xs, xe:xe }
	return, coord_r
end

;
pro projt, x,y,z, xt,yt, coord_d, coord_t
;	projection of cartesian coordinates onto a tangential plane

;     deltax=x-coord_t.xxto(0)
;     deltay=y-coord_t.xxto(1)
      cosals = coord_t.atmat(1,1)
      cosaps = coord_t.atmat(0,0)
      sinals = coord_t.atmat(1,2)
      sinaps =-coord_t.atmat(2,0)
      den=cosals*cosaps
;     xt=(cosals*deltax + sinals*sinaps*deltay)/den
;     yt= cosaps*deltay/den
;     zd=-(sinaps*cosals*deltax + sinals*deltay)/den
;     z=zd +coord_t.xxto(2)
	mat1= unitmat(4)
	mat1(0,3)= -coord_t.xxto(0)
	mat1(1,3)= -coord_t.xxto(1)
	mat2 = [[ cosals/den,        0.0,       -cosals*sinaps/den, 0.0 ], $
		[ sinals*sinaps/den, cosaps/den, sinals/den,        0.0 ], $
		[ 0.0,               0.0,        1.0,               0.0 ], $
		[ 0.0,               0.0,        coord_t.xxto(2), 1.0 ] ]
	mat = mat2#mat1
	xt = mat(0,0)*x + mat(0,1)*y + mat(0,2)*z + mat(0,3)
	yt = mat(1,0)*x + mat(1,1)*y + mat(1,2)*z + mat(1,3)
	z  = mat(2,0)*x + mat(2,1)*y + mat(2,2)*z + mat(2,3)
      return
end

pro projs, x,y,z, ap,al, coord_d, coord_s
;  projection of cartesian coordinates onto the spherical surface 

	rsp = coord_s.rsp
	z=x	; define dimension of z
	z(*) = coord_s.xxsp(2)
	r2 = (x - coord_s.xxsp(0))^2 + (y - coord_s.xxsp(1))^2
	loc = where (r2 lt rsp*rsp)
	z(loc) = sqrt(rsp*rsp-r2(loc))+coord_s.xxsp(2)
	ctos, x,y,z, ap,al,h, coord_d, coord_s
	return
end

pro projr, x,y,z, apr,alr, coord_d, coord_r
;  projection of cartesian coordinates onto the spherical surface 

	rsp = coord_r.rsp
	z=x	; define dimension of z
	z(*) = coord_r.xxsp(2)
	r2 = (x - coord_r.xxsp(0))^2 + (y - coord_r.xxsp(1))^2
	loc = where (r2 lt rsp*rsp)
	z(loc) = sqrt(rsp*rsp-r2(loc))+coord_r.xxsp(2)
	ctor, x,y,z, apr,alr,h, coord_d, coord_r
	return
end

pro stoc, ap,al,h, x,y,z, coord_s, coord_d
	stod, ap,al,h, xd,yd,zd, coord_s, coord_d
	dtoc, xd,yd,zd, x,y,z, coord_d
	return
end

pro ctos, x,y,z, ap,al,h, coord_d, coord_s
	ctod, x,y,z, xd,yd,zd, coord_d
	dtos, xd,yd,zd, ap,al,h, coord_d, coord_s
	return
end

pro rtoc, apr,alr,h, x,y,z, coord_r, coord_d
	rsp = coord_r.rsp
	cl0 = coord_r.cl0
	halfpi = 0.5*3.141592	
	convsx, h+rsp, halfpi-alr, apr-cl0, zr,xr,yr
;	xr=conventional y
;	yr=conventional z
;	zr=conventional x
	mat=coord_r.armat
;	x = mat(0,0)*xr + mat(0,1)*yr + mat(0,2)*zr + coord_r.xxsp(0)
;	y = mat(1,0)*xr + mat(1,1)*yr + mat(1,2)*zr + coord_r.xxsp(1)
;	z = mat(2,0)*xr + mat(2,1)*yr + mat(2,2)*zr + coord_r.xxsp(2)
	x = mat(0,0)*xr + mat(0,1)*yr + mat(0,2)*zr + mat(0,3)
	y = mat(1,0)*xr + mat(1,1)*yr + mat(1,2)*zr + mat(1,3)
	z = mat(2,0)*xr + mat(2,1)*yr + mat(2,2)*zr + mat(2,3)
	return
end

pro ctor, x,y,z, apr,alr,h, coord_d, coord_r
	rsp = coord_r.rsp
	cl0 = coord_r.cl0
	pi=3.141592
	halfpi = 0.5*3.141592	
	mat=coord_r.arinv
	xr= mat(0,0)*x + mat(0,1)*y + mat(0,2)*z + mat(0,3)
	yr= mat(1,0)*x + mat(1,1)*y + mat(1,2)*z + mat(1,3)
	zr= mat(2,0)*x + mat(2,1)*y + mat(2,2)*z + mat(2,3)
;	xx = x - coord_r.xxsp(0)
;	yy = y - coord_r.xxsp(1)
;	zz = z - coord_r.xxsp(2)
;	xr= xx*armat(0,0) + yy*armat(1,0) + zz*armat(2,0)
;	yr= xx*armat(0,1) + yy*armat(1,1) + zz*armat(2,1)
;	zr= xx*armat(0,2) + yy*armat(1,2) + zz*armat(2,2)
;	conventional x = zr
;	conventional y = xr
; 	conventional z = yr
	convxs, zr,xr,yr, r,theta,phi
	apr=phi+cl0
	alr=halfpi-theta
	h=r-rsp
; commented out -- abs(phi) le pi but abs(apr) need not be so
;	xs=coord_r.xs
;	xe=coord_r.xe
;	loc = where( apr lt xs-pi )
;	if (loc(0) ge 0) then apr(loc)=apr(loc)+2*pi
;	loc = where( apr gt xe+pi )
;	if (loc(0) ge 0) then apr(loc)=apr(loc)-2*pi
	return
end

pro ttod, xt,yt,zt, xd,yd,zd, coord_t, coord_d
      mat = coord_t.atmat
      xd = mat(0,0)*xt + mat(0,1)*yt + mat(0,2)*zt + mat(0,3)
      yd = mat(1,0)*xt + mat(1,1)*yt + mat(1,2)*zt + mat(1,3)
      zd = mat(2,0)*xt + mat(2,1)*yt + mat(2,2)*zt + mat(2,3)
;     xd = mat(0,0)*xt + mat(0,1)*yt + mat(0,2)*zt + coord_t.xxto(0)
;     yd = mat(1,0)*xt + mat(1,1)*yt + mat(1,2)*zt + coord_t.xxto(1)
;     zd = mat(2,0)*xt + mat(2,1)*yt + mat(2,2)*zt + coord_t.xxto(2)
      return
end

pro dtot, xd,yd,zd, xt,yt,zt, coord_d, coord_t
	mat = coord_t.atinv
	xt = mat(0,0)*xd + mat(0,1)*yd + mat(0,2)*zd + mat(0,3)
	yt = mat(1,0)*xd + mat(1,1)*yd + mat(1,2)*zd + mat(1,3)
	zt = mat(2,0)*xd + mat(2,1)*yd + mat(2,2)*zd + mat(2,3)
;	xx = xd - coord_t.xxtod(0)
;	yy = yd - coord_t.xxtod(1)
;	zz = zd - coord_t.xxtod(2)
;	xt = xx*atmat(0,0) + yy*atmat(1,0) + zz*atmat(2,0)
;	yt = xx*atmat(0,1) + yy*atmat(1,1) + zz*atmat(2,1)
;	zt = xx*atmat(0,2) + yy*atmat(1,2) + zz*atmat(2,2)
	return
end

pro ttoc, xt,yt,zt, x,y,z, coord_t, coord_d
;	ttod, xt,yt,zt, xd,yd,zd, coord_t, coord_d
;	dtoc, xd,yd,zc, x,y,z, coord_d
	mat = coord_d.admat # coord_t.atmat
	x = mat(0,0)*xt + mat(0,1)*yt + mat(0,2)*zt + mat(0,3)
	y = mat(1,0)*xt + mat(1,1)*yt + mat(1,2)*zt + mat(1,3)
	z = mat(2,0)*xt + mat(2,1)*yt + mat(2,2)*zt + mat(2,3)
	return
end

pro ctot, x,y,z, xt,yt,zt, coord_d, coord_t
;	ctod, x,y,z, xd,yd,zd, coord_d
;	dtot, xd,yd,zd, xt,yt,zt, coord_d, coord_t
	mat = coord_t.atinv # coord_d.adinv
	xt = mat(0,0)*x + mat(0,1)*y + mat(0,2)*z + mat(0,3)
	yt = mat(1,0)*x + mat(1,1)*y + mat(1,2)*z + mat(1,3)
	zt = mat(2,0)*x + mat(2,1)*y + mat(2,2)*z + mat(2,3)
	return
end

pro dtoc, xd,yd,zd, x,y,z, coord_d
	mat = coord_d.admat
	x = mat(0,0)*xd + mat(0,1)*yd + mat(0,2)*zd + mat(0,3)
	y = mat(1,0)*xd + mat(1,1)*yd + mat(1,2)*zd + mat(1,3)
	z = mat(2,0)*xd + mat(2,1)*yd + mat(2,2)*zd + mat(2,3)
;	xdu = coord_d.xdu
;	ydu = coord_d.ydu
;	zdu = coord_d.zdu
;	x = xdu(0)*xd + ydu(0)*yd + coord_d.xxsp(0)
;	y = xdu(1)*xd + ydu(1)*yd + coord_d.xxsp(1)
;	z = zd + coord_d.xxsp(2) + coord_d.rsp
	return
end

pro ctod, x,y,z, xd,yd,zd, coord_d
	mat = coord_d.adinv
	xd = mat(0,0)*x + mat(0,1)*y + mat(0,2)*z + mat(0,3)
	yd = mat(1,0)*x + mat(1,1)*y + mat(1,2)*z + mat(1,3)
	zd = mat(2,0)*x + mat(2,1)*y + mat(2,2)*z + mat(2,3)
;	xdu = coord_d.xdu
;	ydu = coord_d.ydu
;	zdu = coord_d.zdu
;	xx = x - coord_d.xxsp(0)
;	yy = y - coord_d.xxsp(1)
;	xd = xx*xdu(0) + yy*xdu(1)
;	yd = xx*ydu(0) + yy*ydu(1)
;	zd = z - coord_d.xxsp(2) -coord_d.rsp
	return
end

pro stod, ap,al,h, xd,yd,zd, coord_s, coord_d
	halfpi=0.5*3.141592
	rsp = coord_s.rsp
	xxsp = coord_s.xxsp
	convsx, h+rsp, halfpi-al, ap, zd,xd,yd
;	xd = conventional y + xxsp(0)
;	yd = conventional z + xxsp(1)
;	zd = conventional x + xxsp(2)
	xd = xd +coord_s.xxsp(0)
	yd = yd +coord_s.xxsp(1)
	zd = zd +coord_s.xxsp(2)
	return
end

pro dtos, xd,yd,zd, ap,al,h, coord_d, coord_s
	halfpi = 0.5*3.141592
	rsp = coord_s.rsp
;	conventional x = zd - xxsp(2)
;	conventional y = xd - xxsp(0)
;	conventional z = yd - xxsp(1)
	convxs, zd -xxsp(2), xd-xxsp(0), yd-xxsp(1), r,theta,phi
	ap = phi
	al = halfpi - theta
	h = r-rsp
	return
end

pro rtod, apr,alr,h, xd,yd,zd, coord_r, coord_d
	rtoc, apr,alr,h, x,y,z, coord_r, coord_d
	ctod, x,y,z, xd,yd,zd, coord_d
	return
end

pro dtor, xd,yd,zd, apr,alr,h, coord_d, coord_r
	dtoc, xd,yd,zd, x,y,z, coord_d
	ctor, x,y,z, apr,alr,h, coord_d, coord_r
	return
end

pro convxs, convx,convy,convz, r,theta,phi
;  conversion between (x,y,z) and (r,theta,phi)
;  conventional definition is adopted here, namely
;  the z-axis is the polar axis (theta=0 or pi)
;  the phi-angle is measured from the x-axis toward the y-axis
;  in the xz-plane, phi=0 in x>0 and phi=pi in x<0
;
	r    = sqrt(convx^2+convy^2+convz^2)
	theta= acos(convz/r)
	phi  = atan(convy,convx)
	return
end

pro convsx, r,theta,phi, convx,convy,convz

	convx=r*sin(theta)*cos(phi)
	convy=r*sin(theta)*sin(phi)
	convz=r*cos(theta)
	return
end

;  relation between latitude(al) and y-value in S and R coordinates
;  -halfpi <= al <= halfpi
;  -halfpi <= y <= halfpi

;  y as a function of al

function yofal, al

	halfpi=0.5*3.141592
	y=al

	loc = where (al ge halfpi)
	if (loc(0) ge 0) then y(loc) = halfpi
	loc = where (al le -halfpi)
	if (loc(0) ge 0) then y(loc) = -halfpi
	loc = where (al ge -halfpi and al lt halfpi)
	if (loc(0) ge 0) then y(loc) =asin(al/halfpi)
	return, y
end
;
;  al as a function of y
;
function alofy, y

	halfpi=0.5*3.141592
	al=y

	loc = where (y ge halfpi)
	if (loc(0) ge 0) then al(loc) = halfpi
	loc = where (y le -halfpi)
	if (loc(0) ge 0) then al(loc) = -halfpi
	loc = where (y ge -halfpi and y lt halfpi)
	if (loc(0) ge 0) then al(loc) =halfpi*sin(y)
	return, al
end

