;+
; NAME :	uthour
; PURPOSE :
;	calculate UT time in unit of hour from a specified timezone 
; CATEGORY :
; 	time zone handling
; CALLING SEQUENCE :
;	ut = uthour(tsys, time)
; INPUTS :
;	tsys = 'GMT+hh', 'JST',,,etc.,  which indicates a timezone
;	time = 'hh:mm:ss' or
;	       a floating point number expressing the time in hour
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	none
; OUTPUTS :
;	UThour in float
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1999 May
;-
;------------------------------------------------------------------------
;
function uthour, tsys, time

	tntbl=['GMT-12', 'GMT-11', 'HST', 'YST', 'PST', $
               'MST', 'CST', 'EST', 'AST', 'GMT-3',	$
               'GMT-2', 'GMT-1', 'GMT', 'MET', 'EET',	$
               'GMT+3', 'GMT+4', 'GMT+5', 'GMT+6', 'GMT+7',	$
               'WST', 'JST', 'EST', 'GMT+11', 'NZST' ]
;  daylight saving time
	tdtbl=['***-12', '***-11', 'HDT', 'YDT', 'PDT',	$
               'MDT', 'CDT', 'EDT', 'ADT', '***-3',		$
               '***-2', '***-1', 'BST', 'METDST', 'EETDST',	$
               '***+3', '***+4', '***+5', '***+6', '***+7',	$
               '***+8', '***+9', '***+10', '***+11', 'NZDT' ]

	if n_params() eq 0 then begin
	  tsys=' '
	  print,'enter timezone (GMT+hh, JST, etc.)'
	  read, tsys
	  time = ' '
	  print, 'enter time (hh:mm:ss)'
	  read, time
	endif

	if n_elements(tsys) eq 0 then tsys = 'ut'

	tsysu=strtrim(strupcase(tsys),2)

	if tsysu eq 'UT' or tsysu eq 'GMT' then $
		tdif = 0 else begin
	if strmid(tsysu,0,2) eq 'UT' then $
		tdif=fix(strmid(tsysu,2,4)) else begin
	if strmid(tsysu,0,3) eq 'GMT' then $
		tdif=fix(strmid(tsysu,3,3)) else begin
	for tdif=-12, 12 do begin
		if strmid(tsysu,0,strlen(tntbl(tdif+12))) eq tntbl(tdif+12) $
		then goto, found
	endfor
	for tdif=-11, 13 do begin
		if strmid(tsysu,0,strlen(tdtbl(tdif+11))) eq tdtbl(tdif+11) $
		then goto, found
	endfor
	print,'(func.uthour) unknown timezone:',tsys,', assume UT'
	tdif=0
	endelse
	endelse
	endelse
found:

	type = size(time)
	if type(1) eq 7 then begin	; time = string
		if strmid(time,2,1) ne ':' or strmid(time,5,1) ne ':' then begin
			print,'(func.uthour) unrecognizable time format  ',time, $
				', assume 00:00:00 UT'
			ut = 0.0
			return, ut
		endif	
		hour	= double(strmid(time,0,2)) - tdif
		min	= double(strmid(time,3,2))
		sec	= double(strmid(time,6,2))
		ut= ( sec/60.0d0 + min )/60.0d0 + hour
	endif else $
		ut= time - tdif

	return, ut
end
