;+
; NAME :	writel
; PURPOSE :
;	write magnetic field line data
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	writel, ldata [,filename=filename]
; INPUTS :
;	ldata : a structure containing field line parameters and data
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	filename : name of field line file
; OUTPUTS :
;	ldata : a structure containing field line parameters and data
; COMMON BLOCKS : 	none
; SIDE EFFECTS :
;	if filename is not supplied, it has to be typed in from keyboard
; RESTRICTIONS :
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 2000 September
;-
;------------------------------------------------------------------------
;
pro writel, ldata, filename=loffile
;
	if not keyword_set(loffile) then begin
		loffile = ' '
		read,'(proc.writel) enter output file name > ',loffile
	endelse

	openw, u_lof, loffile, /get_lun

	lhdr1 = deflh1()
	lhdr2 = deflh2()
	lhdr3 = deflh3()

	nf0 = data.nf
	nf  = 0
	for n=0,nf0-1 do begin
		if ldata.np(n) gt 0 and ldata.loc(n) ge 0 then nf=nf+1
	endfor
	
	lhdr1.np = 1
	lhdr1.nfinit = ldata.nfinit
	lhdr1.nex = ldata.nex
	lhdr1.ns1 = ldata.ns1
	lhdr1.ns2 = ldata.ns2
	lhdr1.ns3 = ldata.ns3
	lhdr1.bmin1 = ldata.bmin1
	lhdr1.bmin2 = ldata.bmin2
	lhdr1.flxpwr = ldata.fxpwr

	lhdr2.kindl = ldata.kindl
	lhdr2.nbytel = sign(ldata.nbytel,myint())
	lhdr2.ncomp = ldata.ncomp
	lhdr2.xfactor = ldata.xfactor
	lhdr2.bfactor = ldata.bfactor
	lhdr2.lfile = ldata.lfile
	lhdr2.mtag = ldata.mtag
	lhdr2.ltag = ldata.ltag
	ldata.rrss = rrss
	lhdr3.chksum = ldata.chksum
	lhdr3.rrss = ldata.rrss

	writeu, u_lof, lhdr1
	writeu, u_lof, lhdr2
	if ldata.nex eq 1 then writeu, u_lof, lhdr3

	lhdr1 = deflh1()
	nbytel = ldata.nbytel
	ncomp = ldata.ncomp

	for n=0, nf0-1 do begin
		np = ldata.np(n)
		loc = ldata.loc(n)
		if np gt 0 and loc ge 0 then begin
			lhdr1.np = np
			writeu, u_lof, lhdr1

			if ( abs(nbytel) eq 2) then begin
				if (ncomp eq 3 ) then $
					xyzb = intarr(3,np) $
				else $
					xyzb = intarr(8,np)
			endif else begin
				if (ncomp eq 3 ) then $
					xyzb = lonarr(3,np) $
				else $
					xyzb = lonarr(8,np)
			endelse
			
			xyzb(0,*) = ldata.heap(loc:loc+np-1,0)/xfactor
			xyzb(1,*) = ldata.heap(loc:loc+np-1,1)/xfactor
			xyzb(2,*) = ldata.heap(loc:loc+np-1,2)/xfactor
			if ncomp eq 8 then begin
				xyzb(3,*) = ldata.heap(loc:loc+np-1,3)/xfactor
				xyzb(4,*) = ldata.heap(loc:loc+np-1,4)/bfactor
				xyzb(5,*) = ldata.heap(loc:loc+np-1,5)/bfactor
				xyzb(6,*) = ldata.heap(loc:loc+np-1,6)/bfactor
				xyzb(7,*) = ldata.heap(loc:loc+np-1,7)/bfactor
			endif

			writeu, u_lof, xyzb
			nlast = ( abs(nbytel)*ncomp*np) mod 80
			if nlast ne 0 then begin
				padstr = replicate(' ', 80 - nlast )
				writeu, u_lof, padstr
			endif
		endif
	endfor

	close, u_lof
	free_lun, u_lof
	return
end
