;+
; NAME :	writem
; PURPOSE :
;	write magnetogram data to a file
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	writem, mparam, rdata
;		[,filename=filename] [,/nolist]
; INPUTS :
;	mparam : magnetogram parameters
;	rdata  : magnetogram data
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	filename : magnetogram file name
;	/nolist: if set, sessages are not printed
; OUTPUTS :
;	data are written to a file
; COMMON BLOCKS : 	none
; SIDE EFFECTS :
;	if filename is not specified, it should be typed in from keyboard
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	TS 2000/09 byte order taken into account in mparam.nbytem
;	TS 2000/10 unfilled data marked by ifl2/4
;-
;------------------------------------------------------------------------
;
pro writem, mparam,rdata, filename=filename, nolist=nolist
;
	if keyword_set(filename) then begin
		magfile = filename
	endif else begin
		magfile = ' '
		read,'(proc.writem) enter output file name > ',magfile
	endelse
	openw, unitm, magfile, /get_lun

	specs = defspecs(nspec)
	nbytem = abs(mparam.nbytem)
	if myint() gt 0 then mparam.nbytem = nbytem else mparam.nbytem=-nbytem
	nx = long(mparam.nx)
	ny = long(mparam.ny)	; changed from fix to long 94/10

	writemh1, mparam, unitm
	uflval, ufl,iufl2,iufl4

	for i=0, nspec-1 do begin
		k = mparam.status.(i)
		if k ne -1 then begin
			spec = specs(i)
			factor = mparam.factors.(i)
			alpha = mparam.alpha
			mhdr2 = defmh2( spec=spec, factor=factor, alpha=alpha )
			if not keyword_set(nolist) then begin
				print,'spec,factor=', $
					mhdr2.spec, mhdr2.factor
				if alpha ne 0.0 then $
					print,'alpha=',alpha
			endif

			writeu, unitm, mhdr2
			loc = where(rdata(*,*,k) eq ufl, count)
			if nbytem eq 2 then begin
				buf = fix(rdata(*,*,k)/factor)
				if count gt 0 then buf(loc) = iufl2
			endif else begin
				buf = long(rdata(*,*,k)/factor)
				if count gt 0 then buf(loc) = iufl4
			endelse
			writeu, unitm, buf

			nlast = (nbytem*nx*ny) mod 80
			if ( nlast ne 0 ) then begin
				padstr = replicate(' ', 80 - nlast )
				writeu, unitm, padstr
			endif
		endif
	endfor

	stat = fstat(unitm)
	if not keyword_set(nolist) then $
		print, 'file=',magfile,', data size in bytes = ',stat.cur_ptr
	free_lun, unitm
	return
end
