;+
; NAME :	writemh1
; PURPOSE :
;	write magnetogram header part-1
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	writemh1, mparam, unitm
; INPUTS :
;	mparam
;	unitm = logical i/o unit number for output
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	none
; OUTPUTS :
;	mhdr1 is written to a file
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;	TS 95/10/31: add wavel
;-
;------------------------------------------------------------------------
;
pro writemh1, mparam, unitm
;
        str6 = string(' ', format='(a6)') 
        str8 = string(' ', format='(a8)')
        str16= string(' ', format='(a16)')
        str80 = string(' ', format='(a80)')

	mhdr1 = defmh1()

	pi=3.141592
        mhdr1.kindm = 	string( mparam.kindm,	format='(i8)')
        mhdr1.nbytem  =	string( mparam.nbytem,	format='(i4)')
        mhdr1.nframe  =	string( mparam.nframe,	format='(i4)')
        mhdr1.tsys =	strmid( strtrim(mparam.tsys,1)   +str6, 0, 6) 
        mhdr1.inst =	strmid( strtrim(mparam.inst,1)   +str8, 0, 8)
        mhdr1.mfile =	strmid( strtrim(mparam.mfile,1)  +str8, 0, 8)
        mhdr1.dateo =	strmid( strtrim(mparam.dateo,1)+str8, 0, 8)
        mhdr1.stime =	strmid( strtrim(mparam.stime,1)  +str8, 0, 8)
        mhdr1.etime =	strmid( strtrim(mparam.etime,1)  +str8, 0, 8)
        mhdr1.ftag =	strmid( strtrim(mparam.ftag,1)   +str16, 0, 16)
        mhdr1.nx =	string( mparam.nx,	format='(i4)')
        mhdr1.ny =	string( mparam.ny,	format='(i4)')
        mhdr1.xs =	string( mparam.xs,	format='(f12.5)')
        mhdr1.ys =	string( mparam.ys,	format='(f12.5)')
        mhdr1.dx =	string( mparam.dx,	format='(f12.5)')
        mhdr1.dy =	string( mparam.dy,	format='(f12.5)')
        mhdr1.ydeg =	string( 180.0*mparam.ypos/pi,	format='(f12.5)')
        mhdr1.mesh =	string( mparam.mesh,	format='(i4)')
        mhdr1.wavel=	string( mparam.wavel,	format='(f8.2)')
        mhdr1.rsp =	string( mparam.rsp,	format='(f12.5)')
        mhdr1.pdeg =	string( 180.0*mparam.prad/pi,	format='(f12.5)')
        mhdr1.b0deg =	string( 180.0*mparam.b0rad/pi,	format='(f12.5)')
        mhdr1.cl0deg =	string( 180.0*mparam.cl0rad/pi,	format='(f12.5)')
        mhdr1.xt =	string( mparam.xxtod(0),	format='(f12.5)')
        mhdr1.yt =	string( mparam.xxtod(1),	format='(f12.5)')
        mhdr1.title =	strmid( strtrim(mparam.title,1) +str80, 0, 80)

        writeu, unitm, mhdr1
	return
end
