/*	lips2lib.c
	CANON LaserShot lips2 library  (A4 printer)
*/

#include <stdio.h>
#include <string.h>
#include <math.h>

#include "lbpnop.c"
#include "makebitwiseimage.c"

#ifdef LBPONLY
#include "lbponly.h"
#endif

#define LPstrlen 50
#define SPOOL 0
#define LBP 1
#define SPOOLoutfile  "LBPout"
#define SPOOLsetfile  "LBPset"
#define HOLD 1
#define NOHOLD 0
#define NODATA 0
#define EXIST 1
#define EJECT '\f'
#define ESC '\33'
#define IS2 '\36'
#define SI '\17'
#define LBPMAXBUF 2000
#define On 1
#define Off 0
#define LBPSET 1
#define LBPDRAW 0
#define PORTRAIT 0
#define LANDSCAPE 1
/*	A4	*/
#define LBPlmax 28600
#define LBPsmax 19900
		/*	l=long-side, s=short-side, unit = 1/100 mm	*/
#define LBPlmaxdot 2710
#define LBPsmaxdot 1888
#define STANDARD_HEIGHT  250.0
	/*	standard character height= 2.5mm	*/
#define LBPdotsize 10.58333
	/*	LBP dot = 1/240 inch = 10.58 * 1/100mm		*/

FILE *fp;			/*	pointer for output file */
FILE *fps;			/*	pointer for setup file	*/

/*	int = long (4-byte) integer = fortran integer	*/

static int lbp_device;
static int lbp_output;
static int lbp_filenumber = 1;
static int lbp_pagecount = 0;
static int lbp_data = NODATA;
static int lbp_set  = NODATA;
static int lbp_vectormode = Off;
static int lbp_orientation = PORTRAIT;
static int lbp_xmax = LBPsmax;
static int lbp_ymax = LBPlmax;
static int lbp_xmaxdot = LBPsmaxdot;
static int lbp_ymaxdot = LBPlmaxdot;
static int xs=0, ys=0;			/*	start point of VDC	*/
static int xe=32767, ye=32767;		/*	end point of VDC	*/
static int xoffset=0, yoffset=0;	/*	origin of LBP	*/
static int lmargin=2000, tmargin=2000;
					/*	left and top margins	*/
static float scalefactor = 10.0;
static float xfactor = 10.0, yfactor = 10.0;
static float heightfactor = 1.0;
static float widthratio = 1.0;
static int textorientation = 0;
static int textpath = 0;	/*	horizontal	*/

/*	set default values according to news cgi	*/

static int vpunits = 1;
	/*	default viewport specification unit of cgi:1/10 mm
		Internally, this lips2lib uses 1/100mm as unit length */
static int hatchindex = 2;
	/*	default hatch pattern = vertical parallel lines	*/
static int fillstyle = 1;
	/*	In order to perform hatching and interior filling,
		fillstyle should be passed between 'hatchindex' and 
		'interiorstyle' routines	*/
static int edgevisibility = 1;
	/*	default edge visibility	= invisible	*/
static int markertype = 1;
      /*    marker = dot      */


extern unsigned int int_r[],int_g[],int_b[], max_colorindex;
#define DIRECTCOLOR 0
#define INDEXEDCOLOR 1
extern int colormode;   /* DIRECTCOLOR or INDEXEDCOLOR      */

static int ibuffer[LBPMAXBUF];
static char cbuffer[LBPMAXBUF];
static char CSI[]={ESC,'['}, DCS[]={ESC,'P'},ST[]={ESC,'\\'};
static char lbp_outfile[LPstrlen], lbp_setfile[LPstrlen];
static char LPdevice[LPstrlen], LPname[LPstrlen];

extern struct common_sys
	{int mycode,myint,keyin,mag_stdin,lst,msg,msglvl,errflg;
	float ufl; short int iufl2,maxi2; long int iufl4,maxi4}
	sys_;   /*      fortran common/sys/     */

extern struct common_lbp
	{char LPdevice[LPstrlen], LPname[LPstrlen]}
	lbpstr_;	/*	fortran common/lbpstr/	*/



static int debug=0;



/*	internal functions	*/

static LBP_fputc(c, set_or_draw)
	char c; int set_or_draw;
{
	fputc(c,fp);
	if( set_or_draw == LBPSET && lbp_device == SPOOL )	{
		fputc(c,fps);
		lbp_set = EXIST;
	}
	else
		lbp_data = EXIST;
}

static LBP_fputs(s, set_or_draw )
	char *s; int set_or_draw;
/*	*s is a null-terminated string	*/
{
	fputs(s,fp);
	if( set_or_draw == LBPSET && lbp_device == SPOOL )	{
		fputs(s,fps);
		lbp_set = EXIST;
	}
	else
		lbp_data = EXIST;
}

static print(str,nbyte)
/*    print character string
	'str' need not be null-terminated	*/
      char *str;
      int nbyte;
{
      int w;
      w = fwrite(str,nbyte,1,fp);
      lbp_data=EXIST;
      return w;
}

static makevectordata( ndata, data_in, data_out )
/*	convert data <data_in> into lips vector-data format	*/
	int data_in[],ndata;
	unsigned char data_out[];
{
	int p,q,i,d,dd,s;

	q = 0;
	for (p = 0; p < ndata; p++)	{
		d = data_in[p];
		if (d < 0) s = 0; 
			else s = 0x10;
		d = abs(d);
		dd = d;

		if ( dd >= 65536 )	{
			data_out[q++] = (d/65536) | 0x40;
			d %= 65536;
		} 
		if ( dd >= 1024 )	{
			data_out[q++] = (d/1024) | 0x40;
			d %= 1024;
		}
		if ( dd >= 16 )	{
			data_out[q++] = (d/16) | 0x40 ;
			d %= 16;
		}
		data_out[q++] = d | 0x20 | s ;
	}
	data_out[q] = '\0';
	return;
}

static vectordata1( d, set_or_draw )
	int d, set_or_draw;
/*	vectordata-one
	Convert data <d> into lips vector-data format
	and put it into output stream	*/
{
	int q,i,dd,s;

	q = 0;
	if (d < 0) s = 0; 
		else s = 0x10;
	d = abs(d);
	dd = d;

	if ( dd >= 65536 )	{
		cbuffer[q++] = (d/65536) | 0x40;
		d %= 65536;
	} 
	if ( dd >= 1024 )	{
		cbuffer[q++] = (d/1024) | 0x40;
		d %= 1024;
	}
	if ( dd >= 16 )	{
		cbuffer[q++] = (d/16) | 0x40 ;
		d %= 16;
	}		
	cbuffer[q++] = d | 0x20 | s ;
	cbuffer[q] = '\0';
	LBP_fputs(cbuffer, set_or_draw);
/*
printf("\n (lips2lib/vectordata1) %s",cbuffer);
*/
	return;
}

static makexylist(np,x,y,buffer)
	int np,x[],y[],buffer[];
{
	int i,j;

	buffer[0] = xconv( x[0] );
	buffer[1] = yconv( y[0] );
	j=2;
	if (2*np >LBPMAXBUF)	{
		np = LBPMAXBUF/2;
		printf("\n (lips2lib/makexylist) buffer overflow!");
	}
	for (i=1; i<np; i++)	{
		buffer[j++] = xfactor *( x[i] - x[i-1] );
		buffer[j++] = yfactor *( y[i] - y[i-1] );
	}
}

/*	conversion from VDC to LBP coordinates	*/

static xconv(x)
	int x;
{
	return ( (int)( xfactor * x + xoffset + lmargin ) );
}

static yconv(y)
	int y;
{
	int yoff = 667;
		/*	difference between graphic and text origins	*/

	if (lbp_vectormode == On)
		return ( (int)( yfactor * y + yoffset +yoff + tmargin ) );
	else
		return ( (int)( yfactor * y + yoffset + tmargin ) );
}

static lconv(a)
	int a;
{
	return ( (int)( scalefactor * a ) );
}

static angle_to_x(angle)
	float angle;
{
	float amplitude = 100.0;
	double pi = 3.141592;
	return ( (int)( amplitude * cos ( pi*angle/180.0) ) );
}

static angle_to_y(angle)
	float angle;
{
	float amplitude = 100.0;
	double pi = 3.141592;
	return ( (int)( amplitude * sin ( pi*angle/180.0) ) );
}

static execute(str)
	char *str;
/*	execute shell script	*/
{
	int status;
	if( (status = system(str)) != 0 ) printf(
	"\n (lips2lib/execute) error: command execution failed, command=%s",
		str );
	return (status);
}

static fcopy(fpi,fpo)
	FILE *fpi, *fpo;
/*	copy file	*/
{
	int c;
	fflush(fpi);	/*	flush output to fpi if any	*/
	rewind(fpi);
	while ( (c=getc(fpi) )!= EOF)	putc(c,fpo);
}

static openfile(filenumber)
	int filenumber;
{
	if (filenumber == 0)	{
/*	Create setfile as new, allowing read operation	*/
		sprintf(lbp_setfile,"%s%d.ls",SPOOLsetfile,getpid());
		if(debug)
			printf("\n (lips2lib/openfile) open setfile--%s",lbp_setfile);
		fps = fopen(lbp_setfile,"w+");
		if(fps == NULL)
			printf("\n (lips2lib/openfile) open error %s",lbp_setfile);
	}
	else	{
/*	Create outfile as new	*/
		sprintf(lbp_outfile,"%s%d-%d.ls",SPOOLoutfile,getpid(),filenumber);
		if(debug)
			printf("\n (lips2lib/openfile) open outfile--%s",lbp_outfile);
		fp = fopen(lbp_outfile,"w");
		if(fp == NULL)
			printf("\n (lips2lib/openfile) open error %s",lbp_outfile);
	}
}


/*	LBP functions whose effects are different from CGI	*/
 
LBP_initialize_(initmode)
	int *initmode;
/*	abs(initmode) =1:	portrait orientation, output to spool
	               2:	landscape orientation,    "
		       3:	same as 1, output hold
		       4:	same as 2, output hold
		       5:	same as 1, direct output to LBP
		       6:	same as 2,          "
	Otherwise assume initmode=1 
	For color print (not suppoorted in LIPS), 10 is added
	In case of direct output, the printer cannot be shared until
	this program ends
	No CRT output if initmode<0	*/

{
	int imode = 1;

	if ( sys_.msglvl > 2 )	debug = 1;

      imode = (int)(abs(*initmode)) % 10;

	if ( imode == 0 || imode > 6 ) imode = 1;

	if ( imode >4)	{
		lbp_device = LBP;	lbp_output = NOHOLD;
	}
	else if ( imode > 2 )	{
		lbp_device = SPOOL;	lbp_output = HOLD;
	}
	else	{ 
		lbp_device = SPOOL;	lbp_output = NOHOLD;
	}

	strncpy(LPdevice, lbpstr_.LPdevice, LPstrlen);
	strncpy(LPname, lbpstr_.LPname, LPstrlen);
	*strchr(LPdevice,' ') = '\0';
	*strchr(LPname,' ') = '\0';

	if (lbp_device == SPOOL)	{
		openfile(0);	/*	open setfile	*/
		openfile(lbp_filenumber);	/*	open outfile	*/
	}
	if (lbp_device == LBP)	{
		fp = fopen(LPdevice,"w");
		fps = fp;
		if (fp == NULL)
			printf("\n (LBP_initialize) open error %s",LPdevice);
	}

	if ( (imode -1) % 2 == 1 )	landscape();
	else 				portrait();
/*	these should be set before initset()	*/

	initset();
}	

static initset()
{
	softreset(fp);
	if (lbp_device == SPOOL)	softreset(fps);
/*	Only the first reset is stored in a setfile
	and will be executed prior to every print out	*/
 
/*	shift JIS mode	*/
	LBP_fputc(ESC,LBPSET);
	LBP_fputs("%0",LBPSET);

/*	full paint mode	*/
	LBP_fputs(CSI,LBPSET);
	LBP_fputs("2&z",LBPSET);

/*	character scaling by string mode	*/
	LBP_fputs(CSI,LBPSET);
	LBP_fputs("?28h",LBPSET);

/*	unit of character position specification = 1/100mm	*/
	LBP_fputs(CSI,LBPSET);
	LBP_fputs("11h",LBPSET);
	LBP_fputs(CSI,LBPSET);
	LBP_fputs("?6 I",LBPSET);

/*	specified HMI is forced to all character sets	*/
/*	LBP_fputs(CSI,LBPSET);
	LBP_fputs("?23h",LBPSET);
*/
	forced_vectormode();	/*	pageorientation() is called 	*/

/*	end picture (close vector mode of previous job)	*/
	LBP_fputs("%",LBPSET);
	LBP_fputc(IS2,LBPSET);

/*	begin picture	*/
	LBP_fputs("#",LBPSET);
	LBP_fputc(IS2,LBPSET);

/*	set scaling factor for vector mode	*/
	LBP_fputs("!0",LBPSET);
	vectordata1(-2, LBPSET);		/*	unit=1/100mm	*/
	vectordata1(1, LBPSET);			/*	enlargement factor =1	*/
	LBP_fputc(IS2,LBPSET);

/*	begin picture body	*/
	LBP_fputs("$",LBPSET);
	LBP_fputc(IS2,LBPSET);

/*	set hatch index because lips has no default value	*/
	LBP_hatchindex_(&hatchindex);

/*	set edge visibility because default values are different 
	between cgi and lips	*/
	LBP_edgevisibility_(&edgevisibility);

/*	set marker type because default values are different
	between cgi and lips
	markertype = 1	*/
	LBP_markertype_(&markertype);
/*
	These as well as other "setting" commands need to be stored
	and sent to printer, if LBP_clear clears the "drawing" data
*/
}

LBP_vdcextent_(x1,y1,x2,y2)
	int *x1,*y1,*x2,*y2;
/*	extent of virtual display space

	lips supports three units for the specification of VDC
	(1) m x 1/100mm (2) m x 1/720inch (3) physical dots
	where m is a magnification factor

	in text mode, character size etc. can be specified in four units
	(1) 1/100mm (2) 1/720inch (3) 1/1000inch (4) dots

	in this package we use 1/100mm as the unit

	effective range of VDC is from -32640 to 32639 in dots
	*/
{
	xs = *x1;	ys = *y1;
	xe = *x2;	ye = *y2;
}

LBP_viewportunits_(units)
	int *units;
/*	units=0 				:percent relative
	      1 (default here)		:1/10 mm
	      2				:absolute (dots)	*/
{
	vpunits = *units;
}

LBP_viewport_(x1,y1,x2,y2)
	int *x1,*y1,*x2,*y2;

/*	viewport specification for viewportunits=1,2

      (0,0) is at the upper left corner of the screen
      VDC (xs,ys),(xe,ye) are mapped to viewport (x1,y1),(x2,y2)
*/
{
	float xx1,yy1,xx2,yy2;
	int xul,yul,xlr,ylr;

	xx1= *x1;	xx2= *x2;
	yy1= *y1;	yy2= *y2;

	if (vpunits == 1)	{	/*	(x1,y1),(x2,y2) are in units of 1/10mm	*/
		xx1 *= 10.0;	xx2 *= 10.0;
		yy1 *= 10.0;	yy2 *= 10.0;
		/*	(xx1,yy1),(xx2,yy2) are in units of 1/100mm	*/
	}
	if (vpunits == 2)	{	/*	(x1,y1),(x2,y2) are in dots	*/
		xx1 *= LBPdotsize;	xx2 *= LBPdotsize;
		yy1 *= LBPdotsize;	yy2 *= LBPdotsize;
		/*	(xx1,yy1),(xx2,yy2) are in units of 1/100mm	*/
	}

	xfactor = ( xx2 - xx1 )/( xe - xs );
	yfactor = ( yy2 - yy1 )/( ye - ys );

	if ( fabs((double)xfactor) > fabs((double)yfactor) ) 
		scalefactor= fabs(yfactor);
	else
		scalefactor= fabs(xfactor);

	xoffset = xx1 - xfactor * xs;
	yoffset = yy1 - yfactor * ys;

	/*	printer's y-axis in text mode is always directed downward
		y-axis in graphic mode is made downward, too	*/
	xul = (xx1<xx2)? xx1:xx2;
	yul = (yy1<yy2)? yy1:yy2;	/*	(xul,yul)=upper left corner	*/
	xlr = (xx1>xx2)? xx1:xx2;
	ylr = (yy1>yy2)? yy1:yy2;	/*	(xlr,ylr)=lower right corner	*/

	vectormode();
	LBP_fputs("(",LBPSET);
	vectordata1(xul, LBPSET);
	vectordata1(yul, LBPSET);
	vectordata1(xlr, LBPSET);
	vectordata1(ylr, LBPSET);
	LBP_fputc(IS2,LBPSET);		/*	set viewport	*/

if(debug)	{
	printf("\n (LBP_viewport) x,yfactor=%f %f",xfactor,yfactor);
	printf("\n                offset=%d %d",xoffset,yoffset);
	}
}

LBP_viewportf_(fx1,fy1,fx2,fy2)
	float *fx1,*fy1,*fx2,*fy2;
/*	viewport specification for viewportunits=0 (percent relative)	*/
{
	int x1,y1,x2,y2;

	if (vpunits == 0 )	{
		x1 = 0.01*lbp_xmaxdot*(*fx1);
		y1 = 0.01*lbp_ymaxdot*(*fy1);
		x2 = 0.01*lbp_xmaxdot*(*fx2);
		y2 = 0.01*lbp_ymaxdot*(*fy2);
		vpunits = 2;
		LBP_viewport_( &x1,&y1,&x2,&y2 );
		vpunits = 0;
	}
}

static softreset(fp)
	FILE *fp;
/*	soft reset
	print out is made if data are stored.	*/
{
	fputc(ESC,fp);
	fputs("<",fp);
}

LBP_reset_()
{
	softreset(fp);
}

LBP_clear_()
{
	if (lbp_data == NODATA && lbp_set == NODATA ) return;
	if (lbp_device == LBP)	{
/*	eject paper	if the output is not spooled	*/
		eject();
		lbp_pagecount++;
		lbp_data=NODATA;
		lbp_set =NODATA;
		return;
	}

/*	If the output is spooled, abandon the previous output file and
	create a new file               */
	fclose(fp);
	openfile(lbp_filenumber);	/*	re-open as empty file	*/
	fcopy(fps,fp);
		/*	copy setfile onto outfile	*/
	lbp_data=NODATA;
	return;
}

LBP_terminate_()
{
/*	Reset all the settings of text and vector modes by soft reset.
	After soft reset, the mode becomes the text mode.	*/

/*    In case of direct output, reset printer.
	A print-out is made if the printer has data.   */

	if (lbp_device == LBP) 	{
		softreset(fp);
		fclose(fp);
	}

/*    in case of spooled output, simply delete files     */

	else 	{
		fclose(fp);
		fclose(fps);
		unlink(lbp_outfile);
		unlink(lbp_setfile);
	}
}

static linewidth(width)
	int width;
{
	if (width < -4) width = -4;	
	if (width >= 0) width = -1;
	vectormode();
	LBP_fputs("F1",LBPSET);
	vectordata1(width, LBPSET);
	LBP_fputc(IS2,LBPSET);
}

LBP_linewidth_(iwidth)
	int *iwidth;
/*    linewidth specification in VDC unit, integer   */
/*	LBP:	jwidth=	-1:	line width=1 dot
	      		-2:	           3 dots
	      		-3:	           5 dots
	      		-4:	           7 dots	*/
{
	int jwidth;
	
	jwidth = lconv( *iwidth ) /LBPdotsize;
			/*	conversion to LBP dots		*/
	jwidth = - (jwidth +1)/2;
	linewidth(jwidth);
}

LBP_linefwidth_(fwidth)
      float *fwidth;
/*    linewidth specification in unit of pixels, floating)  */
/*    LBP:  jwidth=     -1:   line width=1 dot
                        -2:              3 dots
                        -3:              5 dots
                        -4:              7 dots */
{
      int jwidth;

      jwidth = *fwidth +0.5;
	jwidth = -jwidth;
/*	fwidth=2.0 means second thinnest line, rather than line of
	double width. if we use
	jwidth = - ( jwidth +1)/2;
	fwidth=3.0 gives doulbe width line on print outs, but 
	looks too thick on graphic display
*/
	linewidth(jwidth);
}

static markersize(size)
	int size;
{
	vectormode();
	LBP_fputs("B",LBPSET);
	vectordata1(size, LBPSET);
	LBP_fputc(IS2,LBPSET);
}

LBP_markersize_(isize)
	int *isize;
/*	marker size specification in VDC unit, integer	*/
{
	int jsize;
	
	jsize = lconv (*isize);
	markersize( jsize);
}

LBP_markerfsize_(fsize)
	float *fsize;
/*	marker size specification in unit of pixels, floating		*/
{
	int jsize;
	
	jsize = (*fsize) * LBPdotsize;
			/*	fsize is now in 1/100 mm	*/
	markersize( jsize);
}

LBP_restrictedtext_(wx,wy,x,y,flag,string,len)
	int *wx,*wy,*x,*y,*flag,len;
	char *string;
/*	FORTRAN calling sequence: call restrictedtext(wx,wy,x,y,flag,string)
	flag=0: string not final
	    =1: final                   
	this parameter is ignored in LBP
	there is no equivalent function in LBP; text-routine is executed	*/
{       
	LBP_text_(x,y,string,len);
}

LBP_appendtext_(flag,string,len)
	int *flag,len;
	char *string;
/*	flag=0: string not final
	    =1: final                   
	this parameter is ignored in LBP	*/
{
	textmode();
	print(string,len);
}



/*	functions specific to LBP	*/


static landscape()
{
	lbp_orientation = LANDSCAPE;
	lbp_xmax = LBPlmax;
	lbp_ymax = LBPsmax;
	lbp_xmaxdot = LBPlmaxdot;
	lbp_ymaxdot = LBPsmaxdot; 
}

static portrait()
{
	lbp_orientation = PORTRAIT;
	lbp_xmax = LBPsmax;
	lbp_ymax = LBPlmax;
	lbp_xmaxdot = LBPsmaxdot;
	lbp_ymaxdot = LBPlmaxdot; 
}

static vectormode()
{
	if (lbp_vectormode == Off)	forced_vectormode();
}
 
static forced_vectormode()
{
	LBP_fputs(CSI,LBPSET);
	LBP_fputs("0",LBPSET);	/*	without changing the origin	*/
	LBP_fputs("&}",LBPSET);
	lbp_vectormode = On;
	pageorientation();
}

static textmode()
{
	if (lbp_vectormode == On)	forced_textmode();
}

static forced_textmode()
{	
	LBP_fputs("}p",LBPSET);
	LBP_fputc(IS2,LBPSET);
	lbp_vectormode = Off;
	pageorientation();
}
		
/*	eject paper	*/
static eject()
{
	fputc(EJECT,fp);
}

LBP_hcopy_()
{
	FILE *fpo;
	char oldfilename[LPstrlen];

	if (lbp_data == NODATA && lbp_set == NODATA) return;

/*	in case of direct output, eject paper
	although hcopy is intended to produce a non-destructive copy
	of the screen, cummulative output is not possible in case of
	direct output	*/
	if (lbp_device == LBP)	{
		eject();
		lbp_pagecount++;
		lbp_data = NODATA;
		lbp_set  = NODATA;
		return;
	}

/*	in case of spooled output	*/
	fclose(fp);
	strcpy(oldfilename,lbp_outfile);
	fpo = fopen(oldfilename,"r+");
	if(fpo == NULL)
		printf("\n (LBP_hcopy) open error %s",oldfilename);
	lbp_filenumber++;
	openfile(lbp_filenumber);	/*	set new outfile name	*/
	fcopy(fpo,fp);
	softreset(fpo);	/*	softreset should not be written to fp 
					because plotting continues on fp	*/
	fclose(fpo);

	if (lbp_output == NOHOLD)	{
		strcpy(cbuffer,"lpr -r -s -l -h -P");
		strcat(cbuffer,LPname);
		strcat(cbuffer," ");
		strcat(cbuffer,oldfilename);
		execute(cbuffer);
		/*	print old outfile	*/
	}
	lbp_pagecount++;
}

static pageorientation()
/*    
	page orientation specifications of text mode
	textorientation = 0(right), 1(up), 2(left), 3(down)
	textpath = 3(down), otherwise 0

	since the text mode does not care whether the vector mode
	is in portrait or landscape, this should be taken into account
	(in vector mode: lbp_orientation = 0(portrait) or 1(landscape))
*/
{
	int absolute_orientation;

	if (lbp_vectormode == On )	{
		absolute_orientation = lbp_orientation;
		LBP_fputs("}p",LBPSET);
		LBP_fputc(IS2,LBPSET);
		/*	change to text mode, without updating lbp_vectormode	*/

	}
	else	{
		absolute_orientation = (textorientation + 
					textpath + 
					lbp_orientation) % 4;
	}

	LBP_fputs(CSI,LBPSET);
	sprintf(cbuffer,"%d%%r",absolute_orientation);
	LBP_fputs(cbuffer,LBPSET);

	if (lbp_vectormode == On)	{
		LBP_fputs(CSI,LBPSET);
		LBP_fputs("0",LBPSET);	
		LBP_fputs("&}",LBPSET);	/*	back to vector mode	*/
	}
}

static locate(x,y)
/*	locate current text pointer at (x,y)	*/
	int x,y;
{
	int xx,yy;
	static int stored_orientation=0, stored_path=0;
	xx = xconv( x);
	yy = yconv( y);

	textmode();
	/*	temporarily switch to normal text orientation	*/
	if ( textorientation != 0 || textpath !=0 )	{
		stored_orientation = textorientation;
		stored_path        = textpath;
		textorientation = 0;
		textpath = 0;
		pageorientation();
	}
	LBP_fputs(CSI,LBPDRAW);
	sprintf(cbuffer,"%d;%df",yy,xx);
	LBP_fputs(cbuffer,LBPDRAW);

	/*	return to the original text orientation	*/
	if ( stored_orientation != 0 || stored_path !=0 )	{
		textorientation = stored_orientation;
		textpath = stored_path;
		stored_orientation = 0;
		stored_path = 0;
		pageorientation();
	}
}

static arc1_(openclose,cx,cy,r,angles,anglee,type_cgi)
	int *openclose,*cx,*cy,*r,*type_cgi;
	float *angles,*anglee;
/*	openclose=	0:	open arc
			1:	closed arc

	for closed arc:
	type_cgi=1,(type_lbp=0):	pie (default)
	         0,          1 :	chord			
*/
{
	float angs, ange;

	angs = *angles;	ange = *anglee;
	vectormode();
	LBP_fputs("}6",LBPDRAW);
	vectordata1(*openclose, LBPDRAW);
	vectordata1( xconv(*cx), LBPDRAW);
	vectordata1( yconv(*cy), LBPDRAW);
	vectordata1( lconv(*r), LBPDRAW);
      if (xfactor < 0.0)      {
            angs = 180.0 - angs;
            ange = 180.0 - ange;
      }
      if (-yfactor < 0.0)     {
            angs = -angs;
            ange = -ange;
      }
	if (angs < -180.0)	{
		angs += 360.0;
		ange += 360.0;
	}
	if (angs >  180.0)	{
		angs -= 360.0;
		ange -= 360.0;
	}

	vectordata1( (int)angs, LBPDRAW);
	vectordata1( (int)( ange - angs), LBPDRAW);
	vectordata1( 1- (*type_cgi), LBPDRAW);
	LBP_fputc(IS2,LBPDRAW);
}

LBP_arc_(cx,cy,r,angles,anglee)
	int *cx,*cy,*r;
	float *angles,*anglee;
/*	callable from cgi	*/
{
	int openclose = 0;	/*	open arc	*/
	int type=0;		/*	dummy		*/
	arc1_(&openclose,cx,cy,r,angles,anglee,&type);
}

static ellipticalarc_(openclose,cx,cy,a,b,anglea,angles,anglee,type_cgi)
	int *openclose,*cx,*cy,*a,*b,*type_cgi;
	float *anglea,*angles,*anglee;
/*	openclose=	0:	open arc
		  	1:	closed arc
	type_cgi=1,(type_lbp=0):	pie (default)
	         0,          1 :	chord			*/
{
	float anga, angs, ange;

	anga = *anglea;	angs = *angles;	ange = *anglee;
	vectormode();
      if (xfactor < 0.0)      {
            anga = 180.0 - anga;
            angs = 180.0 - angs;
            ange = 180.0 - ange;
      }
      if (-yfactor < 0.0)     {
            anga = -anga;
            angs = -angs;
            ange = -ange;
      }

	LBP_fputs("};",LBPDRAW);
	vectordata1(*openclose, LBPDRAW);
	vectordata1( xconv(*cx), LBPDRAW);
	vectordata1( yconv(*cy), LBPDRAW);
	vectordata1( lconv(*a), LBPDRAW);
	vectordata1( lconv(*b), LBPDRAW);
	vectordata1( angle_to_x(anga), LBPDRAW);
	vectordata1( angle_to_y(anga), LBPDRAW);
	if (xfactor*(-yfactor) > 0.0)	{
		vectordata1( angle_to_x(angs), LBPDRAW);
		vectordata1( angle_to_y(angs), LBPDRAW);
		vectordata1( angle_to_x(ange), LBPDRAW);
		vectordata1( angle_to_y(ange), LBPDRAW);
	}
	else	{
		vectordata1( angle_to_x(ange), LBPDRAW);
		vectordata1( angle_to_y(ange), LBPDRAW);
		vectordata1( angle_to_x(angs), LBPDRAW);
		vectordata1( angle_to_y(angs), LBPDRAW);
	}
	vectordata1( 1- (*type_cgi), LBPDRAW);
	LBP_fputc(IS2,LBPDRAW);
}

LBP_elliparc_(cx,cy,a,b,anglea,angles,anglee)
	int *cx,*cy,*a,*b;
	float *anglea,*angles,*anglee;
{
	int openclose = 0;	/*	open arc	*/
	int type=0;		/*	dummy		*/
	ellipticalarc_(&openclose,cx,cy,a,b,anglea,angles,anglee,&type);
}



/*	functions usable both in LBP and CGI	*/

LBP_clipindicator_(indicator)
	int *indicator;
/*	CGI:
	indicator=0(default):	clip rectangle
	         =1:			view surface
	         =2:			off		
	LBP:
	indicator=0:			clip on
	          1(default)	clip off		*/
{
	int i;
	vectormode();
	if (*indicator == 2) i=1;
		else i=0;
	LBP_fputs("*",LBPSET);
	vectordata1(i, LBPSET);
	LBP_fputc(IS2,LBPSET);
}

LBP_cliprectangle_(x1,y1,x2,y2)
	int *x1,*y1,*x2,*y2;
{	
	vectormode();
	LBP_fputs(")",LBPSET);
	vectordata1( xconv(*x1), LBPSET);
	vectordata1( xconv(*x2), LBPSET);
	vectordata1( yconv(*y1), LBPSET);
	vectordata1( yconv(*y2), LBPSET);
	LBP_fputc(IS2,LBPSET);

	LBP_fputs("*",LBPSET);
	vectordata1(0, LBPSET);
	LBP_fputc(IS2,LBPSET);	/*	make clipping on	*/
}

LBP_polyline_(np,x,y)
	int *np,x[],y[];
{
	int is=0, n, nmax=200;

	vectormode();
	
	do	{
		n= ( *np - is > nmax )? nmax:(*np - is);
		makexylist(n,x+is,y+is,ibuffer);
		makevectordata(2*n,ibuffer,cbuffer);	
		is+= n-1;

		LBP_fputs("1",LBPDRAW);
		LBP_fputs(cbuffer,LBPDRAW);
		LBP_fputc(IS2,LBPDRAW);
	} while( is < *np -1 );
}

LBP_disjointline_(np,x,y)
      int *np,x[],y[];
{
      int i, two=2;

      for (i=0; i< (*np) -1; i+=2) {
            LBP_polyline_(&two, x+i, y+i);
      }
}


LBP_linetype_(type)
	int *type;
/*
	type=1(default):		solid
	    =2:			dashed	--> lips short dashed
	    =3:			dotted	--> lips short dotted
	    =4:			dash-dot
	    =5:			dash-dot-dot

	lbp_linetype = mapping[type -1]
*/
{
	static int mapping[] = {0, -1, -2, 3, 4};

	if (*type >= 1 && *type <= 5 )	{
		vectormode();
		LBP_fputs("E1",LBPSET);
		vectordata1( mapping[(*type) -1], LBPSET);
		LBP_fputc(IS2,LBPSET);
	}
}

LBP_linecolor_(r,g,b)
	int *r,*g,*b;
{
}

LBP_polymarker_(np,x,y) 
	int *np,x[],y[];
{
	int is=0, n, nmax=200;

	vectormode();
	do {
            n= ( *np - is > nmax )? nmax:(*np - is);
            makexylist(n,x+is,y+is,ibuffer);
            makevectordata(2*n,ibuffer,cbuffer);
            is+= n;

		LBP_fputs("0",LBPDRAW);
            LBP_fputs(cbuffer,LBPDRAW);
            LBP_fputc(IS2,LBPDRAW);
      } while( is < *np );
}

LBP_markertype_(type)
	int *type;
/*
	type=1:		dot (default of cgi)
	    =2:		plus
	    =3:		asterisk (default of lips, which is overridden)
	    =4:		circle
	    =5:		cross	
	    =6:		square		(added for lips)
	    =7:		filled square	(added for lips)
	    =8:		triangle		(added for lips)
	    =9		filled triangle	(added for lips)

	lbp_markertype = mapping[type-1]
*/
{
	static int mapping[]={1, 2, 3, 4, 5, 0, -1, -2, -3};

	if (*type >= 1 && *type <= 9 )	{
		vectormode();
		LBP_fputs("A",LBPSET);
		vectordata1(mapping[(*type) -1], LBPSET);
		LBP_fputc(IS2,LBPSET);
	}
}

LBP_markercolor_(r,g,b)
      int *r,*g,*b;
/*    r is also used as color index */
{
}

static setinterior(style,index,edge)
	int style,index,edge;
/*
	style=1:	hollow (default)
	    	2:	solid
 		3:	user-defined pattern
	  	4:	hatch
	     	5:	empty (not applicable to LBP)

	lbp_interiorstyle= stylemapping[style -1]

	for style=4, hatch index should be specified

	index=1:          horizontal parallel lines
           =2(default): vertical parallel lines
           =3:          positive slope parallel lines
           =4:          negative slope parallel lines
           =5:          horizontal/vertical crosshatch
           =6:          positive/negative slope crosshatch

	lbp_hatchindex = indexmapping[index -1]

	edge = 0: visible
             1: invisivble (default)
*/
{
	static int stylemapping[] = {0, 1, -10, -4, 0};
	static int indexmapping[] = {-5, -4, -2, -3, -7, -6};
	int lbp_style, lbp_edge;

	lbp_edge = 1 - edge;

	if (style == 1)	lbp_edge = 1;
		/*	for interiorstyle = hollow, edge is drawn	*/

	if (style >= 1 && style <= 5 )	{
		lbp_style = stylemapping[style -1];
		if ( style == 4) lbp_style = indexmapping[index -1];
		LBP_fputs("I",LBPSET);
		vectordata1(lbp_style, LBPSET);
		vectordata1(lbp_edge, LBPSET);
		LBP_fputc(IS2,LBPSET);
	}
}

LBP_interiorstyle_(style)
	int *style;
{
	fillstyle = *style;
	setinterior( fillstyle, hatchindex, edgevisibility);
}

LBP_hatchindex_(index)
	int *index;
{
	hatchindex = *index;
	setinterior( fillstyle, hatchindex, edgevisibility);
}

LBP_edgevisibility_(edge)
	int *edge;
{
	edgevisibility = *edge;
	setinterior( fillstyle, hatchindex, edgevisibility);
}

/*	simulate color paint by black halftone fill	*/
LBP_fillcolor_(r,g,b)
      int *r,*g,*b;
/*    r,g,b = 0 to 255
      if colormode=INDEXEDCOLOR, r contains color index (1,2,,,256)     */
{
	static int fillmapping[] = {1,-8,-7,-6,-5,-4,-3,-2,-1,0};
	int intensity, lbp_style, lbp_edge;

	lbp_edge = 0;	/*	no edges drawn	*/

	if (colormode == DIRECTCOLOR)	{
		intensity = (*r) + (*g) + (*b);
	} else	{
		intensity = int_r[(*r)-1] +int_g[(*r)-1] +int_b[(*r)-1];
	}
	intensity = (int) (9.9*intensity/(3.0*256.0) );
	if (0 <= intensity && intensity < 10  )	{
		lbp_style = fillmapping[9-intensity];
	} else	{
		lbp_style = 0;
	}
/*	intensity=0 is black for screen
	intensity=0 is mapped to lbp_style=9 (empty)
	intensity=9 is mapped to lbp_style=0 (fill black)	*/

	vectormode();
	LBP_fputs("I",LBPSET);
	vectordata1(lbp_style, LBPSET);
	vectordata1(lbp_edge, LBPSET);
	LBP_fputc(IS2,LBPSET);
}

LBP_polygon_(np,x,y) 
	int *np,x[],y[];
{
	int is=0, n, nmax=200;

	vectormode();
	do {
            n= ( *np - is > nmax )? nmax:(*np - is);
            makexylist(n,x+is,y+is,ibuffer);
            makevectordata(2*n,ibuffer,cbuffer);
            is+= n-1;

		LBP_fputs("2",LBPDRAW);
            LBP_fputs(cbuffer,LBPDRAW);
            LBP_fputc(IS2,LBPDRAW);
      } while( is < *np -1 );
}

LBP_rectangle_(x1,y1,x2,y2)
	int *x1,*y1,*x2,*y2;
{
	int xx1,xx2,yy1,yy2;

	vectormode();
	LBP_fputs("}:",LBPDRAW);
	xx1= xconv(*x1);	xx2= xconv(*x2);
	yy1= yconv(*y1);	yy2= yconv(*y2);

	if (xx1 < xx2)	{
		vectordata1( xx1, LBPDRAW);
		vectordata1( xx2, LBPDRAW);
	}
	else	{
		vectordata1( xx2, LBPDRAW);
		vectordata1( xx1, LBPDRAW);
	}
	if (yy1 < yy2)	{
		vectordata1( yy1, LBPDRAW);
		vectordata1( yy2, LBPDRAW);
	}
	else	{
		vectordata1( yy2, LBPDRAW);
		vectordata1( yy1, LBPDRAW);
	}
	LBP_fputc(IS2,LBPDRAW);
}

LBP_circle_(cx,cy,r)
	int *cx,*cy,*r;
{
	vectormode();
	LBP_fputs("5",LBPDRAW);
	vectordata1( xconv(*cx), LBPDRAW);
	vectordata1( yconv(*cy), LBPDRAW);
	vectordata1( lconv(*r), LBPDRAW);
	LBP_fputc(IS2,LBPDRAW);
}

LBP_arcclose_(cx,cy,r,angles,anglee,type)
	int *cx,*cy,*r,*type;
	float *angles, *anglee;
{
	int openclose = 1;	/*	closed arc	*/
	arc1_(&openclose,cx,cy,r,angles,anglee,type);
}

LBP_ellipse_(cx,cy,a,b,anglea)
	int *cx,*cy,*a,*b;
	float *anglea;
{
	float anga;

	anga = *anglea;
      if (xfactor < 0.0) anga = 180.0 - anga;
      if (yfactor > 0.0) anga = -anga;

	LBP_fputs("};",LBPDRAW);
	vectormode();
	LBP_fputs("}7",LBPDRAW);
	vectordata1( xconv(*cx), LBPDRAW);
	vectordata1( yconv(*cy), LBPDRAW);
	vectordata1( lconv(*a), LBPDRAW);
	vectordata1( lconv(*b), LBPDRAW);
	vectordata1( (int)(anga), LBPDRAW);
	LBP_fputc(IS2,LBPDRAW);
}

LBP_elliparcclose_(cx,cy,a,b,anglea,angles,anglee,type)
	int *cx,*cy,*a,*b,*type;
	float *anglea,*angles,*anglee;
/*	type=	0:	pie (default)
		1:	chord		*/
{
	int openclose = 1;	/*	closed arc	*/
	ellipticalarc_(&openclose,cx,cy,a,b,anglea,angles,anglee,type);
}

LBP_text_(x,y,string,len)
	int *x,*y,len;
	char *string;
/*	FORTRAN calling sequence: call text(x,y,string)	*/
{
	textmode();
	locate( *x,*y );
	print(string,len);
}

LBP_textnf_(x,y,string,len)
	int *x,*y,len;
	char *string;
{
	LBP_text_(x,y,string,len);
}

LBP_textcolor_(r,g,b)
      int *r,*g,*b;
/*    r is also used as color index */
{
}

LBP_chrheight_(height)
        int *height;
/*    height is in VDC unit	*/
{
	heightfactor = lconv(*height) / STANDARD_HEIGHT;
}

LBP_chrwidth_(ratio)
	float *ratio;
/*	ratio: width to height ratio	*/
{
	widthratio = *ratio;
	chrsize(heightfactor,widthratio);
}

static chrsize(heightfactor,widthratio)
	float heightfactor,widthratio;
{
	int iheight = 100, iwidth = 100;
	float widthfactor;

	if (heightfactor > 1.5 )	
		{ heightfactor=2.0; iheight = 200;}
	else if (heightfactor < 0.5 )
		{ heightfactor=0.5; iheight =  50;}	
	else 
		heightfactor=1.0;
	widthfactor = heightfactor * widthratio; 

	if (widthfactor > 1.5 )	
		{ widthfactor=2.0; iwidth = 200;}
	else if (widthfactor < 0.5 )
		{ widthfactor=0.5; iwidth =  50;}
	else
		widthfactor=1.0;

	textmode();
	LBP_fputs(CSI,LBPSET);
	sprintf(cbuffer,"%d;%d B",iheight,iwidth);
	LBP_fputs(cbuffer,LBPSET);
}

LBP_chrspacing_(spacing)
	float *spacing;
/*	character spacing in units of character height
	standard spacing -- spacing=0	*/
{
	int hmi;
	hmi = ( 0.677 + (*spacing) ) 
		* widthratio * heightfactor * STANDARD_HEIGHT;

if(debug) printf("\n (LBP_chrspacing) hmi= %d",hmi);

	LBP_fputc(SI,LBPSET);	/*	hmi set -- G0 set	*/
	LBP_fputs(CSI,LBPSET);
	sprintf(cbuffer,";%d G",hmi);
	LBP_fputs(cbuffer,LBPSET);

	hmi = ( 2.0 * 0.677 + (*spacing) )
		* widthratio * heightfactor * STANDARD_HEIGHT;
	LBP_fputc(ESC,LBPSET);LBP_fputs("n",LBPSET);	/*	hmi set - G2 set	*/
	LBP_fputs(CSI,LBPSET);
	sprintf(cbuffer,";%d G",hmi);
	LBP_fputs(cbuffer,LBPSET);

	LBP_fputc(SI,LBPSET);	/*	return to G0 set	*/
}

LBP_chrorientation_(angleb,angleu)
	float *angleb,*angleu;
/*	direction of base-vector -- angleb
	             up-vector   -- angleu	*/
{
	textorientation = ( *angleb + 0.5 )/90.0;
	pageorientation();
}


LBP_textpath_(path)
	int *path;
/*	path =0:	left
	      1:	up
	      2:	right (default)
	      3:	down		

	only right and down are supported here	*/
{
	textmode();
	if (*path == 3)	{	/* down	*/
		LBP_fputs(CSI,LBPSET);
		LBP_fputs("15 S",LBPSET);
		textpath = 3;
	}
	else	{
		LBP_fputs(CSI,LBPSET);
		LBP_fputs("0 S",LBPSET);
		textpath = 0;
	}
}

LBP_fontindex_(index_f)
	int *index_f;
{
/*	index_f = 1,2,,,	*/
/*	LBP_fputs(CSI,LBPSET);

	int index;
	index = *index_f -1;
	sprintf(cbuffer,"%dy",index );
	LBP_fputs(cbuffer,LBPSET);
*/
}

LBP_colormode_(mode)
      int *mode;
{
}

LBP_colortable_(startindex_f,ncolor,r,g,b)
      unsigned int *startindex_f,*ncolor,r[],g[],b[];
{
}

LBP_byteimage_(xo,yo,nx,ny,x1,x2,y1,y2,image)
      int *xo,*yo,*nx,*ny,*x1,*x2,*y1,*y2;
      unsigned char *image;
/*	(xo,yo) = VDC coordinates of the upper left corner of
	          the image to be displayed on the screen
	(nx,ny) = size of the full image array
	(x1,y1)-(x2,y2) = range of the image subarray to be displayed
*/
{
      int nplane=8;
      unsigned short *bitmap;
      unsigned int size;

      size = (((*nx)+15)/16)*(*ny)*16;
      bitmap = (unsigned short *)malloc(size);
      makebitwiseimage(image,*nx,*ny,nplane,bitmap);
      LBP_bitwiseimage_(xo,yo,nx,ny,x1,x2,y1,y2,nplane,bitmap);
      free(bitmap);
}

LBP_bitwiseimage_(xo,yo,nx,ny,x1,x2,y1,y2,nplane,bitmap)
      int *xo,*yo,*nx,*ny,*x1,*x2,*y1,*y2,*nplane;
      unsigned short *bitmap;
/*	(xo,yo) = VDC coordinates of the upper left corner of
	          the image to be displayed on the screen
	(nx,ny) = size of the full image array
	(x1,y1)-(x2,y2) = range of the image subarray to be displayed
*/
{
/*	bit pattern whose dot density is proportional to image intensity	*/
	static unsigned short mask[8][16]={
		0,0,0,0,0,0,0, 0x2000,
		0,0,0,0,0,0,0, 0,						/*	bit0	*/
		0,0,0,0,0,0,0, 0x2000,
		0,0,0,0,0,0,0, 0x0020,					/*	bit1	*/
		0,0,0, 0x0202, 0,0,0,0,					
		0,0,0, 0x0202, 0,0,0,0,					/*	bit2	*/
		0,0,0, 0x2020, 0,0,0, 0x0202,
		0,0,0, 0x2020, 0,0,0, 0x0202,				/*	bit3	*/
		0, 0x8888, 0,0, 0, 0x8888, 0,0,
		0, 0x8888, 0,0, 0, 0x8888, 0,0,			/*	bit4	*/
		0, 0x2222, 0, 0x8888, 0, 0x2222, 0, 0x8888,
		0, 0x2222, 0, 0x8888, 0, 0x2222, 0, 0x8888,	/*	bit5	*/
		0x5555, 0, 0x5555, 0, 0x5555, 0, 0x5555, 0,
		0x5555, 0, 0x5555, 0, 0x5555, 0, 0x5555, 0,	/*	bit6	*/
		0xaaaa, 0x5555, 0xaaaa, 0x5555, 
		0xaaaa, 0x5555, 0xaaaa, 0x5555,
		0xaaaa, 0x5555, 0xaaaa, 0x5555,
		0xaaaa, 0x5555, 0xaaaa, 0x5555			/*	bit7	*/
		};

	unsigned short word, *wp;
	int factor, ni, byte_width, nbyte, mapsize, x, xx, y, z, i, j;
	static int factors[]={240,120,80,60};

	if ( *nplane > 0 && *nplane < 5 )	factor = *nplane;
	else						factor = 1;

      ni = ( (15+ *nx) / 16 ) -1;
	byte_width = 2*( ( *x2 - *x1 + 16 ) / 16 );
	nbyte = byte_width * ((*y2) - (*y1) +1);

	locate( *xo,*yo );
	textmode();
	LBP_fputs(CSI,LBPDRAW);
	sprintf(cbuffer,"%d;%d;%d.r", nbyte,byte_width,factors[factor-1]);
	LBP_fputs(cbuffer,LBPDRAW);

	mapsize = (ni+1)* (*ny);
	y = (*yo);
      for( j=(*y1); j <= (*y2); j++)  {
		xx = ( (*x1)/16 ) *16;
		x = (*xo) + xx;
		for( i=(*x1)/16; i<=(*x2)/16; i++)	{	
			wp=bitmap+j*(ni+1)+i;

			if( *nplane == 1 )
				word = *wp;
			else	{
				word = 0;
				for (z=0; z<8; z++, wp+=mapsize)	{
					word |= ( mask[z][y%16] & *wp );
				}
			}

			if (xx < (*x1) )	{
/*				word=left (*x1-xx) bits are zero;	*/
				word << ((*x1)-xx);
				word >> ((*x1)-xx);
			}
			else if (xx > (*x2) )	{
/*				word=right (xx-*x2) bits are zero;	*/
				word >> (xx- (*x2));
				word << (xx- (*x2));
			}
			xx += 16;
			x += 16;
			fwrite( &word, 2, 1, fp );
		}
		y++;
	}
}

