/*	pslib.c
	PostScript library
*/

#include <stdio.h>
#include <string.h>
#include <math.h>

#include "lbpnop.c"

#ifdef LBPONLY
#include "lbponly.h"
#endif

#define LPstrlen 50
#define SPOOL 0
#define LBP 1
#define SPOOLoutfile  "LBPout"
#define SPOOLsetfile  "LBPset"
#define HOLD 1
#define NOHOLD 0
#define NODATA 0
#define EXIST 1
#define LBPMAXBUF 2000
#define On 1
#define Off 0
#define LBPSET 1
#define LBPDRAW 0
#define PORTRAIT 0
#define LANDSCAPE 1

/*	A4
#define LBPlmax 870.71
#define LBPsmax 564.09
		l=long-side = 286 mm = 810.71 pt,
		s=short-side = 199 mm = 564.09 pt
*/

/*	Letter size 11 x 9 inches	*/
#define LBPlmax 792.0
#define LBPsmax 648.0

#define PSunit_mm 0.352778
	/*	postscript default unit length = 1/72 inch = 1pt = 0.352778 mm	*/
#define STANDARD_HEIGHT  10.0
	/*	standard character height= 10 pt	*/
#define LBPdotsize 0.18
	/*	assume LBP dot = 1/400 inch = 72/400 pt = 0.18 pt	*/

#define MAXFONTNAMES 2
static char *fontnamelist[MAXFONTNAMES]={"Courier", "Helvetica"};

FILE *fp;               /*    pointer for output file */
FILE *fps;              /*    pointer for setup file  */

/*	int = long (4-byte) integer = fortran integer	*/

static int lbp_device;
static int lbp_output;
static int lbp_filenumber = 1;
static int lbp_pagecount = 0;
static int lbp_data = NODATA;
static int lbp_set  = NODATA;
static int lbp_xmax = LBPsmax;
static int lbp_ymax = LBPlmax;
static int xs=0, ys=0;			/*	start point of VDC	*/
static int xe=32767, ye=32767;		/*	end point of VDC	*/
static float xoffset=0, yoffset=0;	/*	origin of LBP	*/
static float lmargin=20/PSunit_mm, tmargin=20/PSunit_mm;
					/*	left and top margins = 20mm	*/
static int lbp_font = 1;	/* 	initial font = fontnamelist[0]	*/
static float scalefactor = 0.1/PSunit_mm;
static float xfactor = 0.1/PSunit_mm, yfactor = 0.1/PSunit_mm;
static float chrheight = STANDARD_HEIGHT;
static float widthratio = 1.0;
static float chrspacing = 0.0;
static float textbaseangle = 0.0;	/*	horizontal	*/
static int textpath = 0;	/*	toward left		*/

/*	set default values according to news cgi	*/

static int vpunits = 1;
	/*	default viewport specification unit of cgi:1/10 mm
		this pslib uses 1/72 inch as the unit length */
static int hatchindex = 2;
	/*	default hatch pattern = vertical parallel lines	*/
static int fillstyle = 1;
	/*	in order to perform hatching and interior filling,
		fillstyle shoube be passed between 'hatchindex' and 
		'interiorstyle' routines	*/
static int edgevisibility = 1;
	/*	default edge visibility	= invisible	*/
static int markertype = 1;
	/*	marker = dot	*/


static int colorprint = 0;
	/*	initial mode is colorprint = off		*/ 
static struct rgbtriplet {int r; int g; int b;}
	currentcolor, linecolor, textcolor, fillcolor, markercolor;
static float currentint = 0;
	/*	initial gray level is 0(black)	*/
#define DIRECTCOLOR 0
#define INDEXEDCOLOR 1
extern int colormode;	/* DIRECTCOLOR or INDEXEDCOLOR	*/
extern unsigned int max_colorindex;
static unsigned int int_r[256],int_g[256],int_b[256];

static int ibuffer[LBPMAXBUF];
static char cbuffer[LBPMAXBUF];
static char lbp_outfile[LPstrlen], lbp_setfile[LPstrlen];
static char LPdevice[LPstrlen], LPname[LPstrlen];

extern struct common_sys
	{int mycode,myint,keyin,mag_stdin,lst,msg,msglvl,errflg;
	float ufl; short int iufl2,maxi2; long int iufl4,maxi4;}
	sys_;   /*      fortran common/sys/     */

extern struct common_lbp
	{char LPdevice[LPstrlen], LPname[LPstrlen];}
	lbpstr_;	/*	fortran common/lbpstr/	*/


static int debug=0;




/*	internal functions	*/

static LBP_fputs(s, set_or_draw )
	char *s; int set_or_draw;
/*	*s is a null-terminated string	*/
{
	fputs(s,fp);
	fputc('\n',fp);
	if( set_or_draw == LBPSET && lbp_device == SPOOL )    {
		fputs(s,fps);
		fputc('\n',fps);
		lbp_set = EXIST;
	}
	else
		lbp_data = EXIST;
}

/*	conversion from VDC to LBP coordinates	*/

static float xconv(x)
	int x;
{
	return ( (float) ( xfactor * x + xoffset + lmargin ) );
}

static float yconv(y)
	int y;
{
	return ( (float) ( yfactor * y + yoffset - tmargin ) );
}

static float lconv(a)
	int a;
{
	return ( (float) ( scalefactor * a ) );
}

static execute(str)
      char *str;
/*    execute shell script    */
{
      int status;
      if( (status = system(str)) != 0 ) printf(
      "\n (pslib/execute) error: command execution failed, command=%s",
            str );
      return (status);
}

static fcopy(fpi,fpo)
      FILE *fpi, *fpo;
/*    copy file   */
{
      int c;
      fflush(fpi);      /*    flush output to fpi if any    */
      rewind(fpi);
      while ( (c=getc(fpi) )!= EOF) putc(c,fpo);
}

static openfile(filenumber)
      int filenumber;
{
      if (filenumber == 0)    {
/*    Create setfile as new, allowing read operation  */
            sprintf(lbp_setfile,"%s%d.ps",SPOOLsetfile,getpid());
            if(debug)
			printf("\n (pslib/openfile) open setfile--%s",lbp_setfile);
            fps = fopen(lbp_setfile,"w+");
            if(fps == NULL) printf("\n open error %s",lbp_setfile);
      }
      else  {
/*	Create outfile as new   */
      	sprintf(lbp_outfile, 
			"%s%d-%d.ps",SPOOLoutfile,getpid(),filenumber);
            if(debug)
			printf("\n (pslib/openfile) open outfile--%s",lbp_outfile);
            fp = fopen(lbp_outfile,"w");
            if(fp == NULL) printf("\n open error %s",lbp_outfile);
      }
}

/*	LBP functions whose effects are different from CGI	*/
 
LBP_initialize_(initmode)
	int *initmode;
/*	abs(initmode) =1:	portrait orientation, output to spool
	               2:	landscape orientation,    "
		       3:	same as 1, output hold
		       4:	same as 2, output hold
		       5:	same as 1, direct output to LBP
		       6:	same as 2,          "
	otherwise assume initmode=1 
	for color print, 10 is added
      in case of direct output, the printer cannot be shared until
      this program ends.
	no CRT output if initmode<0	*/

{
	int imode;

	if ( sys_.msglvl > 2 )	debug=1;

	if ( (int)(abs(*initmode)) / 10 == 1 )	colorprint = 1;
	else							colorprint = 0;

	if ( colorprint )	LBP_colormode_(&colormode);

	imode = (int)(abs(*initmode)) % 10;

	if ( imode == 0 || imode > 6 ) imode = 1;

	if ( imode >4)	{
		lbp_device = LBP;	lbp_output = NOHOLD;
	}
	else if ( imode > 2 )	{
		lbp_device = SPOOL;	lbp_output = HOLD;
	}
	else	{ 
		lbp_device = SPOOL;	lbp_output = NOHOLD;
	}

	strncpy(LPdevice, lbpstr_.LPdevice, LPstrlen);
	strncpy(LPname, lbpstr_.LPname, LPstrlen);
	*strchr(LPdevice,' ') = '\0';
	*strchr(LPname,' ') = '\0';

      if (lbp_device == SPOOL)      {
            openfile(0);   /*    open setfile      */
            openfile(lbp_filenumber);   /*    open outfile      */
      }
	if (lbp_device == LBP)  {
		fp = fopen(LPdevice,"w");
		fps = fp;
		if (fp == NULL)
			printf("\n (LBP_initialize) open error %s",LPdevice);
	}

	initset();

	if ( (imode -1) % 2 == 1 )	landscape();
	else 				portrait();
/*	these should be set after initset	*/

}	

static initset()
{
	header();
	macrodef();
/*
	These as well as other "setting" commands need to be stored
	and sent to printer, if LBP_clear clears the "drawing" data
*/
}

static header()
{
	int font;
	char *ctime();
	long time(), clock;

      LBP_fputs("%!PS-Adobe-1.0",LBPSET);

      strcpy(cbuffer,"%%DocumentFonts:");
      for (font=0; font<MAXFONTNAMES; font++)   {
            strcat(cbuffer, " ");
            strcat(cbuffer, fontnamelist[font]);
      }
      LBP_fputs(cbuffer, LBPSET);

      LBP_fputs("%%Title: magpack2 output", LBPSET);

      LBP_fputs("%%Creator: proc.disp/plot with pslib", LBPSET);

      strcpy(cbuffer,"%%CreationDate: ");
      clock = time(0);
      strcat(cbuffer,ctime(&clock));
      LBP_fputs(cbuffer, LBPSET);

      LBP_fputs("%%Pages: (atend)", LBPSET);

      sprintf(cbuffer,"%%%%BoundingBox: 0 0 %.2f %.2f",
            LBPsmax, LBPlmax);
      LBP_fputs(cbuffer,LBPSET);

      LBP_fputs("%%EndComments", LBPSET);

      LBP_fputs("/initstate save def",LBPSET);
/*      LBP_fputs("initgraphics",LBPSET);  this interferes with eps	*/
      LBP_fputs("clear",LBPSET);
}
static macrodef()
{
	int markersize=5;

	static char
	*solid = "/solid {[] 0 setdash} def",
	*dotted = "/dotted {[1 2] 0 setdash} def",
	*dashed = "/dashed {[5 2] 0 setdash} def",
	*dashdot = "/dashdot {[5 2 1 2] 0 setdash} def",
	*dashdotdot = "/dashdotdot {[5 2 1 2 1 2] 0 setdash} def",

	*dotmark = "/dotmark {0 0 rlineto stroke} def",
	*asterisk = "/asterisk {0 markersize rmoveto \
			0 markersize -2 mul rlineto \
			markersize -0.866 mul markersize 0.5 mul rmoveto \
			markersize 1.732 mul markersize rlineto \
			0 markersize neg rmoveto \
			markersize -1.732 mul markersize rlineto stroke} def",
	*plusmark = "/plusmark {markersize 0 rmoveto \
			markersize -2 mul 0 rlineto \
			markersize markersize rmoveto \
			0 markersize -2 mul rlineto stroke} def",
	*circlemark = "/circlemark {currentpoint newpath \
			markersize 0 360 arc stroke} def",
	*crossmark = "/crossmark {markersize markersize rmoveto \
			markersize -2 mul dup rlineto \
			0 markersize 2 mul rmoveto \
			markersize 2 mul dup neg rlineto stroke} def",
	*squaremark = "/squaremark {markersize markersize rmoveto \
			markersize -2 mul 0 rlineto \
			0 markersize -2 mul rlineto \
			markersize 2 mul 0 rlineto \
			closepath gsave stroke grestore} def",
	*filledsquaremark = "/filledsquaremark \
			{squaremark fill} def",
	*trianglemark = "/trianglemark {0 markersize rmoveto \
			markersize -0.866 mul markersize -1.5 mul rlineto \
			markersize 1.732 mul 0 rlineto \
			closepath gsave stroke grestore} def",
	*filledtrianglemark = "/filledtrianglemark \
			{trianglemark fill} def",

	*verticals = "/verticals { gsave clip pathbbox \
		/ury exch def /urx exch def /lly exch def /llx exch def \
		llx linespace urx {dup lly moveto ury lineto} for \
		stroke grestore } def",

	*horizontals = "/horizontals { gsave clip pathbbox \
		/ury exch def /urx exch def /lly exch def /llx exch def \
		lly linespace ury {dup llx exch moveto urx exch lineto} for \
		stroke grestore } def",

	*parallellines = "/parallellines { gsave linetilt rotate pathbbox clip \
		/ury exch def /urx exch def /lly exch def /llx exch def \
		lly linespace ury {dup llx exch moveto urx exch lineto} for \
		stroke linetilt neg rotate grestore } def", 

	*positiveslopes = " /positiveslopes { \
		/linetilt 45 def parallellines } def",

	*negativeslopes = " /negativeslopes { \
		/linetilt -45 def parallellines } def";

/*	initiaize font	*/
	LBP_fontindex_(&lbp_font);

/*	set hatch index because lips has no default value	*/
	LBP_hatchindex_(&hatchindex);

/*	set edge visibility because postscript has no defaults value	*/
	LBP_edgevisibility_(&edgevisibility);

/*	set marker type and marker size because postscript has no default values	*/
	LBP_markersize_(&markersize);
	LBP_markertype_(&markertype);

/*	set line types and markers	*/
	LBP_fputs(solid, LBPSET);
	LBP_fputs(dotted, LBPSET);
	LBP_fputs(dashed, LBPSET);
	LBP_fputs(dashdot, LBPSET);
	LBP_fputs(dashdotdot, LBPSET);

	LBP_fputs(dotmark, LBPSET);
	LBP_fputs(plusmark, LBPSET);
	LBP_fputs(asterisk, LBPSET);
	LBP_fputs(circlemark, LBPSET);
	LBP_fputs(crossmark, LBPSET);
	LBP_fputs(squaremark, LBPSET);
	LBP_fputs(filledsquaremark, LBPSET);
	LBP_fputs(trianglemark, LBPSET);
	LBP_fputs(filledtrianglemark, LBPSET);

/*	set hatching operations	*/
	LBP_fputs("/linespace 5 def", LBPSET);
	LBP_fputs("/linetilt 0 def", LBPSET);
	LBP_fputs(parallellines, LBPSET);
	LBP_fputs(verticals, LBPSET);
	LBP_fputs(horizontals, LBPSET);
	LBP_fputs(positiveslopes, LBPSET);
	LBP_fputs(negativeslopes, LBPSET);

/*	short commands	*/
	LBP_fputs("/N {newpath} def", LBPSET);
	LBP_fputs("/M {moveto} def", LBPSET);
	LBP_fputs("/D {lineto} def", LBPSET);
	LBP_fputs("/S {gsave stroke grestore} def", LBPSET);
	LBP_fputs("/W {setlinewidth} def", LBPSET);
	LBP_fputs("/C {setrgbcolor} def", LBPSET);
	LBP_fputs("/G {setgray} def", LBPSET);
}

LBP_vdcextent_(x1,y1,x2,y2)
	int *x1,*y1,*x2,*y2;
/*	extent of virtual display space

	default unit of length in postscript is 1/72 inch = 0.352778 mm
	*/
{
	xs = *x1;	ys = *y1;
	xe = *x2;	ye = *y2;
}

LBP_viewportunits_(units)
	int *units;
/*	units=0 				:percent relative
	      1 (default here)		:1/10 mm
	      2				:absolute (dots)	*/
{
	vpunits = *units;
}

LBP_viewport_(x1,y1,x2,y2)
	int *x1,*y1,*x2,*y2;

/*	viewport specification for viewportunits=1,2

      VDC (xs,ys),(xe,ye) are mapped to viewport (x1,y1),(x2,y2)

      if (x1,y1)=(0,0), this corresponds to the upper left corner of the screen
	in the postscript default the origin is at the lower left corner of
	the paper
	the upper left corner of the screen should appear on the printer at
	(lmargin, lbp_ymax -tmargin)
	left asnd top margins are added in xconv/yconv

*/
{
	float xx1,yy1,xx2,yy2, fctr;

	xx1= *x1;	xx2= *x2;
	yy1= *y1;	yy2= *y2;

	if (vpunits == 1)	{	/*	(x1,y1),(x2,y2) are in units of 1/10mm	*/
		fctr = 0.1/PSunit_mm;
		xx1 *= fctr;	xx2 *= fctr;
		yy1 *= fctr;	yy2 *= fctr;
	}
	if (vpunits == 2)	{	/*	(x1,y1),(x2,y2) are in dots	*/
		xx1 *= LBPdotsize;	xx2 *= LBPdotsize;
		yy1 *= LBPdotsize;	yy2 *= LBPdotsize;
	}
		/*	(xx1,yy1),(xx2,yy2) are in units of 1/72 inch 	*/

	yy1 = lbp_ymax - yy1;	yy2 = lbp_ymax - yy2;

	xfactor = ( xx2 - xx1 )/( xe - xs );
	yfactor = ( yy2 - yy1 )/( ye - ys );

	if ( fabs((double)xfactor) > fabs((double)yfactor) ) 
		scalefactor= fabs(yfactor);
	else
		scalefactor= fabs(xfactor);

	xoffset = xx1 - xfactor * xs;
	yoffset = yy1 - yfactor * ys;

	/*	set clippath?	*/

/*
printf("\n x1,x2=%d %d",*x1,*x2);
printf("\n y1,y2=%d %d",*y1,*y2);
printf("\n xx1,xx2=%f %f",xx1,xx2);
printf("\n yy1,yy2=%f %f",yy1,yy2);
printf("\n x,yfactor=%f %f",xfactor,yfactor);
printf("\n offset=%f %f",xoffset,yoffset);
*/
}

LBP_viewportf_(fx1,fy1,fx2,fy2)
	float *fx1,*fy1,*fx2,*fy2;
/*	viewport specification for viewportunits=0 (percent relative)	*/
{
	int x1,y1,x2,y2;

	if (vpunits == 0 )	{
		x1 = 0.01*lbp_xmax*PSunit_mm*10*(*fx1);
		y1 = 0.01*lbp_ymax*PSunit_mm*10*(*fy1);
		x2 = 0.01*lbp_xmax*PSunit_mm*10*(*fx2);
		y2 = 0.01*lbp_ymax*PSunit_mm*10*(*fy2);
		vpunits = 1;	/*	use 1/10 mm as the unit	*/
		LBP_viewport_( &x1,&y1,&x2,&y2 );
		vpunits = 0;
	}
}

LBP_reset_()
{
}

LBP_clear_()
{
	if (lbp_data == NODATA && lbp_set == NODATA ) return;

/*	erase page if the output is not spooled	*/
	if (lbp_device == LBP)	{
		LBP_fputs("erasepage", LBPDRAW);
		lbp_data=NODATA;
		lbp_set =NODATA;
		return;
	}

/*	If the output is spooled, abandon the previous output file and
	create a new file		*/

	fclose(fp);
	openfile(lbp_filenumber);
	fcopy(fps,fp);
            /*    copy setfile onto outfile     */
	lbp_data=NODATA;
}

static trailer(fp)
	FILE *fp;
{
	fputs("%%Trailer\n", fp);
	fputs("initstate restore\n", fp);
	sprintf(cbuffer, "%%%%Pages: %d\n",lbp_pagecount);
	fputs(cbuffer, fp);
}

LBP_terminate_()
{
/*	end of print out job	*/

/*	If the output is not spooled, erase page and reset printer	*/
	if (lbp_device == LBP) 	{
		LBP_fputs("erasepage", LBPDRAW);
		trailer(fp);
		fclose(fp);
	}

/*	If the output is spooled, simply delete files	*/

	else 	{
		fclose(fp);
		fclose(fps);
            unlink(lbp_outfile);
            unlink(lbp_setfile);
	}
}

static linewidth(width)
	float width;
{
	if (width < 0) width = 1;	
	sprintf(cbuffer, "%.2f W", width);
	LBP_fputs(cbuffer,LBPSET);
}

LBP_linewidth_(iwidth)
	int *iwidth;
/*    linewidth specification in VDC unit, integer   */
{
	float width;
	
	width = lconv( *iwidth );
	linewidth(width);
}

LBP_linefwidth_(fwidth)
      float *fwidth;
/*    linewidth specification in unit of pixels, floating  */
{
      float width;

      width = (*fwidth)*LBPdotsize;
			/*	conversion to postscript length scale	*/
	linewidth(width);

}

static markersize(size)
	float size;
{
	sprintf(cbuffer, "/markersize %.2f def", size);
	LBP_fputs(cbuffer,LBPSET);
}

LBP_markersize_(isize)
	int *isize;
/*	marker size specification in VDC unit, integer	*/
{
	float size;
	
	size = lconv (*isize);
	markersize( size);
}

LBP_markerfsize_(fsize)
	float *fsize;
/*	marker size specification in unit of pixels, floating	*/
{
	float size;
	
	size = (*fsize) * LBPdotsize;
			/*	size is now in 1/72 inch	*/
	markersize( size);
}

LBP_restrictedtext_(wx,wy,x,y,flag,string,len)
	int *wx,*wy,*x,*y,*flag,len;
	char *string;
/*	FORTRAN calling sequence: call restrictedtext(wx,wy,x,y,flag,string)
	flag=0: string not final
	    =1: final                   
	this parameter is ignored in LBP
	there is no equivalent function in LBP; text-routine is executed	*/
{       
	LBP_text_(x,y,string,len);
}

LBP_appendtext_(flag,string,len)
	int *flag,len;
	char *string;
/*	flag=0: string not final
	    =1: final                   
	string is of length len and not null-terminated
	this parameter is ignored in LBP	*/
{

	sprintf(cbuffer, "gsave currentpoint 2 copy translate %.2f rotate",
		textbaseangle );
	LBP_fputs(cbuffer, LBPDRAW);
	sprintf(cbuffer, "%.2f 0 (", chrspacing);
	strncat(cbuffer, string, len);
	strcat(cbuffer, ") ashow currentpoint /y exch def /x exch def \
		 grestore y add exch x add exch moveto");
	LBP_fputs(cbuffer, LBPDRAW);
/*
	grestore resets the currentpoint to the state saved by gsave
	therefore the currentpoint defined within gsave-grestore is 
	brought outside via variables x and y
*/
}


/*	functions specific to LBP	*/


static landscape()
{
	sprintf(cbuffer, "%.2f 0 translate 90 rotate", LBPsmax);
	LBP_fputs(cbuffer, LBPSET);
	lbp_xmax = LBPlmax;
	lbp_ymax = LBPsmax;
}

static portrait()
{
	lbp_xmax = LBPsmax;
	lbp_ymax = LBPlmax;
}

LBP_hcopy_()
{
	FILE *fpo;
	char oldfilename[20];

	if (lbp_data == NODATA && lbp_set == NODATA ) return;

	lbp_pagecount++;
	sprintf(cbuffer, "%%%%Page: %d %d", lbp_pagecount, lbp_pagecount);

/*    in case of direct output, copypage and continue      */
      if (lbp_device == LBP)  {
		LBP_fputs("copypage", LBPDRAW);
		LBP_fputs(cbuffer, LBPDRAW);
            lbp_data = NODATA;
            lbp_set  = NODATA;
            return;
      }

/*    in case of spooled output     */

	fclose(fp);
      strcpy(oldfilename,lbp_outfile);
      fpo = fopen(oldfilename,"r+");
      if(fpo == NULL)
            printf("\n (LBP_hcopy) open error %s",oldfilename);
      lbp_filenumber++;
      openfile(lbp_filenumber);     /*    set new outfile name    */

      fcopy(fpo,fp);
	fputs("showpage\n", fpo);
	fputs(cbuffer, fpo);
	fputs("\n", fpo);
	trailer(fpo);
		/*	showpage and trailer should be written to fpo	*/
      fclose(fpo);

	if (lbp_output == NOHOLD)	{
		strcpy(cbuffer,"lpr -r -s -h -P");
		strcat(cbuffer,LPname);
            strcat(cbuffer," ");
		strcat(cbuffer, oldfilename);
		execute(cbuffer);
	}
}

LBP_setrgbcolor_(r,g,b)
	int *r,*g,*b;
/*    r,g,b = 0 to 255
      if colormode=INDEXEDCOLOR, r contains color index (1,2,,,256)	*/
{
	float rr,gg,bb;

	if (colorprint &&
	  ( currentcolor.r != (*r) ||
	    currentcolor.g != (*g) ||
	    currentcolor.b != (*b) ) )	{
		if (colormode == DIRECTCOLOR)	{
			rr = (float)(*r)/255.0;
			gg = (float)(*g)/255.0;
			bb = (float)(*b)/255.0;
		}
		else	{
			rr = (float)(int_r[(*r)-1])/255.0;
			gg = (float)(int_g[(*r)-1])/255.0;
			bb = (float)(int_b[(*r)-1])/255.0;
		}
		sprintf(cbuffer, "%.3f %.3f %.3f C", rr,gg,bb);
		LBP_fputs(cbuffer, LBPSET);
	} else	{
/*	reset gray level to black	*/
		if (currentint != 0.0)	{
			sprintf(cbuffer, "0 G");
			LBP_fputs(cbuffer, LBPSET);
		}
	}
	currentcolor.r = (*r);
	currentcolor.g = (*g);
	currentcolor.b = (*b);
	currentint = 0.0;
}

LBP_setgrayscale_(r,g,b)
	int *r,*g,*b;
/*    r,g,b = 0 to 255
      if colormode=INDEXEDCOLOR, r contains color index (1,2,,,256)	*/
{
	float rr,gg,bb,intensity;
	if (colormode == DIRECTCOLOR)	{
		rr = (float)(*r)/255.0;
		gg = (float)(*g)/255.0;
		bb = (float)(*b)/255.0;
	}
	else	{
		rr = (float)(int_r[(*r)-1])/255.0;
		gg = (float)(int_g[(*r)-1])/255.0;
		bb = (float)(int_b[(*r)-1])/255.0;
	}
	intensity = (rr+gg+bb)/3.0;
	if (intensity != currentint)	{
		sprintf(cbuffer, "%.3f G", 1-intensity);
/*	intensity=0 is black, intensity=1 is white
	for halftone print-out, make it reversed	*/
		LBP_fputs(cbuffer, LBPSET);
		currentint = intensity;
	}
}

static arc1_(openclose,cx,cy,r,angles,anglee,type_cgi)
	int openclose,type_cgi;
	float cx,cy,r,angles,anglee;
/*	openclose=	0:	open arc
			1:	closed arc
	type_cgi=1,(type_lbp=0):	pie (default)
	         0,          1 :	chord		

	arguments to this routine are passed by value
	cx, cy, r should have been converted to printer coordinates by
	the caller in order to enable origin/scaling changes in the caller
*/
{
	LBP_linecolor_(&linecolor.r, &linecolor.g, &linecolor.b);

	LBP_fputs("N", LBPDRAW);
	if( openclose == 1 && type_cgi == 1 )	{
            sprintf(cbuffer, "%.2f %.2f M", cx, cy );
		LBP_fputs(cbuffer, LBPDRAW);
	}

	if (xfactor < 0.0)	{
		angles = 180.0 - angles;
		anglee = 180.0 - anglee;
	}
	if (yfactor < 0.0)	{
		angles = -angles;
		anglee = -anglee;
	}
	sprintf(cbuffer, "%.2f %.2f %.2f %.2f %.2f", 
		cx, cy, r, angles, anglee);
	LBP_fputs(cbuffer, LBPDRAW);

	if (xfactor*yfactor > 0.0)
		LBP_fputs("arc", LBPDRAW);
	else
		LBP_fputs("arcn", LBPDRAW);

	if( openclose == 1 )	{
		LBP_fputs("closepath", LBPDRAW);
		dofill( fillstyle, hatchindex, edgevisibility );
	}
	else
		LBP_fputs("S",LBPDRAW);
}

LBP_arc_(cx,cy,r,angles,anglee)
	int *cx,*cy,*r;
	float *angles,*anglee;
/*	callable from cgi	*/
{
	int openclose = 0;	/*	open arc	*/
	int type=0;		/*	dummy		*/
	arc1_(openclose,xconv(*cx),yconv(*cy),lconv(*r),*angles,*anglee,type);
}

static ellipticalarc_(openclose,cx,cy,a,b,anglea,angles,anglee,type_cgi)
	int *openclose,*cx,*cy,*a,*b,*type_cgi;
	float *anglea,*angles,*anglee;
/*	openclose=	0:	open arc
		  	1:	closed arc

	for closed arc:
	type_cgi=1,(type_lbp=0):	pie (default)
	         0,          1 :	chord			*/
{
	float ratio, anga;
	
      anga = *anglea;
      if (xfactor < 0.0) anga = 180.0 - anga;
      if (yfactor < 0.0) anga = -anga;

	ratio = ( (float)(*b)) / ( (float)(*a));

	LBP_linecolor_(&linecolor.r, &linecolor.g, &linecolor.b);

	sprintf(cbuffer,
	"gsave %.2f %.2f translate %.2f rotate 1 %f scale",
		xconv(*cx), yconv(*cy), anga, ratio );
	LBP_fputs(cbuffer, LBPDRAW);
	arc1_(*openclose, 0.0, 0.0, lconv(*a), *angles, *anglee, *type_cgi);
	LBP_fputs("grestore", LBPDRAW);
}

LBP_elliparc_(cx,cy,a,b,anglea,angles,anglee)
	int *cx,*cy,*a,*b;
	float *anglea,*angles,*anglee;
{
	int openclose = 0;	/*	open arc	*/
	int type=0;		/*	dummy		*/
	ellipticalarc_(&openclose,cx,cy,a,b,anglea,angles,anglee,&type);
}



/*	functions usable both in LBP and CGI	*/

LBP_clipindicator_(indicator)
	int *indicator;
/*	CGI:
	indicator=0(default):	clip rectangle
	         =1:			view surface
	         =2:			off		
	LBP:
	indicator=0:			clip on
	          1(default)	clip off		*/
{
}

LBP_cliprectangle_(x1,y1,x2,y2)
	int *x1,*y1,*x2,*y2;
{	
	float xx1,xx2,yy1,yy2;

	xx1= xconv(*x1);	xx2= xconv(*x2);
	yy1= yconv(*y1);	yy2= yconv(*y2);

	LBP_fputs("N", LBPDRAW);
	sprintf(cbuffer, "%.2f %.2f M", xx1,yy1);
	LBP_fputs(cbuffer, LBPDRAW);
	sprintf(cbuffer, "%.2f %.2f D", xx2,yy1);
	LBP_fputs(cbuffer, LBPDRAW);
	sprintf(cbuffer, "%.2f %.2f D", xx2,yy2);
	LBP_fputs(cbuffer, LBPDRAW);
	sprintf(cbuffer, "%.2f %.2f D", xx1,yy2);
	LBP_fputs(cbuffer, LBPDRAW);
	LBP_fputs("closepath clip", LBPDRAW);
}

LBP_polyline_(np,x,y)
	int *np,x[],y[];
{
	int i;

	LBP_linecolor_(&linecolor.r, &linecolor.g, &linecolor.b);

	LBP_fputs("N", LBPDRAW);
	sprintf(cbuffer, "%.2f %.2f M", xconv(x[0]), yconv(y[0]));
	LBP_fputs(cbuffer, LBPDRAW);

	for (i=1; i< *np; i++)	{
		sprintf(cbuffer, "%.2f %.2f D", xconv(x[i]), yconv(y[i]));
		LBP_fputs(cbuffer, LBPDRAW);
	}
		
	LBP_fputs("S", LBPDRAW);
}

LBP_disjointline_(np,x,y)
	int *np,x[],y[];
{
	int i, two=2;

	for (i=0; i< (*np) -1; i+=2)	{
		LBP_polyline_(&two, x+i, y+i);
	}
}

LBP_linetype_(type)
	int *type;
/*
	type=1(default):		solid
	    =2:			dashed	--> LBP's short dashed
	    =3:			dotted	--> LBP's short dotted
	    =4:			dash-dot
	    =5:			dash-dot-dot

*/
{
	static char *linetypes[]={"solid",
			"dashed","dotted","dashdot","dashdotdot"};

	if (*type >= 1 && *type <= 5 )	{
		LBP_fputs(linetypes[(*type)-1],LBPSET);
	}
}

LBP_linecolor_(r,g,b)
	int *r,*g,*b;
/*    r,g,b = 0 to 255
      if colormode=INDEXEDCOLOR, r contains color index (1,2,,,256)	*/
{
	LBP_setrgbcolor_(r,g,b);
/*
	call setrgbcolor regardless of colorprint = T or F
	setrgbcolor sets grayscale to 0 (black) if colorprint = F
*/
	linecolor.r = (*r);
	linecolor.g = (*g);
	linecolor.b = (*b);
}

LBP_polymarker_(np,x,y) 
	int *np,x[],y[];
{
	static char *markertypes[] = {"dotmark", "plusmark", "asterisk",
		"circlemark", "crossmark", "squaremark", "filledsquaremark", 
		"trianglemark", "filledtrianglemark" };

	int i;

	LBP_markercolor_(&markercolor.r,
		&markercolor.g, &markercolor.b);

	LBP_fputs("N", LBPDRAW);
	for (i=0; i< *np; i++)	{
		sprintf(cbuffer, "%.2f %.2f M %s", 
			xconv(x[i]), yconv(y[i]), markertypes[markertype -1]);
		LBP_fputs(cbuffer, LBPDRAW);
	}
}

LBP_markertype_(type)
	int *type;
/*
	type=1:		dot (default of cgi)
	    =2:		plus
	    =3:		asterisk (default of lips, which is overridden)
	    =4:		circle
	    =5:		cross	
	    =6:		square		(LBP only)
	    =7:		filled square	(LBP only)
	    =8:		triangle		(LBP only)
	    =9		filled triangle	(LBP only)
*/
{
	if (*type >= 1 && *type <= 9 )	{
		markertype = *type;
	}
}

LBP_markercolor_(r,g,b)
	int *r,*g,*b;
/*    r,g,b = 0 to 255
      if colormode=INDEXEDCOLOR, r contains color index (1,2,,,256)	*/
{
	LBP_setrgbcolor_(r,g,b);
	markercolor.r = (*r);
	markercolor.g = (*g);
	markercolor.b = (*b);
}

static dofill(style,index,edge)
	int style,index,edge;
/*
	style=1:	hollow (default)
	    	2:	solid
 		3:	user-defined pattern
	  	4:	hatch
	     	5:	empty (not applicable to LBP)

	for style=4, hatch index should be specified

	index=1:          horizontal parallel lines
           =2(default): vertical parallel lines
           =3:          positive slope parallel lines
           =4:          negative slope parallel lines
           =5:          horizontal/vertical crosshatch
           =6:          positive/negative slope crosshatch

	edge = 0: visible
             1: invisivble (default)
*/
{
	LBP_fillcolor_(&fillcolor.r, &fillcolor.g, &fillcolor.b);

	if (style == 1)	edge = 0;
		/*	for interiorstyle = hollow, edge is drawn	*/

	if (edge == 0) LBP_fputs("S", LBPDRAW);

	if (style == 2) LBP_fputs("gsave fill grestore", LBPDRAW);

	if (style == 4)	{
		switch (index) {
		case 1:	LBP_fputs("horizontals", LBPDRAW);	break;
		case 2:	LBP_fputs("verticals", LBPDRAW);	break;
		case 3:	LBP_fputs("positiveslopes", LBPDRAW);	break;
		case 4:	LBP_fputs("negativeslopes", LBPDRAW);	break;
		case 5:	LBP_fputs("horizontals verticals", LBPDRAW);	break;
		case 6:	LBP_fputs("positiveslopes negativeslopes", LBPDRAW);
				break;
		}
	}
}

LBP_interiorstyle_(style)
	int *style;
{
	fillstyle = *style;
}

LBP_hatchindex_(index)
	int *index;
{
	hatchindex = *index;
}

LBP_edgevisibility_(edge)
	int *edge;
{
	edgevisibility = *edge;
}

LBP_fillcolor_(r,g,b)
	int *r,*g,*b;
/*    r,g,b = 0 to 255
      if colormode=INDEXEDCOLOR, r contains color index (1,2,,,256)	*/
{
	if (colorprint)	{
		LBP_setrgbcolor_(r,g,b);
	} else	{
		LBP_setgrayscale_(r,g,b);
	}
	fillcolor.r = (*r);
	fillcolor.g = (*g);
	fillcolor.b = (*b);
}

LBP_polygon_(np,x,y) 
	int *np,x[],y[];
{
	int i;

	LBP_linecolor_(&linecolor.r, &linecolor.g, &linecolor.b);

      LBP_fputs("N", LBPDRAW);
      sprintf(cbuffer, "%.2f %.2f M", xconv(x[0]), yconv(y[0]));
      LBP_fputs(cbuffer, LBPDRAW);

      for (i=1; i< *np; i++)	{
            sprintf(cbuffer, "%.2f %.2f D", xconv(x[i]), yconv(y[i]));
		LBP_fputs(cbuffer, LBPDRAW);
	}

      LBP_fputs("closepath", LBPDRAW);

	dofill( fillstyle, hatchindex, edgevisibility);
}

LBP_rectangle_(x1,y1,x2,y2)
	int *x1,*y1,*x2,*y2;
{
	float xx1,xx2,yy1,yy2;

	xx1= xconv(*x1);	xx2= xconv(*x2);
	yy1= yconv(*y1);	yy2= yconv(*y2);

	LBP_linecolor_(&linecolor.r, &linecolor.g, &linecolor.b);

	LBP_fputs("N", LBPDRAW);
	sprintf(cbuffer, "%.2f %.2f M", xx1,yy1);
	LBP_fputs(cbuffer, LBPDRAW);
	sprintf(cbuffer, "%.2f %.2f D", xx2,yy1);
	LBP_fputs(cbuffer, LBPDRAW);
	sprintf(cbuffer, "%.2f %.2f D", xx2,yy2);
	LBP_fputs(cbuffer, LBPDRAW);
	sprintf(cbuffer, "%.2f %.2f D", xx1,yy2);
	LBP_fputs(cbuffer, LBPDRAW);
	LBP_fputs("closepath", LBPDRAW);

	dofill( fillstyle, hatchindex, edgevisibility);
}

LBP_circle_(cx,cy,r)
	int *cx,*cy,*r;
{
	LBP_linecolor_(&linecolor.r, &linecolor.g, &linecolor.b);

	LBP_fputs("N", LBPDRAW);
	sprintf(cbuffer, "%.2f %.2f %.2f 0 360 arc closepath",
		xconv(*cx), yconv(*cy), lconv(*r) );
	LBP_fputs(cbuffer, LBPDRAW);

	dofill( fillstyle, hatchindex, edgevisibility);
}

LBP_arcclose_(cx,cy,r,angles,anglee,type)
	int *cx,*cy,*r,*type;
	float *angles, *anglee;
{
	int openclose = 1;	/*	closed arc	*/
	arc1_(openclose,xconv(*cx),yconv(*cy),lconv(*r),
		*angles, *anglee, *type);
}

LBP_ellipse_(cx,cy,a,b,anglea)
	int *cx,*cy,*a,*b;
	float *anglea;
{
	int openclose = 1, type_cgi=0;	/*	closed arc by chord	*/
      float ratio, anga;

      anga = *anglea;
      if (xfactor < 0.0) anga = 180.0 - anga;
      if (yfactor < 0.0) anga = -anga;

	ratio = ( (float)(*b)) / ( (float)(*a));

	LBP_linecolor_(&linecolor.r, &linecolor.g, &linecolor.b);

      sprintf(cbuffer,
      "gsave %.2f %.2f translate %.2f rotate 1 %f scale",
		xconv(*cx), yconv(*cy), anga, ratio );
      LBP_fputs(cbuffer, LBPDRAW);
      arc1_(openclose, 0.0, 0.0, lconv(*a), 0.0, 360.0, type_cgi);
      LBP_fputs("grestore", LBPDRAW);

	dofill( fillstyle, hatchindex, edgevisibility);
}

LBP_elliparcclose_(cx,cy,a,b,anglea,angles,anglee,type)
	int *cx,*cy,*a,*b,*type;
	float *anglea,*angles,*anglee;
/*	type=	0:	pie (default)
		1:	chord		*/
{
	int openclose = 1;	/*	closed arc	*/
	ellipticalarc_(&openclose,cx,cy,a,b,anglea,angles,anglee,type);
}

LBP_text_(x,y,string,len)
	int *x,*y,len;
	char *string;
/*	FORTRAN calling sequence: call text(x,y,string)
	string is of length len and not null-terminated	*/
{
	LBP_textcolor_(&textcolor.r, &textcolor.g, &textcolor.b );

	sprintf(cbuffer, "gsave %.2f %.2f 2 copy translate %.2f rotate",
		xconv(*x), yconv(*y), textbaseangle );
	LBP_fputs(cbuffer, LBPDRAW);

	sprintf(cbuffer, "0 0 moveto %.2f 0 (", chrspacing);
	strncat(cbuffer, string, len);
	strcat(cbuffer, ") ashow currentpoint /y exch def /x exch def \
		 grestore y add exch x add exch moveto");
	LBP_fputs(cbuffer, LBPDRAW);
}

LBP_textnf_(x,y,string,len)
	int *x,*y,len;
	char *string;
{
	LBP_text_(x,y,string,len);
}

LBP_textcolor_(r,g,b)
	int *r,*g,*b;
/*    r,g,b = 0 to 255
      if colormode=INDEXEDCOLOR, r contains color index (1,2,,,256)	*/
{
	LBP_setrgbcolor_(r,g,b);
	textcolor.r = (*r);
	textcolor.g = (*g);
	textcolor.b = (*r);
}

LBP_chrheight_(iheight)
        int *iheight;
/*    height is in VDC unit	*/
{
	chrheight = lconv(*iheight);
	sprintf(cbuffer, "/%s findfont %.2f scalefont setfont",
		fontnamelist[lbp_font-1], chrheight);
	LBP_fputs(cbuffer, LBPSET);
}

LBP_chrwidth_(ratio)
	float *ratio;
/*	ratio: width to height ratio	*/
{
	sprintf(cbuffer, "/%s findfont [%.2f 0 0 %.2f 0 0] makefont setfont",
		fontnamelist[lbp_font-1], chrheight, chrheight*(*ratio));
	LBP_fputs(cbuffer, LBPSET);
}

LBP_chrspacing_(spacing)
	float *spacing;
/*	character spacing in units of character height
	standard spacing -- spacing=0	*/
{
	chrspacing = chrheight*(*spacing);
}

LBP_chrorientation_(angleb,angleu)
	float *angleb,*angleu;
/*	direction of base-vector -- angleb
	             up-vector   -- angleu	*/
{
	textbaseangle = *angleb;
}


LBP_textpath_(path)
	int *path;
/*	path =0:	left
	      1:	up
	      2:	right (default)
	      3:	down		*/
{
	textpath = *path;
}

LBP_fontindex_(index_f)
	int *index_f;
{
/*	index_f = 1,2,,,	*/

	lbp_font = *index_f ;
	sprintf(cbuffer,"/%s findfont %.2f scalefont setfont",
		fontnamelist[lbp_font -1], chrheight );
	LBP_fputs(cbuffer,LBPSET);
}

LBP_colormode_(mode)
      int *mode;
/*    the argument 'mode' is dummy and external variable 'colormode'
	is referenced 	*/
{
	if ( colormode == INDEXEDCOLOR)	{
		linecolor.r = 1;
		textcolor.r = 1;
		fillcolor.r = 1;
		markercolor.r = 1;
		int_r[0] = 0;
		int_g[0] = 0;
		int_b[0] = 0;
	}
	else	{
		linecolor.r = 0;
		linecolor.g = 0;
		linecolor.b = 0;
		textcolor.r = 0;
		textcolor.g = 0;
		textcolor.b = 0;
		fillcolor.r = 0;
		fillcolor.g = 0;
		fillcolor.b = 0;
		markercolor.r = 0;
		markercolor.g = 0;
		markercolor.b = 0;
	}
}

LBP_colortable_(startindex_f,ncolor,r,g,b)
      unsigned int *startindex_f,*ncolor,r[],g[],b[];
/*    startindex_f = 1,2,,,256
      ncolor = 1,2,,,256      */
{
      int i, startindex;

      startindex = *startindex_f -1;
      for(i=0; i< *ncolor; i++)     {
            int_r[startindex+i]  = r[i];
            int_g[startindex+i]  = g[i];
            int_b[startindex+i]  = b[i];
      }
	if ( *startindex_f == 1 )	{
            int_r[startindex]  = 0;
            int_g[startindex]  = 0;
            int_b[startindex]  = 0;
	}
/*	startindex_f =1 means default foreground color,
	which is black on white paper	*/
}

LBP_byteimage_(xo,yo,nx,ny,x1,x2,y1,y2,image)
      int *xo,*yo,*nx,*ny,*x1,*x2,*y1,*y2;
      unsigned char *image;
{
	int x,y,width,height;
	unsigned char *p;

	LBP_fputs("gsave N", LBPDRAW);
	sprintf(cbuffer, "%.2f %.2f M", xconv(*xo), yconv(*yo));
	LBP_fputs(cbuffer, LBPDRAW);
	
	width = LBPdotsize*abs((*x2)-(*x1)+1);	/* in unit of dots	*/
	height= LBPdotsize*abs((*y2)-(*y1)+1);
	sprintf(cbuffer, "%.2f %.2f scale",LBPDRAW);
	LBP_fputs(cbuffer, LBPDRAW);
		/*	reserve a rentangular area of width*height dots	*/

	sprintf(cbuffer, "[ %d %d 8 %.2f 0 0 %.2f 0 %.2f ] {",
		(int)width, (int)height, width, height,height);
	LBP_fputs(cbuffer, LBPDRAW);

	for (y = *y1; y <= *y2; y++)	{
		p = image + (*nx)*(y-1);
		for (x = *x1; x <= *x2; x++)	{
			sprintf(cbuffer, "<0x%2.2x>",*(p++));
			fputs(cbuffer,fp);
		}
		fputc('\n',fp);
	}
	LBP_fputs("} image grestore", LBPDRAW);
}

LBP_bitwiseimage_(xo,yo,nx,ny,x1,x2,y1,y2,nplane,bitmap)
      int *xo,*yo,*nx,*ny,*x1,*x2,*y1,*y2,*nplane;
      unsigned short *bitmap;
{
}


