/*
	CALCOMP to UNIXPLOT/LBP interface subroutines
	callable from FORTRAN programs
	linetype of unixplot is fixed to 'solid'

	plots(x,y,imode)
	plot(x,y,mode)
	symbol(x,y,h,str,angle,nchar)
	scale(f)
	newpen(ipen)
	plote()

	for tektronix 4013 display or emulator:
	f77 ,,, /home/common/plotlib/plot-lbp.o
	        /home/common/plotlib/4013lib.o 
	        /home/common/lbplib/lips2lib.o (or other printer driver)
	        -l4013
*/

#include <string.h>

float plot_scalefactor;
static float xo, yo;
static int imode0,line, ixx[2], iyy[2];


/*	dummy parameters for color plotting	*/
unsigned int int_r[256],int_g[256],int_b[256];
int colormode;

/*	scale conversion 
	calcomp routines: unit=1cm
	lbp               unit=1/10mm
*/

#define LBPFACTOR 100.0

int lbpconv(coord, org)
	float coord, org;
{
	return ( (int) ( LBPFACTOR * ( plot_scalefactor * coord + org ) ) );
}

plots_(x,y,imode)
	float *x,*y;
	int *imode;
/*
	initialize the device and set the origin at (x,y)
	imode: 1 -- lbp = portrait  orientation, with screen output
	       2 --       landscape orientation, with screen output
	      -1 -- lbp = portrait  orientation, no   screen output
	      -2 --       landscape orientation, no   screen output
	       0 -- no lbp output (screen output only)
*/

{
	int vpunits=1, minx=0, miny=0;
/*	int maxx=1820, maxy=2010;		B5 size	*/
	int maxx=1820, maxy=2160;
	/*	  x-size = B5-size, y-size = screen height -2cm	*/

	float cwidth=0.5, cspace=0.05;

	imode0 = *imode;
	plot_scalefactor = 1.0;
	xo = 0.0;
	yo = 0.0;
	ixx[0] = lbpconv(*x, xo);
	iyy[0] = lbpconv(*y, yo);
	xo = *x;
	yo = *y;
	line=1;

	if ( imode0 != 0) {
		LBP_initialize_(&imode0);
		LBP_vdcextent_(&minx, &miny, &maxx, &maxy );
		LBP_viewportunits_( &vpunits );
		LBP_viewport_( &minx, &maxy, &maxx, &miny );
			/*	origin = lower left corner	*/
/*		LBP_chrwidth_(&cwidth);			*/
/*		LBP_chrspacing_(&cspace);		*/

/*	origin = upper left
		LBP_viewport_( &minx, &miny, &maxx, &maxy );	*/

	}
/*	if ( imode0 >= 0) openpl();	*/
}

plot_(x,y,mode)
	float *x, *y;
	int *mode;
/*
  |mode|=3: move point to (x,y)
         2: draw line  to (x,y)
        12: draw short-dashed line to (x,y)
        22: draw dotted line to (x,y)
        32: draw dot-dashed line to (x,y)
        42: draw long-dhashed line to (x,y)
       999: print out
      >999: erase output
  if mode<0, origin is moved to (x,y)
*/
{
	int ix,iy, np1=1, np2=2;

	if( *mode >= 999) {
		ixx[0] = lbpconv(*x, xo);
		iyy[0] = lbpconv(*y, yo);
		if ( *mode == 999) {
			if( imode0 != 0 )	LBP_hcopy_();
		}
		else	{
			if ( imode0 >= 0 ) erase();
			if ( imode0 != 0 ) LBP_clear_();
		}
		return;
	}

	ixx[1] = lbpconv( *x , xo );
	iyy[1] = lbpconv( *y , yo );
	ix = plotconv( *x , xo );
	iy = plotconv( *y , yo );

	if ( abs(*mode)/10 +1 != line && abs(*mode)%10 == 2 ) {
		line = abs(*mode)/10 +1;
		if ( imode0 != 0 ) LBP_linetype_( &line );
	}

	if ( (abs( *mode) % 10) == 2 ) {
		if ( imode0 != 0 ) LBP_polyline_( &np2, ixx, iyy);
		if ( imode0 >= 0 ) cont(ix,iy);
/*		if ( imode0 >= 0 ) printf("\ncont,%4x %4x, %4x %4x",
			ix/32, ix%32, iy/32, iy%32);
*/
	}
	else {
		if ( imode0 != 0 ) LBP_polyline_( &np1, &ixx[1],&iyy[1]);
		if ( imode0 >= 0 ) move(ix,iy);	
/*		if ( imode0 >= 0 ) printf("\nmove,%4x %4x, %4x %4x",
			ix/32, ix%32, iy/32, iy%32);
*/
	}
	ixx[0] = ixx[1];
	iyy[0] = iyy[1];
	if ( *mode < 0 ) {
		xo += (*x)*plot_scalefactor;
		yo += (*y)*plot_scalefactor;
	}
}

symbol_(x,y,h,str,angle,nchar,lstr)
	float *x, *y, *h, *angle;
	int *nchar, lstr;
	char *str;
{
	char buf[257], *strncpy();
	int ix,iy, hh, n, np1=1;
	float angleu;

	hh = lbpconv( *h , 0.0 );
	ixx[0] = lbpconv( *x , xo);
	iyy[0] = lbpconv( *y , yo);
	ix = plotconv( *x , xo);
	iy = plotconv( *y , yo);
	n= *nchar;

	if ( n >= 0 ) {
		if ( imode0 != 0 ) {
			angleu = *angle + 90.0;
			LBP_chrorientation_( angle, &angleu );
			LBP_chrheight_( &hh );
			LBP_text_( &ixx[0], &iyy[0], str, n );
		}
		if ( imode0 >= 0 ) {
			move(ix,iy);
/*		if ( imode0 >= 0 ) printf("\nmove,%4x %4x, %4x %4x",
			ix/32, ix%32, iy/32, iy%32);
*/
          		strncpy(buf,str,n);
            	buf[n]='\0';
			label(buf);
/*		printf("\nlabel,%s",buf);
*/
		}
	}
	else if ( imode0 != 0 ) {
/*		LBP_markersize_( &hh );	*/
		n = -n;
		if ( n < 1 ) n = 1;
		if ( n > 5 ) n = 1;
		LBP_markertype_( &n );
		LBP_polymarker_( &np1,&ixx[0],&iyy[0]);
	}
}

factor_(f)
	float *f;
{
	plot_scalefactor = *f;
}

newpen_(ipen)
	int *ipen;
{
	float f;

	if ( imode0 != 0) {
		f = (*ipen)/10 +1;
		LBP_linefwidth_( &f );
	}

}

plote_()
{
	if ( imode0 != 0 ) LBP_terminate_();
	if ( imode0 >= 0 ) closepl();
}
